@@ -432,7 +432,7 @@ nc_grid_index(stream_t *streamptr, int gridID)
 {
   int index = 0;
   int vlistID = streamptr->vlistID;
-  int ngrids = vlistNgrids(vlistID);
+  int ngrids = vlistNumGrids(vlistID);
   for (index = 0; index < ngrids; ++index)
     if (streamptr->ncgrid[index].gridID == gridID) break;
 
@@ -855,12 +855,13 @@ cdfCheckVarname(int fileID, char name[CDI_MAX_NAME])
       int ncvarID;
       char varname[CDI_MAX_NAME];
       snprintf(varname, sizeof(varname), "%s", name);
-      char *varname2 = varname + strlen(varname);
+      size_t len = strlen(varname);
+      char *varname2 = varname + len;
       unsigned iz = 0;
 
       do
         {
-          if (iz) sprintf(varname2, "_%u", iz + 1);
+          if (iz) snprintf(varname2, sizeof(varname) - len, "_%u", iz + 1);
 
           if (nc_inq_varid(fileID, varname, &ncvarID) != NC_NOERR) break;
 
@@ -891,13 +892,14 @@ cdfGenVarname(int fileID, char name[CDI_MAX_NAME], int pnum, int pcat, int *pdis
   else
     snprintf(varname, sizeof(varname), "param%d.%d.%d", pnum, pcat, *pdis);
 
-  char *varname2 = varname + strlen(varname);
+  size_t len = strlen(varname);
+  char *varname2 = varname + len;
   int ncvarID;
   unsigned iz = 0;
 
   do
     {
-      if (iz) sprintf(varname2, "_%u", iz + 1);
+      if (iz) snprintf(varname2, sizeof(varname) - len, "_%u", iz + 1);
 
       if (nc_inq_varid(fileID, varname, &ncvarID) != NC_NOERR) break;
 
@@ -979,7 +981,7 @@ cdfDefVar(stream_t *streamptr, int varID)
     cdfGenVarname(fileID, name, pnum, pcat, &pdis, &code);
 
   int dtype = vlistInqVarDatatype(vlistID, varID);
-  const nc_type xtype = cdfDefDatatype(dtype, streamptr);
+  nc_type xtype = cdfDefDatatype(dtype, streamptr);
 
   if (streamptr->ncmode == 2)
     {
@@ -1109,7 +1111,7 @@ cdfWriteGridTraj(stream_t *streamptr, int gridID)
 
 static void
 cdf_write_var_data(int fileID, int vlistID, int varID, int ncvarID, int dtype, size_t nvals, size_t xsize, size_t ysize,
-                   bool swapxy, size_t *start, size_t *count, int memtype, const void *data, size_t nmiss)
+                   bool swapxy, size_t *start, size_t *count, int memtype, const void *data, size_t numMissVals)
 {
   const double *pdata_dp = (const double *) data;
   double *mdata_dp = NULL;
@@ -1133,7 +1135,7 @@ cdf_write_var_data(int fileID, int vlistID, int varID, int ncvarID, int dtype, s
             memcpy(mdata_sp, pdata_sp, nvals * sizeof(float));
             pdata_sp = mdata_sp;
 
-            if (nmiss > 0)
+            if (numMissVals > 0)
               {
                 for (size_t i = 0; i < nvals; ++i)
                   {
@@ -1163,7 +1165,7 @@ cdf_write_var_data(int fileID, int vlistID, int varID, int ncvarID, int dtype, s
             memcpy(mdata_dp, pdata_dp, nvals * sizeof(double));
             pdata_dp = mdata_dp;
 
-            if (nmiss > 0)
+            if (numMissVals > 0)
               {
                 for (size_t i = 0; i < nvals; ++i)
                   {
@@ -1261,7 +1263,7 @@ cdf_write_var_data(int fileID, int vlistID, int varID, int ncvarID, int dtype, s
               }
           }
 
-        Message("nvals = %zu, nmiss = %d, missval = %g, minval = %g, maxval = %g", nvals, nmiss, missval, fmin, fmax);
+        Message("nvals = %zu, numMissVals = %d, missval = %g, minval = %g, maxval = %g", nvals, numMissVals, missval, fmin, fmax);
       }
   }
 
@@ -1362,7 +1364,7 @@ cdfDefineStartAndCount(stream_t *streamptr, int varID, int xid, int yid, int zid
   int vlistID = streamptr->vlistID;
   int fileID = streamptr->fileID;
 
-  const long ntsteps = streamptr->ntsteps;
+  long ntsteps = streamptr->ntsteps;
   if (CDI_Debug) Message("ntsteps = %ld", ntsteps);
 
   int timetype = vlistInqVarTimetype(vlistID, varID);
@@ -1407,7 +1409,7 @@ cdfDefineStartAndCount(stream_t *streamptr, int varID, int xid, int yid, int zid
 }
 
 void
-cdf_write_var(stream_t *streamptr, int varID, int memtype, const void *data, size_t nmiss)
+cdf_write_var(stream_t *streamptr, int varID, int memtype, const void *data, size_t numMissVals)
 {
   if (streamptr->accessmode == 0) cdfEndDef(streamptr);
 
@@ -1436,12 +1438,12 @@ cdf_write_var(stream_t *streamptr, int varID, int memtype, const void *data, siz
 
   int dtype = vlistInqVarDatatype(vlistID, varID);
 
-  if (nmiss > 0) cdfDefVarMissval(streamptr, varID, dtype, 1);
+  if (numMissVals > 0) cdfDefVarMissval(streamptr, varID, dtype, 1);
 
   size_t nvals = gridInqSize(gridID) * (size_t) (zaxisInqSize(zaxisID));
 
   bool swapxy = false;
-  cdf_write_var_data(fileID, vlistID, varID, ncvarID, dtype, nvals, xsize, ysize, swapxy, start, count, memtype, data, nmiss);
+  cdf_write_var_data(fileID, vlistID, varID, ncvarID, dtype, nvals, xsize, ysize, swapxy, start, count, memtype, data, numMissVals);
 }
 
 static void
@@ -1455,7 +1457,7 @@ cdfDefineStartAndCountChunk(stream_t *streamptr, const int rect[][2], int varID,
   int vlistID = streamptr->vlistID;
   int fileID = streamptr->fileID;
 
-  const long ntsteps = streamptr->ntsteps;
+  long ntsteps = streamptr->ntsteps;
   if (CDI_Debug) Message("ntsteps = %ld", ntsteps);
 
   int timetype = vlistInqVarTimetype(vlistID, varID);
@@ -1502,7 +1504,7 @@ cdfDefineStartAndCountChunk(stream_t *streamptr, const int rect[][2], int varID,
 }
 
 void
-cdf_write_var_chunk(stream_t *streamptr, int varID, int memtype, const int rect[][2], const void *data, size_t nmiss)
+cdf_write_var_chunk(stream_t *streamptr, int varID, int memtype, const int rect[][2], const void *data, size_t numMissVals)
 {
   if (streamptr->accessmode == 0) cdfEndDef(streamptr);
 
@@ -1533,12 +1535,12 @@ cdf_write_var_chunk(stream_t *streamptr, int varID, int memtype, const int rect[
 
   int dtype = vlistInqVarDatatype(vlistID, varID);
 
-  if (nmiss > 0) cdfDefVarMissval(streamptr, varID, dtype, 1);
+  if (numMissVals > 0) cdfDefVarMissval(streamptr, varID, dtype, 1);
 
   size_t nvals = gridInqSize(gridID) * (size_t) (zaxisInqSize(zaxisID));
 
   bool swapxy = false;
-  cdf_write_var_data(fileID, vlistID, varID, ncvarID, dtype, nvals, xsize, ysize, swapxy, start, count, memtype, data, nmiss);
+  cdf_write_var_data(fileID, vlistID, varID, ncvarID, dtype, nvals, xsize, ysize, swapxy, start, count, memtype, data, numMissVals);
 }
 
 static void
@@ -1552,7 +1554,7 @@ cdfDefineStartAndCountSlice(stream_t *streamptr, int varID, int levelID, int dim
   int vlistID = streamptr->vlistID;
   int fileID = streamptr->fileID;
 
-  const long ntsteps = streamptr->ntsteps;
+  long ntsteps = streamptr->ntsteps;
   if (CDI_Debug) Message("ntsteps = %ld", ntsteps);
 
   int timetype = vlistInqVarTimetype(vlistID, varID);
@@ -1593,7 +1595,7 @@ cdfDefineStartAndCountSlice(stream_t *streamptr, int varID, int levelID, int dim
 }
 
 void
-cdf_write_var_slice(stream_t *streamptr, int varID, int levelID, int memtype, const void *data, size_t nmiss)
+cdf_write_var_slice(stream_t *streamptr, int varID, int levelID, int memtype, const void *data, size_t numMissVals)
 {
   if (streamptr->accessmode == 0) cdfEndDef(streamptr);
 
@@ -1620,19 +1622,19 @@ cdf_write_var_slice(stream_t *streamptr, int varID, int levelID, int memtype, co
 
   int dtype = vlistInqVarDatatype(vlistID, varID);
 
-  if (nmiss > 0) cdfDefVarMissval(streamptr, varID, dtype, 1);
+  if (numMissVals > 0) cdfDefVarMissval(streamptr, varID, dtype, 1);
 
   size_t nvals = gridInqSize(gridID);
 
-  cdf_write_var_data(fileID, vlistID, varID, ncvarID, dtype, nvals, xsize, ysize, swapxy, start, count, memtype, data, nmiss);
+  cdf_write_var_data(fileID, vlistID, varID, ncvarID, dtype, nvals, xsize, ysize, swapxy, start, count, memtype, data, numMissVals);
 }
 
 void
-cdf_write_record(stream_t *streamptr, int memtype, const void *data, size_t nmiss)
+cdf_write_record(stream_t *streamptr, int memtype, const void *data, size_t numMissVals)
 {
   int varID = streamptr->record->varID;
   int levelID = streamptr->record->levelID;
-  cdf_write_var_slice(streamptr, varID, levelID, memtype, data, nmiss);
+  cdf_write_var_slice(streamptr, varID, levelID, memtype, data, numMissVals);
 }
 
 #endif
