@@ -20,7 +20,6 @@
 #include "cdo_vlist.h"
 #include "param_conversion.h"
 #include "util_string.h"
-#include "field_functions.h"
 
 /*
   Transformation from potential to in situ temperature according to Bryden, 1973,
@@ -133,8 +132,8 @@ get_code(int vlistID1, int varID, const std::string &cname)
       auto varname = string_to_lower(cdo::inq_var_name(vlistID1, varID));
       auto stdname = string_to_lower(cdo::inq_key_string(vlistID1, varID, CDI_KEY_STDNAME));
 
-      if (varname == "s" || stdname == "sea_water_salinity") { code = 5; }
-      else if (varname == "t") { code = 2; }
+      if (varname == "s" || varname == "so" || stdname == "sea_water_salinity") { code = 5; }
+      else if (varname == "t" || varname == "to") { code = 2; }
 
       if (stdname == cname) code = 2;
     }
@@ -155,23 +154,23 @@ struct IOSettings
 };
 
 IOSettings
-configureOutput(const std::function<void(const int, const int)> &outputSettingFunc, int vlistID1, int thoID, int saoID,
+configureOutput(const std::function<void(int, int)> &outputSettingFunc, int vlistID, int thoID, int saoID,
                 FieldVector &tho, FieldVector &sao, FieldVector &tis, Varray<double> &pressure)
 {
   double pin = (cdo_operator_argc() == 1) ? parameter_to_double(cdo_operator_argv(0)) : -1.0;
 
-  VarList varList1;
-  varList_init(varList1, vlistID1);
+  VarList varList(vlistID);
+  const auto &vars = varList.vars;
 
-  auto units = varList1[thoID].units;
+  auto units = vars[thoID].units;
   if (units.empty()) units = "Celcius";
 
-  auto gridID = vlistGrid(vlistID1, 0);
-  auto gridsize = vlist_check_gridsize(vlistID1);
+  auto gridID = vlistGrid(vlistID, 0);
+  auto gridsize = vlist_check_gridsize(vlistID);
 
-  auto nlevels1 = varList1[saoID].nlevels;
-  auto nlevels2 = varList1[thoID].nlevels;
-  auto zaxisID = varList1[thoID].zaxisID;
+  auto nlevels1 = vars[saoID].nlevels;
+  auto nlevels2 = vars[thoID].nlevels;
+  auto zaxisID = vars[thoID].zaxisID;
 
   if (nlevels1 != nlevels2) cdo_abort("temperature and salinity have different number of levels!");
   auto nlevels = nlevels1;
@@ -198,17 +197,16 @@ configureOutput(const std::function<void(const int, const int)> &outputSettingFu
       tho[levelID].resize(gridsize);
       sao[levelID].resize(gridsize);
       tis[levelID].resize(gridsize);
-      tho[levelID].missval = varList1[thoID].missval;
-      sao[levelID].missval = varList1[saoID].missval;
+      tho[levelID].missval = vars[thoID].missval;
+      sao[levelID].missval = vars[saoID].missval;
       tis[levelID].missval = tho[levelID].missval;
     }
 
   int datatype = CDI_DATATYPE_FLT32;
-  if (varList1[thoID].datatype == CDI_DATATYPE_FLT64 && varList1[saoID].datatype == CDI_DATATYPE_FLT64)
-    datatype = CDI_DATATYPE_FLT64;
+  if (vars[thoID].dataType == CDI_DATATYPE_FLT64 && vars[saoID].dataType == CDI_DATATYPE_FLT64) datatype = CDI_DATATYPE_FLT64;
 
   auto vlistID2 = vlistCreate();
-  vlistDefNtsteps(vlistID2, vlistNtsteps(vlistID1));
+  vlistDefNtsteps(vlistID2, vlistNtsteps(vlistID));
 
   auto tisID2 = vlistDefVar(vlistID2, gridID, zaxisID, TIME_VARYING);
 
@@ -226,7 +224,7 @@ configureOutput(const std::function<void(const int, const int)> &outputSettingFu
   vlistDefVarMissval(vlistID2, saoID2, sao[0].missval);
   vlistDefVarDatatype(vlistID2, saoID2, datatype);
 
-  auto taxisID1 = vlistInqTaxis(vlistID1);
+  auto taxisID1 = vlistInqTaxis(vlistID);
   auto taxisID2 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID2, taxisID2);
 
@@ -236,8 +234,21 @@ configureOutput(const std::function<void(const int, const int)> &outputSettingFu
   return IOSettings{ streamID2, vlistID2, gridsize, nlevels, taxisID1, taxisID2, tisID2, saoID2 };
 }
 
-class ModuleAdisit
+class Adisit : public Process
 {
+public:
+  using Process::Process;
+  inline static CdoModule module = {
+    .name = "Adisit",
+    .operators = { { "adisit", 0, 0, "", AdisitHelp }, { "adipot", 0, 0, "", AdisitHelp } },
+    .aliases = {},
+    .mode = EXPOSED,     // Module mode: 0:intern 1:extern
+    .number = CDI_REAL,  // Allowed number type
+    .constraints = { 1, 1, NoRestriction },
+  };
+  inline static RegisterEntry<Adisit> registration = RegisterEntry<Adisit>(module);
+
+  int ADISIT, ADIPOT;
   int thoID = -1, saoID = -1;
 
   CdoStreamID streamID1;
@@ -258,16 +269,12 @@ class ModuleAdisit
   FieldVector tho, sao, tis;
   Varray<double> pressure;
 
-  int ADISIT;
-
 public:
   void
-  init(void *process)
+  init() override
   {
-    cdo_initialize(process);
-
-    ADISIT = cdo_operator_add("adisit", 0, 0, "");
-    auto ADIPOT = cdo_operator_add("adipot", 0, 0, "");
+    ADISIT = module.get_id("adisit");
+    ADIPOT = module.get_id("adipot");
 
     operatorID = cdo_operator_id();
 
@@ -278,7 +285,7 @@ public:
 
     std::string cname_ADISIT{ "sea_water_potential_temperature" };
     std::string cname_ADIPOT{ "sea_water_temperature" };
-    std::string cname = std::string{ (operatorID == ADISIT) ? cname_ADISIT : cname_ADIPOT };
+    std::string cname = (operatorID == ADISIT) ? cname_ADISIT : cname_ADIPOT;
 
     for (int varID = 0; varID < nvars; ++varID)
       {
@@ -319,8 +326,9 @@ public:
     tisID2 = configResults.tisID2;
     saoID2 = configResults.saoID2;
   }
+
   void
-  run()
+  run() override
   {
     int tsID = 0;
     while (true)
@@ -335,8 +343,8 @@ public:
           {
             int varID, levelID;
             cdo_inq_record(streamID1, &varID, &levelID);
-            if (varID == thoID) cdo_read_record(streamID1, tho[levelID].vec_d.data(), &tho[levelID].nmiss);
-            if (varID == saoID) cdo_read_record(streamID1, sao[levelID].vec_d.data(), &sao[levelID].nmiss);
+            if (varID == thoID) cdo_read_record(streamID1, tho[levelID].vec_d.data(), &tho[levelID].numMissVals);
+            if (varID == saoID) cdo_read_record(streamID1, sao[levelID].vec_d.data(), &sao[levelID].numMissVals);
 
             if (varID == thoID)
               {
@@ -369,26 +377,13 @@ public:
         tsID++;
       }
   }
+
   void
-  close()
+  close() override
   {
     cdo_stream_close(streamID2);
     cdo_stream_close(streamID1);
 
     vlistDestroy(vlistID2);
-
-    cdo_finish();
   }
 };
-
-void *
-Adisit(void *process)
-{
-  ModuleAdisit adisit;
-
-  adisit.init(process);
-  adisit.run();
-  adisit.close();
-
-  return nullptr;
-}
