@@ -7,7 +7,6 @@
 
 #include <cdi.h>
 #include <algorithm>
-#include "julian_date.h"
 
 #include "cdo_options.h"
 #include "cdo_timer.h"
@@ -93,7 +92,7 @@ get_parameter(void)
       KVList kvlist;
       // kvlist.name = cdo_module_name();
       kvlist.name = "CDIwrite";
-      if (kvlist.parse_arguments(pargc, pargv) != 0) cdo_abort("Parse error!");
+      if (kvlist.parse_arguments(pargv) != 0) cdo_abort("Parse error!");
       if (Options::cdoVerbose) kvlist.print();
 
       for (const auto &kv : kvlist)
@@ -127,8 +126,20 @@ verify_parameter(Params &params)
   params.nsteps = std::max(params.nsteps, 1);
 }
 
-class ModuleCDIWrite
+class CDIwrite : public Process
 {
+public:
+  using Process::Process;
+  inline static CdoModule module = {
+    .name = "CDIwrite",
+    .operators = { { "cdiwrite" } },
+    .aliases = {},
+    .mode = EXPOSED,     // Module mode: 0:intern 1:extern
+    .number = CDI_REAL,  // Allowed number type
+    .constraints = { 0, 1, NoRestriction },
+  };
+  inline static RegisterEntry<CDIwrite> registration = RegisterEntry<CDIwrite>(module);
+
   MemType memtype = Options::CDO_Memtype;
   int filetype = -1, datatype = -1;
   char sinfo[64] = { 0 };
@@ -148,10 +159,8 @@ class ModuleCDIWrite
 
 public:
   void
-  init(void *process)
+  init() override
   {
-    cdo_initialize(process);
-
     if (Options::cdoVerbose) cdo_print("parameter: nruns/nvars/nlevs/nsteps/grid/varysteps");
 
     params = get_parameter();
@@ -213,8 +222,9 @@ public:
 
     vlistDefNtsteps(vlistID, params.nsteps);
   }
+
   void
-  run()
+  run() override
   {
     for (int irun = 0; irun < params.nruns; ++irun)
       {
@@ -296,20 +306,10 @@ public:
 
     if (params.nruns > 1) print_stat("(mean)", memtype, datatype, filetype, nvalues, dataSize, fileSize, runTimeSum / params.nruns);
   }
+
   void
-  close()
+  close() override
   {
     vlistDestroy(vlistID);
-
-    cdo_finish();
   }
 };
-void *
-CDIwrite(void *process)
-{
-  ModuleCDIWrite cdiWrite;
-  cdiWrite.init(process);
-  cdiWrite.run();
-  cdiWrite.close();
-  return nullptr;
-}
