@@ -16,11 +16,23 @@
 #include "process_int.h"
 #include "cdi_lockedIO.h"
 
-class ModuleChange_e5slm
+class Change_e5slm : public Process
 {
+public:
+  using Process::Process;
+  inline static CdoModule module = {
+    .name = "Change_e5slm",
+    .operators = { { "change_e5slm" }, { "change_e5lsm" }, { "change_e5mask" } },
+    .aliases = {},
+    .mode = INTERNAL,    // Module mode: 0:intern 1:extern
+    .number = CDI_REAL,  // Allowed number type
+    .constraints = { 1, 1, NoRestriction },
+  };
+  inline static RegisterEntry<Change_e5slm> registration = RegisterEntry<Change_e5slm>(module);
+
   int nrecs;
   int varID, levelID;
-  size_t nmiss;
+  size_t numMissVals;
 
   CdoStreamID streamID1;
   int taxisID1;
@@ -38,16 +50,16 @@ class ModuleChange_e5slm
 
 public:
   void
-  init(void *process)
+  init() override
   {
-    cdo_initialize(process);
-
     streamID1 = cdo_open_read(0);
 
-    const auto vlistID1 = cdo_stream_inq_vlist(streamID1);
+    auto vlistID1 = cdo_stream_inq_vlist(streamID1);
+    VarList varList1(vlistID1);
+
     taxisID1 = vlistInqTaxis(vlistID1);
 
-    const auto vlistID2 = vlistDuplicate(vlistID1);
+    auto vlistID2 = vlistDuplicate(vlistID1);
     taxisID2 = taxisDuplicate(taxisID1);
     vlistDefTaxis(vlistID2, taxisID2);
 
@@ -60,10 +72,13 @@ public:
     const char *fn_slm = cdo_operator_argv(0).c_str();
 
     /* read SLM */
-    const auto streamIDslm = stream_open_read_locked(fn_slm);
-    const auto vlistIDslm = streamInqVlist(streamIDslm);
+    auto streamIDslm = stream_open_read_locked(fn_slm);
+    auto vlistIDslm = streamInqVlist(streamIDslm);
 
-    gridsize = gridInqSize(vlistInqVarGrid(vlistIDslm, 0));
+    {
+      VarList varListSLM(vlistIDslm);
+      gridsize = varListSLM.vars[0].gridsize;
+    }
 
     array = Varray<double>(gridsize);
     cland = Varray<double>(gridsize);
@@ -72,50 +87,50 @@ public:
     streamInqTimestep(streamIDslm, 0);
 
     streamInqRecord(streamIDslm, &varID, &levelID);
-    streamReadRecord(streamIDslm, cland.data(), &nmiss);
+    streamReadRecord(streamIDslm, cland.data(), &numMissVals);
 
-    if (nmiss) cdo_abort("SLM with missing values are unsupported!");
+    if (numMissVals) cdo_abort("SLM with missing values are unsupported!");
 
-    const auto mm = varray_min_max(cland);
+    auto mm = varray_min_max(cland);
     if (mm.min < 0 || mm.max > 1) cdo_warning("Values of SLM out of bounds! (minval=%g, maxval=%g)", mm.min, mm.max);
 
     streamClose(streamIDslm);
 
     for (size_t i = 0; i < gridsize; ++i) lsea[i] = cland[i] <= 0;
 
-    const auto nvars = vlistNvars(vlistID1);
-    codes = std::vector<short>(nvars);
+    auto numVars = varList1.numVars();
+    codes = std::vector<short>(numVars);
 
-    for (varID = 0; varID < nvars; ++varID)
+    for (varID = 0; varID < numVars; ++varID)
       {
-        if (gridsize != gridInqSize(vlistInqVarGrid(vlistID1, varID))) cdo_abort("gridsize differ!");
-
-        auto code = vlistInqVarCode(vlistID1, varID);
-        auto varname = cdo::inq_var_name(vlistID1, varID);
+        const auto &var = varList1.vars[varID];
+        if (gridsize != var.gridsize) cdo_abort("gridsize differ!");
 
+        auto code = var.code;
         if (code < 0)
           {
             // clang-format off
-          if      (varname == "SLM")       code = 172;
-          else if (varname == "ALAKE")     code = 99;
-          else if (varname == "WS")        code = 140;
-          else if (varname == "AZ0")       code = 173;
-          else if (varname == "ALB")       code = 174;
-          else if (varname == "VGRAT")     code = 198;
-          else if (varname == "FOREST")    code = 212;
-          else if (varname == "FAO")       code = 226;
-          else if (varname == "WSMX")      code = 229;
-          else if (varname == "GLAC")      code = 232;
-          else if (varname == "VLTCLIM")   code = 71;
-          else if (varname == "VGRATCLIM") code = 70;
+            if      (var.name == "SLM")       code = 172;
+            else if (var.name == "ALAKE")     code = 99;
+            else if (var.name == "WS")        code = 140;
+            else if (var.name == "AZ0")       code = 173;
+            else if (var.name == "ALB")       code = 174;
+            else if (var.name == "VGRAT")     code = 198;
+            else if (var.name == "FOREST")    code = 212;
+            else if (var.name == "FAO")       code = 226;
+            else if (var.name == "WSMX")      code = 229;
+            else if (var.name == "GLAC")      code = 232;
+            else if (var.name == "VLTCLIM")   code = 71;
+            else if (var.name == "VGRATCLIM") code = 70;
             // clang-format on
           }
 
         codes[varID] = code;
       }
   }
+
   void
-  run()
+  run() override
   {
     int tsID = 0;
     while ((nrecs = cdo_stream_inq_timestep(streamID1, tsID)))
@@ -126,7 +141,7 @@ public:
         for (int recID = 0; recID < nrecs; ++recID)
           {
             cdo_inq_record(streamID1, &varID, &levelID);
-            cdo_read_record(streamID1, array.data(), &nmiss);
+            cdo_read_record(streamID1, array.data(), &numMissVals);
 
             auto code = codes[varID];
             if (code == 172)
@@ -154,28 +169,17 @@ public:
               }
 
             cdo_def_record(streamID2, varID, levelID);
-            cdo_write_record(streamID2, array.data(), nmiss);
+            cdo_write_record(streamID2, array.data(), numMissVals);
           }
 
         tsID++;
       }
   }
+
   void
-  close()
+  close() override
   {
     cdo_stream_close(streamID1);
     cdo_stream_close(streamID2);
-
-    cdo_finish();
   }
 };
-
-void *
-Change_e5slm(void *process)
-{
-  ModuleChange_e5slm change_e5slm;
-  change_e5slm.init(process);
-  change_e5slm.run();
-  change_e5slm.close();
-  return nullptr;
-}
