@@ -43,15 +43,33 @@ static auto func_comp
           }
       };
 
-class ModuleComp
+class Comp : public Process
 {
-  enum
+  enum class FillType
   {
-    FILL_NONE,
-    FILL_TS,
-    FILL_REC
+    NONE,
+    TS,
+    REC,
   };
-  int filltype = FILL_NONE;
+
+public:
+  using Process::Process;
+  inline static CdoModule module = {
+    .name = "Comp",
+    .operators = { { "eq", FieldFunc_EQ, 0, CompHelp },
+                   { "ne", FieldFunc_NE, 0, CompHelp },
+                   { "le", FieldFunc_LE, 0, CompHelp },
+                   { "lt", FieldFunc_LT, 0, CompHelp },
+                   { "ge", FieldFunc_GE, 0, CompHelp },
+                   { "gt", FieldFunc_GT, 0, CompHelp } },
+    .aliases = {},
+    .mode = EXPOSED,     // Module mode: 0:intern 1:extern
+    .number = CDI_REAL,  // Allowed number type
+    .constraints = { 2, 1, NoRestriction },
+  };
+  inline static RegisterEntry<Comp> registration = RegisterEntry<Comp>(module);
+
+  FillType fillType{ FillType::NONE };
   Varray2D<double> vardata;
 
   CdoStreamID streamID1;
@@ -65,23 +83,15 @@ class ModuleComp
   double *arrayx1;
   double *arrayx2;
 
-  VarList varList1, varList2;
+  VarList varList1;
+  VarList varList2;
 
   Varray<double> vaIn1, vaIn2, vaOut;
 
 public:
   void
-  init(void *process)
+  init() override
   {
-    cdo_initialize(process);
-
-    cdo_operator_add("eq", FieldFunc_EQ, 0, nullptr);
-    cdo_operator_add("ne", FieldFunc_NE, 0, nullptr);
-    cdo_operator_add("le", FieldFunc_LE, 0, nullptr);
-    cdo_operator_add("lt", FieldFunc_LT, 0, nullptr);
-    cdo_operator_add("ge", FieldFunc_GE, 0, nullptr);
-    cdo_operator_add("gt", FieldFunc_GT, 0, nullptr);
-
     auto operatorID = cdo_operator_id();
     operFunc = cdo_operator_f1(operatorID);
 
@@ -103,15 +113,15 @@ public:
 
     auto fillstream1 = false;
 
-    if (vlistNrecs(vlistID1) != 1 && vlistNrecs(vlistID2) == 1)
+    if (vlistNumRecords(vlistID1) != 1 && vlistNumRecords(vlistID2) == 1)
       {
-        filltype = FILL_REC;
+        fillType = FillType::REC;
         cdo_print("Filling up stream2 >%s< by copying the first record.", cdo_get_stream_name(1));
         if (ntsteps2 != 1) cdo_abort("stream2 has more than 1 timestep!");
       }
-    else if (vlistNrecs(vlistID1) == 1 && vlistNrecs(vlistID2) != 1)
+    else if (vlistNumRecords(vlistID1) == 1 && vlistNumRecords(vlistID2) != 1)
       {
-        filltype = FILL_REC;
+        fillType = FillType::REC;
         cdo_print("Filling up stream1 >%s< by copying the first record.", cdo_get_stream_name(0));
         if (ntsteps1 != 1) cdo_abort("stream1 has more than 1 timestep!");
         fillstream1 = true;
@@ -120,7 +130,7 @@ public:
         std::swap(taxisID1, taxisID2);
       }
 
-    if (filltype == FILL_NONE) vlist_compare(vlistID1, vlistID2, CmpVlist::All);
+    if (fillType == FillType::NONE) vlist_compare(vlistID1, vlistID2, CmpVarList::All);
 
     auto gridsizemax = vlistGridsizeMax(vlistID1);
     vaIn1 = Varray<double>(gridsizemax);
@@ -132,16 +142,16 @@ public:
 
     if (Options::cdoVerbose) cdo_print("Number of timesteps: file1 %d, file2 %d", ntsteps1, ntsteps2);
 
-    if (filltype == FILL_NONE)
+    if (fillType == FillType::NONE)
       {
         if (ntsteps1 != 1 && ntsteps2 == 1)
           {
-            filltype = FILL_TS;
+            fillType = FillType::TS;
             cdo_print("Filling up stream2 >%s< by copying the first timestep.", cdo_get_stream_name(1));
           }
         else if (ntsteps1 == 1 && ntsteps2 != 1)
           {
-            filltype = FILL_TS;
+            fillType = FillType::TS;
             cdo_print("Filling up stream1 >%s< by copying the first timestep.", cdo_get_stream_name(0));
             fillstream1 = true;
             std::swap(streamID1, streamID2);
@@ -149,7 +159,7 @@ public:
             std::swap(taxisID1, taxisID2);
           }
 
-        if (filltype == FILL_TS) cdo_fill_ts(vlistID2, vardata);
+        if (fillType == FillType::TS) cdo_fill_ts(vlistID2, vardata);
       }
 
     if (fillstream1)
@@ -158,8 +168,8 @@ public:
         arrayx2 = vaIn1.data();
       }
 
-    varList_init(varList1, vlistID1);
-    varList_init(varList2, vlistID2);
+    varList1 = VarList(vlistID1);
+    varList2 = VarList(vlistID2);
 
     auto vlistID3 = vlistDuplicate(vlistID1);
 
@@ -173,7 +183,7 @@ public:
   }
 
   void
-  run()
+  run() override
   {
     int tsID = 0;
     while (true)
@@ -181,7 +191,7 @@ public:
         auto nrecs = cdo_stream_inq_timestep(streamID1, tsID);
         if (nrecs == 0) break;
 
-        if (tsID == 0 || filltype == FILL_NONE)
+        if (tsID == 0 || fillType == FillType::NONE)
           {
             auto nrecs2 = cdo_stream_inq_timestep(streamID2, tsID);
             if (nrecs2 == 0) cdo_abort("Input streams have different number of timesteps!");
@@ -192,39 +202,43 @@ public:
 
         for (int recID = 0; recID < nrecs; ++recID)
           {
-            size_t nmiss1 = 0, nmiss2 = 0;
+            size_t numMissVals1 = 0, numMissVals2 = 0;
             int varID, levelID;
             cdo_inq_record(streamID1, &varID, &levelID);
-            cdo_read_record(streamID1, arrayx1, &nmiss1);
+            cdo_read_record(streamID1, arrayx1, &numMissVals1);
 
-            if (tsID == 0 || filltype == FILL_NONE)
+            if (tsID == 0 || fillType == FillType::NONE)
               {
-                if (recID == 0 || filltype != FILL_REC)
+                if (recID == 0 || fillType != FillType::REC)
                   {
                     cdo_inq_record(streamID2, &varID, &levelID);
-                    cdo_read_record(streamID2, arrayx2, &nmiss2);
+                    cdo_read_record(streamID2, arrayx2, &numMissVals2);
                   }
 
-                if (filltype == FILL_TS)
+                if (fillType == FillType::TS)
                   {
-                    auto offset = varList2[varID].gridsize * levelID;
-                    array_copy(varList2[varID].gridsize, arrayx2, &vardata[varID][offset]);
+                    auto gridsize = varList2.vars[varID].gridsize;
+                    auto offset = gridsize * levelID;
+                    array_copy(gridsize, arrayx2, &vardata[varID][offset]);
                   }
               }
-            else if (filltype == FILL_TS)
+            else if (fillType == FillType::TS)
               {
-                auto offset = varList2[varID].gridsize * levelID;
-                array_copy(varList2[varID].gridsize, &vardata[varID][offset], arrayx2);
+                auto gridsize = varList2.vars[varID].gridsize;
+                auto offset = gridsize * levelID;
+                array_copy(gridsize, &vardata[varID][offset], arrayx2);
               }
 
-            auto datatype1 = varList1[varID].datatype;
-            auto gridsize1 = varList1[varID].gridsize;
-            auto missval1 = varList1[varID].missval;
+            const auto &var1 = varList1.vars[varID];
+            auto datatype1 = var1.dataType;
+            auto gridsize1 = var1.gridsize;
+            auto missval1 = var1.missval;
 
-            auto xvarID = (filltype == FILL_REC) ? 0 : varID;
-            auto datatype2 = varList2[xvarID].datatype;
-            auto gridsize2 = varList2[xvarID].gridsize;
-            auto missval2 = varList2[xvarID].missval;
+            auto xvarID = (fillType == FillType::REC) ? 0 : varID;
+            const auto &var2 = varList2.vars[xvarID];
+            auto datatype2 = var2.dataType;
+            auto gridsize2 = var2.gridsize;
+            auto missval2 = var2.missval;
 
             if (gridsize1 != gridsize2)
               cdo_abort("Streams have different gridsize (gridsize1 = %zu; gridsize2 = %zu)!", gridsize1, gridsize2);
@@ -245,10 +259,10 @@ public:
                   }
               }
 
-            if (nmiss1 > 0) cdo_check_missval(missval1, varList1[varID].name);
-            // if (nmiss2 > 0) cdo_check_missval(missval2, varList2[varID].name);
+            if (numMissVals1 > 0) cdo_check_missval(missval1, varList1.vars[varID].name);
+            // if (numMissVals2 > 0) cdo_check_missval(missval2, varList2.vars[varID].name);
 
-            auto hasMissvals = (nmiss1 > 0 || nmiss2 > 0);
+            auto hasMissvals = (numMissVals1 > 0 || numMissVals2 > 0);
             // clang-format off
             if      (operFunc == FieldFunc_EQ) func_comp(hasMissvals, ngp, missval1, missval2, vaIn1, vaIn2, vaOut, binary_op_EQ);
             else if (operFunc == FieldFunc_NE) func_comp(hasMissvals, ngp, missval1, missval2, vaIn1, vaIn2, vaOut, binary_op_NE);
@@ -259,9 +273,9 @@ public:
             else cdo_abort("Operator not implemented!");
             // clang-format on
 
-            auto nmissOut = varray_num_mv(ngp, vaOut, missval1);
+            auto numMissValsOut = varray_num_mv(ngp, vaOut, missval1);
             cdo_def_record(streamID3, varID, levelID);
-            cdo_write_record(streamID3, vaOut.data(), nmissOut);
+            cdo_write_record(streamID3, vaOut.data(), numMissValsOut);
           }
 
         tsID++;
@@ -269,23 +283,10 @@ public:
   }
 
   void
-  close()
+  close() override
   {
     cdo_stream_close(streamID3);
     cdo_stream_close(streamID2);
     cdo_stream_close(streamID1);
-
-    cdo_finish();
   }
 };
-
-void *
-Comp(void *process)
-{
-  ModuleComp comp;
-  comp.init(process);
-  comp.run();
-  comp.close();
-
-  return nullptr;
-}
