@@ -7,10 +7,24 @@
 
 #include <cdi.h>
 
+#include "arithmetic.h"
 #include "process_int.h"
 
-class ModuleComplextorect
+class Complextorect : public Process
 {
+public:
+  using Process::Process;
+  inline static CdoModule module = {
+    .name = "Complextorect",
+    .operators = { { "complextorect" }, { "complextopol" } },
+    .aliases = {},
+    .mode = EXPOSED,     // Module mode: 0:intern 1:extern
+    .number = CDI_COMP,  // Allowed number type
+    .constraints = { 1, 2, OnlyFirst },
+  };
+  inline static RegisterEntry<Complextorect> registration = RegisterEntry<Complextorect>(module);
+
+  int COMPLEXTORECT, COMPLEXTOPOL;
 
   CdoStreamID streamID1;
   int taxisID1;
@@ -30,16 +44,12 @@ class ModuleComplextorect
   Varray<double> array3;
   VarList varList1;
 
-  int COMPLEXTORECT, COMPLEXTOPOL;
-
 public:
   void
-  init(void *process)
+  init() override
   {
-    cdo_initialize(process);
-
-    COMPLEXTORECT = cdo_operator_add("complextorect", 0, 0, nullptr);
-    COMPLEXTOPOL = cdo_operator_add("complextopol", 0, 0, nullptr);
+    COMPLEXTORECT = module.get_id("complextorect");
+    COMPLEXTOPOL = module.get_id("complextopol");
 
     operatorID = cdo_operator_id();
 
@@ -71,17 +81,17 @@ public:
 
     cdo_def_vlist(streamID2, vlistID2);
     cdo_def_vlist(streamID3, vlistID3);
-    varList_init(varList1, vlistID1);
+    varList1 = VarList(vlistID1);
 
     auto gridsizemax = vlistGridsizeMax(vlistID1);
     array1 = Varray<double>(2 * gridsizemax);
     array2 = Varray<double>(gridsizemax);
     array3 = Varray<double>(gridsizemax);
   }
-  void
-  run()
-  {
 
+  void
+  run() override
+  {
     int tsID = 0;
     while (true)
       {
@@ -101,12 +111,14 @@ public:
             cdo_def_record(streamID2, varID, levelID);
             cdo_def_record(streamID3, varID, levelID);
 
-            auto gridsize = varList1[varID].gridsize;
+            const auto &var1 = varList1.vars[varID];
+            auto gridsize = var1.gridsize;
 
-            size_t nmiss;
-            cdo_read_record(streamID1, array1.data(), &nmiss);
+            size_t numMissVals;
+            cdo_read_record(streamID1, array1.data(), &numMissVals);
 
-            auto missval1 = varList1[varID].missval;
+            auto is_EQ = dbl_is_equal;
+            auto missval1 = var1.missval;
             auto missval2 = missval1;
 
             if (operatorID == COMPLEXTORECT)
@@ -121,22 +133,23 @@ public:
               {
                 for (size_t i = 0; i < gridsize; ++i)
                   {
-                    array2[i] = SQRTMN(ADDMN(MULMN(array1[2 * i], array1[2 * i]), MULMN(array1[2 * i + 1], array1[2 * i + 1])));
-                    array3[i] = (DBL_IS_EQUAL(array1[2 * i], missval1) || DBL_IS_EQUAL(array1[2 * i + 1], missval1))
+                    array2[i] = SQRTM(ADDM(MULM(array1[2 * i], array1[2 * i]), MULM(array1[2 * i + 1], array1[2 * i + 1])));
+                    array3[i] = (is_EQ(array1[2 * i], missval1) || is_EQ(array1[2 * i + 1], missval1))
                                     ? missval1
                                     : std::atan2(array1[2 * i + 1], array1[2 * i]);
                   }
               }
 
-            cdo_write_record(streamID2, array2.data(), nmiss);
-            cdo_write_record(streamID3, array3.data(), nmiss);
+            cdo_write_record(streamID2, array2.data(), numMissVals);
+            cdo_write_record(streamID3, array3.data(), numMissVals);
           }
 
         tsID++;
       }
   }
+
   void
-  close()
+  close() override
   {
     cdo_stream_close(streamID3);
     cdo_stream_close(streamID2);
@@ -144,18 +157,5 @@ public:
 
     vlistDestroy(vlistID2);
     vlistDestroy(vlistID3);
-
-    cdo_finish();
   }
 };
-void *
-Complextorect(void *process)
-{
-  ModuleComplextorect complextorect;
-
-  complextorect.init(process);
-  complextorect.run();
-  complextorect.close();
-
-  return nullptr;
-}
