@@ -51,16 +51,16 @@ selEndOfPeriod(Field &periods, const Field &history, const Field &current, int i
 
   if (!isLastTimestep)
     {
-      if (current.nmiss || history.nmiss)
+      if (current.numMissVals || history.numMissVals)
         {
 #ifdef _OPENMP
 #pragma omp parallel for default(shared)
 #endif
           for (size_t i = 0; i < len; ++i)
             {
-              if (!dbl_is_equal(harray[i], hmissval))
+              if (dbl_is_not_equal(harray[i], hmissval))
                 {
-                  if (!dbl_is_equal(carray[i], cmissval))
+                  if (dbl_is_not_equal(carray[i], cmissval))
                     parray[i] = (dbl_is_equal(carray[i], 0.0) && is_not_equal(harray[i], 0.0)) ? harray[i] : pmissval;
                   else  // DBL_IS_EQUAL(carray[i], cmissval)
                     parray[i] = (is_not_equal(harray[i], 0.0)) ? harray[i] : pmissval;
@@ -79,7 +79,7 @@ selEndOfPeriod(Field &periods, const Field &history, const Field &current, int i
     }
   else
     {
-      if (current.nmiss)
+      if (current.numMissVals)
         {
 #ifdef _OPENMP
 #pragma omp parallel for default(shared)
@@ -99,11 +99,22 @@ selEndOfPeriod(Field &periods, const Field &history, const Field &current, int i
         }
     }
 
-  periods.nmiss = varray_num_mv(len, parray, pmissval);
+  periods.numMissVals = varray_num_mv(len, parray, pmissval);
 }
 
-class ModuleConsecstat
+class Consecstat : public Process
 {
+public:
+  using Process::Process;
+  inline static CdoModule module = {
+    .name = "Consecstat",
+    .operators = { { "consects", CONSECTS, 0, ConsecstatHelp }, { "consecsum", CONSECSUM, 0, "refval", ConsecstatHelp } },
+    .aliases = {},
+    .mode = EXPOSED,     // Module mode: 0:intern 1:extern
+    .number = CDI_REAL,  // Allowed number type
+    .constraints = { 1, 1, NoRestriction },
+  };
+  inline static RegisterEntry<Consecstat> registration = RegisterEntry<Consecstat>(module);
   CdiDateTime vDateTime{};
   CdiDateTime histDateTime{};
   double refval = 0.0;
@@ -118,19 +129,16 @@ class ModuleConsecstat
   int otaxisID;
 
   int operatorID;
-  int nvars;
+  int numVars;
 
-  VarList varList;
+  VarList varList1;
   Field field;
-  FieldVector2D vars, hist, periods;
+  FieldVector2D varsData, histData, periodsData;
 
 public:
   void
-  init(void *process)
+  init() override
   {
-    cdo_initialize(process);
-    cdo_operator_add("consecsum", CONSECSUM, 0, "refval");
-    cdo_operator_add("consects", CONSECTS, 0, nullptr);
     operatorID = cdo_operator_id();
 
     if (operatorID == CONSECSUM)
@@ -144,32 +152,29 @@ public:
     otaxisID = taxisDuplicate(itaxisID);
     vlistDefTaxis(ovlistID, otaxisID);
 
-    varList_init(varList, ovlistID);
+    varList1 = VarList(ivlistID);
 
     field.resize(vlistGridsizeMax(ovlistID));
 
-    nvars = vlistNvars(ivlistID);
-    fields_from_vlist(ivlistID, vars, FIELD_VEC, 0);
-    if (operatorID == CONSECTS)
-      {
-        fields_from_vlist(ivlistID, hist, FIELD_VEC);
-        fields_from_vlist(ivlistID, periods, FIELD_VEC);
-      }
+    numVars = varList1.numVars();
+    field2D_init(varsData, varList1, FIELD_VEC, 0);
+    if (operatorID == CONSECTS) field2D_init(histData, varList1, FIELD_VEC);
+    if (operatorID == CONSECTS) field2D_init(periodsData, varList1, FIELD_VEC);
 
-    for (int varID = 0; varID < nvars; ++varID) cdiDefKeyString(ovlistID, varID, CDI_KEY_UNITS, "steps");  // TODO
+    for (int varID = 0; varID < numVars; ++varID) cdiDefKeyString(ovlistID, varID, CDI_KEY_UNITS, "steps");  // TODO
 
     ostreamID = cdo_open_write(1);
     cdo_def_vlist(ostreamID, ovlistID);
   }
-  void
-  run()
-  {
 
+  void
+  run() override
+  {
     int itsID = 0;
     int otsID = 0;
     while (true)
       {
-        const auto nrecs = cdo_stream_inq_timestep(istreamID, itsID);
+        auto nrecs = cdo_stream_inq_timestep(istreamID, itsID);
         if (nrecs == 0) break;
 
         vDateTime = taxisInqVdatetime(itaxisID);
@@ -193,32 +198,33 @@ public:
           {
             int varID, levelID;
             cdo_inq_record(istreamID, &varID, &levelID);
-            cdo_read_record(istreamID, field.vec_d.data(), &field.nmiss);
-            field.grid = varList[varID].gridID;
-            field.size = varList[varID].gridsize;
-            field.missval = varList[varID].missval;
+            cdo_read_record(istreamID, field.vec_d.data(), &field.numMissVals);
+            const auto &var1 = varList1.vars[varID];
+            field.grid = var1.gridID;
+            field.size = var1.gridsize;
+            field.missval = var1.missval;
 
-            field2_sumtr(vars[varID][levelID], field, refval);
+            field2_sumtr(varsData[varID][levelID], field, refval);
 
             switch (operatorID)
               {
               case CONSECSUM:
                 cdo_def_record(ostreamID, varID, levelID);
-                cdo_write_record(ostreamID, vars[varID][levelID].vec_d.data(), vars[varID][levelID].nmiss);
+                cdo_write_record(ostreamID, varsData[varID][levelID].vec_d.data(), varsData[varID][levelID].numMissVals);
                 break;
               case CONSECTS:
                 if (itsID != 0)
                   {
-                    selEndOfPeriod(periods[varID][levelID], hist[varID][levelID], vars[varID][levelID], false);
+                    selEndOfPeriod(periodsData[varID][levelID], histData[varID][levelID], varsData[varID][levelID], false);
                     cdo_def_record(ostreamID, varID, levelID);
-                    cdo_write_record(ostreamID, periods[varID][levelID].vec_d.data(), periods[varID][levelID].nmiss);
+                    cdo_write_record(ostreamID, periodsData[varID][levelID].vec_d.data(), periodsData[varID][levelID].numMissVals);
                   }
 #ifdef _OPENMP
 #pragma omp parallel for default(shared) schedule(static)
-                for (size_t i = 0; i < vars[varID][levelID].size; ++i)
-                  hist[varID][levelID].vec_d[i] = vars[varID][levelID].vec_d[i];
+                for (size_t i = 0; i < varsData[varID][levelID].size; ++i)
+                  histData[varID][levelID].vec_d[i] = varsData[varID][levelID].vec_d[i];
 #else
-                hist[varID][levelID].vec_d = vars[varID][levelID].vec_d;
+                histData[varID][levelID].vec_d = varsData[varID][levelID].vec_d;
 #endif
                 break;
               default: printf(SWITCHWARN, __func__); break;
@@ -235,35 +241,23 @@ public:
         taxisDefVdatetime(otaxisID, vDateTime);
         cdo_def_timestep(ostreamID, otsID - 1);
 
-        for (int varID = 0; varID < nvars; ++varID)
+        for (int varID = 0; varID < numVars; ++varID)
           {
-            const auto nlevels = varList[varID].nlevels;
+            const auto nlevels = varList1.vars[varID].nlevels;
             for (int levelID = 0; levelID < nlevels; ++levelID)
               {
-                selEndOfPeriod(periods[varID][levelID], hist[varID][levelID], vars[varID][levelID], true);
+                selEndOfPeriod(periodsData[varID][levelID], histData[varID][levelID], varsData[varID][levelID], true);
                 cdo_def_record(ostreamID, varID, levelID);
-                cdo_write_record(ostreamID, periods[varID][levelID].vec_d.data(), periods[varID][levelID].nmiss);
+                cdo_write_record(ostreamID, periodsData[varID][levelID].vec_d.data(), periodsData[varID][levelID].numMissVals);
               }
           }
       }
   }
+
   void
-  close()
+  close() override
   {
     cdo_stream_close(istreamID);
     cdo_stream_close(ostreamID);
-
-    cdo_finish();
   }
 };
-void *
-Consecstat(void *process)
-{
-  ModuleConsecstat consecstat;
-
-  consecstat.init(process);
-  consecstat.run();
-  consecstat.close();
-
-  return nullptr;
-}
