@@ -51,14 +51,14 @@ field_deltat(const Field &field0, const Field &field1, Field &field2, double fac
 {
   if (field1.memType != field2.memType) cdo_abort("Interal error, memType of field1 and field2 differ!");
 
-  if (field1.nmiss || field0.nmiss)
+  if (field1.numMissVals || field0.numMissVals)
     {
       if (field1.memType == MemType::Float)
         varray_deltat(field1.size, field0.vec_f, field1.vec_f, field2.vec_f, factor, field1.missval);
       else
         varray_deltat(field1.size, field0.vec_d, field1.vec_d, field2.vec_d, factor, field1.missval);
 
-      field2.nmiss = field_num_mv(field2);
+      field2.numMissVals = field_num_mv(field2);
     }
   else
     {
@@ -69,8 +69,21 @@ field_deltat(const Field &field0, const Field &field1, Field &field2, double fac
     }
 }
 
-class ModuleDeltat
+class Deltat : public Process
 {
+public:
+  using Process::Process;
+  inline static CdoModule module = {
+    .name = "Deltat",
+    .operators = { { "deltat", DeltatHelp }, { "timederivative", 0, 1, DeltatHelp } },
+    .aliases = {},
+    .mode = EXPOSED,     // Module mode: 0:intern 1:extern
+    .number = CDI_REAL,  // Allowed number type
+    .constraints = { 1, 1, NoRestriction },
+  };
+  inline static RegisterEntry<Deltat> registration = RegisterEntry<Deltat>(module);
+
+private:
   CdoStreamID streamID1;
   CdoStreamID streamID2;
 
@@ -78,22 +91,15 @@ class ModuleDeltat
   int taxisID2;
   int calendar;
   VarList varList1;
-  FieldVector2D vars;
+  FieldVector2D varsData;
 
   Field field1, field2;
   bool ldivdt;
 
 public:
   void
-  init(void *process)
+  init() override
   {
-    cdo_initialize(process);
-
-    // clang-format off
-  cdo_operator_add("deltat",            0,     0, nullptr);
-  cdo_operator_add("timederivative",    0,     1, nullptr);
-    // clang-format on
-
     auto operatorID = cdo_operator_id();
     ldivdt = cdo_operator_f2(operatorID);
 
@@ -110,15 +116,15 @@ public:
 
     calendar = taxisInqCalendar(taxisID1);
 
-    varList_init(varList1, vlistID1);
+    varList1 = VarList(vlistID1);
 
     streamID2 = cdo_open_write(1);
     cdo_def_vlist(streamID2, vlistID2);
 
-    fields_from_vlist(vlistID1, vars, FIELD_VEC | FIELD_NAT);
+    field2D_init(varsData, vlistID1, FIELD_VEC | FIELD_NAT);
   }
   void
-  run()
+  run() override
   {
     int tsID = 0;
     auto nrecs = cdo_stream_inq_timestep(streamID1, tsID);
@@ -129,7 +135,7 @@ public:
       {
         int varID, levelID;
         cdo_inq_record(streamID1, &varID, &levelID);
-        cdo_read_record(streamID1, vars[varID][levelID]);
+        cdo_read_record(streamID1, varsData[varID][levelID]);
       }
 
     tsID++;
@@ -150,12 +156,13 @@ public:
           {
             int varID, levelID;
             cdo_inq_record(streamID1, &varID, &levelID);
-            field1.init(varList1[varID]);
+            const auto &var1 = varList1.vars[varID];
+            field1.init(var1);
             cdo_read_record(streamID1, field1);
 
-            auto &field0 = vars[varID][levelID];
+            auto &field0 = varsData[varID][levelID];
 
-            field2.init(varList1[varID]);
+            field2.init(var1);
             field_deltat(field0, field1, field2, idtInSec);
 
             field_copy(field1, field0);
@@ -170,21 +177,9 @@ public:
   }
 
   void
-  close()
+  close() override
   {
     cdo_stream_close(streamID2);
     cdo_stream_close(streamID1);
-
-    cdo_finish();
   }
 };
-void *
-Deltat(void *process)
-{
-  ModuleDeltat deltat;
-  deltat.init(process);
-  deltat.run();
-  deltat.close();
-
-  return nullptr;
-}
