@@ -8,7 +8,8 @@
 /*
    This module contains the following operators:
 
-      Derivepar     gheight           Geopotential height
+      Derivepar     gheight           Geopotential height on full-levels
+      Derivepar     gheight_half      Geopotential height on half-levels
       Derivepar     sealevelpressure  Sea level pressure
 */
 
@@ -22,10 +23,8 @@
 #include "cdo_zaxis.h"
 #include "cdo_options.h"
 
-void geopot_height_halflevel(double *gheight, const double *const ta_fl, const double *const hus_fl, const double *const p_hl,
-                             long ngp, long nlev);
-void geopot_height_fulllevel(double *gheight, const double *const ta_fl, const double *const hus_fl, const double *const p_hl,
-                             long ngp, long nlev);
+void geopot_height_half(double *gheight, const double *ta_fl, const double *hus_fl, const double *p_hl, long ngp, long nlev);
+void geopot_height_full(double *gheight, const double *ta_fl, const double *hus_fl, const double *p_hl, long ngp, long nlev);
 
 static void
 check_range_var2d(int stepNum, const Varray<double> &var2d, double rMin, double rMax, const char *varname)
@@ -58,8 +57,21 @@ check_range_var3d(int stepNum, int nlevels, size_t gridsize, const Varray<double
     }
 }
 
-class ModuleDerivepar
+class Derivepar : public Process
 {
+public:
+  using Process::Process;
+  inline static CdoModule module = {
+    .name = "Derivepar",
+    .operators = { { "gheight", DeriveparHelp }, { "gheight_half", DeriveparHelp }, { "sealevelpressure", DeriveparHelp } },
+    .aliases = { { "gheight_full", "gheight" }, { "gheighthalf", "gheight_half" } },
+    .mode = EXPOSED,     // Module mode: 0:intern 1:extern
+    .number = CDI_REAL,  // Allowed number type
+    .constraints = { 1, 1, NoRestriction },
+  };
+  inline static auto registration = RegisterEntry<Derivepar>(module);
+
+  int GHEIGHT_FULL, GHEIGHT_HALF, SEALEVELPRESSURE;
   int surfaceID = -1;
   int presID = -1;
 
@@ -73,35 +85,33 @@ class ModuleDerivepar
   size_t gridsize;
   VarIDs varIDs;
   int zaxisID_ML = -1;
-  int numHybridLevels = 0, numFullLevels = 0, numHalfLevels = 0;
+  int numHybridLevels = 0;
+  int numFullLevels = 0;
+  int numHalfLevels = 0;
 
   Varray<double> array;
   Varray<double> sgeopot;
   Varray<double> ps;
   Varray<double> temp;
 
-  Varray<double> hum, gheight;
+  Varray<double> hum;
+  Varray<double> gheight;
   Varray<double> halfPress;
-  Varray<double> fullPress, sealevelpressure;
+  Varray<double> fullPress;
+  Varray<double> sealevelpressure;
   Varray<double> vct;
 
   VarList varList1;
 
   int operatorID;
 
-  int GHEIGHT, GHEIGHTHALF, SEALEVELPRESSURE;
-
 public:
   void
-  init(void *process)
+  init() override
   {
-    cdo_initialize(process);
-
-    // clang-format off
-    GHEIGHT          = cdo_operator_add("gheight",            0, 0, nullptr);
-    GHEIGHTHALF      = cdo_operator_add("gheighthalf",        0, 0, nullptr);
-    SEALEVELPRESSURE = cdo_operator_add("sealevelpressure",   0, 0, nullptr);
-    // clang-format on
+    GHEIGHT_FULL = module.get_id("gheight");
+    GHEIGHT_HALF = module.get_id("gheight_half");
+    SEALEVELPRESSURE = module.get_id("sealevelpressure");
 
     operatorID = cdo_operator_id();
 
@@ -119,28 +129,33 @@ public:
     int vctSize = vct.size();
 
     if (Options::cdoVerbose)
-      for (int i = 0; i < vctSize / 2; ++i) cdo_print("vct: %5d %25.17f %25.17f", i, vct[i], vct[vctSize / 2 + i]);
+      {
+        auto numAB = vctSize / 2;
+        for (int i = 0; i < 10; ++i) cdo_print("vct: %5d %25.17f %25.17f", i, vct[i], vct[numAB + i]);
+        cdo_print("vct:   ...");
+        for (int i = numAB - 10; i < numAB; ++i) cdo_print("vct: %5d %25.17f %25.17f", i, vct[i], vct[numAB + i]);
+      }
 
     if (zaxisID_ML == -1) cdo_abort("No 3D variable with hybrid sigma pressure coordinate found!");
 
-    varList_init(varList1, vlistID1);
-    varListSetUniqueMemtype(varList1);
+    varList1 = VarList(vlistID1);
+    varList_set_unique_memtype(varList1);
 
-    auto nvars = vlistNvars(vlistID1);
+    auto numVars = varList1.numVars();
 
-    varIDs = search_varIDs(varList1, vlistID1, numFullLevels);
+    varIDs = varList_search_varIDs(varList1, numFullLevels);
 
     if (Options::cdoVerbose)
       {
         cdo_print("Found:");
         // clang-format off
-      if (-1 != varIDs.humID)     cdo_print("  %s -> %s", var_stdname(specific_humidity), varList1[varIDs.humID].name);
-      if (-1 != varIDs.tempID)    cdo_print("  %s -> %s", var_stdname(air_temperature), varList1[varIDs.tempID].name);
-      if (-1 != varIDs.psID)      cdo_print("  %s -> %s", var_stdname(surface_air_pressure), varList1[varIDs.psID].name);
-      if (-1 != varIDs.lnpsID)    cdo_print("  LOG(%s) -> %s", var_stdname(surface_air_pressure), varList1[varIDs.lnpsID].name);
-      if (-1 != varIDs.sgeopotID) cdo_print("  %s -> %s", var_stdname(surface_geopotential), varList1[varIDs.sgeopotID].name);
-      if (-1 != varIDs.geopotID)  cdo_print("  %s -> %s", var_stdname(geopotential), varList1[varIDs.geopotID].name);
-      if (-1 != varIDs.gheightID) cdo_print("  %s -> %s", var_stdname(geopotential_height), varList1[varIDs.gheightID].name);
+        if (-1 != varIDs.humID)     cdo_print("  %s -> %s", var_stdname(specific_humidity), varList1.vars[varIDs.humID].name);
+        if (-1 != varIDs.tempID)    cdo_print("  %s -> %s", var_stdname(air_temperature), varList1.vars[varIDs.tempID].name);
+        if (-1 != varIDs.psID)      cdo_print("  %s -> %s", var_stdname(surface_air_pressure), varList1.vars[varIDs.psID].name);
+        if (-1 != varIDs.lnpsID)    cdo_print("  LOG(%s) -> %s", var_stdname(surface_air_pressure), varList1.vars[varIDs.lnpsID].name);
+        if (-1 != varIDs.sgeopotID) cdo_print("  %s -> %s", var_stdname(surface_geopotential), varList1.vars[varIDs.sgeopotID].name);
+        if (-1 != varIDs.geopotID)  cdo_print("  %s -> %s", var_stdname(geopotential), varList1.vars[varIDs.geopotID].name);
+        if (-1 != varIDs.gheightID) cdo_print("  %s -> %s", var_stdname(geopotential_height), varList1.vars[varIDs.gheightID].name);
         // clang-format on
       }
 
@@ -149,24 +164,23 @@ public:
 
     if (varIDs.tempID == -1) cdo_abort("%s not found!", var_stdname(air_temperature));
 
-    for (int varID = 0; varID < nvars; ++varID)
+    for (int varID = 0; varID < numVars; ++varID)
       {
-        auto gridID = varList1[varID].gridID;
-        auto zaxisID = varList1[varID].zaxisID;
-
         if (operatorID == SEALEVELPRESSURE) varIDs.humID = -1;
 
-        if (gridInqType(gridID) == GRID_SPECTRAL && zaxisInqType(zaxisID) == ZAXIS_HYBRID)
-          cdo_abort("Spectral data on model level unsupported!");
+        const auto &var = varList1.vars[varID];
+        if (var.gridType == GRID_SPECTRAL && var.zaxisType == ZAXIS_HYBRID) cdo_abort("Spectral data on model level unsupported!");
 
-        if (gridInqType(gridID) == GRID_SPECTRAL) cdo_abort("Spectral data unsupported!");
+        if (var.gridType == GRID_SPECTRAL) cdo_abort("Spectral data unsupported!");
       }
+
     array = Varray<double>(gridsize);
     sgeopot = Varray<double>(gridsize);
     ps = Varray<double>(gridsize);
     temp = Varray<double>(gridsize * numFullLevels);
     halfPress = Varray<double>(gridsize * (numFullLevels + 1));
-    if (operatorID == GHEIGHT || operatorID == GHEIGHTHALF)
+
+    if (operatorID == GHEIGHT_FULL || operatorID == GHEIGHT_HALF)
       {
         if (varIDs.humID == -1)
           cdo_warning("%s not found - using algorithm without %s!", var_stdname(specific_humidity), var_stdname(specific_humidity));
@@ -191,7 +205,7 @@ public:
         else
           cdo_print("%s not found - using bottom layer of %s!", var_stdname(surface_geopotential), var_stdname(geopotential));
 
-        varray_fill(sgeopot, 0.0);
+        ranges::fill(sgeopot, 0.0);
       }
 
     presID = varIDs.lnpsID;
@@ -218,12 +232,12 @@ public:
       int var_id = -1;
       int varID = -1;
 
-      if (operatorID == GHEIGHT)
+      if (operatorID == GHEIGHT_FULL)
         {
           var_id = geopotential_height;
           varID = vlistDefVar(vlistID2, gridID0, zaxisID_ML, TIME_VARYING);
         }
-      else if (operatorID == GHEIGHTHALF)
+      else if (operatorID == GHEIGHT_HALF)
         {
           auto zaxisID_ML_Half = zaxisCreate(ZAXIS_HYBRID_HALF, numHalfLevels);
           zaxisDefVct(zaxisID_ML_Half, 2 * numHalfLevels, vct.data());
@@ -254,8 +268,9 @@ public:
     streamID2 = cdo_open_write(1);
     cdo_def_vlist(streamID2, vlistID2);
   }
+
   void
-  run()
+  run() override
   {
     int tsID = 0;
     while (true)
@@ -270,8 +285,8 @@ public:
           {
             int varID, levelID;
             cdo_inq_record(streamID1, &varID, &levelID);
-            size_t nmiss;
-            cdo_read_record(streamID1, array.data(), &nmiss);
+            size_t numMissVals;
+            cdo_read_record(streamID1, array.data(), &numMissVals);
 
             auto offset = gridsize * levelID;
 
@@ -305,32 +320,22 @@ public:
             check_range_var2d(tsID + 1, sgeopot, MIN_FIS, MAX_FIS, "Orography");
           }
 
-        check_range_var3d(tsID + 1, varList1[varIDs.tempID].nlevels, gridsize, temp, MIN_T, MAX_T, "Temperature");
+        check_range_var3d(tsID + 1, varList1.vars[varIDs.tempID].nlevels, gridsize, temp, MIN_T, MAX_T, "Temperature");
 
-        if (varIDs.humID != -1) check_range_var3d(tsID + 1, varList1[varIDs.humID].nlevels, gridsize, hum, -0.1, MAX_Q, "Humidity");
+        if (varIDs.humID != -1)
+          check_range_var3d(tsID + 1, varList1.vars[varIDs.humID].nlevels, gridsize, hum, -0.1, MAX_Q, "Humidity");
 
-        if (operatorID == GHEIGHT || operatorID == GHEIGHTHALF)
+        if (operatorID == GHEIGHT_FULL || operatorID == GHEIGHT_HALF)
           {
             vct_to_hybrid_pressure((double *) nullptr, halfPress.data(), vct.data(), ps.data(), numFullLevels, gridsize);
             array_copy(gridsize, sgeopot.data(), gheight.data() + gridsize * numFullLevels);
-            if (operatorID == GHEIGHT)
-              geopot_height_fulllevel(gheight.data(), temp.data(), hum.data(), halfPress.data(), gridsize, numFullLevels);
+            if (operatorID == GHEIGHT_FULL)
+              geopot_height_full(gheight.data(), temp.data(), hum.data(), halfPress.data(), gridsize, numFullLevels);
             else
-              geopot_height_halflevel(gheight.data(), temp.data(), hum.data(), halfPress.data(), gridsize, numFullLevels);
+              geopot_height_half(gheight.data(), temp.data(), hum.data(), halfPress.data(), gridsize, numFullLevels);
 
             int varID = 0;
-            auto nlevels = (operatorID == GHEIGHT) ? numFullLevels : numHalfLevels;
-            for (int levelID = 0; levelID < nlevels; ++levelID)
-              {
-                cdo_def_record(streamID2, varID, levelID);
-                cdo_write_record(streamID2, gheight.data() + levelID * gridsize, 0);
-              }
-          }
-        else if (operatorID == GHEIGHTHALF)
-          {
-
-            int varID = 0;
-            auto nlevels = numHalfLevels;
+            auto nlevels = (operatorID == GHEIGHT_FULL) ? numFullLevels : numHalfLevels;
             for (int levelID = 0; levelID < nlevels; ++levelID)
               {
                 cdo_def_record(streamID2, varID, levelID);
@@ -341,9 +346,8 @@ public:
           {
             vct_to_hybrid_pressure(fullPress.data(), halfPress.data(), vct.data(), ps.data(), numFullLevels, gridsize);
 
-            extrapolate_P(sealevelpressure.data(), &halfPress[gridsize * (numFullLevels)],
-                          &fullPress[gridsize * (numFullLevels - 1)], sgeopot.data(), &temp[gridsize * (numFullLevels - 1)],
-                          gridsize);
+            extrapolate_P(sealevelpressure.data(), &halfPress[gridsize * numFullLevels], &fullPress[gridsize * (numFullLevels - 1)],
+                          sgeopot.data(), &temp[gridsize * (numFullLevels - 1)], gridsize);
 
             cdo_def_record(streamID2, 0, 0);
             cdo_write_record(streamID2, sealevelpressure.data(), 0);
@@ -354,25 +358,13 @@ public:
         tsID++;
       }
   }
-  void
-  close()
-  {
 
+  void
+  close() override
+  {
     cdo_stream_close(streamID2);
     cdo_stream_close(streamID1);
 
     vlistDestroy(vlistID2);
-
-    cdo_finish();
   }
 };
-
-void *
-Derivepar(void *process)
-{
-  ModuleDerivepar derivepar;
-  derivepar.init(process);
-  derivepar.run();
-  derivepar.close();
-  return nullptr;
-}
