@@ -107,11 +107,12 @@ writeTimesteps(int MaxMonths, int recentYear, FieldVector3D &cei, int frequency,
           for (int recID = 0; recID < maxRecs; ++recID)
             {
               auto [varIDo, levelIDo] = recList[recID].get();
-              if (*otsID && varList1[varIDo].isConstant) continue;
+              if (*otsID && varList1.vars[varIDo].isConstant) continue;
 
               fieldc_div(cei[loopmonth - 2][0][levelIDo], (double) (tempdpm[loopmonth - 2] / 100.0));
               cdo_def_record(streamID4, varIDo, levelIDo);
-              cdo_write_record(streamID4, cei[loopmonth - 2][0][levelIDo].vec_d.data(), cei[loopmonth - 2][0][levelIDo].nmiss);
+              cdo_write_record(streamID4, cei[loopmonth - 2][0][levelIDo].vec_d.data(),
+                               cei[loopmonth - 2][0][levelIDo].numMissVals);
             }
           (*otsID)++;
         }
@@ -123,11 +124,11 @@ writeTimesteps(int MaxMonths, int recentYear, FieldVector3D &cei, int frequency,
       for (int recID = 0; recID < maxRecs; ++recID)
         {
           auto [varIDo, levelIDo] = recList[recID].get();
-          if (*otsID && varList1[varIDo].isConstant) continue;
+          if (*otsID && varList1.vars[varIDo].isConstant) continue;
 
           if (func2 == FieldFunc_Avg) fieldc_div(cei[0][0][levelIDo], (double) (tempdpy / 100.0));
           cdo_def_record(streamID4, varIDo, levelIDo);
-          cdo_write_record(streamID4, cei[0][0][levelIDo].vec_d.data(), cei[0][0][levelIDo].nmiss);
+          cdo_write_record(streamID4, cei[0][0][levelIDo].vec_d.data(), cei[0][0][levelIDo].numMissVals);
         }
       (*otsID)++;
     }
@@ -142,8 +143,8 @@ calculateOuterPeriod(Field &field, int MaxMonths, int recentYear, int endOfCalc,
 
   auto cdiStream = streamOpenRead(cdo_get_stream_name(0));
 
-  const auto cdiVlistID = streamInqVlist(cdiStream);
-  const auto cdiTaxisID = vlistInqTaxis(cdiVlistID);
+  auto cdiVlistID = streamInqVlist(cdiStream);
+  auto cdiTaxisID = vlistInqTaxis(cdiVlistID);
   std::vector<int> tempdpm(MaxMonths);
   int tempdpy = 0;
   for (int i = 0; i < MaxMonths; ++i) tempdpm[i] = 0;
@@ -154,10 +155,10 @@ calculateOuterPeriod(Field &field, int MaxMonths, int recentYear, int endOfCalc,
 
   while (true)
     {
-      const auto nrecs = streamInqTimestep(cdiStream, tsID++);
+      auto nrecs = streamInqTimestep(cdiStream, tsID++);
       if (nrecs == 0) break;
 
-      const auto vDateTime = taxisInqVdatetime(cdiTaxisID);
+      auto vDateTime = taxisInqVdatetime(cdiTaxisID);
       cdiDate_decode(vDateTime.date, &year, &month, &day);
       if (!lHasStarted && year != recentYear)
         continue;
@@ -189,7 +190,7 @@ calculateOuterPeriod(Field &field, int MaxMonths, int recentYear, int endOfCalc,
       for (int recID = 0; recID < nrecs; ++recID)
         {
           streamInqRecord(cdiStream, &varID, &levelID);
-          streamReadRecord(cdiStream, field.vec_d.data(), &field.nmiss);
+          streamReadRecord(cdiStream, field.vec_d.data(), &field.numMissVals);
 
           Field &pctls = varsPtemp[dayOfYear][0][levelID];
           if (selection == func_selle)
@@ -229,7 +230,7 @@ etccdi_op(ETCCDI_REQUEST &request)
 {
   constexpr int MaxDays = 373;
   constexpr int MaxMonths = 12;
-  FieldVector2D vars2[MaxDays];
+  FieldVector2D varsData2[MaxDays];
   HistogramSet hsets[MaxDays];
 
   const int operatorID = cdo_operator_id();
@@ -260,35 +261,38 @@ etccdi_op(ETCCDI_REQUEST &request)
       for (int month = 0; month < MaxMonths; ++month) dpm[month + year * MaxMonths] = 0;
     }
 
-  const auto streamID1 = cdo_open_read(0);
-  const auto streamID2 = cdo_open_read(1);
-  const auto streamID3 = cdo_open_read(2);
+  auto streamID1 = cdo_open_read(0);
+  auto streamID2 = cdo_open_read(1);
+  auto streamID3 = cdo_open_read(2);
 
-  const auto vlistID1 = cdo_stream_inq_vlist(streamID1);
-  const auto vlistID2 = cdo_stream_inq_vlist(streamID2);
-  const auto vlistID3 = cdo_stream_inq_vlist(streamID3);
-  const auto vlistID4 = vlistDuplicate(vlistID1);
+  auto vlistID1 = cdo_stream_inq_vlist(streamID1);
+  auto vlistID2 = cdo_stream_inq_vlist(streamID2);
+  auto vlistID3 = cdo_stream_inq_vlist(streamID3);
+  auto vlistID4 = vlistDuplicate(vlistID1);
 
   vlist_unpack(vlistID4);
 
-  vlist_compare(vlistID1, vlistID2, CmpVlist::All);
-  vlist_compare(vlistID1, vlistID3, CmpVlist::All);
+  VarList varList1(vlistID1);
+  VarList varList2(vlistID2);
 
-  const auto taxisID1 = vlistInqTaxis(vlistID1);
-  const auto taxisID2 = vlistInqTaxis(vlistID2);
-  const auto taxisID3 = vlistInqTaxis(vlistID3);
+  varList_compare(varList1, varList2);
+  varList_compare(varList1, VarList(vlistID3));
+
+  auto taxisID1 = vlistInqTaxis(vlistID1);
+  auto taxisID2 = vlistInqTaxis(vlistID2);
+  auto taxisID3 = vlistInqTaxis(vlistID3);
   // TODO - check that time axes 2 and 3 are equal
 
-  const auto taxisID4 = taxisDuplicate(taxisID1);
+  auto taxisID4 = taxisDuplicate(taxisID1);
   if (taxisHasBounds(taxisID4)) taxisDeleteBounds(taxisID4);
   vlistDefTaxis(vlistID4, taxisID4);
 
-  const auto streamID4 = cdo_open_write(3);
+  auto streamID4 = cdo_open_write(3);
 
-  const auto nvars = vlistNvars(vlistID1);
-  bool lOnlyOneVar = true;
+  auto numVars = varList1.numVars();
+  auto lOnlyOneVar = true;
 
-  if (nvars == 1)
+  if (numVars == 1)
     {
       cdiDefKeyString(vlistID4, 0, CDI_KEY_NAME, request.name);
       cdiDefKeyString(vlistID4, 0, CDI_KEY_LONGNAME, request.longname);
@@ -303,40 +307,40 @@ etccdi_op(ETCCDI_REQUEST &request)
 
   cdo_def_vlist(streamID4, vlistID4);
 
-  const auto maxrecs = vlistNrecs(vlistID1);
-  std::vector<RecordInfo> recList(maxrecs);
+  auto maxRecords = varList1.numRecords();
+  std::vector<RecordInfo> recordList(maxRecords);
 
-  const auto gridsizemax = vlistGridsizeMax(vlistID1);
+  auto gridsizemax = vlistGridsizeMax(vlistID1);
 
   Field field;
 
-  VarList varList1;
-  varList_init(varList1, vlistID1);
+  FieldVector3D varsData1(sumboot * (MaxDays - 1) + 1);
+  FieldVector3D cei(sumboot * MaxMonths);
+  FieldVector3D varsPtemp(MaxDays);
 
-  FieldVector3D vars1(sumboot * (MaxDays - 1) + 1), cei(sumboot * MaxMonths), varsPtemp(MaxDays);
   for (int dayOfYear = 0; dayOfYear < MaxDays; dayOfYear++)
     {
-      fields_from_vlist(vlistID1, vars1[dayOfYear], FIELD_VEC | FIELD_MEMTYPE);
+      field2D_init(varsData1[dayOfYear], varList1, FIELD_VEC | FIELD_MEMTYPE);
       wdaysSrc[dayOfYear] = false;
       for (int year = 0; year < sumboot; year++) wdaysRead[dayOfYear + year * (MaxDays - 1)] = false;
     }
 
   for (int dayOfYear = MaxDays; dayOfYear < sumboot * (MaxDays - 1) + 1; dayOfYear++)
-    fields_from_vlist(vlistID1, vars1[dayOfYear], FIELD_VEC | FIELD_MEMTYPE);
+    field2D_init(varsData1[dayOfYear], varList1, FIELD_VEC | FIELD_MEMTYPE);
 
   int tsID = 0;
 
   while (true)
     {
-      const auto nrecs = cdo_stream_inq_timestep(streamID2, tsID);
+      auto nrecs = cdo_stream_inq_timestep(streamID2, tsID);
       if (nrecs == 0) break;
 
       if (nrecs != cdo_stream_inq_timestep(streamID3, tsID))
         cdo_abort("Number of records at time step %d of %s and %s differ!", tsID + 1, cdo_get_stream_name(1),
                   cdo_get_stream_name(2));
 
-      const auto vDateTime2 = taxisInqVdatetime(taxisID2);
-      const auto vDateTime3 = taxisInqVdatetime(taxisID3);
+      auto vDateTime2 = taxisInqVdatetime(taxisID2);
+      auto vDateTime3 = taxisInqVdatetime(taxisID3);
 
       if (cdiDate_get(vDateTime2.date) != cdiDate_get(vDateTime3.date))
         cdo_abort("Verification dates at time step %d of %s and %s differ!", tsID + 1, cdo_get_stream_name(1),
@@ -348,36 +352,35 @@ etccdi_op(ETCCDI_REQUEST &request)
 
       if (dayOfYear < 0 || dayOfYear >= MaxDays) cdo_abort("Day %d out of range!", dayOfYear);
 
-      if (!vars2[dayOfYear].size())
+      if (!varsData2[dayOfYear].size())
         {
           wdaysSrc[dayOfYear] = true;
-          fields_from_vlist(vlistID2, vars2[dayOfYear], FIELD_VEC | FIELD_MEMTYPE);
-          fields_from_vlist(vlistID2, varsPtemp[dayOfYear], FIELD_VEC);
-          hsets[dayOfYear].create(nvars);
+          field2D_init(varsData2[dayOfYear], varList2, FIELD_VEC | FIELD_MEMTYPE);
+          field2D_init(varsPtemp[dayOfYear], varList2, FIELD_VEC);
+          hsets[dayOfYear].create(numVars);
 
-          for (int varID = 0; varID < nvars; ++varID)
-            hsets[dayOfYear].createVarLevels(varID, varList1[varID].nlevels, varList1[varID].gridsize);
+          for (const auto &var : varList1.vars) hsets[dayOfYear].createVarLevels(var.ID, var.nlevels, var.gridsize);
         }
 
       for (int recID = 0; recID < nrecs; ++recID)
         {
           int varID, levelID;
           cdo_inq_record(streamID2, &varID, &levelID);
-          cdo_read_record(streamID2, vars2[dayOfYear][varID][levelID]);
-          varsPtemp[dayOfYear][varID][levelID].nmiss = vars2[dayOfYear][varID][levelID].nmiss;
+          cdo_read_record(streamID2, varsData2[dayOfYear][varID][levelID]);
+          varsPtemp[dayOfYear][varID][levelID].numMissVals = varsData2[dayOfYear][varID][levelID].numMissVals;
         }
 
       for (int recID = 0; recID < nrecs; ++recID)
         {
           int varID, levelID;
           cdo_inq_record(streamID3, &varID, &levelID);
-          field.init(varList1[varID]);
+          field.init(varList1.vars[varID]);
           cdo_read_record(streamID3, field);
 
-          hsets[dayOfYear].defVarLevelBounds(varID, levelID, vars2[dayOfYear][varID][levelID], field);
+          hsets[dayOfYear].defVarLevelBounds(varID, levelID, varsData2[dayOfYear][varID][levelID], field);
         }
       // fieldsFree(vlistID2, vars2[dayOfYear]);
-      // fields_from_vlist(vlistID2, vars2[dayOfYear], FIELD_VEC);
+      // field2D_init(vars2[dayOfYear], varList2, FIELD_VEC);
 
       tsID++;
     }
@@ -389,10 +392,10 @@ etccdi_op(ETCCDI_REQUEST &request)
   int year;
   while (true)
     {
-      const auto nrecs = cdo_stream_inq_timestep(streamID1, tsID++);
+      auto nrecs = cdo_stream_inq_timestep(streamID1, tsID++);
       if (nrecs == 0) break;
 
-      const auto vDateTime = taxisInqVdatetime(taxisID1);
+      auto vDateTime = taxisInqVdatetime(taxisID1);
 
       int month, day;
       cdiDate_decode(vDateTime.date, &year, &month, &day);
@@ -419,9 +422,9 @@ etccdi_op(ETCCDI_REQUEST &request)
                 {
                   int varID, levelID;
                   cdo_inq_record(streamID1, &varID, &levelID);
-                  cdo_read_record(streamID1, vars1[dayOfYear + (year - request.startboot) * (MaxDays - 1)][varID][levelID]);
+                  cdo_read_record(streamID1, varsData1[dayOfYear + (year - request.startboot) * (MaxDays - 1)][varID][levelID]);
 
-                  if (tsID == 0) recList[recID].set(varID, levelID);
+                  if (tsID == 0) recordList[recID].set(varID, levelID);
                 }
             }
           else
@@ -436,9 +439,9 @@ etccdi_op(ETCCDI_REQUEST &request)
 
   for (year = 0; year < sumboot; year++)
     {
-      fields_from_vlist(vlistID1, cei[year * MaxMonths], FIELD_VEC);
+      field2D_init(cei[year * MaxMonths], varList1, FIELD_VEC);
       if (frequency == 8)
-        for (int month = 1; month < MaxMonths; ++month) fields_from_vlist(vlistID1, cei[year * MaxMonths + month], FIELD_VEC);
+        for (int month = 1; month < MaxMonths; ++month) field2D_init(cei[year * MaxMonths + month], varList1, FIELD_VEC);
     }
 
   // printf("Wir beginnen nun mit der Schleife.\n");
@@ -458,10 +461,11 @@ etccdi_op(ETCCDI_REQUEST &request)
     }
   if (Options::cdoVerbose) cdo_print("Calculated window days");
 
-  for (int varID = 0; varID < nvars; ++varID)
+  for (int varID = 0; varID < numVars; ++varID)
     {
-      if (varList1[varID].isConstant) continue;
-      for (int levelID = 0; levelID < varList1[varID].nlevels; ++levelID)
+      const auto &var1 = varList1.vars[varID];
+      if (var1.isConstant) continue;
+      for (int levelID = 0; levelID < var1.nlevels; ++levelID)
         {
           if (request.func2 == FieldFunc_Sum)
             {
@@ -471,7 +475,7 @@ etccdi_op(ETCCDI_REQUEST &request)
                     {
                       if (wdaysRead[loopdoy + ytoadd * (MaxDays - 1)])
                         {
-                          auto &source = vars1[loopdoy + ytoadd * (MaxDays - 1)][varID][levelID];
+                          auto &source = varsData1[loopdoy + ytoadd * (MaxDays - 1)][varID][levelID];
                           auto &hset = hsets[1];
                           hset.addVarLevelValues(varID, levelID, source);
                         }
@@ -481,7 +485,7 @@ etccdi_op(ETCCDI_REQUEST &request)
           else
             {
 #ifdef _OPENMP
-#pragma omp parallel for shared(sumboot, vars1, request, varID, levelID, hsets, wdays, wdaysRead) schedule(dynamic)
+#pragma omp parallel for shared(sumboot, varsData1, request, varID, levelID, hsets, wdays, wdaysRead) schedule(dynamic)
 #endif
               for (int loopdoy = 1; loopdoy < MaxDays; loopdoy++)
                 {
@@ -491,9 +495,8 @@ etccdi_op(ETCCDI_REQUEST &request)
                         {
                           for (int ano = 0; ano < request.ndates; ano++)
                             {
-                              auto &source
-                                  = vars1[wdays[ytoadd * request.ndates * (MaxDays - 1) + (loopdoy - 1) * request.ndates + ano + 1]]
-                                         [varID][levelID];
+                              auto &source = varsData1[wdays[ytoadd * request.ndates * (MaxDays - 1)
+                                                             + (loopdoy - 1) * request.ndates + ano + 1]][varID][levelID];
                               auto &hset = hsets[loopdoy];
                               hset.addVarLevelValues(varID, levelID, source);
                             }
@@ -508,10 +511,11 @@ etccdi_op(ETCCDI_REQUEST &request)
   if (Options::cdoVerbose) cdo_print("Added 30 years to histograms");
   if (lOnlyOneVar && ((!lOnlyRefPeriod && firstYear != request.startboot) || request.func2 == FieldFunc_Sum))
     {
-      for (int varID = 0; varID < nvars; ++varID)
+      for (int varID = 0; varID < numVars; ++varID)
         {
-          if (varList1[varID].isConstant) continue;
-          for (int levelID = 0; levelID < varList1[varID].nlevels; ++levelID)
+          const auto &var1 = varList1.vars[varID];
+          if (var1.isConstant) continue;
+          for (int levelID = 0; levelID < var1.nlevels; ++levelID)
             {
               if (request.func2 == FieldFunc_Sum)
                 {
@@ -536,7 +540,7 @@ etccdi_op(ETCCDI_REQUEST &request)
         }
       field.resize(gridsizemax);
       calculateOuterPeriod(field, MaxMonths, firstYear, request.startboot, cei, varsPtemp, frequency, taxisID4, streamID4, &otsID,
-                           varList1, recList, selection, request.func2);
+                           varList1, recordList, selection, request.func2);
     }
   else if (!lOnlyRefPeriod && firstYear != request.startboot)
     cdo_warning("Since you have more than one variable in the input file, only the bootstrapping period can be calculated");
@@ -546,13 +550,13 @@ etccdi_op(ETCCDI_REQUEST &request)
       for (bootsyear = request.startboot; bootsyear < request.endboot + 1; bootsyear++)
         {
           if (Options::cdoVerbose) cdo_print("Bootsyear: %d", bootsyear);
-          for (int varID = 0; varID < nvars; ++varID)
+          for (int varID = 0; varID < numVars; ++varID)
             {
-              if (varList1[varID].isConstant) continue;
-              for (int levelID = 0; levelID < varList1[varID].nlevels; ++levelID)
+              if (varList1.vars[varID].isConstant) continue;
+              for (int levelID = 0; levelID < varList1.vars[varID].nlevels; ++levelID)
                 {
 #ifdef _OPENMP
-#pragma omp parallel for shared(sumboot, wdaysRead, request, vars1, varID, levelID, hsets, wdays, cei) schedule(dynamic)
+#pragma omp parallel for shared(sumboot, wdaysRead, request, varsData1, varID, levelID, hsets, wdays, cei) schedule(dynamic)
 #endif
                   for (int loopdoy = 1; loopdoy < MaxDays; loopdoy++)
                     {
@@ -566,14 +570,14 @@ etccdi_op(ETCCDI_REQUEST &request)
                                       = ytoadd * request.ndates * (MaxDays - 1) + (loopdoy - 1) * request.ndates + ano + 1;
                                   if ((int((wdays[recentWday] - 1) / (MaxDays - 1)) + request.startboot) == bootsyear)
                                     {
-                                      auto &source = vars1[wdays[recentWday]][varID][levelID];
+                                      auto &source = varsData1[wdays[recentWday]][varID][levelID];
                                       auto &hset = hsets[loopdoy];
                                       hset.subVarLevelValues(varID, levelID, source);
                                     }
                                   // percyear cannot be smaller than request.startboot
                                   if ((int((wdays[recentWday] - 1) / (MaxDays - 1)) + request.startboot) == bootsyear - 1)
                                     {
-                                      auto &source = vars1[wdays[recentWday]][varID][levelID];
+                                      auto &source = varsData1[wdays[recentWday]][varID][levelID];
                                       auto &hset = hsets[loopdoy];
                                       hset.addVarLevelValues(varID, levelID, source);
                                     }
@@ -587,7 +591,7 @@ etccdi_op(ETCCDI_REQUEST &request)
                       if (subyear != bootsyear)
                         {
 #ifdef _OPENMP
-#pragma omp parallel for shared(sumboot, request, vars1, varID, levelID, hsets, wdaysRead, varsPtemp, vars2, cei, subyear, \
+#pragma omp parallel for shared(sumboot, request, varsData1, varID, levelID, hsets, wdaysRead, varsPtemp, varsData2, cei, subyear, \
                                 bootsyear, wdays, frequency) schedule(dynamic)
 #endif
                           for (int loopdoy = 1; loopdoy < MaxDays; loopdoy++)
@@ -602,7 +606,7 @@ etccdi_op(ETCCDI_REQUEST &request)
                                               = ytoadd * request.ndates * (MaxDays - 1) + (loopdoy - 1) * request.ndates + ano + 1;
                                           if ((int((wdays[recentWday] - 1) / (MaxDays - 1)) + request.startboot) == subyear)
                                             {
-                                              auto &source = vars1[wdays[recentWday]][varID][levelID];
+                                              auto &source = varsData1[wdays[recentWday]][varID][levelID];
                                               auto &hset = hsets[loopdoy];
                                               if (hset.addVarLevelValues(varID, levelID, source) == 1)
                                                 cdo_print("'%d', '%d", loopdoy, wdays[recentWday]);
@@ -618,8 +622,9 @@ etccdi_op(ETCCDI_REQUEST &request)
                                   auto &pctls = varsPtemp[loopdoy][varID][levelID];
                                   hsets[loopdoy].getVarLevelPercentiles(pctls, varID, levelID, request.pn);
                                   /*** Compare data with percentile ***/
-                                  auto &source = vars1[loopdoy + (bootsyear - request.startboot) * (MaxDays - 1)][varID][levelID];
-                                  auto &toCompare = vars2[loopdoy][varID][levelID];
+                                  auto &source
+                                      = varsData1[loopdoy + (bootsyear - request.startboot) * (MaxDays - 1)][varID][levelID];
+                                  auto &toCompare = varsData2[loopdoy][varID][levelID];
                                   field_copy(source, toCompare);
                                   if (selection == func_selle)
                                     vfarselle(toCompare, pctls);
@@ -662,7 +667,7 @@ etccdi_op(ETCCDI_REQUEST &request)
                                               = ytoadd * request.ndates * (MaxDays - 1) + (loopdoy - 1) * request.ndates + ano + 1;
                                           if ((int((wdays[recentWday] - 1) / (MaxDays - 1)) + request.startboot) == subyear)
                                             {
-                                              auto &source = vars1[wdays[recentWday]][varID][levelID];
+                                              auto &source = varsData1[wdays[recentWday]][varID][levelID];
                                               auto &hset = hsets[loopdoy];
                                               if (hset.subVarLevelValues(varID, levelID, source) == 1)
                                                 cdo_print("'%d', '%d", loopdoy, wdays[recentWday]);
@@ -697,15 +702,15 @@ etccdi_op(ETCCDI_REQUEST &request)
                   define_mid_of_time(frequency, taxisID4, bootsyear, month, MaxMonths);
                   cdo_def_timestep(streamID4, otsID);
 
-                  for (int recID = 0; recID < maxrecs; ++recID)
+                  for (int recID = 0; recID < maxRecords; ++recID)
                     {
-                      auto [varIDo, levelIDo] = recList[recID].get();
-                      if (otsID && varList1[varIDo].isConstant) continue;
+                      auto [varIDo, levelIDo] = recordList[recID].get();
+                      if (otsID && varList1.vars[varIDo].isConstant) continue;
 
                       cdo_def_record(streamID4, varIDo, levelIDo);
                       cdo_write_record(
                           streamID4, cei[(bootsyear - request.startboot) * MaxMonths + (month - 2)][varIDo][levelIDo].vec_d.data(),
-                          cei[(bootsyear - request.startboot) * MaxMonths + (month - 2)][varIDo][levelIDo].nmiss);
+                          cei[(bootsyear - request.startboot) * MaxMonths + (month - 2)][varIDo][levelIDo].numMissVals);
                     }
                   otsID++;
                 }
@@ -715,14 +720,14 @@ etccdi_op(ETCCDI_REQUEST &request)
               define_mid_of_time(frequency, taxisID4, bootsyear, 0, MaxMonths);
               cdo_def_timestep(streamID4, otsID);
 
-              for (int recID = 0; recID < maxrecs; ++recID)
+              for (int recID = 0; recID < maxRecords; ++recID)
                 {
-                  auto [varIDo, levelIDo] = recList[recID].get();
-                  if (otsID && varList1[varIDo].isConstant) continue;
+                  auto [varIDo, levelIDo] = recordList[recID].get();
+                  if (otsID && varList1.vars[varIDo].isConstant) continue;
 
                   cdo_def_record(streamID4, varIDo, levelIDo);
                   cdo_write_record(streamID4, cei[(bootsyear - request.startboot) * MaxMonths][varIDo][levelIDo].vec_d.data(),
-                                   cei[(bootsyear - request.startboot) * MaxMonths][varIDo][levelIDo].nmiss);
+                                   cei[(bootsyear - request.startboot) * MaxMonths][varIDo][levelIDo].numMissVals);
                 }
               otsID++;
             }
@@ -735,10 +740,10 @@ etccdi_op(ETCCDI_REQUEST &request)
       if (frequency == 8)
         for (int loopmonth = 1; loopmonth < MaxMonths; loopmonth++) field_fill(cei[loopmonth][0][0], 0.);
 
-      for (int varID = 0; varID < nvars; ++varID)
+      for (int varID = 0; varID < numVars; ++varID)
         {
-          if (varList1[varID].isConstant) continue;
-          for (int levelID = 0; levelID < varList1[varID].nlevels; ++levelID)
+          if (varList1.vars[varID].isConstant) continue;
+          for (int levelID = 0; levelID < varList1.vars[varID].nlevels; ++levelID)
             {
 #ifdef _OPENMP
 #pragma omp parallel for shared(request, wdaysRead, varID, levelID, hsets, varsPtemp) schedule(dynamic)
@@ -755,7 +760,7 @@ etccdi_op(ETCCDI_REQUEST &request)
                               // percyear cannot be smaller than request.startboot
                               if ((int((wdays[recentWday] - 1) / (MaxDays - 1)) + request.startboot) == bootsyear - 1)
                                 {
-                                  auto &source = vars1[wdays[recentWday]][varID][levelID];
+                                  auto &source = varsData1[wdays[recentWday]][varID][levelID];
                                   auto &hset = hsets[loopdoy];
                                   hset.addVarLevelValues(varID, levelID, source);
                                 }
@@ -771,11 +776,11 @@ etccdi_op(ETCCDI_REQUEST &request)
             }
         }
       field.resize(gridsizemax);
-      field.missval = vars1[1][0][0].missval;
-      field.size = vars1[1][0][0].size;
-      field.grid = vars1[1][0][0].grid;
+      field.missval = varsData1[1][0][0].missval;
+      field.size = varsData1[1][0][0].size;
+      field.grid = varsData1[1][0][0].grid;
       calculateOuterPeriod(field, MaxMonths, request.endboot + 1, lastYear + 1, cei, varsPtemp, frequency, taxisID4, streamID4,
-                           &otsID, varList1, recList, selection, request.func2);
+                           &otsID, varList1, recordList, selection, request.func2);
     }
 
   cdo_stream_close(streamID4);
@@ -784,35 +789,47 @@ etccdi_op(ETCCDI_REQUEST &request)
   cdo_stream_close(streamID1);
 }
 
-class ModuleEcaEtccdi
+class EcaEtccdi : public Process
 {
+public:
+  using Process::Process;
+  inline static CdoModule module = {
+    .name = "EcaEtccdi",
+    .operators = { { "etccdi_tx90p", func_selge, CMP_DATE, EcaEtccdiHelp },
+                   { "etccdi_tx10p", func_selle, CMP_DATE, EcaEtccdiHelp },
+                   { "etccdi_tn90p", func_selge, CMP_DATE, EcaEtccdiHelp },
+                   { "etccdi_tn10p", func_selle, CMP_DATE, EcaEtccdiHelp },
+                   { "etccdi_r95p", func_selge, CMP_DATE, EcaEtccdiHelp },
+                   { "etccdi_r99p", func_selge, CMP_DATE, EcaEtccdiHelp },
+                   { "etccdi", 0, CMP_DATE, EcaEtccdiHelp } },
+    .aliases = {},
+    .mode = EXPOSED,     // Module mode: 0:intern 1:extern
+    .number = CDI_REAL,  // Allowed number type
+    .constraints = { 3, 1, FilesOnly },
+  };
+  inline static RegisterEntry<EcaEtccdi> registration = RegisterEntry<EcaEtccdi>(module);
+
+  int ETCCDI_TX90P, ETCCDI_R99P, ETCCDI_R95P, ETCCDI_TX10P, ETCCDI_TN90P, ETCCDI_TN10P;
   ETCCDI_REQUEST request;
 
 public:
   void
-  init(void *process)
+  init() override
   {
-    cdo_initialize(process);
-
     if (cdo_operator_argc() < 3) cdo_abort("Too few arguments!");
     if (cdo_operator_argc() > 4) cdo_abort("Too many arguments!");
 
-    int ETCCDI_TX90P, ETCCDI_R99P, ETCCDI_R95P, ETCCDI_TX10P, ETCCDI_TN90P, ETCCDI_TN10P;
-
-    ETCCDI_TX90P = cdo_operator_add("etccdi_tx90p", func_selge, CMP_DATE, nullptr);
-    ETCCDI_R99P = cdo_operator_add("etccdi_r99p", func_selge, CMP_DATE, nullptr);
-    ETCCDI_R95P = cdo_operator_add("etccdi_r95p", func_selge, CMP_DATE, nullptr);
-    ETCCDI_TX10P = cdo_operator_add("etccdi_tx10p", func_selle, CMP_DATE, nullptr);
-    ETCCDI_TN90P = cdo_operator_add("etccdi_tn90p", func_selge, CMP_DATE, nullptr);
-    ETCCDI_TN10P = cdo_operator_add("etccdi_tn10p", func_selle, CMP_DATE, nullptr);
-    cdo_operator_add("etccdi", 0, CMP_DATE, nullptr);
-
-    
+    ETCCDI_TX90P = module.get_id("etccdi_tx90p");
+    ETCCDI_R99P = module.get_id("etccdi_r99p");
+    ETCCDI_R95P = module.get_id("etccdi_r95p");
+    ETCCDI_TX10P = module.get_id("etccdi_tx10p");
+    ETCCDI_TN90P = module.get_id("etccdi_tn90p");
+    ETCCDI_TN10P = module.get_id("etccdi_tn10p");
 
     request.ndates = parameter_to_int(cdo_operator_argv(0));
     request.startboot = parameter_to_int(cdo_operator_argv(1));
 
-    const auto operatorID = cdo_operator_id();
+    auto operatorID = cdo_operator_id();
     request.compare_type = cdo_operator_f2(cdo_operator_id());
     if (cdo_operator_argc() == 4 && 'm' == cdo_operator_argv(3)[0]) { request.compare_type = CMP_MONTH; }
 
@@ -886,27 +903,17 @@ public:
         request.pn = 10;
       }
   }
+
   void
-  run()
+  run() override
   {
     etccdi_op(request);
     /*  else
         EcaEtccdi(-1, ndates, startboot, endboot); */
   }
+
   void
-  close()
+  close() override
   {
-    cdo_finish();
   }
 };
-
-void *
-EcaEtccdi(void *process)
-{
-  ModuleEcaEtccdi ecaEtccdi;
-  ecaEtccdi.init(process);
-  ecaEtccdi.run();
-  ecaEtccdi.close();
-
-  return nullptr;
-}
