@@ -21,12 +21,24 @@
 
 // NO MISSING VALUE SUPPORT ADDED SO FAR
 
-class ModuleEofcoeff3d
+class Eofcoeff3d : public Process
 {
+public:
+  using Process::Process;
+  inline static CdoModule module = {
+    .name = "Eofcoeff3d",
+    .operators = { { "eofcoeff3d", EofcoeffHelp } },
+    .aliases = {},
+    .mode = EXPOSED,     // Module mode: 0:intern 1:extern
+    .number = CDI_REAL,  // Allowed number type
+    .constraints = { 2, OBASE, OnlyFirst },
+  };
+  inline static RegisterEntry<Eofcoeff3d> registration = RegisterEntry<Eofcoeff3d>(module);
+
   double missval1 = -999, missval2 = -999;
   int varID, levelID;
   int nrecs;
-  size_t nmiss;
+  size_t numMissVals;
 
   CdoStreamID streamID1;
   CdoStreamID streamID2;
@@ -34,14 +46,17 @@ class ModuleEofcoeff3d
   int vlistID1;
   int vlistID2;
 
+  VarList varList1;
+  VarList varList2;
+
   int gridID1;
 
   int taxisID2;
 
   size_t gridsize;
-  int nvars;
+  int numVars;
   int neof;
-  int nlevs;
+  int numLevels;
   int taxisID3;
 
   std::string fileSuffix;
@@ -53,45 +68,42 @@ class ModuleEofcoeff3d
 
 public:
   void
-  init(void *process)
+  init() override
   {
-    cdo_initialize(process);
-
-    if (process_self().m_ID != 0) cdo_abort("This operator can't be combined with other operators!");
-
-    cdo_operator_add("eofcoeff3d", 0, 0, nullptr);
-
     streamID1 = cdo_open_read(0);
     streamID2 = cdo_open_read(1);
 
     vlistID1 = cdo_stream_inq_vlist(streamID1);
     vlistID2 = cdo_stream_inq_vlist(streamID2);
 
+    varList1 = VarList(vlistID1);
+    varList2 = VarList(vlistID2);
+
     // taxisID1 = vlistInqTaxis(vlistID1);
     taxisID2 = vlistInqTaxis(vlistID2);
     taxisID3 = taxisDuplicate(taxisID2);
 
-    gridID1 = vlistInqVarGrid(vlistID1, 0);
-    int gridID2 = vlistInqVarGrid(vlistID2, 0);
+    gridID1 = varList1.vars[0].gridID;
+    int gridID2 = varList2.vars[0].gridID;
 
     gridsize = vlistGridsizeMax(vlistID1);
     if (gridsize != vlistGridsizeMax(vlistID2)) cdo_abort("Gridsize of input files does not match!");
 
-    if (vlistNgrids(vlistID2) > 1 || vlistNgrids(vlistID1) > 1) cdo_abort("Too many different grids in input");
+    if (vlistNumGrids(vlistID2) > 1 || vlistNumGrids(vlistID1) > 1) cdo_abort("Too many different grids in input");
 
-    nvars = (vlistNvars(vlistID1) == vlistNvars(vlistID2)) ? vlistNvars(vlistID1) : -1;
-    nlevs = zaxisInqSize(vlistInqVarZaxis(vlistID1, 0));
+    numVars = (varList1.numVars() == varList2.numVars()) ? varList1.numVars() : -1;
+    numLevels = varList1.vars[0].nlevels;
 
     if (gridID1 != gridID2) cdo_compare_grids(gridID1, gridID2);
 
     fileSuffix = FileUtils::gen_suffix(cdo_inq_filetype(streamID1), vlistID1, cdo_get_stream_name(0));
 
-    eof = FieldVector3D(nvars);
-    for (varID = 0; varID < nvars; ++varID) eof[varID].resize(nlevs);
+    eof = FieldVector3D(numVars);
+    for (varID = 0; varID < numVars; ++varID) eof[varID].resize(numLevels);
   }
 
   void
-  run()
+  run() override
   {
     while (1)
       {
@@ -101,18 +113,18 @@ public:
         for (int recID = 0; recID < nrecs; ++recID)
           {
             cdo_inq_record(streamID1, &varID, &levelID);
-            missval1 = vlistInqVarMissval(vlistID1, varID);
+            missval1 = varList1.vars[varID].missval;
             eof[varID][levelID].resize((eofID == 0) ? 1 : eofID + 1);
             eof[varID][levelID][eofID].grid = gridID1;
             eof[varID][levelID][eofID].missval = missval1;
             eof[varID][levelID][eofID].resize(gridsize);
-            varray_fill(gridsize, eof[varID][levelID][eofID].vec_d, missval1);
+            ranges::fill(eof[varID][levelID][eofID].vec_d, missval1);
 
-            if (varID >= nvars) cdo_abort("Internal error - varID >= nvars");
-            if (levelID >= nlevs) cdo_abort("Internal error - levelID >= nlevs");
+            if (varID >= numVars) cdo_abort("Internal error - varID >= nvars");
+            if (levelID >= numLevels) cdo_abort("Internal error - levelID >= nlevs");
 
-            cdo_read_record(streamID1, eof[varID][levelID][eofID].vec_d.data(), &nmiss);
-            eof[varID][levelID][eofID].nmiss = nmiss;
+            cdo_read_record(streamID1, eof[varID][levelID][eofID].vec_d.data(), &numMissVals);
+            eof[varID][levelID][eofID].numMissVals = numMissVals;
           }
         eofID++;
       }
@@ -121,7 +133,7 @@ public:
 
     if (Options::cdoVerbose) cdo_print("%s contains %i eof's", cdo_get_stream_name(0), neof);
     // Create 1x1 Grid for output
-    const auto gridID3 = gridCreate(GRID_LONLAT, 1);
+    auto gridID3 = gridCreate(GRID_LONLAT, 1);
     gridDefXsize(gridID3, 1);
     gridDefYsize(gridID3, 1);
     double xvals = 0, yvals = 0;
@@ -129,7 +141,7 @@ public:
     gridDefYvals(gridID3, &yvals);
 
     double zvals = 0.;
-    const auto zaxisID3 = zaxisCreate(ZAXIS_GENERIC, 1);
+    auto zaxisID3 = zaxisCreate(ZAXIS_GENERIC, 1);
     zaxisDefLevels(zaxisID3, &zvals);
     cdiDefKeyString(zaxisID3, CDI_GLOBAL, CDI_KEY_NAME, "zaxis_Reduced");
     cdiDefKeyString(zaxisID3, CDI_GLOBAL, CDI_KEY_LONGNAME,
@@ -137,16 +149,16 @@ public:
 
     // Create var-list and time-axis for output
 
-    const auto vlistID3 = vlistDuplicate(vlistID2);
+    auto vlistID3 = vlistDuplicate(vlistID2);
 
-    const auto ngrids = vlistNgrids(vlistID3);
+    auto ngrids = vlistNumGrids(vlistID3);
     for (int i = 0; i < ngrids; ++i) vlistChangeGridIndex(vlistID3, i, gridID3);
 
-    const auto nzaxis = vlistNzaxis(vlistID3);
+    auto nzaxis = vlistNumZaxis(vlistID3);
     for (int i = 0; i < nzaxis; ++i) vlistChangeZaxisIndex(vlistID3, i, zaxisID3);
 
     vlistDefTaxis(vlistID3, taxisID3);
-    for (varID = 0; varID < nvars; ++varID) vlistDefVarTimetype(vlistID3, varID, TIME_VARYING);
+    for (varID = 0; varID < numVars; ++varID) vlistDefVarTimetype(vlistID3, varID, TIME_VARYING);
 
     // open streams for eofcoeff output
 
@@ -168,14 +180,14 @@ public:
     Field in;
     in.resize(gridsize);
     in.grid = gridID1;
-    FieldVector2D out(nvars);
-    for (varID = 0; varID < nvars; ++varID)
+    FieldVector2D out(numVars);
+    for (varID = 0; varID < numVars; ++varID)
       {
         out[varID].resize(neof);
         for (eofID = 0; eofID < neof; eofID++)
           {
             out[varID][eofID].missval = missval1;
-            out[varID][eofID].nmiss = 0;
+            out[varID][eofID].numMissVals = 0;
             out[varID][eofID].resize(1);
           }
       }
@@ -186,7 +198,7 @@ public:
         nrecs = cdo_stream_inq_timestep(streamID2, tsID);
         if (nrecs == 0) break;
 
-        for (varID = 0; varID < nvars; ++varID)
+        for (varID = 0; varID < numVars; ++varID)
           for (eofID = 0; eofID < neof; eofID++)
             {
               out[varID][eofID].vec_d[0] = 0;
@@ -199,14 +211,14 @@ public:
         for (int recID = 0; recID < nrecs; ++recID)
           {
             cdo_inq_record(streamID2, &varID, &levelID);
-            cdo_read_record(streamID2, in.vec_d.data(), &in.nmiss);
-            missval2 = vlistInqVarMissval(vlistID2, varID);
+            cdo_read_record(streamID2, in.vec_d.data(), &in.numMissVals);
+            missval2 = varList2.vars[varID].missval;
 
             for (eofID = 0; eofID < neof; eofID++)
               {
                 if (recID == 0) cdo_def_timestep(streamIDs[eofID], tsID);
 
-                nmiss = 0;
+                numMissVals = 0;
                 for (size_t i = 0; i < gridsize; ++i)
                   {
                     if (!DBL_IS_EQUAL(in.vec_d[i], missval2) && !DBL_IS_EQUAL(eof[varID][levelID][eofID].vec_d[i], missval1))
@@ -215,26 +227,26 @@ public:
                         out[varID][eofID].vec_d[0] += in.vec_d[i] * eof[varID][levelID][eofID].vec_d[i];
                       }
                     else
-                      nmiss += 1;
+                      numMissVals += 1;
                   }
                 /*
-                if ( nmiss ) {
-                  out[varID][eofID].nmiss=1;
+                if ( numMissVals ) {
+                  out[varID][eofID].numMissVals=1;
                   out[varID][eofID].vec_d[0] = missval2;
                 }
                 */
               }
 
-            if (varID >= nvars) cdo_abort("Internal error - varID >= nvars");
-            if (levelID >= nlevs) cdo_abort("Internal error - levelID >= nlevs");
+            if (varID >= numVars) cdo_abort("Internal error - varID >= nvars");
+            if (levelID >= numLevels) cdo_abort("Internal error - levelID >= nlevs");
           }
 
         for (eofID = 0; eofID < neof; eofID++)
           {
-            for (varID = 0; varID < nvars; ++varID)
+            for (varID = 0; varID < numVars; ++varID)
               {
                 cdo_def_record(streamIDs[eofID], varID, 0);
-                cdo_write_record(streamIDs[eofID], out[varID][eofID].vec_d.data(), out[varID][eofID].nmiss);
+                cdo_write_record(streamIDs[eofID], out[varID][eofID].vec_d.data(), out[varID][eofID].numMissVals);
               }
           }
 
@@ -243,24 +255,11 @@ public:
   }
 
   void
-  close()
+  close() override
   {
     for (eofID = 0; eofID < neof; eofID++) cdo_stream_close(streamIDs[eofID]);
 
     cdo_stream_close(streamID2);
     cdo_stream_close(streamID1);
-
-    cdo_finish();
   }
 };
-
-void *
-Eofcoeff3d(void *process)
-{
-  ModuleEofcoeff3d eofcoeff3d;
-  eofcoeff3d.init(process);
-  eofcoeff3d.run();
-  eofcoeff3d.close();
-
-  return nullptr;
-}
