@@ -72,7 +72,7 @@ exprs_from_file(const std::vector<std::string> &exprArgv)
   return buffer.str();
 }
 
-#define MAX_PARAMS 4096
+constexpr int MaxParams = 4096;
 
 static std::size_t
 replace_all(std::string &inout, const std::string &what, const std::string &with)
@@ -95,9 +95,9 @@ exprs_expand(std::string &exprString, const VarList &varList)
   if (exprString.find(templateName) != std::string::npos)
     {
       replaceTemplate = true;
-      for (size_t varID = 0, n = varList.size(); varID < n; ++varID)
+      for (const auto &var : varList.vars)
         {
-          if (templateName == varList[varID].name)
+          if (templateName == var.name)
             {
               replaceTemplate = false;
               break;
@@ -114,10 +114,10 @@ exprs_expand(std::string &exprString, const VarList &varList)
           if (string.find(templateName) == std::string::npos) { exprStringNew += string + ";"; }
           else
             {
-              for (size_t varID = 0, n = varList.size(); varID < n; ++varID)
+              for (const auto &var : varList.vars)
                 {
                   auto tmpString = string;
-                  replace_all(tmpString, templateName, varList[varID].name);
+                  replace_all(tmpString, templateName, var.name);
                   exprStringNew += tmpString + ";";
                 }
             }
@@ -130,76 +130,61 @@ exprs_expand(std::string &exprString, const VarList &varList)
 }
 
 static void
-params_init(std::vector<ParamEntry> &params, const VarList &varList, int vlistID)
+params_init(std::vector<ParamEntry> &params, const VarList &varList)
 {
-  auto nvars = vlistNvars(vlistID);
-  for (int varID = 0; varID < nvars; ++varID)
+  for (const auto &var : varList.vars)
     {
-      const auto &var = varList[varID];
+      auto &param = params[var.ID];
+      auto stdname = cdo::inq_key_string(varList.vlistID, var.ID, CDI_KEY_STDNAME);
 
-      auto stdname = cdo::inq_key_string(vlistID, varID, CDI_KEY_STDNAME);
-
-      params[varID].type = ParamType::VAR;
-      params[varID].isValid = true;
-      params[varID].hasMV = true;
-      params[varID].gridID = var.gridID;
-      params[varID].zaxisID = var.zaxisID;
-      params[varID].datatype = var.datatype;
-      params[varID].steptype = var.timetype;
-      params[varID].nlat = gridInqYsize(var.gridID);
-      params[varID].ngp = var.gridsize;
-      params[varID].nlev = var.nlevels;
-      params[varID].missval = var.missval;
-      params[varID].name = var.name;
-      if (var.longname.size()) params[varID].longname = var.longname;
-      if (var.units.size()) params[varID].units = var.units;
-      if (stdname.size()) params[varID].stdname = stdname;
+      param.type = ParamType::VAR;
+      param.isValid = true;
+      param.hasMV = true;
+      param.gridID = var.gridID;
+      param.zaxisID = var.zaxisID;
+      param.datatype = var.dataType;
+      param.steptype = var.timeType;
+      param.nlat = gridInqYsize(var.gridID);
+      param.ngp = var.gridsize;
+      param.nlev = var.nlevels;
+      param.missval = var.missval;
+      param.name = var.name;
+      if (var.longname.size()) param.longname = var.longname;
+      if (var.units.size()) param.units = var.units;
+      if (stdname.size()) param.stdname = stdname;
     }
 }
 
 static void
 params_delete(const std::vector<ParamEntry> &params)
 {
-  for (int varID = 0; varID < MAX_PARAMS; ++varID)
+  for (auto &param : params)
     {
-      if (params[varID].data) delete[] params[varID].data;
+      if (param.data) delete[] param.data;
     }
 }
 
 static void
 params_add_coord(ParseParamType &parseArg, int coord, int cdiID, size_t size, const std::string &units, const std::string &longname)
 {
-  auto ncoords = parseArg.ncoords;
+  auto ncoords = parseArg.numCoords;
   if (ncoords >= parseArg.maxCoords) cdo_abort("Too many coordinates (limit=%d)", parseArg.maxCoords);
 
-  parseArg.coords[ncoords].needed = false;
-  parseArg.coords[ncoords].coord = coord;
-  parseArg.coords[ncoords].cdiID = cdiID;
-  parseArg.coords[ncoords].size = size;
-  if (units.size()) parseArg.coords[ncoords].units = units;
-  if (longname.size()) parseArg.coords[ncoords].longname = longname;
+  auto &coords = parseArg.coords[ncoords];
+  coords.needed = false;
+  coords.coord = coord;
+  coords.cdiID = cdiID;
+  coords.size = size;
+  if (units.size()) coords.units = units;
+  if (longname.size()) coords.longname = longname;
 
-  parseArg.ncoords++;
-}
-
-int
-params_get_coord_ID(const ParseParamType &parseArg, int coord, int cdiID)
-{
-  auto ncoords = parseArg.ncoords;
-  for (int coordID = 0; coordID < ncoords; ++coordID)
-    {
-      if (parseArg.coords[coordID].coord == coord && parseArg.coords[coordID].cdiID == cdiID) return coordID;
-    }
-
-  cdo_abort("%s: coordinate %c not found!", __func__, coord);
-
-  return -1;
+  parseArg.numCoords++;
 }
 
 static void
 params_add_coordinates(int vlistID, ParseParamType &parseArg)
 {
-  auto ngrids = vlistNgrids(vlistID);
+  auto ngrids = vlistNumGrids(vlistID);
   for (int index = 0; index < ngrids; ++index)
     {
       auto gridID = vlistGrid(vlistID, index);
@@ -214,7 +199,7 @@ params_add_coordinates(int vlistID, ParseParamType &parseArg)
       params_add_coord(parseArg, 'g', gridID, size, "", "grid cell indices");
     }
 
-  auto nzaxis = vlistNzaxis(vlistID);
+  auto nzaxis = vlistNumZaxis(vlistID);
   for (int index = 0; index < nzaxis; ++index)
     {
       auto zaxisID = vlistZaxis(vlistID, index);
@@ -235,12 +220,13 @@ params_add_ts(ParseParamType &parseArg)
   auto varID = parseArg.nparams;
   if (varID >= parseArg.maxparams) cdo_abort("Too many parameter (limit=%d)", parseArg.maxparams);
 
-  params[varID].name = "_timestep_info";
-  params[varID].gridID = parseArg.pointID;
-  params[varID].zaxisID = parseArg.surfaceID;
-  params[varID].steptype = TIME_VARYING;
-  params[varID].ngp = CoordIndex::LEN;
-  params[varID].nlev = 1;
+  auto &param = params[varID];
+  param.name = "_timestep_info";
+  param.gridID = parseArg.pointID;
+  param.zaxisID = parseArg.surfaceID;
+  param.steptype = TIME_VARYING;
+  param.ngp = CoordIndex::LEN;
+  param.nlev = 1;
 
   parseArg.nparams++;
   parseArg.cnparams++;
@@ -251,25 +237,25 @@ params_add_ts(ParseParamType &parseArg)
 static void
 parse_param_init(ParseParamType &parseArg, int vlistID, int pointID, int zonalID, int surfaceID)
 {
-  auto nvars = vlistNvars(vlistID);
-  auto ngrids = vlistNgrids(vlistID);
-  auto nzaxis = vlistNzaxis(vlistID);
+  auto numVars = vlistNvars(vlistID);
+  auto ngrids = vlistNumGrids(vlistID);
+  auto nzaxis = vlistNumZaxis(vlistID);
   auto maxCoords = ngrids * 5 + nzaxis * 3;
 
-  parseArg.maxparams = MAX_PARAMS;
-  parseArg.params.resize(MAX_PARAMS);
-  parseArg.nparams = nvars;
-  parseArg.cnparams = nvars;
-  parseArg.nvars1 = nvars;
+  parseArg.maxparams = MaxParams;
+  parseArg.params.resize(MaxParams);
+  parseArg.nparams = numVars;
+  parseArg.cnparams = numVars;
+  parseArg.numVars1 = numVars;
   parseArg.init = true;
   parseArg.debug = (Options::cdoVerbose != 0);
   parseArg.pointID = pointID;
   parseArg.zonalID = zonalID;
   parseArg.surfaceID = surfaceID;
-  parseArg.needed.resize(nvars);
+  parseArg.needed.resize(numVars);
   parseArg.coords.resize(maxCoords);
   parseArg.maxCoords = maxCoords;
-  parseArg.ncoords = 0;
+  parseArg.numCoords = 0;
 }
 
 static int
@@ -277,7 +263,7 @@ genZonalID(int vlistID)
 {
   int zonalID = -1;
 
-  auto ngrids = vlistNgrids(vlistID);
+  auto ngrids = vlistNumGrids(vlistID);
   for (int index = 0; index < ngrids; ++index)
     {
       auto gridID = vlistGrid(vlistID, index);
@@ -327,19 +313,23 @@ set_date_and_time(ParamEntry &varts, int calendar, int tsID, const CdiDateTime &
   varts.data[CoordIndex::DPY] = days_per_year(calendar, year);
 }
 
-static void
-addOperators(void)
+class Exprf : public Process
 {
-  // clang-format off
-  cdo_operator_add("expr",   1, 1, "expressions");
-  cdo_operator_add("exprf",  1, 0, "expr script filename");
-  cdo_operator_add("aexpr",  0, 1, "expressions");
-  cdo_operator_add("aexprf", 0, 0, "expr script filename");
-  // clang-format on
-}
+public:
+  using Process::Process;
+  inline static CdoModule module = {
+    .name = "Exprf",
+    .operators = { { "expr", 1, 1, "expressions", ExprHelp },
+                   { "exprf", 1, 0, "exprscriptfilename", ExprHelp },
+                   { "aexpr", 0, 1, "expressions", ExprHelp },
+                   { "aexprf", 0, 0, "exprscriptfilename", ExprHelp } },
+    .aliases = {},
+    .mode = EXPOSED,     // Module mode: 0:intern 1:extern
+    .number = CDI_REAL,  // Allowed number type
+    .constraints = { 1, 1, NoRestriction },
+  };
+  inline static RegisterEntry<Exprf> registration = RegisterEntry<Exprf>(module);
 
-class ModuleExprf
-{
   CdoStreamID streamID1;
   CdoStreamID streamID2;
 
@@ -350,8 +340,8 @@ class ModuleExprf
 
   int vartsID;
 
-  int nvars1;
-  int nvars2;
+  int numVars1;
+  int numVars2;
 
   ParseParamType parseArg;
   CdiDateTime vDateTime0{};
@@ -366,12 +356,8 @@ class ModuleExprf
 
 public:
   void
-  init(void *process)
+  init() override
   {
-    cdo_initialize(process);
-
-    addOperators();
-
     auto operatorID = cdo_operator_id();
     bool replacesVariables = cdo_operator_f1(operatorID);
     bool readsCommandLine = cdo_operator_f2(operatorID);
@@ -385,13 +371,12 @@ public:
     streamID1 = cdo_open_read(0);
     auto vlistID1 = cdo_stream_inq_vlist(streamID1);
 
-    VarList varList1;
-    varList_init(varList1, vlistID1);
+    VarList varList1(vlistID1);
 
     exprString = exprs_expand(exprString, varList1);
     if (Options::cdoVerbose) cdo_print(exprString);
 
-    nvars1 = vlistNvars(vlistID1);
+    numVars1 = varList1.numVars();
 
     pointID = gridCreate(GRID_GENERIC, 1);
     zonalID = genZonalID(vlistID1);
@@ -400,10 +385,10 @@ public:
     parse_param_init(parseArg, vlistID1, pointID, zonalID, surfaceID);
 
     auto &params = parseArg.params;
-    params_init(parseArg.params, varList1, vlistID1);
+    params_init(parseArg.params, varList1);
 
     // Set all input variables to 'needed' if replacing is switched off
-    for (int varID = 0; varID < nvars1; ++varID) parseArg.needed[varID] = !replacesVariables;
+    for (int varID = 0; varID < numVars1; ++varID) parseArg.needed[varID] = !replacesVariables;
 
     // init function rand()
     std::srand(Options::Random_Seed);
@@ -426,7 +411,7 @@ public:
     parseArg.init = false;
 
     if (Options::cdoVerbose)
-      for (int varID = 0; varID < nvars1; ++varID)
+      for (int varID = 0; varID < numVars1; ++varID)
         if (parseArg.needed[varID]) cdo_print("Needed var: %d %s", varID, params[varID].name);
 
     if (Options::cdoVerbose)
@@ -442,13 +427,13 @@ public:
     if (!replacesVariables)
       {
         int pidx = 0;
-        for (int varID = 0; varID < nvars1; ++varID)
+        for (int varID = 0; varID < numVars1; ++varID)
           {
             params[varID].select = false;
             if (!params[varID].remove)
               {
                 varIDmap[pidx++] = varID;
-                auto nlevels = varList1[varID].nlevels;
+                auto nlevels = varList1.vars[varID].nlevels;
                 // printf("Replace %d nlevs %d\n", varID, nlevels);
                 for (int levID = 0; levID < nlevels; levID++) vlistDefFlag(vlistID1, varID, levID, true);
               }
@@ -460,8 +445,8 @@ public:
     for (int pidx = 0, n = parseArg.nparams; pidx < n; pidx++)
       {
         const auto &param = params[pidx];
-        if (pidx < nvars1 && !param.select) continue;
-        if (pidx >= nvars1)
+        if (pidx < numVars1 && !param.select) continue;
+        if (pidx >= numVars1)
           {
             if (param.type == ParamType::CONST) continue;
             if (param.name[0] == '_') continue;
@@ -490,15 +475,15 @@ public:
 
     if (Options::cdoVerbose)
       {
-        for (int varID = 0; varID < nvars1; ++varID)
+        for (int varID = 0; varID < numVars1; ++varID)
           if (parseArg.needed[varID]) cdo_print("needed: %d %s", varID, parseArg.params[varID].name);
-        cdo_print("vlistNvars(vlistID1)=%d, vlistNvars(vlistID2)=%d", vlistNvars(vlistID1), vlistNvars(vlistID2));
+        cdo_print("numVars1=%d, numVars2=%d", numVars1, numVars2);
       }
 
-    nvars2 = vlistNvars(vlistID2);
-    if (nvars2 == 0) cdo_abort("No output variable found!");
+    numVars2 = vlistNvars(vlistID2);
+    if (numVars2 == 0) cdo_abort("No output variable found!");
 
-    for (int varID = 0; varID < nvars1; ++varID)
+    for (int varID = 0; varID < numVars1; ++varID)
       {
         if (parseArg.needed[varID])
           {
@@ -507,13 +492,13 @@ public:
           }
       }
 
-    for (int varID = parseArg.nvars1, n = parseArg.nparams; varID < n; ++varID)
+    for (int varID = parseArg.numVars1, n = parseArg.nparams; varID < n; ++varID)
       {
         auto nItems = std::max((size_t) 4, params[varID].ngp * params[varID].nlev);
         params[varID].data = new double[nItems];
       }
 
-    for (int i = 0, n = parseArg.ncoords; i < n; ++i)
+    for (int i = 0, n = parseArg.numCoords; i < n; ++i)
       {
         if (parseArg.coords[i].needed)
           {
@@ -564,7 +549,7 @@ public:
                   }
                 else if (coord == 'd')
                   {
-                    varray_fill(nlev, cdata, 1.0);
+                    ranges::fill(cdata, 1.0);
                     if (zaxisInqLbounds(zaxisID, nullptr) && zaxisInqUbounds(zaxisID, nullptr))
                       {
                         std::vector<double> lbounds(nlev), ubounds(nlev);
@@ -579,7 +564,7 @@ public:
           }
       }
 
-    for (int varID = parseArg.nvars1, n = parseArg.nparams; varID < n; ++varID)
+    for (int varID = parseArg.numVars1, n = parseArg.nparams; varID < n; ++varID)
       {
         auto coord = params[varID].coord;
         if (coord)
@@ -624,7 +609,7 @@ public:
   }
 
   void
-  run()
+  run() override
   {
     int tsID = 0;
     while (true)
@@ -643,10 +628,10 @@ public:
 
         cdo_def_timestep(streamID2, tsID);
 
-        // for (int varID = 0; varID < nvars1; ++varID) printf(">>> %s %d\n", params[varID].name.c_str(), params[varID].isValid);
-        for (int varID = 0; varID < nvars1; ++varID) params[varID].isValid = true;
-        for (int varID = 0; varID < nvars1; ++varID)
-          if (tsID == 0 || params[varID].steptype != TIME_CONSTANT) params[varID].nmiss = 0;
+        // for (int varID = 0; varID < numVars1; ++varID) printf(">>> %s %d\n", params[varID].name.c_str(), params[varID].isValid);
+        for (int varID = 0; varID < numVars1; ++varID) params[varID].isValid = true;
+        for (int varID = 0; varID < numVars1; ++varID)
+          if (tsID == 0 || params[varID].steptype != TIME_CONSTANT) params[varID].numMissVals = 0;
 
         for (int recID = 0; recID < nrecs; ++recID)
           {
@@ -656,21 +641,22 @@ public:
               {
                 auto offset = params[varID].ngp * levelID;
                 auto vardata = params[varID].data + offset;
-                size_t nmiss;
-                cdo_read_record(streamID1, vardata, &nmiss);
-                params[varID].nmiss += nmiss;
+                size_t numMissVals;
+                cdo_read_record(streamID1, vardata, &numMissVals);
+                params[varID].numMissVals += numMissVals;
 
-                if (nmiss > 0) cdo_check_missval(params[varID].missval, params[varID].name);
+                if (numMissVals > 0) cdo_check_missval(params[varID].missval, params[varID].name);
               }
           }
 
-        for (int varID = 0; varID < nvars2; ++varID)
+        for (int varID = 0; varID < numVars2; ++varID)
           {
             auto pidx = varIDmap[varID];
-            if (pidx < nvars1) continue;
+            if (pidx < numVars1) continue;
 
-            params[pidx].nmiss = 0;
-            varray_fill(params[pidx].ngp * params[pidx].nlev, params[pidx].data, 0.0);
+            auto &param = params[pidx];
+            param.numMissVals = 0;
+            ranges::fill_n(param.data, param.ngp * param.nlev, 0.0);
           }
 
         parseArg.cnparams = vartsID + 1;
@@ -684,7 +670,7 @@ public:
           yylex_destroy(scanner);
         }
 
-        for (int varID = 0; varID < nvars2; ++varID)
+        for (int varID = 0; varID < numVars2; ++varID)
           {
             auto pidx = varIDmap[varID];
 
@@ -698,9 +684,9 @@ public:
               {
                 auto offset = ngp * levelID;
                 double *vardata = params[pidx].data + offset;
-                auto nmiss = array_num_mv(ngp, vardata, missval);
+                auto numMissVals = array_num_mv(ngp, vardata, missval);
                 cdo_def_record(streamID2, varID, levelID);
-                cdo_write_record(streamID2, vardata, nmiss);
+                cdo_write_record(streamID2, vardata, numMissVals);
               }
           }
 
@@ -709,7 +695,7 @@ public:
   }
 
   void
-  close()
+  close() override
   {
     cdo_stream_close(streamID2);
     cdo_stream_close(streamID1);
@@ -717,21 +703,5 @@ public:
     vlistDestroy(vlistID2);
 
     params_delete(parseArg.params);
-
-    gridDestroy(pointID);
-    if (zonalID != -1) gridDestroy(zonalID);
-
-    cdo_finish();
   }
 };
-
-void *
-Exprf(void *process)
-{
-  ModuleExprf expr;
-  expr.init(process);
-  expr.run();
-  expr.close();
-
-  return nullptr;
-}
