@@ -23,8 +23,6 @@
 #endif
 #include "cdo_fftw3.h"
 
-#include <mutex>
-
 #include <cdi.h>
 
 #include "cdo_vlist.h"
@@ -34,15 +32,11 @@
 #include "griddes.h"
 #include "cimdOmp.h"
 
-#ifdef HAVE_LIBFFTW3
-static std::mutex fftwMutex;
-#endif
-
 static int
 vlistGetFirstReg2DGrid(int vlistID)
 {
   // find first gaussian grid
-  auto ngrids = vlistNgrids(vlistID);
+  auto ngrids = vlistNumGrids(vlistID);
   for (int index = 0; index < ngrids; ++index)
     {
       auto gridID = vlistGrid(vlistID, index);
@@ -53,8 +47,22 @@ vlistGetFirstReg2DGrid(int vlistID)
   return -1;
 }
 
-class ModuleFC
+class FC : public Process
 {
+public:
+  using Process::Process;
+  inline static CdoModule module = {
+    .name = "FC",
+    .operators
+    = { { "fc2sp" }, { "sp2fc" }, { "fc2gp" }, { "gp2fc" }, { "fourier2grid", 1, 0, nullptr }, { "grid2fourier", 1, 0, nullptr } },
+    .aliases = {},
+    .mode = EXPOSED,     // Module mode: 0:intern 1:extern
+    .number = CDI_BOTH,  // Allowed number type
+    .constraints = { 1, 1, NoRestriction },
+  };
+  inline static RegisterEntry<FC> registration = RegisterEntry<FC>(module);
+
+  int FC2SP, SP2FC, FC2GP, GP2FC, GRID2FOURIER, FOURIER2GRID;
   int gridID1 = -1, gridID2 = -1;
   size_t nlon = 0, nlat = 0;
   int ntr = 0;
@@ -65,6 +73,7 @@ class ModuleFC
   CdoStreamID streamID2;
 
   int vlistID1;
+  VarList varList1;
 
   int taxisID1;
   int taxisID2;
@@ -79,24 +88,21 @@ class ModuleFC
   FC_Transformation fcTrans;
   SP_Transformation spTrans;
 
-  int FC2SP, SP2FC, FC2GP, GP2FC, GRID2FOURIER, FOURIER2GRID;
-
 public:
   void
-  init(void *process)
+  init() override
   {
-    cdo_initialize(process);
 
     operator_check_argc(0);
 
     dataIsUnchanged = data_is_unchanged();
 
-    FC2SP = cdo_operator_add("fc2sp", 0, 0, nullptr);
-    SP2FC = cdo_operator_add("sp2fc", 0, 0, nullptr);
-    FC2GP = cdo_operator_add("fc2gp", 0, 0, nullptr);
-    GP2FC = cdo_operator_add("gp2fc", 0, 0, nullptr);
-    GRID2FOURIER = cdo_operator_add("grid2fourier", 1, 0, nullptr);
-    FOURIER2GRID = cdo_operator_add("fourier2grid", 1, 0, nullptr);
+    FC2SP = module.get_id("fc2sp");
+    SP2FC = module.get_id("sp2fc");
+    FC2GP = module.get_id("fc2gp");
+    GP2FC = module.get_id("gp2fc");
+    GRID2FOURIER = module.get_id("grid2fourier");
+    FOURIER2GRID = module.get_id("fourier2grid");
 
     operatorID = cdo_operator_id();
     auto operfunc = cdo_operator_f1(operatorID);
@@ -106,6 +112,8 @@ public:
     vlistID1 = cdo_stream_inq_vlist(streamID1);
     auto vlistID2 = vlistDuplicate(vlistID1);
 
+    varList1 = VarList(vlistID1);
+
     taxisID1 = vlistInqTaxis(vlistID1);
     taxisID2 = taxisDuplicate(taxisID1);
     vlistDefTaxis(vlistID2, taxisID2);
@@ -268,19 +276,19 @@ public:
 
     // printf("nfc %d, ntr %d, nlat %zu, nlon %zu\n", nfc, ntr, nlat, nlon);
 
-    auto nvars = vlistNvars(vlistID2);
-    vars = std::vector<bool>(nvars);
-    for (int varID = 0; varID < nvars; ++varID) vars[varID] = gridID1 == vlistInqVarGrid(vlistID1, varID);
+    auto numVars = varList1.numVars();
+    vars = std::vector<bool>(numVars);
+    for (const auto &var : varList1.vars) vars[var.ID] = (gridID1 == var.gridID);
 
     if (gridID1 != -1) vlistChangeGrid(vlistID2, gridID1, gridID2);
     if (operatorID == GRID2FOURIER)
       {
-        for (int varID = 0; varID < nvars; ++varID)
+        for (int varID = 0; varID < numVars; ++varID)
           if (vars[varID]) vlistDefVarDatatype(vlistID2, varID, CDI_DATATYPE_CPX64);
       }
     else if (operatorID == FOURIER2GRID)
       {
-        for (int varID = 0; varID < nvars; ++varID)
+        for (int varID = 0; varID < numVars; ++varID)
           if (vars[varID]) vlistDefVarDatatype(vlistID2, varID, CDI_DATATYPE_FLT32);
       }
 
@@ -299,8 +307,9 @@ public:
         array2.resize(gridsize);
       }
   }
+
   void
-  run()
+  run() override
   {
     int tsID = 0;
     while (true)
@@ -318,11 +327,11 @@ public:
 
             if (vars[varID])
               {
-                size_t nmiss;
-                cdo_read_record(streamID1, array1.data(), &nmiss);
-                if (nmiss) cdo_abort("Missing values unsupported for spectral/fourier data!");
+                size_t numMissVals;
+                cdo_read_record(streamID1, array1.data(), &numMissVals);
+                if (numMissVals) cdo_abort("Missing values unsupported for spectral/fourier data!");
 
-                gridID1 = vlistInqVarGrid(vlistID1, varID);
+                gridID1 = varList1.vars[varID].gridID;
                 if (operatorID == FC2SP)
                   four2spec(spTrans, gridID1, array1, gridID2, array2);
                 else if (operatorID == SP2FC)
@@ -336,7 +345,7 @@ public:
                 else if (operatorID == GRID2FOURIER)
                   grid2fourier(gridID1, array1, gridID2, array2);
                 cdo_def_record(streamID2, varID, levelID);
-                cdo_write_record(streamID2, array2.data(), nmiss);
+                cdo_write_record(streamID2, array2.data(), numMissVals);
               }
             else
               {
@@ -344,9 +353,9 @@ public:
                 if (dataIsUnchanged) { cdo_copy_record(streamID2, streamID1); }
                 else
                   {
-                    size_t nmiss;
-                    cdo_read_record(streamID1, array1.data(), &nmiss);
-                    cdo_write_record(streamID2, array1.data(), nmiss);
+                    size_t numMissVals;
+                    cdo_read_record(streamID1, array1.data(), &numMissVals);
+                    cdo_write_record(streamID2, array1.data(), numMissVals);
                   }
               }
           }
@@ -354,24 +363,11 @@ public:
         tsID++;
       }
   }
+
   void
-  close()
+  close() override
   {
     cdo_stream_close(streamID2);
     cdo_stream_close(streamID1);
-
-    cdo_finish();
   }
 };
-
-void *
-FC(void *process)
-{
-  ModuleFC fc;
-
-  fc.init(process);
-  fc.run();
-  fc.close();
-
-  return nullptr;
-}
