@@ -11,12 +11,13 @@
 
 #ifdef HAVE_LIBFFTW3
 #include <fftw3.h>
+#include <mutex>
+static std::mutex fftwMutex;
 #endif
 
 #include <cdi.h>
 
 #include "process_int.h"
-#include "cdo_vlist.h"
 #include "param_conversion.h"
 #include "cdo_fft.h"
 #include "cdo_options.h"
@@ -132,8 +133,20 @@ fourier_intrinsic(int sign, int varID, int levelID, int nts, size_t gridsize, do
     }
 }
 
-class ModuleFourier
+class Fourier : public Process
 {
+public:
+  using Process::Process;
+  inline static CdoModule module = {
+    .name = "Fourier",
+    .operators = { { "fourier", FourierHelp } },
+    .aliases = {},
+    .mode = EXPOSED,     // Module mode: 0:intern 1:extern
+    .number = CDI_COMP,  // Allowed number type
+    .constraints = { 1, 1, NoRestriction },
+  };
+  inline static RegisterEntry<Fourier> registration = RegisterEntry<Fourier>(module);
+
   size_t nalloc = 0;
 
   CdoStreamID streamID1;
@@ -145,19 +158,17 @@ class ModuleFourier
   int vlistID1;
 
   bool use_fftw = false;
-  int nvars;
+  int numVars;
   int sign;
 
   VarList varList;
-  FieldVector3D vars;
+  FieldVector3D varsData;
   std::vector<CdiDateTime> vDateTimes;
 
 public:
   void
-  init(void *process)
+  init() override
   {
-    cdo_initialize(process);
-
     if (Options::Use_FFTW)
       {
 #ifdef HAVE_LIBFFTW3
@@ -185,12 +196,13 @@ public:
     streamID2 = cdo_open_write(1);
     cdo_def_vlist(streamID2, vlistID2);
 
-    varList_init(varList, vlistID1);
+    varList = VarList(vlistID1);
 
-    nvars = vlistNvars(vlistID1);
+    numVars = vlistNvars(vlistID1);
   }
+
   void
-  run()
+  run() override
   {
     int tsID = 0;
     while (true)
@@ -203,22 +215,22 @@ public:
             constexpr size_t NALLOC_INC = 1024;
             nalloc += NALLOC_INC;
             vDateTimes.resize(nalloc);
-            vars.resize(nalloc);
+            varsData.resize(nalloc);
           }
 
         vDateTimes[tsID] = taxisInqVdatetime(taxisID1);
 
-        fields_from_vlist(vlistID1, vars[tsID]);
+        field2D_init(varsData[tsID], varList);
 
         for (int recID = 0; recID < nrecs; ++recID)
           {
             int varID, levelID;
             cdo_inq_record(streamID1, &varID, &levelID);
-            auto gridsize = varList[varID].gridsize;
-            vars[tsID][varID][levelID].resize(2 * gridsize);
-            size_t nmiss;
-            cdo_read_record(streamID1, vars[tsID][varID][levelID].vec_d.data(), &nmiss);
-            vars[tsID][varID][levelID].nmiss = nmiss;
+            auto gridsize = varList.vars[varID].gridsize;
+            varsData[tsID][varID][levelID].resize(2 * gridsize);
+            size_t numMissVals;
+            cdo_read_record(streamID1, varsData[tsID][varID][levelID].vec_d.data(), &numMissVals);
+            varsData[tsID][varID][levelID].numMissVals = numMissVals;
           }
 
         tsID++;
@@ -235,6 +247,7 @@ public:
           {
             fm.in_fft = fftw_alloc_complex(nts);
             fm.out_fft = fftw_alloc_complex(nts);
+            std::scoped_lock lock(fftwMutex);
             fm.plan = fftw_plan_dft_1d(nts, fm.in_fft, fm.out_fft, sign, FFTW_ESTIMATE);
           }
         if (Options::cdoVerbose) fftw_print_plan(fourierMemory[0].plan);
@@ -242,27 +255,28 @@ public:
       }
     else
       {
-        const bool isPower2 = ((nts & (nts - 1)) == 0);
+        auto isPowerOf2 = (nts > 0 && (nts & (nts - 1)) == 0);
         for (auto &fm : fourierMemory)
           {
             fm.real.resize(nts);
             fm.imag.resize(nts);
-            if (!isPower2) fm.work_r.resize(nts);
-            if (!isPower2) fm.work_i.resize(nts);
+            if (!isPowerOf2) fm.work_r.resize(nts);
+            if (!isPowerOf2) fm.work_i.resize(nts);
           }
       }
 
-    for (int varID = 0; varID < nvars; ++varID)
+    for (int varID = 0; varID < numVars; ++varID)
       {
-        auto missval = varList[varID].missval;
-        auto gridsize = varList[varID].gridsize;
-        auto nlevels = varList[varID].nlevels;
+        const auto &var = varList.vars[varID];
+        auto missval = var.missval;
+        auto gridsize = var.gridsize;
+        auto nlevels = var.nlevels;
         for (int levelID = 0; levelID < nlevels; ++levelID)
           {
             if (use_fftw)
-              fourier_fftw(sign, varID, levelID, nts, gridsize, missval, vars, fourierMemory);
+              fourier_fftw(sign, varID, levelID, nts, gridsize, missval, varsData, fourierMemory);
             else
-              fourier_intrinsic(sign, varID, levelID, nts, gridsize, missval, vars, fourierMemory);
+              fourier_intrinsic(sign, varID, levelID, nts, gridsize, missval, varsData, fourierMemory);
           }
       }
 
@@ -273,6 +287,7 @@ public:
           {
             fftw_free(fm.in_fft);
             fftw_free(fm.out_fft);
+            std::scoped_lock lock(fftwMutex);
             fftw_destroy_plan(fm.plan);
           }
         fftw_cleanup();
@@ -284,39 +299,26 @@ public:
         taxisDefVdatetime(taxisID2, vDateTimes[tsID]);
         cdo_def_timestep(streamID2, tsID);
 
-        for (int varID = 0; varID < nvars; ++varID)
+        for (int varID = 0; varID < numVars; ++varID)
           {
-            auto nlevels = varList[varID].nlevels;
+            auto nlevels = varList.vars[varID].nlevels;
             for (int levelID = 0; levelID < nlevels; ++levelID)
               {
-                if (!vars[tsID][varID][levelID].empty())
+                if (!varsData[tsID][varID][levelID].empty())
                   {
-                    auto nmiss = vars[tsID][varID][levelID].nmiss;
+                    auto numMissVals = varsData[tsID][varID][levelID].numMissVals;
                     cdo_def_record(streamID2, varID, levelID);
-                    cdo_write_record(streamID2, vars[tsID][varID][levelID].vec_d.data(), nmiss);
+                    cdo_write_record(streamID2, varsData[tsID][varID][levelID].vec_d.data(), numMissVals);
                   }
               }
           }
       }
   }
+
   void
-  close()
+  close() override
   {
     cdo_stream_close(streamID2);
     cdo_stream_close(streamID1);
-
-    cdo_finish();
   }
 };
-
-void *
-Fourier(void *process)
-{
-  ModuleFourier furier;
-
-  furier.init(process);
-  furier.run();
-  furier.close();
-
-  return nullptr;
-}
