@@ -26,7 +26,6 @@
 #include "param_conversion.h"
 #include <mpim_grid.h>
 #include "cdo_options.h"
-#include "progress.h"
 #include "cimdOmp.h"
 #include "field_functions.h"
 
@@ -216,23 +215,12 @@ gen_boxgrid(int gridID1, size_t xinc, size_t yinc)
       gridDefXsize(gridID2, nlon2);
       gridDefYsize(gridID2, nlat2);
     }
-  else
-    {
-      cdo_abort("Unsupported grid: %s", gridNamePtr(gridtype));
-    }
+  else { cdo_abort("Unsupported grid: %s", gridNamePtr(gridtype)); }
 
   if (gridtype == GRID_GAUSSIAN || gridtype == GRID_LONLAT) { gen_boxgrid_reg2D(gridID1, xinc, yinc, gridID2); }
-  else if (gridtype == GRID_GENERIC)
-    {
-    }
-  else if (gridtype == GRID_CURVILINEAR)
-    {
-      gen_boxgrid_curv2D(gridID1, xinc, yinc, gridID2);
-    }
-  else
-    {
-      cdo_abort("Unsupported grid: %s", gridNamePtr(gridtype));
-    }
+  else if (gridtype == GRID_GENERIC) {}
+  else if (gridtype == GRID_CURVILINEAR) { gen_boxgrid_curv2D(gridID1, xinc, yinc, gridID2); }
+  else { cdo_abort("Unsupported grid: %s", gridNamePtr(gridtype)); }
 
   return gridID2;
 }
@@ -268,7 +256,7 @@ gridbox_stat(const Field &field1, Field &field2, size_t xinc, size_t yinc, int s
       auto ompthID = cdo_omp_get_thread_num();
 
       auto &field = fields[ompthID];
-      field.nmiss = 0;
+      field.numMissVals = 0;
 
       auto ilat = ig / nlon2;
       auto ilon = ig - ilat * nlon2;
@@ -284,7 +272,7 @@ gridbox_stat(const Field &field1, Field &field2, size_t xinc, size_t yinc, int s
               auto index = jj * nlon1 + ii;
               if (ii >= nlon1) break;
               field.vec_d[isize] = (field1.memType == MemType::Float) ? (double) field1.vec_f[index] : field1.vec_d[index];
-              if (dbl_is_equal(field.vec_d[isize], field.missval)) field.nmiss++;
+              if (dbl_is_equal(field.vec_d[isize], field.missval)) field.numMissVals++;
               if (useWeight) field.weightv[isize] = field1.weightv[index];
               isize++;
             }
@@ -301,28 +289,32 @@ gridbox_stat(const Field &field1, Field &field2, size_t xinc, size_t yinc, int s
   field_num_mv(field2);
 }
 
-static void
-add_operators(void)
+class Gridboxstat : public Process
 {
-  // clang-format off
-  cdo_operator_add("gridboxrange",  FieldFunc_Range,  0, nullptr);
-  cdo_operator_add("gridboxmin",    FieldFunc_Min,    0, nullptr);
-  cdo_operator_add("gridboxmax",    FieldFunc_Max,    0, nullptr);
-  cdo_operator_add("gridboxsum",    FieldFunc_Sum,    0, nullptr);
-  cdo_operator_add("gridboxmean",   FieldFunc_Meanw,  1, nullptr);
-  cdo_operator_add("gridboxavg",    FieldFunc_Avgw,   1, nullptr);
-  cdo_operator_add("gridboxvar",    FieldFunc_Varw,   1, nullptr);
-  cdo_operator_add("gridboxvar1",   FieldFunc_Var1w,  1, nullptr);
-  cdo_operator_add("gridboxstd",    FieldFunc_Stdw,   1, nullptr);
-  cdo_operator_add("gridboxstd1",   FieldFunc_Std1w,  1, nullptr);
-  cdo_operator_add("gridboxskew",   FieldFunc_Skew,   0, nullptr);
-  cdo_operator_add("gridboxkurt",   FieldFunc_Kurt,   0, nullptr);
-  cdo_operator_add("gridboxmedian", FieldFunc_Median, 0, nullptr);
-  // clang-format on
-}
+public:
+  using Process::Process;
+  inline static CdoModule module = {
+    .name = "Gridboxstat",
+    .operators = { { "gridboxrange", FieldFunc_Range, 0, GridboxstatHelp },
+                   { "gridboxmin", FieldFunc_Min, 0, GridboxstatHelp },
+                   { "gridboxmax", FieldFunc_Max, 0, GridboxstatHelp },
+                   { "gridboxsum", FieldFunc_Sum, 0, GridboxstatHelp },
+                   { "gridboxmean", FieldFunc_Meanw, 1, GridboxstatHelp },
+                   { "gridboxavg", FieldFunc_Avgw, 1, GridboxstatHelp },
+                   { "gridboxstd", FieldFunc_Stdw, 1, GridboxstatHelp },
+                   { "gridboxstd1", FieldFunc_Std1w, 1, GridboxstatHelp },
+                   { "gridboxvar", FieldFunc_Varw, 1, GridboxstatHelp },
+                   { "gridboxvar1", FieldFunc_Var1w, 1, GridboxstatHelp },
+                   { "gridboxskew", FieldFunc_Skew, 0, GridboxstatHelp },
+                   { "gridboxkurt", FieldFunc_Kurt, 0, GridboxstatHelp },
+                   { "gridboxmedian", FieldFunc_Median, 0, GridboxstatHelp } },
+    .aliases = {},
+    .mode = EXPOSED,     // Module mode: 0:intern 1:extern
+    .number = CDI_REAL,  // Allowed number type
+    .constraints = { 1, 1, NoRestriction },
+  };
+  inline static RegisterEntry<Gridboxstat> registration = RegisterEntry<Gridboxstat>(module);
 
-class ModuleGridboxstat
-{
   int lastgrid = -1;
   bool wstatus = false;
 
@@ -335,7 +327,8 @@ class ModuleGridboxstat
   int taxisID2;
 
   Field field1, field2;
-  VarList varList1, varList2;
+  VarList varList1;
+  VarList varList2;
 
   int xinc;
   int yinc;
@@ -344,17 +337,13 @@ class ModuleGridboxstat
 
 public:
   void
-  init(void *process)
+  init() override
   {
-    cdo_initialize(process);
-
     operator_input_arg("xinc, yinc");
     operator_check_argc(2);
     xinc = parameter_to_int(cdo_operator_argv(0));
     yinc = parameter_to_int(cdo_operator_argv(1));
 
-    add_operators();
-
     auto operatorID = cdo_operator_id();
     operfunc = cdo_operator_f1(operatorID);
 
@@ -372,7 +361,7 @@ public:
     taxisID2 = taxisDuplicate(taxisID1);
     vlistDefTaxis(vlistID2, taxisID2);
 
-    auto ngrids = vlistNgrids(vlistID1);
+    auto ngrids = vlistNumGrids(vlistID1);
     if (ngrids > 1) cdo_abort("Too many different grids!");
 
     auto gridID1 = vlistGrid(vlistID1, 0);
@@ -383,8 +372,8 @@ public:
     streamID2 = cdo_open_write(1);
     cdo_def_vlist(streamID2, vlistID2);
 
-    varList_init(varList1, vlistID1);
-    varList_init(varList2, vlistID2);
+    varList1 = VarList(vlistID1);
+    varList2 = VarList(vlistID2);
 
     auto gridsize1 = gridInqSize(gridID1);
 
@@ -392,7 +381,7 @@ public:
   }
 
   void
-  run()
+  run() override
   {
     int tsID = 0;
     while (true)
@@ -407,7 +396,7 @@ public:
           {
             int varID, levelID;
             cdo_inq_record(streamID1, &varID, &levelID);
-            field1.init(varList1[varID]);
+            field1.init(varList1.vars[varID]);
             cdo_read_record(streamID1, field1);
 
             if (needWeights && field1.grid != lastgrid)
@@ -417,9 +406,9 @@ public:
               }
             if (wstatus != 0 && tsID == 0 && levelID == 0)
               cdo_warning("Grid cell bounds not available, using constant grid cell area weights for variable %s!",
-                          varList1[varID].name);
+                          varList1.vars[varID].name);
 
-            field2.init(varList2[varID]);
+            field2.init(varList2.vars[varID]);
             gridbox_stat(field1, field2, xinc, yinc, operfunc);
 
             cdo_def_record(streamID2, varID, levelID);
@@ -430,22 +419,9 @@ public:
   }
 
   void
-  close()
+  close() override
   {
     cdo_stream_close(streamID2);
     cdo_stream_close(streamID1);
-
-    cdo_finish();
   }
 };
-
-void *
-Gridboxstat(void *process)
-{
-  ModuleGridboxstat gridboxstat;
-  gridboxstat.init(process);
-  gridboxstat.run();
-  gridboxstat.close();
-
-  return nullptr;
-}
