@@ -13,12 +13,28 @@
 #include <cdi.h>
 
 #include "cdo_options.h"
-#include "cdo_vlist.h"
 #include "process_int.h"
 #include "param_conversion.h"
 
-class ModuleHistogram
+class Histogram : public Process
 {
+public:
+  using Process::Process;
+  inline static CdoModule module = {
+    .name = "Histogram",
+    .operators = { { "histcount", HistogramHelp },
+                   { "histsum", HistogramHelp },
+                   { "histmean", HistogramHelp },
+                   { "histfreq", HistogramHelp } },
+    .aliases = {},
+    .mode = EXPOSED,     // Module mode: 0:intern 1:extern
+    .number = CDI_REAL,  // Allowed number type
+    .constraints = { 1, 1, NoRestriction },
+  };
+  inline static RegisterEntry<Histogram> registration = RegisterEntry<Histogram>(module);
+
+  int HISTCOUNT, HISTSUM, HISTMEAN, HISTFREQ;
+
 private:
   CdoStreamID streamID1;
   CdoStreamID streamID2;
@@ -31,7 +47,8 @@ private:
   Varray2D<double> vartcount;
 
   Varray<double> array;
-  VarList varList1, varList2;
+  VarList varList1;
+  VarList varList2;
 
   int nbins;
 
@@ -39,23 +56,14 @@ private:
   int nvars;
   int operatorID;
 
-  int HISTCOUNT;
-  int HISTSUM;
-  int HISTMEAN;
-  int HISTFREQ;
-
 public:
   void
-  init(void *process)
+  init() override
   {
-    cdo_initialize(process);
-
-    // clang-format off
-    HISTCOUNT = cdo_operator_add("histcount", 0, 0, nullptr);
-    HISTSUM   = cdo_operator_add("histsum",   0, 0, nullptr);
-    HISTMEAN  = cdo_operator_add("histmean",  0, 0, nullptr);
-    HISTFREQ  = cdo_operator_add("histfreq",  0, 0, nullptr);
-    // clang-format on
+    HISTCOUNT = module.get_id("histcount");
+    HISTSUM = module.get_id("histsum");
+    HISTMEAN = module.get_id("histmean");
+    HISTFREQ = module.get_id("histfreq");
 
     (void) (HISTSUM);  // unused
 
@@ -74,14 +82,14 @@ public:
       }
 
     streamID1 = cdo_open_read(0);
-    const auto vlistID1 = cdo_stream_inq_vlist(streamID1);
+    auto vlistID1 = cdo_stream_inq_vlist(streamID1);
 
     taxisID1 = vlistInqTaxis(vlistID1);
 
-    const auto vlistID2 = vlistDuplicate(vlistID1);
+    auto vlistID2 = vlistDuplicate(vlistID1);
 
     /* create zaxis for output bins */
-    const auto zaxisID2 = zaxisCreate(ZAXIS_GENERIC, nbins);
+    auto zaxisID2 = zaxisCreate(ZAXIS_GENERIC, nbins);
 
     zaxisDefLevels(zaxisID2, &fltarr[0]);
     zaxisDefLbounds(zaxisID2, &fltarr[0]);
@@ -92,11 +100,11 @@ public:
     cdiDefKeyString(zaxisID2, CDI_GLOBAL, CDI_KEY_UNITS, "level");
 
     /* check zaxis: only 2D fields allowed */
-    const auto nzaxis = vlistNzaxis(vlistID1);
+    auto nzaxis = vlistNumZaxis(vlistID1);
     for (int index = 0; index < nzaxis; ++index)
       {
-        const auto zaxisID = vlistZaxis(vlistID1, index);
-        const auto nlevel = zaxisInqSize(zaxisID);
+        auto zaxisID = vlistZaxis(vlistID1, index);
+        auto nlevel = zaxisInqSize(zaxisID);
         if (nlevel > 1) cdo_abort("Found 3D field with %d levels. Only 2D fields allowed!", nlevel);
         vlistChangeZaxisIndex(vlistID2, index, zaxisID2);
       }
@@ -108,8 +116,8 @@ public:
 
     cdo_def_vlist(streamID2, vlistID2);
 
-    varList_init(varList1, vlistID1);
-    varList_init(varList2, vlistID2);
+    varList1 = VarList(vlistID1);
+    varList2 = VarList(vlistID2);
 
     nvars = vlistNvars(vlistID2);
 
@@ -119,22 +127,23 @@ public:
 
     for (int varID = 0; varID < nvars; ++varID)
       {
-        const auto gridsize = varList1[varID].gridsize;
+        auto gridsize = varList1.vars[varID].gridsize;
         vardata[varID].resize(nbins * gridsize, 0);
         varcount[varID].resize(nbins * gridsize, 0);
         vartcount[varID].resize(gridsize, 0);
       }
 
-    const auto gridsizemax = vlistGridsizeMax(vlistID1);
+    auto gridsizemax = vlistGridsizeMax(vlistID1);
     array = Varray<double>(gridsizemax);
   }
+
   void
-  run()
+  run() override
   {
     int tsID = 0;
     while (true)
       {
-        const auto nrecs = cdo_stream_inq_timestep(streamID1, tsID++);
+        auto nrecs = cdo_stream_inq_timestep(streamID1, tsID++);
         if (nrecs == 0) break;
 
         cdo_taxis_copy_timestep(taxisID2, taxisID1);
@@ -143,13 +152,13 @@ public:
           {
             int varID, levelID;
             cdo_inq_record(streamID1, &varID, &levelID);
-            size_t nmiss;
-            cdo_read_record(streamID1, array.data(), &nmiss);
+            size_t numMissVals;
+            cdo_read_record(streamID1, array.data(), &numMissVals);
 
-            const auto gridsize = varList1[varID].gridsize;
-            const auto missval = varList1[varID].missval;
+            auto gridsize = varList1.vars[varID].gridsize;
+            auto missval = varList1.vars[varID].missval;
 
-            nmiss = 0;
+            numMissVals = 0;
             for (size_t i = 0; i < gridsize; ++i)
               {
                 if (!DBL_IS_EQUAL(array[i], missval))
@@ -158,7 +167,7 @@ public:
                     int index = 0;
                     while (index < nbins)
                       {
-                        const auto offset = gridsize * index;
+                        auto offset = gridsize * index;
                         if (!DBL_IS_EQUAL(vardata[varID][offset + i], missval) && array[i] >= fltarr[index]
                             && array[i] < fltarr[index + 1])
                           {
@@ -171,7 +180,7 @@ public:
                   }
                 else
                   {
-                    nmiss++;  // missing value
+                    numMissVals++;  // missing value
                   }
               }
           }
@@ -181,15 +190,15 @@ public:
 
     for (int varID = 0; varID < nvars; ++varID)
       {
-        const auto gridsize = varList2[varID].gridsize;
-        const auto missval = varList2[varID].missval;
+        auto gridsize = varList2.vars[varID].gridsize;
+        auto missval = varList2.vars[varID].missval;
 
         // fix missing values
 
         for (int index = 0; index < nbins; ++index)
           {
-            size_t nmiss = 0;
-            const auto offset = gridsize * index;
+            size_t numMissVals = 0;
+            auto offset = gridsize * index;
 
             for (size_t i = 0; i < gridsize; ++i)
               {
@@ -208,7 +217,7 @@ public:
                   }
                 else
                   {
-                    nmiss++;
+                    numMissVals++;
                     varcount[varID][offset + i] = missval;
                     vardata[varID][offset + i] = missval;
                   }
@@ -217,29 +226,17 @@ public:
             cdo_def_record(streamID2, varID, index);
 
             if (operatorID == HISTCOUNT)
-              cdo_write_record(streamID2, &varcount[varID][offset], nmiss);
+              cdo_write_record(streamID2, &varcount[varID][offset], numMissVals);
             else
-              cdo_write_record(streamID2, &vardata[varID][offset], nmiss);
+              cdo_write_record(streamID2, &vardata[varID][offset], numMissVals);
           }
       }
   }
 
   void
-  close()
+  close() override
   {
     cdo_stream_close(streamID1);
     cdo_stream_close(streamID2);
-
-    cdo_finish();
   }
 };
-
-void *
-Histogram(void *process)
-{
-  ModuleHistogram histogram;
-  histogram.init(process);
-  histogram.run();
-  histogram.close();
-  return nullptr;
-}
