@@ -16,10 +16,9 @@
 #endif
 
 #include <cdi.h>
+#include <stdlib.h>  // realloc()
 
 #include "cdo_options.h"
-#include "compare.h"
-#include "dmemory.h"
 #include "varray.h"
 #include "process_int.h"
 #include "cdo_default_values.h"
@@ -120,7 +119,7 @@ get_grid_info(double c0, double re, int *nrxp, int *nryp, double *r0p, double *s
 static double
 det_lon_atovs(double r, double r0, double lts, double c, double re)
 {
-  const double pi = M_PI;
+  constexpr double pi = M_PI;
 
   double xla = (r - r0) * c / re / std::cos(lts * pi / 180.); /* longitude */
   xla = 180. * xla / pi;
@@ -131,7 +130,7 @@ det_lon_atovs(double r, double r0, double lts, double c, double re)
 static double
 det_lat_atovs(double s, double s0, double lts, double c, double re)
 {
-  const double pi = M_PI;
+  constexpr double pi = M_PI;
 
   double siphi = (s - s0) * c * std::cos(lts * pi / 180.) / re;
   double phi = 180. * std::asin(siphi) / pi; /* latitude */
@@ -143,7 +142,7 @@ static int
 defLonLatGrid(int nx, int ny, double c0, double lts, double re)
 {
   int gridID;
-  int nrx, nry, i;
+  int nrx, nry;
   double c;
   double r0, s0;
   double r, s;
@@ -160,7 +159,7 @@ defLonLatGrid(int nx, int ny, double c0, double lts, double re)
   Varray<double> xvals(nx), yvals(ny);
   Varray<double> xbounds(nx * 2), ybounds(ny * 2);
 
-  for (i = 0; i < nx; ++i)
+  for (int i = 0; i < nx; ++i)
     {
       r = i + 1;
       xla = det_lon_atovs(r, r0, lts, c, re);
@@ -172,7 +171,7 @@ defLonLatGrid(int nx, int ny, double c0, double lts, double re)
       /* printf("xla[%d]=%g\n", i, xla); */
     }
 
-  for (i = 0; i < ny; ++i)
+  for (int i = 0; i < ny; ++i)
     {
       s = (nry - i - 1) + 1;
       phi = det_lat_atovs(s, s0, lts, c, re);
@@ -246,30 +245,29 @@ scan_pcs_def(char *pcs_def, char proj[128], double *a, double *lon0, double *lat
 {
   char *pcs[64];
   int npcs = 0;
-  int i;
   int len;
   int nfound = 0;
 
-  strcpy(proj, "unknown");
+  std::strcpy(proj, "unknown");
   *a = 1;
   *lon0 = 0;
   *lat0 = 0;
 
   pcs[npcs++] = &pcs_def[0];
   len = (int) strlen(pcs_def);
-  for (i = 0; i < len; ++i)
+  for (int i = 0; i < len; ++i)
     if (pcs_def[i] == ',' && npcs < 64)
       {
         pcs_def[i] = 0;
         pcs[npcs++] = &pcs_def[i + 1];
       }
 
-  for (i = 0; i < npcs; ++i)
+  for (int i = 0; i < npcs; ++i)
     {
       if (memcmp(pcs[i], "proj=", 5) == 0)
         {
           pcs[i] += 5;
-          strcpy(proj, pcs[i]);
+          std::strcpy(proj, pcs[i]);
           nfound++;
         }
       else if (memcmp(pcs[i], "a=", 2) == 0)
@@ -351,7 +349,7 @@ read_geolocation(hid_t loc_id, int nx, int ny, int lprojtype)
   H5Tinsert(proj_tid, "Reference ellipsoid", HOFFSET(proj_t, ellipsoid), str_tid);
   H5Tinsert(proj_tid, "Projection parameter", HOFFSET(proj_t, parameter), fltarr_tid);
 
-  if (projection_name) Free(projection_name);
+  // if (projection_name) free(projection_name); failed with: pointer being freed was not allocated
 
   grp_id = H5Gopen(loc_id, "Geolocation");
 
@@ -421,7 +419,7 @@ read_geolocation(hid_t loc_id, int nx, int ny, int lprojtype)
   /* some CM-SAF files have incorrect entries for some metadata. */
   /* these are corrected in the following sections. */
   /* in case of questions on this, contact frank.kaspar@dwd.de */
-  if (cdo_cmpstr(proj.ellipsoid, "WSG-84")) strcpy(proj.ellipsoid, "WGS-84");
+  if (cdo_cmpstr(proj.ellipsoid, "WSG-84")) std::strcpy(proj.ellipsoid, "WGS-84");
 
   if ((int) region.xmin == -8887500 && (int) region.xmax == -8887500 && (int) region.ymin == 8887500 && (int) region.ymax == 8887500
       && (int) region.dx == 15000 && (int) region.dy == 15000)
@@ -471,8 +469,8 @@ read_geolocation(hid_t loc_id, int nx, int ny, int lprojtype)
               && (int) region.dy == 45000)))
     {
       if (Options::cdoVerbose) cdo_print("Replacing incorrect projection parameters for sinusoidal products:");
-      strcpy(proj.ellipsoid, "WGS-84");
-      strcpy(proj.name, "sinusoidal");
+      std::strcpy(proj.ellipsoid, "WGS-84");
+      std::strcpy(proj.name, "sinusoidal");
       proj.parameter[0] = 0.0;
       proj.parameter[1] = 0.0;
       proj.parameter[2] = 0.0;
@@ -632,8 +630,8 @@ read_region(hid_t loc_id, int nx, int ny)
   double dx = (xmax - xmin) / nx;
   double dy = (ymax - ymin) / ny;
   /*
-  xsize = (int)lround((region.xmax-region.xmin)/region.dx);
-  ysize = (int)lround((region.ymax-region.ymin)/region.dy);
+  xsize = (int) std::lround((region.xmax-region.xmin)/region.dx);
+  ysize = (int) std::lround((region.ymax-region.ymin)/region.dy);
 
   if ( Options::cdoVerbose ) cdo_print("  Size: xsize=%d  ysize=%d", xsize, ysize);
   */
@@ -650,8 +648,8 @@ static void
 read_dataset(hid_t loc_id, const char *name, void *opdata)
 {
   hid_t dataspace;
-  hsize_t dims_out[9]; /* dataset dimensions           */
-  herr_t status;       /* Generic return value		*/
+  hsize_t dims_out[9];  // dataset dimensions
+  herr_t status;        // Generic return value
   hid_t attr, atype, atype_mem;
   int iattr;
   float fattr;
@@ -662,20 +660,19 @@ read_dataset(hid_t loc_id, const char *name, void *opdata)
   int rank;
   int nx = 0, ny = 0, nz = 0, nt = 0;
   size_t gridsize;
-  double *array;
   double addoffset = 0.0, scalefactor = 1.0, missval = cdiInqMissval();
   bool haveAddoffset = false, haveScalefactor = false, haveMissvals = false;
   int nset;
   int ftype = 0;
   int len;
   int dtype = CDI_DATATYPE_FLT32;
-  char attstring[4096]; /* Buffer to read string attribute back */
+  char attstring[4096];  // Buffer to read string attribute back
   char varname[CDI_MAX_NAME];
-  size_t nmiss;
+  size_t numMissVals;
   int num_attrs;
 
   attstring[0] = 0;
-  strcpy(varname, name);
+  std::strcpy(varname, name);
 
   hid_t dset_id = H5Dopen(loc_id, varname);
 
@@ -767,7 +764,7 @@ read_dataset(hid_t loc_id, const char *name, void *opdata)
 
   len = (int) strlen(varname);
   if (len > 0 && ((datasets_t *) opdata)->mergelevel)
-    if (isdigit(varname[len - 1]) && memcmp(varname, "Data", 4) != 0)
+    if (std::isdigit(varname[len - 1]) && memcmp(varname, "Data", 4) != 0)
       {
         if (nt > 1) cdo_abort("Combination of nlevel > 1 and ntime > 1 not implemented!");
 
@@ -887,7 +884,7 @@ read_dataset(hid_t loc_id, const char *name, void *opdata)
               else
                 cdo_warning("Attribute %s has unsupported data type!", attname);
             }
-          else if (strncmp(attname, "no_data", 7) == 0 || strncmp(attname, "nodata", 6) == 0)
+          else if (std::strncmp(attname, "no_data", 7) == 0 || std::strncmp(attname, "nodata", 6) == 0)
             {
               if (atype_class == H5T_FLOAT)
                 {
@@ -929,19 +926,19 @@ read_dataset(hid_t loc_id, const char *name, void *opdata)
           else if (cdo_cmpstr(attname, "description"))
             {
               H5Aread(attr, atype_mem, attstring);
-              if (((datasets_t *) opdata)->obj[nset].description) Free(((datasets_t *) opdata)->obj[nset].description);
+              if (((datasets_t *) opdata)->obj[nset].description) free(((datasets_t *) opdata)->obj[nset].description);
               ((datasets_t *) opdata)->obj[nset].description = strdup(attstring);
             }
           else if (cdo_cmpstr(attname, "title"))
             {
               H5Aread(attr, atype_mem, attstring);
-              if (((datasets_t *) opdata)->obj[nset].title) Free(((datasets_t *) opdata)->obj[nset].title);
+              if (((datasets_t *) opdata)->obj[nset].title) free(((datasets_t *) opdata)->obj[nset].title);
               ((datasets_t *) opdata)->obj[nset].title = strdup(attstring);
             }
           else if (cdo_cmpstr(attname, "time"))
             {
               H5Aread(attr, atype_mem, attstring);
-              if (((datasets_t *) opdata)->obj[nset].time) Free(((datasets_t *) opdata)->obj[nset].time);
+              if (((datasets_t *) opdata)->obj[nset].time) free(((datasets_t *) opdata)->obj[nset].time);
               ((datasets_t *) opdata)->obj[nset].time = strdup(attstring);
             }
           else if (cdo_cmpstr(attname, "unit"))
@@ -956,8 +953,8 @@ read_dataset(hid_t loc_id, const char *name, void *opdata)
         }
 
       int offset = gridsize * (nz - 1);
-      array = ((datasets_t *) opdata)->obj[nset].array;
-      array = (double *) Realloc(array, gridsize * nz * nt * sizeof(double));
+      double *array = ((datasets_t *) opdata)->obj[nset].array;
+      array = (double *) realloc(array, gridsize * nz * nt * sizeof(double));
       ((datasets_t *) opdata)->obj[nset].array = array;
       array = array + offset;
 
@@ -995,13 +992,13 @@ read_dataset(hid_t loc_id, const char *name, void *opdata)
         {
           if (((datasets_t *) opdata)->obj[nset].dtype != dtype) cdo_warning("Data type changes over levels!");
 
-          if (haveAddoffset && !DBL_IS_EQUAL(((datasets_t *) opdata)->obj[nset].addoffset, addoffset))
+          if (haveAddoffset && dbl_is_not_equal(((datasets_t *) opdata)->obj[nset].addoffset, addoffset))
             cdo_warning("Offset changes over levels!");
 
-          if (haveScalefactor && !DBL_IS_EQUAL(((datasets_t *) opdata)->obj[nset].scalefactor, scalefactor))
+          if (haveScalefactor && dbl_is_not_equal(((datasets_t *) opdata)->obj[nset].scalefactor, scalefactor))
             cdo_warning("Scalefactor changes over levels!");
 
-          if (haveMissvals && !DBL_IS_EQUAL(((datasets_t *) opdata)->obj[nset].missval, missval))
+          if (haveMissvals && dbl_is_not_equal(((datasets_t *) opdata)->obj[nset].missval, missval))
             cdo_warning("Missing value changes over levels!");
         }
 
@@ -1009,7 +1006,7 @@ read_dataset(hid_t loc_id, const char *name, void *opdata)
 
       std::vector<bool> mask(gridsize * nt, false);
 
-      nmiss = 0;
+      numMissVals = 0;
 
       auto mm = varray_min_max(gridsize * nt, array);
 
@@ -1028,13 +1025,13 @@ read_dataset(hid_t loc_id, const char *name, void *opdata)
           if (mm.min >= 0 && mm.max <= 32767) dtype = CDI_DATATYPE_INT16;
         }
 
-      haveAddoffset = IS_NOT_EQUAL(addoffset, 0.0);
-      haveScalefactor = IS_NOT_EQUAL(scalefactor, 1.0);
+      haveAddoffset = is_not_equal(addoffset, 0.0);
+      haveScalefactor = is_not_equal(scalefactor, 1.0);
 
       if (haveAddoffset || haveScalefactor)
         {
           for (size_t i = 0; i < gridsize * nt; ++i)
-            if (!DBL_IS_EQUAL(array[i], missval))
+            if (dbl_is_not_equal(array[i], missval))
               {
                 mask[i] = false;
 
@@ -1043,7 +1040,7 @@ read_dataset(hid_t loc_id, const char *name, void *opdata)
               }
             else
               {
-                nmiss++;
+                numMissVals++;
                 mask[i] = true;
               }
         }
@@ -1060,11 +1057,11 @@ read_dataset(hid_t loc_id, const char *name, void *opdata)
       if (Options::cdoVerbose)
         cdo_print("Dataset %s: dtype = %d  minval = %g  maxval = %g  missval = %g", varname, dtype, minval, maxval, missval);
 
-      if (nmiss)
+      if (numMissVals)
         {
           if (!(missval < minval || missval > maxval))
             {
-              if (DBL_IS_EQUAL(missval, 255.) && dtype == CDI_DATATYPE_UINT8)
+              if (dbl_is_equal(missval, 255.) && dtype == CDI_DATATYPE_UINT8)
                 {
                   missval = -255;
                   dtype = CDI_DATATYPE_INT16;
@@ -1154,7 +1151,7 @@ get_global_att(hid_t file_id, const char *obj_path, int vlistID)
   H5T_class_t type_class;
   int attint;
   double attflt;
-  int i, pos;
+  int pos;
   int num_attrs;
   char attstring[4096]; /* Buffer to read string attribute back */
 
@@ -1164,7 +1161,7 @@ get_global_att(hid_t file_id, const char *obj_path, int vlistID)
 
   num_attrs = H5Aget_num_attrs(obj_id);
 
-  for (i = 0; i < num_attrs; ++i)
+  for (int i = 0; i < num_attrs; ++i)
     {
       attr = H5Aopen_idx(obj_id, i);
       atype = H5Aget_type(attr);
@@ -1204,13 +1201,13 @@ get_vdate(int vlistID)
 {
   int64_t vdate = 0;
   int natts;
-  int i, len, type;
+  int len, type;
   char name[CDI_MAX_NAME];
   char attstr[CDI_MAX_NAME];
 
   cdiInqNatts(vlistID, CDI_GLOBAL, &natts);
 
-  for (i = 0; i < natts; ++i)
+  for (int i = 0; i < natts; ++i)
     {
       cdiInqAtt(vlistID, CDI_GLOBAL, i, name, &type, &len);
       if (type == CDI_DATATYPE_TXT)
@@ -1258,191 +1255,213 @@ dsets_init(datasets_t *dsets)
     }
 }
 #endif
-
-void *
-Importcmsaf(void *process)
+class Importcmsaf : public Process
 {
+public:
+  using Process::Process;
+  inline static CdoModule module = {
+    .name = "Importcmsaf",
+    .operators = { { "import_cmsaf", ImportcmsafHelp } },
+    .aliases = {},
+    .mode = EXPOSED,     // Module mode: 0:intern 1:extern
+    .number = CDI_REAL,  // Allowed number type
+    .constraints = { 1, 1, NoRestriction },
+  };
+  inline static RegisterEntry<Importcmsaf> registration = RegisterEntry<Importcmsaf>(module);
 #ifdef HAVE_LIBHDF5
   int gridID = -1, zaxisID;
-  int i;
-  size_t nmiss;
+  size_t numMissVals;
   int ivar;
   int varID, levelID, tsID;
-  herr_t status;  // Generic return value
+  hid_t file_id;
   datasets_t dsets;
 #endif
 
-  cdo_initialize(process);
+  CdoStreamID streamID;
+  int taxisID;
+  int vlistID;
 
-  operator_check_argc(0);
+public:
+  void
+  init() override
+  {
 
-  if (CdoDefault::FileType == CDI_UNDEFID) CdoDefault::FileType = CDI_FILETYPE_NC;
+    operator_check_argc(0);
+
+    if (CdoDefault::FileType == CDI_UNDEFID) CdoDefault::FileType = CDI_FILETYPE_NC;
 
 #ifdef HAVE_LIBHDF5
-  dsets_init(&dsets);
+    dsets_init(&dsets);
 
-  // Open an existing file.
-  hid_t file_id = H5Fopen(cdo_get_stream_name(0), H5F_ACC_RDONLY, H5P_DEFAULT);
-  if (file_id < 0) cdo_abort("H5Fopen failed on %s", cdo_get_stream_name(0));
+    // Open an existing file
+    file_id = H5Fopen(cdo_get_stream_name(0), H5F_ACC_RDONLY, H5P_DEFAULT);
+    if (file_id < 0) cdo_abort("H5Fopen failed on %s", cdo_get_stream_name(0));
 
-  // cmsaf_type = get_cmsaf_type(file_id);
+    // cmsaf_type = get_cmsaf_type(file_id);
 
-  H5Giterate(file_id, "/", nullptr, obj_info, (void *) &dsets);
+    H5Giterate(file_id, "/", nullptr, obj_info, (void *) &dsets);
 
-  if (dsets.numSets == 0) cdo_abort("No dataset found!");
+    if (dsets.numSets == 0) cdo_abort("No dataset found!");
+#endif
+  }
 
-  int gridsize = dsets.obj[0].gridsize;
-  int nx = dsets.obj[0].nx;
-  int ny = dsets.obj[0].ny;
-  int nz = dsets.obj[0].nz;
-  int nt = dsets.obj[0].nt;
+  void
+  run() override
+  {
 
-  for (ivar = 0; ivar < dsets.numSets; ++ivar)
-    if (dsets.obj[ivar].nt > 1)
+#ifdef HAVE_LIBHDF5
+    int gridsize = dsets.obj[0].gridsize;
+    int nx = dsets.obj[0].nx;
+    int ny = dsets.obj[0].ny;
+    int nz = dsets.obj[0].nz;
+    int nt = dsets.obj[0].nt;
+
+    for (ivar = 0; ivar < dsets.numSets; ++ivar)
+      if (dsets.obj[ivar].nt > 1)
+        {
+          nt = dsets.obj[ivar].nt;
+          break;
+        }
+
+    Varray<int> vtimes;
+    if (nt > 1)
       {
-        nt = dsets.obj[ivar].nt;
-        break;
+        vtimes.resize(nt);
+
+        for (int i = 0; i < nt; ++i) vtimes[i] = i * 10000 + 45 * 100;
+
+        if (dsets.obj[ivar].time)
+          {
+            char *pline = dsets.obj[ivar].time;
+            for (int i = 0; i < nt; ++i)
+              {
+                int itime = ((int) strtol(pline, &pline, 10)) * 100;
+                if (itime < 0 || itime > 240000)
+                  {
+                    cdo_warning("Wrong time string!");
+                    break;
+                  }
+                vtimes[i] = itime;
+              }
+          }
       }
 
-  Varray<int> vtimes;
-  if (nt > 1)
-    {
-      vtimes.resize(nt);
-
-      for (i = 0; i < nt; ++i) vtimes[i] = i * 10000 + 45 * 100;
-
-      if (dsets.obj[ivar].time)
-        {
-          char *pline = dsets.obj[ivar].time;
-          for (i = 0; i < nt; ++i)
-            {
-              int itime = ((int) strtol(pline, &pline, 10)) * 100;
-              if (itime < 0 || itime > 240000)
-                {
-                  cdo_warning("Wrong time string!");
-                  break;
-                }
-              vtimes[i] = itime;
-            }
-        }
-    }
-
-  if (Options::cdoVerbose)
-    for (ivar = 0; ivar < dsets.numSets; ++ivar)
-      cdo_print(" Var %d %-20s %dx%d nlev = %d nts = %d", ivar, dsets.obj[ivar].name, nx, ny, nz, dsets.obj[ivar].nt);
-
-  for (ivar = 1; ivar < dsets.numSets; ++ivar)
-    {
-      if (nx != dsets.obj[0].nx || ny != dsets.obj[0].ny) cdo_abort("Gridsize must not change!");
-      if (nz != dsets.obj[0].nz) cdo_abort("Number of levels must not change!");
-    }
-
-  if (dsets.lgeoloc) { gridID = read_geolocation(file_id, nx, ny, dsets.lprojtype); }
-  else if (dsets.lregion) { gridID = read_region(file_id, nx, ny); }
-
-  if (gridID == -1)
-    {
-      gridID = gridCreate(GRID_GENERIC, gridsize);
-      gridDefXsize(gridID, nx);
-      gridDefYsize(gridID, ny);
-    }
-
-  if (nz == 1)
-    zaxisID = zaxisCreate(ZAXIS_SURFACE, 1);
-  else
-    {
-      Varray<double> levels(nz);
-      for (i = 0; i < nz; ++i) levels[i] = i + 1;
-      zaxisID = zaxisCreate(ZAXIS_GENERIC, nz);
-      zaxisDefLevels(zaxisID, levels.data());
-    }
-
-  auto vlistID = vlistCreate();
-
-  auto taxisID = cdo_taxis_create((nt > 1) ? TAXIS_RELATIVE : TAXIS_ABSOLUTE);
-  taxisDefCalendar(taxisID, CALENDAR_STANDARD);
-  vlistDefTaxis(vlistID, taxisID);
-
-  for (ivar = 0; ivar < dsets.numSets; ++ivar)
-    {
-      varID = vlistDefVar(vlistID, gridID, zaxisID, TIME_VARYING);
-      cdiDefKeyString(vlistID, varID, CDI_KEY_NAME, dsets.obj[ivar].name);
-      if (dsets.obj[ivar].description) cdiDefKeyString(vlistID, varID, CDI_KEY_LONGNAME, dsets.obj[ivar].description);
-      if (dsets.obj[ivar].units) cdiDefKeyString(vlistID, varID, CDI_KEY_UNITS, dsets.obj[ivar].units);
-      if (dsets.obj[ivar].title) cdiDefAttTxt(vlistID, varID, "title", (int) strlen(dsets.obj[ivar].title), dsets.obj[ivar].title);
-
-      vlistDefVarDatatype(vlistID, varID, dsets.obj[ivar].dtype);
-      if (dsets.obj[ivar].haveMissvals) vlistDefVarMissval(vlistID, varID, dsets.obj[ivar].missval);
-      if (dsets.obj[ivar].haveScalefactor) cdiDefKeyFloat(vlistID, varID, CDI_KEY_SCALEFACTOR, dsets.obj[ivar].scalefactor);
-      if (dsets.obj[ivar].haveAddoffset) cdiDefKeyFloat(vlistID, varID, CDI_KEY_ADDOFFSET, dsets.obj[ivar].addoffset);
-    }
-
-  get_global_att(file_id, "/", vlistID);
-  if (dsets.lmetadata) get_global_att(file_id, "Metadata", vlistID);
-
-  int vdate = get_vdate(vlistID);
-  if (vdate == 0) vdate = 10101;
-
-  auto streamID = cdo_open_write(1);
-
-  cdo_def_vlist(streamID, vlistID);
-
-  for (tsID = 0; tsID < nt; ++tsID)
-    {
-      const int vtime = (vtimes.empty()) ? 0 : vtimes[tsID];
-      CdiDateTime vDateTime{};
-      vDateTime.date = cdiDate_set(vdate);
-      vDateTime.time = cdiTime_set(vtime);
-      taxisDefVdatetime(taxisID, vDateTime);
-      cdo_def_timestep(streamID, tsID);
-
+    if (Options::cdoVerbose)
       for (ivar = 0; ivar < dsets.numSets; ++ivar)
-        {
-          varID = ivar;
+        cdo_print(" Var %d %-20s %dx%d nlev = %d nts = %d", ivar, dsets.obj[ivar].name, nx, ny, nz, dsets.obj[ivar].nt);
 
-          if (tsID > 0 && dsets.obj[ivar].nt == 1) continue;
+    for (ivar = 1; ivar < dsets.numSets; ++ivar)
+      {
+        if (nx != dsets.obj[0].nx || ny != dsets.obj[0].ny) cdo_abort("Gridsize must not change!");
+        if (nz != dsets.obj[0].nz) cdo_abort("Number of levels must not change!");
+      }
 
-          gridsize = dsets.obj[ivar].gridsize;
-          double missval = dsets.obj[ivar].missval;
+    if (dsets.lgeoloc) { gridID = read_geolocation(file_id, nx, ny, dsets.lprojtype); }
+    else if (dsets.lregion) { gridID = read_region(file_id, nx, ny); }
 
-          for (levelID = 0; levelID < nz; ++levelID)
-            {
-              int offset = gridsize * levelID;
-              if (nz == 1) offset = gridsize * tsID;
-              double *array = dsets.obj[ivar].array + offset;
+    if (gridID == -1)
+      {
+        gridID = gridCreate(GRID_GENERIC, gridsize);
+        gridDefXsize(gridID, nx);
+        gridDefYsize(gridID, ny);
+      }
 
-              auto mm = varray_min_max_mv(gridsize, array, missval);
-              nmiss = gridsize - mm.n;
+    if (nz == 1)
+      zaxisID = zaxisCreate(ZAXIS_SURFACE, 1);
+    else
+      {
+        Varray<double> levels(nz);
+        for (int i = 0; i < nz; ++i) levels[i] = i + 1;
+        zaxisID = zaxisCreate(ZAXIS_GENERIC, nz);
+        zaxisDefLevels(zaxisID, levels.data());
+      }
 
-              if (Options::cdoVerbose)
-                cdo_print(" Write var %d,  level %d, nmiss %zu, missval %g, minval %g, maxval %g", varID, levelID, nmiss, missval,
-                          mm.min, mm.max);
+    vlistID = vlistCreate();
 
-              cdo_def_record(streamID, varID, levelID);
-              cdo_write_record(streamID, array, nmiss);
-            }
-        }
-    }
+    taxisID = cdo_taxis_create((nt > 1) ? TAXIS_RELATIVE : TAXIS_ABSOLUTE);
+    taxisDefCalendar(taxisID, CALENDAR_STANDARD);
+    vlistDefTaxis(vlistID, taxisID);
 
-  // Close file
-  status = H5Fclose(file_id);
-  (void) (status);  // unused
+    for (ivar = 0; ivar < dsets.numSets; ++ivar)
+      {
+        varID = vlistDefVar(vlistID, gridID, zaxisID, TIME_VARYING);
+        cdiDefKeyString(vlistID, varID, CDI_KEY_NAME, dsets.obj[ivar].name);
+        if (dsets.obj[ivar].description) cdiDefKeyString(vlistID, varID, CDI_KEY_LONGNAME, dsets.obj[ivar].description);
+        if (dsets.obj[ivar].units) cdiDefKeyString(vlistID, varID, CDI_KEY_UNITS, dsets.obj[ivar].units);
+        if (dsets.obj[ivar].title)
+          cdiDefAttTxt(vlistID, varID, "title", (int) strlen(dsets.obj[ivar].title), dsets.obj[ivar].title);
 
-  process_def_var_num(vlistNvars(vlistID));
+        vlistDefVarDatatype(vlistID, varID, dsets.obj[ivar].dtype);
+        if (dsets.obj[ivar].haveMissvals) vlistDefVarMissval(vlistID, varID, dsets.obj[ivar].missval);
+        if (dsets.obj[ivar].haveScalefactor) cdiDefKeyFloat(vlistID, varID, CDI_KEY_SCALEFACTOR, dsets.obj[ivar].scalefactor);
+        if (dsets.obj[ivar].haveAddoffset) cdiDefKeyFloat(vlistID, varID, CDI_KEY_ADDOFFSET, dsets.obj[ivar].addoffset);
+      }
 
-  cdo_stream_close(streamID);
+    get_global_att(file_id, "/", vlistID);
+    if (dsets.lmetadata) get_global_att(file_id, "Metadata", vlistID);
 
-  vlistDestroy(vlistID);
-  gridDestroy(gridID);
-  zaxisDestroy(zaxisID);
-  taxisDestroy(taxisID);
+    int vdate = get_vdate(vlistID);
+    if (vdate == 0) vdate = 10101;
 
-  for (ivar = 0; ivar < dsets.numSets; ++ivar) Free(dsets.obj[ivar].array);
+    streamID = cdo_open_write(1);
 
-  cdo_finish();
+    cdo_def_vlist(streamID, vlistID);
+
+    for (tsID = 0; tsID < nt; ++tsID)
+      {
+        int vtime = (vtimes.empty()) ? 0 : vtimes[tsID];
+        CdiDateTime vDateTime{};
+        vDateTime.date = cdiDate_set(vdate);
+        vDateTime.time = cdiTime_set(vtime);
+        taxisDefVdatetime(taxisID, vDateTime);
+        cdo_def_timestep(streamID, tsID);
+
+        for (ivar = 0; ivar < dsets.numSets; ++ivar)
+          {
+            varID = ivar;
+
+            if (tsID > 0 && dsets.obj[ivar].nt == 1) continue;
+
+            gridsize = dsets.obj[ivar].gridsize;
+            double missval = dsets.obj[ivar].missval;
+
+            for (levelID = 0; levelID < nz; ++levelID)
+              {
+                int offset = gridsize * levelID;
+                if (nz == 1) offset = gridsize * tsID;
+                double *array = dsets.obj[ivar].array + offset;
+
+                auto mm = varray_min_max_mv(gridsize, array, missval);
+                numMissVals = gridsize - mm.n;
+
+                if (Options::cdoVerbose)
+                  cdo_print(" Write var %d,  level %d, numMissVals %zu, missval %g, minval %g, maxval %g", varID, levelID,
+                            numMissVals, missval, mm.min, mm.max);
+
+                cdo_def_record(streamID, varID, levelID);
+                cdo_write_record(streamID, array, numMissVals);
+              }
+          }
+      }
 #else
-  cdo_abort("HDF5 support not compiled in!");
+    cdo_abort("HDF5 support not compiled in!");
 #endif
+  }
 
-  return nullptr;
-}
+  void
+  close() override
+  {
+#ifdef HAVE_LIBHDF5
+    // Close file
+    (void) H5Fclose(file_id);
+
+    process_def_var_num(vlistNvars(vlistID));
+
+    cdo_stream_close(streamID);
+    vlistDestroy(vlistID);
+
+    for (ivar = 0; ivar < dsets.numSets; ++ivar) free(dsets.obj[ivar].array);
+
+#endif
+  }
+};
