@@ -13,13 +13,24 @@
 #include <cdi.h>
 
 #include "process_int.h"
-#include "cdo_vlist.h"
 #include <mpim_grid.h>
 
 void genGridIndex(int gridID1, int gridID2, std::vector<long> &index);
 
-class ModuleMergegrid
+class Mergegrid : public Process
 {
+public:
+  using Process::Process;
+  inline static CdoModule module = {
+    .name = "Mergegrid",
+    .operators = { { "mergegrid", MergegridHelp } },
+    .aliases = {},
+    .mode = EXPOSED,     // Module mode: 0:intern 1:extern
+    .number = CDI_REAL,  // Allowed number type
+    .constraints = { 2, 1, NoRestriction },
+  };
+  inline static RegisterEntry<Mergegrid> registration = RegisterEntry<Mergegrid>(module);
+
   CdoStreamID streamID1;
   CdoStreamID streamID2;
   CdoStreamID streamID3;
@@ -30,6 +41,9 @@ class ModuleMergegrid
   int taxisID1;
   int taxisID3;
 
+  VarList varList1;
+  VarList varList2;
+
   Varray<double> array1;
   Varray<double> array2;
 
@@ -40,10 +54,8 @@ class ModuleMergegrid
 
 public:
   void
-  init(void *process)
+  init() override
   {
-    cdo_initialize(process);
-
     operator_check_argc(0);
 
     streamID1 = cdo_open_read(0);
@@ -55,16 +67,18 @@ public:
     streamID2 = cdo_open_read(1);
     vlistID2 = cdo_stream_inq_vlist(streamID2);
 
-    vlist_compare(vlistID1, vlistID2, CmpVlist::Name | CmpVlist::NumLevels);
+    varList1 = VarList(vlistID1);
+    varList2 = VarList(vlistID2);
+    varList_compare(varList1, varList2, CmpVarList::Name | CmpVarList::NumLevels);
 
     int ndiffgrids = 0;
-    for (int index = 1; index < vlistNgrids(vlistID1); ++index)
+    for (int index = 1; index < vlistNumGrids(vlistID1); ++index)
       if (vlistGrid(vlistID1, 0) != vlistGrid(vlistID1, index)) ndiffgrids++;
 
     if (ndiffgrids > 0) cdo_abort("Too many different grids in %s!", cdo_get_stream_name(0));
 
     ndiffgrids = 0;
-    for (int index = 1; index < vlistNgrids(vlistID2); ++index)
+    for (int index = 1; index < vlistNumGrids(vlistID2); ++index)
       if (vlistGrid(vlistID2, 0) != vlistGrid(vlistID2, index)) ndiffgrids++;
 
     if (ndiffgrids > 0) cdo_abort("Too many different grids in %s!", cdo_get_stream_name(1));
@@ -86,8 +100,9 @@ public:
     vlistDefTaxis(vlistID3, taxisID3);
     cdo_def_vlist(streamID3, vlistID3);
   }
+
   void
-  run()
+  run() override
   {
     int tsID = 0;
     while (true)
@@ -108,53 +123,41 @@ public:
           {
             int varID, levelID;
             cdo_inq_record(streamID2, &varID, &levelID);
-            size_t nmiss2;
-            cdo_read_record(streamID2, array2.data(), &nmiss2);
-
-            auto missval2 = vlistInqVarMissval(vlistID2, varID);
+            size_t numMissVals2;
+            cdo_read_record(streamID2, array2.data(), &numMissVals2);
 
             cdo_inq_record(streamID1, &varID, &levelID);
-            size_t nmiss1;
-            cdo_read_record(streamID1, array1.data(), &nmiss1);
+            size_t numMissVals1;
+            cdo_read_record(streamID1, array1.data(), &numMissVals1);
 
-            auto missval1 = vlistInqVarMissval(vlistID1, varID);
+            auto missval1 = varList1.vars[varID].missval;
+            auto missval2 = varList2.vars[varID].missval;
 
             for (size_t i = 0; i < gridsize2; ++i)
               {
-                if (gindex[i] >= 0 && !DBL_IS_EQUAL(array2[i], missval2)) { array1[gindex[i]] = array2[i]; }
+                if (gindex[i] >= 0 && dbl_is_not_equal(array2[i], missval2)) { array1[gindex[i]] = array2[i]; }
               }
 
-            if (nmiss1)
+            if (numMissVals1)
               {
-                nmiss1 = 0;
+                numMissVals1 = 0;
                 for (size_t i = 0; i < gridsize1; ++i)
-                  if (DBL_IS_EQUAL(array1[i], missval1)) nmiss1++;
+                  if (dbl_is_equal(array1[i], missval1)) numMissVals1++;
               }
 
             cdo_def_record(streamID3, varID, levelID);
-            cdo_write_record(streamID3, array1.data(), nmiss1);
+            cdo_write_record(streamID3, array1.data(), numMissVals1);
           }
 
         tsID++;
       }
   }
+
   void
-  close()
+  close() override
   {
     cdo_stream_close(streamID3);
     cdo_stream_close(streamID2);
     cdo_stream_close(streamID1);
-
-    cdo_finish();
   }
 };
-
-void *
-Mergegrid(void *process)
-{
-  ModuleMergegrid mergegrid;
-  mergegrid.init(process);
-  mergegrid.run();
-  mergegrid.close();
-  return nullptr;
-}
