@@ -176,9 +176,21 @@ p_to_uv_grid(long nlon, long nlat, Varray<double> &grid1x_v, Varray<double> &gri
       }
 }
 
-class ModuleMrotuv
+class Mrotuv : public Process
 {
-  size_t nmiss1 = 0, nmiss2 = 0;
+public:
+  using Process::Process;
+  inline static CdoModule module = {
+    .name = "Mrotuv",
+    .operators = { { "mrotuv" } },
+    .aliases = {},
+    .mode = EXPOSED,     // Module mode: 0:intern 1:extern
+    .number = CDI_REAL,  // Allowed number type
+    .constraints = { 1, 2, NoRestriction },
+  };
+  inline static RegisterEntry<Mrotuv> registration = RegisterEntry<Mrotuv>(module);
+
+  size_t numMissVals1 = 0, numMissVals2 = 0;
   int uid = -1, vid = -1;
 
   CdoStreamID streamID1;
@@ -212,27 +224,26 @@ class ModuleMrotuv
 
 public:
   void
-  init(void *process)
+  init() override
   {
-    cdo_initialize(process);
-
     operator_check_argc(0);
 
     streamID1 = cdo_open_read(0);
 
     auto vlistID1 = cdo_stream_inq_vlist(streamID1);
+    VarList varList1(vlistID1);
 
-    auto nvars = vlistNvars(vlistID1);
-    for (int varid = 0; varid < nvars; varid++)
+    auto numVars = vlistNvars(vlistID1);
+    for (int varid = 0; varid < numVars; varid++)
       {
-        auto code = vlistInqVarCode(vlistID1, varid);
+        auto code = varList1.vars[varid].code;
         if (code == 3 || code == 131) uid = varid;
         if (code == 4 || code == 132) vid = varid;
       }
 
     if (uid == -1 || vid == -1)
       {
-        if (nvars == 2)
+        if (numVars == 2)
           {
             uid = 0;
             vid = 1;
@@ -241,18 +252,19 @@ public:
           cdo_abort("U and V not found in %s", cdo_get_stream_name(0));
       }
 
-    nlevs = zaxisInqSize(vlistInqVarZaxis(vlistID1, uid));
-    if (nlevs != zaxisInqSize(vlistInqVarZaxis(vlistID1, vid))) cdo_abort("U and V have different number of levels!");
+    const auto &varU = varList1.vars[uid];
+    const auto &varV = varList1.vars[vid];
+    if (varU.nlevels != varV.nlevels) cdo_abort("U and V have different number of levels!");
 
-    auto gridID1 = vlistInqVarGrid(vlistID1, uid);
-    auto gridID2 = vlistInqVarGrid(vlistID1, vid);
-    gridsize = gridInqSize(gridID1);
-    if (gridID1 != gridID2) cdo_abort("Input grids differ!");
+    auto gridID1 = varU.gridID;
+    gridsize = varU.gridsize;
+    if (varU.gridID != varV.gridID) cdo_abort("Input grids differ!");
 
-    if (gridInqType(gridID1) != GRID_LONLAT && gridInqType(gridID1) != GRID_GAUSSIAN && gridInqType(gridID1) != GRID_CURVILINEAR)
-      cdo_abort("Grid %s unsupported!", gridNamePtr(gridInqType(gridID1)));
+    auto gridType = varU.gridType;
+    if (gridType != GRID_LONLAT && gridType != GRID_GAUSSIAN && gridType != GRID_CURVILINEAR)
+      cdo_abort("Grid %s unsupported!", gridNamePtr(gridType));
 
-    if (gridInqType(gridID1) != GRID_CURVILINEAR) gridID1 = gridToCurvilinear(gridID1);
+    if (gridType != GRID_CURVILINEAR) gridID1 = gridToCurvilinear(gridID1);
 
     if (gridsize != gridInqSize(gridID1)) cdo_abort("Internal problem: gridsize changed!");
 
@@ -319,11 +331,12 @@ public:
     cdo_def_vlist(streamID2, vlistID2);
     cdo_def_vlist(streamID3, vlistID3);
 
-    missval1 = vlistInqVarMissval(vlistID1, uid);
-    missval2 = vlistInqVarMissval(vlistID1, vid);
+    missval1 = varU.missval;
+    missval2 = varV.missval;
   }
+
   void
-  run()
+  run() override
   {
     ufield_v = Varray<double>(gridsize);
     vfield_v = Varray<double>(gridsize);
@@ -356,14 +369,14 @@ public:
             int varID, levelID;
             cdo_inq_record(streamID1, &varID, &levelID);
 
-            if (varID == uid) cdo_read_record(streamID1, urfield[levelID].data(), &nmiss1);
-            if (varID == vid) cdo_read_record(streamID1, vrfield[levelID].data(), &nmiss2);
+            if (varID == uid) cdo_read_record(streamID1, urfield[levelID].data(), &numMissVals1);
+            if (varID == vid) cdo_read_record(streamID1, vrfield[levelID].data(), &numMissVals2);
           }
 
         for (int levelID = 0; levelID < nlevs; ++levelID)
           {
             // remove missing values
-            if (nmiss1 || nmiss2)
+            if (numMissVals1 || numMissVals2)
               {
                 for (size_t i = 0; i < gridsize; ++i)
                   {
@@ -402,33 +415,20 @@ public:
             for (size_t i = 0; i < nlon; ++i) { vfield[nlat - 1][i] = vhelp[nlat - 1][i + 1]; }
 
             cdo_def_record(streamID2, 0, levelID);
-            cdo_write_record(streamID2, ufield_v.data(), nmiss1);
+            cdo_write_record(streamID2, ufield_v.data(), numMissVals1);
             cdo_def_record(streamID3, 0, levelID);
-            cdo_write_record(streamID3, vfield_v.data(), nmiss2);
+            cdo_write_record(streamID3, vfield_v.data(), numMissVals2);
           }
 
         tsID++;
       }
   }
+
   void
-  close()
+  close() override
   {
     cdo_stream_close(streamID3);
     cdo_stream_close(streamID2);
     cdo_stream_close(streamID1);
-
-    cdo_finish();
   }
 };
-
-void *
-Mrotuv(void *process)
-{
-  ModuleMrotuv mrotuv;
-
-  mrotuv.init(process);
-  mrotuv.run();
-  mrotuv.close();
-
-  return nullptr;
-}
