@@ -36,7 +36,7 @@ uv2dv_cfd_W(double missval, double *u, double *v, double *lon, double *lat, size
       double *tmp_v = v + k * gridsize_uv;
       double *tmp_div = div + k * gridsize_uv;
       // Init output array.
-      varray_fill(gridsize_uv, tmp_div, 0.0);
+      ranges::fill_n(tmp_div, gridsize_uv, 0.0);
       // Call the Fortran routine.
 #ifdef HAVE_CF_INTERFACE
       DDVFIDF(tmp_u, tmp_v, lat, lon, inlon, inlat, missval, boundOpt, tmp_div, ierror);
@@ -66,7 +66,7 @@ uv2vr_cfd_W(double missval, double *u, double *v, double *lon, double *lat, size
       double *tmp_v = v + k * gridsize_uv;
       double *tmp_vort = vort + k * gridsize_uv;
       // Init output array.
-      varray_fill(gridsize_uv, tmp_vort, 0.0);
+      ranges::fill_n(tmp_vort, gridsize_uv, 0.0);
       // Call the Fortran routine.
 #ifdef HAVE_CF_INTERFACE
       DVRFIDF(tmp_u, tmp_v, lat, lon, inlon, inlat, missval, boundOpt, tmp_vort, ierror);
@@ -79,10 +79,10 @@ uv2vr_cfd_W(double missval, double *u, double *v, double *lon, double *lat, size
 static int
 find_name(const VarList &varList, const std::string &name)
 {
-  int nvars = varList.size();
-  for (int varID = 0; varID < nvars; ++varID)
+  auto numVars = varList.numVars();
+  for (int varID = 0; varID < numVars; ++varID)
     {
-      if (name == varList[varID].name) return varID;
+      if (name == varList.vars[varID].name) return varID;
     }
 
   return CDI_UNDEFID;
@@ -96,10 +96,10 @@ enum struct OutMode
 };
 
 // Parameter
-OutMode outMode(OutMode::NEW);
-int boundOpt = -1;
-std::string uName;
-std::string vName;
+static OutMode outMode(OutMode::NEW);
+static int boundOpt = -1;
+static std::string uName;
+static std::string vName;
 
 static void
 print_parameter(void)
@@ -123,7 +123,7 @@ set_parameter(void)
 
       KVList kvlist;
       kvlist.name = "PARAMETER";
-      if (kvlist.parse_arguments(pargc, pargv) != 0) cdo_abort("Parse error!");
+      if (kvlist.parse_arguments(pargv) != 0) cdo_abort("Parse error!");
       if (Options::cdoVerbose) kvlist.print();
 
       for (const auto &kv : kvlist)
@@ -152,8 +152,24 @@ set_parameter(void)
   if (Options::cdoVerbose) print_parameter();
 }
 
-class ModuleNCL_wind
+class NCL_wind : public Process
 {
+public:
+  using Process::Process;
+  inline static CdoModule module = {
+    .name = "NCL_wind",
+    // clang-format off
+    .operators = { { "uv2dv_cfd", 0, 0, "[u,v,boundsOpt,outMode]", NCL_windHelp },
+                   { "uv2vr_cfd", 0, 0, "[u,v,boundsOpt,outMode]", NCL_windHelp } },
+    // clang-format on
+    .aliases = {},
+    .mode = EXPOSED,     // Module mode: 0:intern 1:extern
+    .number = CDI_REAL,  // Allowed number type
+    .constraints = { 1, 1, NoRestriction },
+  };
+  inline static RegisterEntry<NCL_wind> registration = RegisterEntry<NCL_wind>(module);
+
+  int UV2DV_CFD, UV2VR_CFD;
 
   CdoStreamID streamID1;
   CdoStreamID streamID2;
@@ -182,17 +198,14 @@ class ModuleNCL_wind
   Varray<double> lon;
   Varray<double> lat;
 
-  int UV2DV_CFD, UV2VR_CFD;
   VarList varList1;
 
 public:
   void
-  init(void *process)
+  init() override
   {
-    cdo_initialize(process);
-
-    UV2DV_CFD = cdo_operator_add("uv2dv_cfd", 0, 0, "[u, v, boundsOpt, outMode]");
-    UV2VR_CFD = cdo_operator_add("uv2vr_cfd", 0, 0, "[u, v, boundsOpt, outMode]");
+    UV2DV_CFD = module.get_id("uv2dv_cfd");
+    UV2VR_CFD = module.get_id("uv2vr_cfd");
 
     operatorID = cdo_operator_id();
 
@@ -209,7 +222,7 @@ public:
     else
       cdo_abort("outMode=%d unsupported!", outMode);
 
-    varList_init(varList1, vlistID1);
+    varList1 = VarList(vlistID1);
 
     varIDu = find_name(varList1, uName);
     varIDv = find_name(varList1, vName);
@@ -217,8 +230,8 @@ public:
     if (varIDu == CDI_UNDEFID) cdo_abort("%s not found!", uName);
     if (varIDv == CDI_UNDEFID) cdo_abort("%s not found!", vName);
 
-    const auto &varU = varList1[varIDu];
-    const auto &varV = varList1[varIDv];
+    const auto &varU = varList1.vars[varIDu];
+    const auto &varV = varList1.vars[varIDv];
 
     auto gridIDu = varU.gridID;
     auto gridIDv = varV.gridID;
@@ -242,7 +255,7 @@ public:
 
     if (nlev != zaxisInqSize(varU.zaxisID)) cdo_abort("u and v must have the same number of level!");
 
-    varIDo = vlistDefVar(vlistID2, gridIDu, zaxisIDu, varU.timetype);
+    varIDo = vlistDefVar(vlistID2, gridIDu, zaxisIDu, varU.timeType);
     if (operatorID == UV2DV_CFD)
       {
         cdiDefKeyString(vlistID2, varIDo, CDI_KEY_NAME, "d");
@@ -279,18 +292,17 @@ public:
   }
 
   void
-  run()
+  run() override
   {
-
-    const auto &varU = varList1[varIDu];
-    const auto &varV = varList1[varIDv];
+    const auto &varU = varList1.vars[varIDu];
+    const auto &varV = varList1.vars[varIDv];
     int tsID = 0;
     while (true)
       {
         auto nrecs = cdo_stream_inq_timestep(streamID1, tsID);
         if (nrecs == 0) break;
 
-        size_t nmissu = 0, nmissv = 0;
+        size_t numMissValsu = 0, numMissValsv = 0;
 
         cdo_taxis_copy_timestep(taxisID2, taxisID1);
         cdo_def_timestep(streamID2, tsID);
@@ -299,34 +311,34 @@ public:
           {
             int varID, levelID;
             cdo_inq_record(streamID1, &varID, &levelID);
-            size_t nmiss;
-            cdo_read_record(streamID1, &array[0], &nmiss);
+            size_t numMissVals;
+            cdo_read_record(streamID1, &array[0], &numMissVals);
 
             if (varID == varIDu || varID == varIDv)
               {
                 if (varID == varIDu)
                   {
                     std::copy_n(&array[0], gridsizeuv, &arrayu[levelID * gridsizeuv]);
-                    nmissu += nmiss;
+                    numMissValsu += numMissVals;
                   }
                 if (varID == varIDv)
                   {
                     std::copy_n(&array[0], gridsizeuv, &arrayv[levelID * gridsizeuv]);
-                    nmissv += nmiss;
+                    numMissValsv += numMissVals;
                   }
               }
 
             if (outMode == OutMode::APPEND)
               {
                 cdo_def_record(streamID2, varID, levelID);
-                cdo_write_record(streamID2, &array[0], nmiss);
+                cdo_write_record(streamID2, &array[0], numMissVals);
               }
           }
 
-        if (nmissu != nmissv)
+        if (numMissValsu != numMissValsv)
           {
             cdo_abort("u and v have different number of missing values!");
-            if (nmissu && !dbl_is_equal(varU.missval, varV.missval))
+            if (numMissValsu && dbl_is_not_equal(varU.missval, varV.missval))
               {
                 for (int levelID = 0; levelID < nlev; ++levelID)
                   {
@@ -345,34 +357,21 @@ public:
         for (int levelID = 0; levelID < nlev; ++levelID)
           {
             auto parray = &arrayo[levelID * gridsizeuv];
-            auto nmiss = array_num_mv(gridsizeuv, parray, varU.missval);
+            auto numMissVals = array_num_mv(gridsizeuv, parray, varU.missval);
             cdo_def_record(streamID2, varIDo, levelID);
-            cdo_write_record(streamID2, parray, nmiss);
+            cdo_write_record(streamID2, parray, numMissVals);
           }
 
         tsID++;
       }
   }
+
   void
-  close()
+  close() override
   {
     cdo_stream_close(streamID1);
     cdo_stream_close(streamID2);
 
     vlistDestroy(vlistID2);
-
-    cdo_finish();
   }
 };
-void *
-NCL_wind(void *process)
-
-{
-  ModuleNCL_wind ncl_wind;
-
-  ncl_wind.init(process);
-  ncl_wind.run();
-  ncl_wind.close();
-
-  return nullptr;
-}
