@@ -18,8 +18,6 @@
 
 #include <cdi.h>
 
-#include <unordered_map>
-
 #include "cdo_options.h"
 #include "process_int.h"
 #include "param_conversion.h"
@@ -76,7 +74,7 @@ outputxyz(size_t gridsize, const Varray<double> &array, double missval, size_t n
   double fmin = 0.0;
   double x, y, z;
   for (size_t i = 0; i < gridsize; ++i)
-    if (!dbl_is_equal(array[i], missval))
+    if (dbl_is_not_equal(array[i], missval))
       {
         if (array[i] < fmin) fmin = array[i];
         fprintf(stdout, "%g\t%g\t%g\t%g\n", lon[i], lat[i], array[i], array[i]);
@@ -225,9 +223,59 @@ outputf(int nelem, const std::string &format, size_t gridsize, const Varray<doub
   fprintf(stdout, "\n");
 }
 
-class ModuleOutput
+class Output : public Process
 {
-  size_t nmiss;
+  enum
+  {
+    knohead,
+    kvalue,
+    kparam,
+    kcode,
+    kname,
+    kx,
+    ky,
+    klon,
+    klat,
+    klev,
+    kbin,
+    kxind,
+    kyind,
+    ktimestep,
+    kdate,
+    ktime,
+    kyear,
+    kmonth,
+    kday
+  };
+  struct KeyLenEntry
+  {
+    std::string key;
+    int idx;
+    int len;
+  };
+
+public:
+  using Process::Process;
+  inline static CdoModule module = {
+    .name = "Output",
+    .operators = { { "output", 0, 1, OutputHelp },
+                   { "outputint", OutputHelp },
+                   { "outputsrv", OutputHelp },
+                   { "outputext", OutputHelp },
+                   { "outputf", OutputHelp },
+                   { "outputts", OutputHelp },
+                   { "outputfld", OutputHelp },
+                   { "outputarr", OutputHelp },
+                   { "outputxyz", OutputHelp },
+                   { "outputtab", OutputtabHelp } },
+    .aliases = { { "outputkey", "outputtab" } },
+    .mode = EXPOSED,     // Module mode: 0:intern 1:extern
+    .number = CDI_BOTH,  // Allowed number type
+    .constraints = { -1, 0, NoRestriction },
+  };
+  inline static RegisterEntry<Output> registration = RegisterEntry<Output>(module);
+  int OUTPUT, OUTPUTINT, OUTPUTSRV, OUTPUTEXT, OUTPUTF, OUTPUTTS, OUTPUTFLD, OUTPUTARR, OUTPUTXYZ, OUTPUTTAB;
+  size_t numMissVals;
   int nelem = 1;
   int index;
   std::string format;
@@ -238,12 +286,6 @@ class ModuleOutput
   int operatorID;
 
   // clang-format off
-  enum {knohead,   kvalue,  kparam,  kcode,  kname,  kx,  ky,  klon,  klat,  klev,  kbin,  kxind,  kyind,  ktimestep,  kdate,  ktime,  kyear,  kmonth,  kday};
-  struct KeyLenEntry {
-    std::string key;
-    int         idx;
-    int         len;
-  };
   std::vector<KeyLenEntry> keyMap = {
     { "nohead",   knohead,    0 },
     { "value",    kvalue,     8 },
@@ -264,28 +306,23 @@ class ModuleOutput
     { "year",     kyear,      5 },
     { "month",    kmonth,     2 },
     { "day",      kday,       2 },
-    // clang-format on
   };
-
-  int OUTPUT, OUTPUTINT, OUTPUTSRV, OUTPUTEXT, OUTPUTF, OUTPUTTS, OUTPUTFLD, OUTPUTARR, OUTPUTXYZ, OUTPUTTAB;
+  // clang-format on
 
 public:
   void
-  init(void *process)
+  init() override
   {
-
-    cdo_initialize(process);
-
-    OUTPUT = cdo_operator_add("output", 0, 1, nullptr);
-    OUTPUTINT = cdo_operator_add("outputint", 0, 0, nullptr);
-    OUTPUTSRV = cdo_operator_add("outputsrv", 0, 0, nullptr);
-    OUTPUTEXT = cdo_operator_add("outputext", 0, 0, nullptr);
-    OUTPUTF = cdo_operator_add("outputf", 0, 0, nullptr);
-    OUTPUTTS = cdo_operator_add("outputts", 0, 0, nullptr);
-    OUTPUTFLD = cdo_operator_add("outputfld", 0, 0, nullptr);
-    OUTPUTARR = cdo_operator_add("outputarr", 0, 0, nullptr);
-    OUTPUTXYZ = cdo_operator_add("outputxyz", 0, 0, nullptr);
-    OUTPUTTAB = cdo_operator_add("outputtab", 0, 0, nullptr);
+    OUTPUT = module.get_id("output");
+    OUTPUTINT = module.get_id("outputint");
+    OUTPUTSRV = module.get_id("outputsrv");
+    OUTPUTEXT = module.get_id("outputext");
+    OUTPUTF = module.get_id("outputf");
+    OUTPUTTS = module.get_id("outputts");
+    OUTPUTFLD = module.get_id("outputfld");
+    OUTPUTARR = module.get_id("outputarr");
+    OUTPUTXYZ = module.get_id("outputxyz");
+    OUTPUTTAB = module.get_id("outputtab");
 
     (void) (OUTPUT);  // unused
 
@@ -356,18 +393,18 @@ public:
       }
     else { operator_check_argc(0); }
   }
+
   void
-  run()
+  run() override
   {
     for (int indf = 0; indf < cdo_stream_cnt(); indf++)
       {
         auto streamID = cdo_open_read(indf);
         auto vlistID = cdo_stream_inq_vlist(streamID);
 
-        VarList varList;
-        varList_init(varList, vlistID);
+        VarList varList(vlistID);
 
-        auto ngrids = vlistNgrids(vlistID);
+        auto ngrids = vlistNumGrids(vlistID);
         int ndiffgrids = 0;
         for (index = 1; index < ngrids; ++index)
           if (vlistGrid(vlistID, 0) != vlistGrid(vlistID, index)) ndiffgrids++;
@@ -408,11 +445,11 @@ public:
               {
                 int varID, levelID;
                 cdo_inq_record(streamID, &varID, &levelID);
-                const auto &var = varList[varID];
+                const auto &var = varList.vars[varID];
 
                 auto code = var.code;
                 auto gridID = var.gridID;
-                auto dig = (var.datatype == CDI_DATATYPE_FLT64) ? Options::CDO_dbl_digits : Options::CDO_flt_digits;
+                auto dig = (var.dataType == CDI_DATATYPE_FLT64) ? Options::CDO_dbl_digits : Options::CDO_flt_digits;
                 gridsize = var.nwpv * var.gridsize;
                 auto nlon = gridInqXsize(gridID);
                 auto nlat = gridInqYsize(gridID);
@@ -427,7 +464,7 @@ public:
                     nlat = 1;
                   }
 
-                cdo_read_record(streamID, array.data(), &nmiss);
+                cdo_read_record(streamID, array.data(), &numMissVals);
 
                 auto vdate = cdiDate_get(vDateTime.date);
                 auto vtime = cdiTime_get(vDateTime.time);
@@ -451,7 +488,7 @@ public:
                     cdiTime_decode(vDateTime.time, &hour, &minute, &second, &ms);
                     double xdate = vdate - (vdate / 100) * 100 + (hour * 3600 + minute * 60 + second) / 86400.0;
                     for (size_t i = 0; i < gridsize; ++i)
-                      if (!dbl_is_equal(array[i], missval))
+                      if (dbl_is_not_equal(array[i], missval))
                         fprintf(stdout, "%g\t%g\t%g\t%.*g\n", xdate, grid_center_lat[i], grid_center_lon[i], dig, array[i]);
                   }
                 else if (operatorID == OUTPUTTAB)
@@ -522,19 +559,9 @@ public:
         cdo_stream_close(streamID);
       }
   }
+
   void
-  close()
+  close() override
   {
-    cdo_finish();
   }
 };
-
-void *
-Output(void *process)
-{
-  ModuleOutput output;
-  output.init(process);
-  output.run();
-  output.close();
-  return nullptr;
-}
