@@ -16,8 +16,21 @@
 #include "printinfo.h"
 #include "cdo_zaxis.h"
 
-class ModulePinfo
+class Pinfo : public Process
 {
+public:
+  using Process::Process;
+  inline static CdoModule module = {
+    .name = "Pinfo",
+    .operators = { { "pinfo" }, { "pinfov" } },
+    .aliases = {},
+    .mode = EXPOSED,     // Module mode: 0:intern 1:extern
+    .number = CDI_REAL,  // Allowed number type
+    .constraints = { 1, 1, NoRestriction },
+  };
+  inline static RegisterEntry<Pinfo> registration = RegisterEntry<Pinfo>(module);
+
+  int PINFO, PINFOV;
   CdoStreamID streamID1;
   CdoStreamID streamID2;
 
@@ -32,16 +45,12 @@ class ModulePinfo
 
   VarList varList1;
 
-  int PINFO, PINFOV;
-
 public:
   void
-  init(void *process)
+  init() override
   {
-    cdo_initialize(process);
-
-    PINFO = cdo_operator_add("pinfo", 0, 0, nullptr);
-    PINFOV = cdo_operator_add("pinfov", 0, 0, nullptr);
+    PINFO = module.get_id("pinfo");
+    PINFOV = module.get_id("pinfov");
 
     (void) (PINFO);  // unused
 
@@ -61,12 +70,13 @@ public:
     streamID2 = cdo_open_write(1);
     cdo_def_vlist(streamID2, vlistID2);
 
-    varList_init(varList1, vlistID1);
+    varList1 = VarList(vlistID1);
 
     gridsizemax = vlistGridsizeMax(vlistID1);
   }
+
   void
-  run()
+  run() override
   {
     Varray<double> array1(gridsizemax), array2(gridsizemax);
 
@@ -98,10 +108,10 @@ public:
 
             int varID, levelID;
             cdo_inq_record(streamID1, &varID, &levelID);
-            size_t nmiss;
-            cdo_read_record(streamID1, array1.data(), &nmiss);
+            size_t numMissVals;
+            cdo_read_record(streamID1, array1.data(), &numMissVals);
 
-            const auto &var = varList1[varID];
+            const auto &var = varList1.vars[varID];
             indg += 1;
             auto gridsize = var.gridsize;
 
@@ -114,15 +124,15 @@ public:
             auto level = cdo_zaxis_inq_level(var.zaxisID, levelID);
             fprintf(stdout, " %7g ", level);
 
-            fprintf(stdout, "%7zu %7zu :", gridsize, nmiss);
+            fprintf(stdout, "%7zu %7zu :", gridsize, numMissVals);
 
-            if (gridInqType(var.gridID) == GRID_SPECTRAL || (gridsize == 1 && nmiss == 0))
+            if (gridInqType(var.gridID) == GRID_SPECTRAL || (gridsize == 1 && numMissVals == 0))
               {
                 fprintf(stdout, "            %#12.5g\n", array1[0]);
               }
             else
               {
-                if (nmiss)
+                if (numMissVals)
                   {
                     auto mmm = varray_min_max_mean_mv(gridsize, array1, var.missval);
                     arrmin = mmm.min;
@@ -143,34 +153,23 @@ public:
                 if (gridsize) { fprintf(stdout, "%#12.5g%#12.5g%#12.5g\n", arrmin, arrmean, arrmax); }
                 else { fprintf(stdout, "                     nan\n"); }
 
-                if (imiss != nmiss && nmiss) fprintf(stdout, "Found %zu of %zu missing values!\n", imiss, nmiss);
+                if (imiss != numMissVals && numMissVals) fprintf(stdout, "Found %zu of %zu missing values!\n", imiss, numMissVals);
               }
 
             varray_copy(gridsize, array1, array2);
 
             cdo_def_record(streamID2, varID, levelID);
-            cdo_write_record(streamID2, array2.data(), nmiss);
+            cdo_write_record(streamID2, array2.data(), numMissVals);
           }
 
         tsID++;
       }
   }
+
   void
-  close()
+  close() override
   {
     cdo_stream_close(streamID1);
     cdo_stream_close(streamID2);
-
-    cdo_finish();
   }
 };
-
-void *
-Pinfo(void *process)
-{
-  ModulePinfo pinfo;
-  pinfo.init(process);
-  pinfo.run();
-  pinfo.close();
-  return nullptr;
-}
