@@ -17,7 +17,6 @@
 #include "griddes.h"
 #include "grid_point_search.h"
 #include <mpim_grid.h>
-#include "gridreference.h"
 #include "cimdOmp.h"
 #include "verifygrid.h"
 #include "field_functions.h"
@@ -165,6 +164,30 @@ read_ybounds_reg2d(int gridID, std::vector<double> &ybounds2d)
   for (size_t i = 0; i < 2 * nlat; ++i) ybounds2d[i] *= DEG2RAD;
 }
 
+static bool
+is_zonal_grid(int gridID, const std::vector<double> &xbounds2d, const std::vector<double> &ybounds2d)
+{
+  bool isZonalGrid{ false };
+
+  auto nlon = gridInqXsize(gridID);
+  auto nlat = gridInqYsize(gridID);
+
+  if (nlon == 1 && xbounds2d[1] > xbounds2d[0])
+    {
+      isZonalGrid = true;
+      for (size_t i = 1; i < nlat; ++i)
+        {
+          if (IS_NOT_EQUAL(ybounds2d[2 * i - 1], ybounds2d[2 * i]))
+            {
+              isZonalGrid = false;
+              break;
+            }
+        }
+    }
+
+  return isZonalGrid;
+}
+
 static double
 calc_maxdist(size_t i, size_t nv, double plon, double plat, const Varray<double> &xbounds, const Varray<double> &ybounds)
 {
@@ -181,8 +204,6 @@ calc_maxdist(size_t i, size_t nv, double plon, double plat, const Varray<double>
       maxdist = std::max(maxdist, sdist);
     }
 
-  maxdist = 1.01 * std::sqrt(maxdist);
-
   return maxdist;
 }
 
@@ -213,14 +234,13 @@ calc_maxdist_rec2d(size_t i, size_t nlon, double plon, double plat, const std::v
       auto sdist = squareDistance(p1, p2);
       maxdist = std::max(maxdist, sdist);
     }
-  maxdist = 1.01 * std::sqrt(maxdist);
 
   return maxdist;
 }
 
 #ifdef USE_YAC
 static size_t
-find_points_yac(std::vector<char> &vmask, size_t cell_no, size_t ncorner, size_t nadds, Varray<size_t> &adds,
+find_points_yac(std::vector<char> &vmask, size_t cell_no, size_t ncorner, size_t numIndices, Varray<size_t> &indices,
                 const Varray<double> &xvals1, const Varray<double> &yvals1, const Varray<double> &xbounds,
                 const Varray<double> &ybounds)
 {
@@ -248,9 +268,9 @@ find_points_yac(std::vector<char> &vmask, size_t cell_no, size_t ncorner, size_t
 
   double centerCoordinates[3];
   size_t nvalues = 0;
-  for (size_t k = 0; k < nadds; ++k)
+  for (size_t k = 0; k < numIndices; ++k)
     {
-      auto index1 = adds[k];
+      auto index1 = indices[k];
       auto lon = xvals1[index1];
       auto lat = yvals1[index1];
 
@@ -260,7 +280,7 @@ find_points_yac(std::vector<char> &vmask, size_t cell_no, size_t ncorner, size_t
         {
           if (vmask[index1] == 0)
             {
-              adds[nvalues] = adds[k];
+              indices[nvalues] = indices[k];
               nvalues++;
             }
           if (vmask[index1] < 127) vmask[index1]++;
@@ -275,7 +295,7 @@ find_points_yac(std::vector<char> &vmask, size_t cell_no, size_t ncorner, size_t
 #endif
 
 static size_t
-find_points(std::vector<char> &vmask, size_t cell_no, size_t ncorner, size_t nadds, Varray<size_t> &adds,
+find_points(std::vector<char> &vmask, size_t cell_no, size_t ncorner, size_t numIndices, Varray<size_t> &indices,
             const Varray<double> &xvals1, const Varray<double> &yvals1, const Varray<double> &xbounds,
             const Varray<double> &ybounds)
 {
@@ -293,11 +313,11 @@ find_points(std::vector<char> &vmask, size_t cell_no, size_t ncorner, size_t nad
 
   auto actualNumberOfCorners = get_actual_number_of_corners(ncorner, cell_corners_xyz_open_cell);
 
-  auto no_duplicates = get_no_duplicates(actualNumberOfCorners, cell_corners_xyz_open_cell, marked_duplicate_indices);
+  auto noDuplicates = get_no_duplicates(actualNumberOfCorners, cell_corners_xyz_open_cell, marked_duplicate_indices);
 
   copy_unique_corners(actualNumberOfCorners, cell_corners_xyz_open_cell, marked_duplicate_indices, cellCorners3D);
 
-  actualNumberOfCorners -= no_duplicates;
+  actualNumberOfCorners -= noDuplicates;
 
   cellCorners3D[actualNumberOfCorners] = cellCorners3D[0];
 
@@ -318,9 +338,9 @@ find_points(std::vector<char> &vmask, size_t cell_no, size_t ncorner, size_t nad
 
   double centerCoordinates[3];
   size_t nvalues = 0;
-  for (size_t k = 0; k < nadds; ++k)
+  for (size_t k = 0; k < numIndices; ++k)
     {
-      auto index1 = adds[k];
+      auto index1 = indices[k];
       auto lon = xvals1[index1];
       auto lat = yvals1[index1];
 
@@ -337,7 +357,7 @@ find_points(std::vector<char> &vmask, size_t cell_no, size_t ncorner, size_t nad
         {
           if (vmask[index1] == 0)
             {
-              adds[nvalues] = adds[k];
+              indices[nvalues] = indices[k];
               nvalues++;
             }
           if (vmask[index1] < 127) vmask[index1]++;
@@ -348,7 +368,7 @@ find_points(std::vector<char> &vmask, size_t cell_no, size_t ncorner, size_t nad
 }
 
 static size_t
-find_points_rec2d(std::vector<char> &vmask, size_t i, size_t nlon2, size_t nadds, Varray<size_t> &adds,
+find_points_rec2d(std::vector<char> &vmask, size_t i, size_t nlon2, size_t numIndices, Varray<size_t> &indices,
                   const Varray<double> &xvals1, const Varray<double> &yvals1, const std::vector<double> &xbounds2d,
                   const std::vector<double> &ybounds2d)
 {
@@ -356,9 +376,9 @@ find_points_rec2d(std::vector<char> &vmask, size_t i, size_t nlon2, size_t nadds
   auto ix = i - iy * nlon2;
 
   size_t nvalues = 0;
-  for (size_t k = 0; k < nadds; ++k)
+  for (size_t k = 0; k < numIndices; ++k)
     {
-      auto index1 = adds[k];
+      auto index1 = indices[k];
       auto x = xvals1[index1];
       auto y = yvals1[index1];
       if (y >= ybounds2d[2 * iy] && y < ybounds2d[2 * iy + 1])
@@ -367,7 +387,7 @@ find_points_rec2d(std::vector<char> &vmask, size_t i, size_t nlon2, size_t nadds
           {
             if (vmask[index1] == 0)
               {
-                adds[nvalues] = index1;
+                indices[nvalues] = index1;
                 nvalues++;
               }
             if (vmask[index1] < 127) vmask[index1]++;
@@ -377,18 +397,43 @@ find_points_rec2d(std::vector<char> &vmask, size_t i, size_t nlon2, size_t nadds
   return nvalues;
 }
 
-static void
-check_vmask(std::vector<char> vmask)
+static long
+get_bin_num(double lon1, double lat1, size_t numBins, const std::vector<double> &xbounds2d, const std::vector<double> &ybounds2)
 {
-  constexpr int max_vals = 128;
-  size_t vm[max_vals] = { 0 };
+  if ((lon1 >= xbounds2d[0] && lon1 <= xbounds2d[1]) || ((lon1 - PI2) >= xbounds2d[0] && (lon1 - PI2) <= xbounds2d[1]))
+    {
+      auto latIsUp = (ybounds2[1] > ybounds2[0]);
+      if (latIsUp)
+        {
+          for (size_t i = 0; i < numBins; ++i)
+            {
+              if (lat1 > ybounds2[i * 2] && lat1 <= ybounds2[i * 2 + 1]) return (long) i;
+            }
+        }
+      else
+        {
+          for (size_t i = 0; i < numBins; ++i)
+            {
+              if (lat1 <= ybounds2[i * 2] && lat1 > ybounds2[i * 2 + 1]) return (long) i;
+            }
+        }
+    }
+
+  return -1;
+}
+
+static void
+check_vmask(const std::vector<char> &vmask)
+{
+  constexpr int maxVals = 128;
+  size_t vm[maxVals] = { 0 };
   auto size = vmask.size();
   for (size_t i = 0; i < size; ++i)
     if (vmask[i] >= 0) vm[(int) vmask[i]]++;
-  for (int i = 0; i < max_vals; ++i)
+  for (int i = 0; i < maxVals; ++i)
     if (vm[i]) cdo_print("Number of source values: %d --> %zu/%zu", i, vm[i], size);
   size_t sum = 0;
-  for (int i = 1; i < max_vals; ++i) sum += vm[i];
+  for (int i = 1; i < maxVals; ++i) sum += vm[i];
   cdo_print("Sum of used source values:     %zu/%zu", sum, size);
 }
 
@@ -407,16 +452,16 @@ gen_mapdata(int gridID1, int gridID2)
   read_coordinates(gridID2, xvals2, yvals2);
 
   auto gridtype2 = gridInqType(gridID2);
-  auto grid2_is_reg2d = (gridtype2 == GRID_GAUSSIAN || gridtype2 == GRID_LONLAT);
+  auto grid2IsReg2d = (gridtype2 == GRID_GAUSSIAN || gridtype2 == GRID_LONLAT);
 
-  auto nlon2 = grid2_is_reg2d ? gridInqXsize(gridID2) : 0;
+  auto nlon2 = grid2IsReg2d ? gridInqXsize(gridID2) : 0;
 
   int gridID2x = -1;
 
   Varray<double> xbounds, ybounds;
   std::vector<double> xbounds2d, ybounds2d;
   size_t nv = 4;
-  if (grid2_is_reg2d)
+  if (grid2IsReg2d)
     {
       read_xbounds_reg2d(gridID2, xbounds2d);
       read_ybounds_reg2d(gridID2, ybounds2d);
@@ -427,102 +472,136 @@ gen_mapdata(int gridID1, int gridID2)
       nv = read_target_cell_bounds(gridID2, xbounds, ybounds);
     }
 
-  // for (size_t i = 0; i < nlon2; ++i) printf("%zu %g %g\n", i+1, xbounds2d[2*i], xbounds2d[2*i+1]);
-  // for (size_t i = 0; i < nlat2; ++i) printf("%zu %g %g\n", i+1, ybounds2d[2*i], ybounds2d[2*i+1]);
+  auto isZonalGrid = (grid2IsReg2d ? is_zonal_grid(gridID2, xbounds2d, ybounds2d) : false);
 
-  cdo::timer timer;
-
-  auto xIsCyclic = false;
-  size_t dims[2] = { gridsize1, 0 };
-  GridPointSearch gps;
-  grid_point_search_create(gps, xIsCyclic, dims, gridsize1, xvals1, yvals1);
-
-  if (Options::cdoVerbose) cdo_print("Point search created: %.2f seconds", timer.elapsed());
-
-  timer.reset();
-
-  auto ndist_max = gridsize1;
-  if (gridsize1 > 1000000) ndist_max /= 4;
-  std::vector<char> vmask(gridsize1, 0);
-  Varray2D<size_t> adds_2D(Threading::ompNumThreads, Varray<size_t>(ndist_max));
-  Varray2D<double> dist_2D(Threading::ompNumThreads, Varray<double>(ndist_max));
-
-  StatInfo statInfo;
+  if (isZonalGrid)
+    {
+      Varray<size_t> zonalBinsNum(gridsize2, 0);
+      Varray<long> indexMap(gridsize1, -1);
 
 #ifdef _OPENMP
 #pragma omp parallel for default(shared) schedule(dynamic)
 #endif
-  for (size_t i = 0; i < gridsize2; ++i)
-    {
-      auto ompthID = cdo_omp_get_thread_num();
-      auto &adds = adds_2D[ompthID];
-      auto &dist = dist_2D[ompthID];
-
-      auto lon2 = xvals2[i];
-      auto lat2 = yvals2[i];
-
-      auto maxdist = grid2_is_reg2d ? calc_maxdist_rec2d(i, nlon2, lon2, lat2, xbounds2d, ybounds2d)
-                                    : calc_maxdist(i, nv, lon2, lat2, xbounds, ybounds);
-
-      auto nadds = grid_point_search_distance_qnearest(gps, maxdist, lon2, lat2, ndist_max, adds.data(), dist.data());
-      // printf("%zu nadds %zu\n", i+1, nadds);
-
-      auto nvalues = grid2_is_reg2d ? find_points_rec2d(vmask, i, nlon2, nadds, adds, xvals1, yvals1, xbounds2d, ybounds2d)
-#ifdef USE_YAC
-                                    : find_points_yac(vmask, i, nv, nadds, adds, xvals1, yvals1, xbounds, ybounds);
-#else
-                                    : find_points(vmask, i, nv, nadds, adds, xvals1, yvals1, xbounds, ybounds);
-#endif
-
-      if (nvalues)
+      for (size_t i = 0; i < gridsize1; ++i)
         {
-          mapdata[i].resize(nvalues);
-          for (size_t k = 0; k < nvalues; ++k) mapdata[i][k] = adds[k];
+          auto binNum = get_bin_num(xvals1[i], yvals1[i], gridsize2, xbounds2d, ybounds2d);
+          if (binNum >= 0)
+            {
+              indexMap[i] = binNum;
+#ifdef _OPENMP
+#pragma omp atomic
+#endif
+              zonalBinsNum[binNum]++;
+            }
         }
 
-      // if (Options::cdoVerbose) printf("%zu nadds %zu nvalues %zu  maxdist %g\n", i+1, nadds, nvalues, maxdist);
-      if (Options::cdoVerbose) statInfo.add(nadds, nvalues, maxdist);
+      for (size_t i = 0; i < gridsize2; ++i) mapdata[i].reserve(zonalBinsNum[i]);
+
+      for (size_t i = 0; i < gridsize1; ++i)
+        {
+          if (indexMap[i] >= 0) mapdata[indexMap[i]].push_back(i);
+        }
     }
+  else
+    {
+      cdo::timer timer;
 
-  if (Options::cdoVerbose) statInfo.print();
+      auto xIsCyclic = false;
+      size_t dims[2] = { gridsize1, 0 };
+      GridPointSearch gps;
+      grid_point_search_create(gps, xIsCyclic, dims, gridsize1, xvals1, yvals1);
 
-  if (Options::cdoVerbose) check_vmask(vmask);
+      if (Options::cdoVerbose) cdo_print("Point search created: %.2f seconds", timer.elapsed());
 
-  if (Options::cdoVerbose) cdo_print("Point search qnearest: %.2f seconds", timer.elapsed());
+      timer.reset();
 
-  grid_point_search_delete(gps);
+      auto ndistMax = gridsize1;
+      if (gridsize1 > 1000000) ndistMax /= 4;
+      std::vector<char> vmask(gridsize1, 0);
+      Varray2D<size_t> indices_2D(Threading::ompNumThreads, Varray<size_t>(ndistMax));
+      Varray2D<double> dist_2D(Threading::ompNumThreads, Varray<double>(ndistMax));
 
-  if (gridID2x != -1) gridDestroy(gridID2x);
+      StatInfo statInfo;
+
+#ifdef _OPENMP
+#pragma omp parallel for default(shared) schedule(dynamic)
+#endif
+      for (size_t i = 0; i < gridsize2; ++i)
+        {
+          auto ompthID = cdo_omp_get_thread_num();
+          auto &indices = indices_2D[ompthID];
+          auto &dist = dist_2D[ompthID];
+
+          auto lon2 = xvals2[i];
+          auto lat2 = yvals2[i];
+
+          auto maxdist = grid2IsReg2d ? calc_maxdist_rec2d(i, nlon2, lon2, lat2, xbounds2d, ybounds2d)
+                                      : calc_maxdist(i, nv, lon2, lat2, xbounds, ybounds);
+
+          if (maxdist < 2.0) { maxdist = 1.01 * std::sqrt(maxdist); }
+          else { cdo_abort("Search radius of target grid cell[%zu] > 90 degrees!", i + 1); }
+
+          auto numIndices = grid_point_search_distance_qnearest(gps, maxdist, lon2, lat2, ndistMax, indices, dist);
+          // printf("%zu numIndices %zu\n", i+1, numIndices);
+
+          auto nvalues = grid2IsReg2d
+                             ? find_points_rec2d(vmask, i, nlon2, numIndices, indices, xvals1, yvals1, xbounds2d, ybounds2d)
+#ifdef USE_YAC
+                             : find_points_yac(vmask, i, nv, numIndices, indices, xvals1, yvals1, xbounds, ybounds);
+#else
+                             : find_points(vmask, i, nv, numIndices, indices, xvals1, yvals1, xbounds, ybounds);
+#endif
+
+          if (nvalues)
+            {
+              mapdata[i].resize(nvalues);
+              for (size_t k = 0; k < nvalues; ++k) mapdata[i][k] = indices[k];
+            }
+
+          // if (Options::cdoVerbose) printf("%zu numIndices %zu nvalues %zu  maxdist %g\n", i+1, numIndices, nvalues, maxdist);
+          if (Options::cdoVerbose) statInfo.add(numIndices, nvalues, maxdist);
+        }
+
+      if (Options::cdoVerbose) statInfo.print();
+
+      if (Options::cdoVerbose) check_vmask(vmask);
+
+      if (Options::cdoVerbose) cdo_print("Point search qnearest: %.2f seconds", timer.elapsed());
+
+      grid_point_search_delete(gps);
+
+      if (gridID2x != -1) gridDestroy(gridID2x);
+    }
 
   return mapdata;
 }
 
 template <typename T>
 static T
-remap_kernel(int operfunc, const Varray<size_t> &adds, size_t &nmiss2, Field &field, Varray<T> &fieldvec, const Varray<T> &vec1,
-             T missval)
+remap_kernel(int operfunc, const Varray<size_t> &indices, size_t &numMissVals2, Field &field, Varray<T> &fieldvec,
+             const Varray<T> &vec1, T missval)
 {
   T value;
-  auto nvalues = adds.size();
+  auto nvalues = indices.size();
   if (nvalues)
     {
-      field.nmiss = 0;
+      field.numMissVals = 0;
       for (size_t k = 0; k < nvalues; ++k)
         {
-          auto v1 = vec1[adds[k]];
+          auto v1 = vec1[indices[k]];
           fieldvec[k] = v1;
-          if (dbl_is_equal(v1, missval)) field.nmiss++;
+          if (dbl_is_equal(v1, missval)) field.numMissVals++;
         }
 
       field.size = nvalues;
       field.missval = missval;
       value = field_function(field, operfunc);
-      if (dbl_is_equal(value, missval)) nmiss2++;
+      if (dbl_is_equal(value, missval)) numMissVals2++;
     }
   else
     {
       value = missval;
-      nmiss2++;
+      numMissVals2++;
     }
 
   return value;
@@ -536,14 +615,14 @@ remap_field(const Varray2D<size_t> &mapdata, const Field &field1, Field &field2,
   auto gridsize2 = gridInqSize(field2.grid);
   cdo::timer timer;
 
-  size_t nmiss2 = 0;
+  size_t numMissVals2 = 0;
 #ifdef _OPENMP
-#pragma omp parallel for default(shared) schedule(static) reduction(+ : nmiss2)
+#pragma omp parallel for default(shared) schedule(static) reduction(+ : numMissVals2)
 #endif
   for (size_t i = 0; i < gridsize2; ++i)
     {
-      const auto &adds = mapdata[i];
-      auto nvalues = adds.size();
+      const auto &indices = mapdata[i];
+      auto nvalues = indices.size();
       auto missval = field1.missval;
 
       auto ompthID = cdo_omp_get_thread_num();
@@ -556,9 +635,9 @@ remap_field(const Varray2D<size_t> &mapdata, const Field &field1, Field &field2,
 
       double rvalue = 0.0;
       if (field1.memType == MemType::Float)
-        rvalue = remap_kernel(operfunc, adds, nmiss2, field, field.vec_f, field1.vec_f, (float) missval);
+        rvalue = remap_kernel(operfunc, indices, numMissVals2, field, field.vec_f, field1.vec_f, (float) missval);
       else
-        rvalue = remap_kernel(operfunc, adds, nmiss2, field, field.vec_d, field1.vec_d, missval);
+        rvalue = remap_kernel(operfunc, indices, numMissVals2, field, field.vec_d, field1.vec_d, missval);
 
       if (field2.memType == MemType::Float)
         field2.vec_f[i] = rvalue;
@@ -566,53 +645,53 @@ remap_field(const Varray2D<size_t> &mapdata, const Field &field1, Field &field2,
         field2.vec_d[i] = rvalue;
     }
 
-  field2.nmiss = nmiss2;
+  field2.numMissVals = numMissVals2;
 
   if (Options::cdoVerbose) cdo_print("Remap: %.3f seconds", timer.elapsed());
 }
 
-static void
-add_operators(void)
+class Remapstat : public Process
 {
-  // clang-format off
-  cdo_operator_add("remaprange",  FieldFunc_Range,  0, nullptr);
-  cdo_operator_add("remapmin",    FieldFunc_Min,    0, nullptr);
-  cdo_operator_add("remapmax",    FieldFunc_Max,    0, nullptr);
-  cdo_operator_add("remapsum",    FieldFunc_Sum,    0, nullptr);
-  cdo_operator_add("remapmean",   FieldFunc_Mean,   0, nullptr);
-  cdo_operator_add("remapavg",    FieldFunc_Avg,    0, nullptr);
-  cdo_operator_add("remapstd",    FieldFunc_Std,    0, nullptr);
-  cdo_operator_add("remapstd1",   FieldFunc_Std1,   0, nullptr);
-  cdo_operator_add("remapvar",    FieldFunc_Var,    0, nullptr);
-  cdo_operator_add("remapvar1",   FieldFunc_Var1,   0, nullptr);
-  cdo_operator_add("remapskew",   FieldFunc_Skew,   0, nullptr);
-  cdo_operator_add("remapkurt",   FieldFunc_Kurt,   0, nullptr);
-  cdo_operator_add("remapmedian", FieldFunc_Median, 0, nullptr);
-  // clang-format on
-}
+public:
+  using Process::Process;
+  inline static CdoModule module = {
+    .name = "Remapstat",
+    .operators = { { "remaprange", FieldFunc_Range, 0, RemapstatHelp },
+                   { "remapmin", FieldFunc_Min, 0, RemapstatHelp },
+                   { "remapmax", FieldFunc_Max, 0, RemapstatHelp },
+                   { "remapsum", FieldFunc_Sum, 0, RemapstatHelp },
+                   { "remapmean", FieldFunc_Mean, 0, RemapstatHelp },
+                   { "remapavg", FieldFunc_Avg, 0, RemapstatHelp },
+                   { "remapstd", FieldFunc_Std, 0, RemapstatHelp },
+                   { "remapstd1", FieldFunc_Std1, 0, RemapstatHelp },
+                   { "remapvar", FieldFunc_Var, 0, RemapstatHelp },
+                   { "remapvar1", FieldFunc_Var1, 0, RemapstatHelp },
+                   { "remapskew", FieldFunc_Skew, 0, RemapstatHelp },
+                   { "remapkurt", FieldFunc_Kurt, 0, RemapstatHelp },
+                   { "remapmedian", FieldFunc_Median, 0, RemapstatHelp } },
+    .aliases = {},
+    .mode = EXPOSED,     // Module mode: 0:intern 1:extern
+    .number = CDI_REAL,  // Allowed number type
+    .constraints = { 1, 1, NoRestriction },
+  };
+  inline static RegisterEntry<Remapstat> registration = RegisterEntry<Remapstat>(module);
 
-class ModuleRemapstat
-{
   CdoStreamID streamID1;
   CdoStreamID streamID2;
 
   int taxisID1;
   int taxisID2;
 
-  Field field1, field2;
-  VarList varList1, varList2;
+  VarList varList1;
+  VarList varList2;
 
   Varray2D<size_t> mapdata;
   int operfunc;
 
 public:
   void
-  init(void *process)
+  init() override
   {
-    cdo_initialize(process);
-
-    add_operators();
-
     auto operatorID = cdo_operator_id();
     operfunc = cdo_operator_f1(operatorID);
 
@@ -644,7 +723,7 @@ public:
     taxisID2 = taxisDuplicate(taxisID1);
     vlistDefTaxis(vlistID2, taxisID2);
 
-    auto ngrids = vlistNgrids(vlistID1);
+    auto ngrids = vlistNumGrids(vlistID1);
     auto gridID1 = vlistGrid(vlistID1, 0);
     for (int index = 0; index < ngrids; ++index)
       {
@@ -665,12 +744,15 @@ public:
 
     mapdata = gen_mapdata(gridID1, gridID2);
 
-    varList_init(varList1, vlistID1);
-    varList_init(varList2, vlistID2);
+    varList1 = VarList(vlistID1);
+    varList2 = VarList(vlistID2);
   }
+
   void
-  run()
+  run() override
   {
+    Field field1, field2;
+
     int tsID = 0;
     while (true)
       {
@@ -684,10 +766,10 @@ public:
           {
             int varID, levelID;
             cdo_inq_record(streamID1, &varID, &levelID);
-            field1.init(varList1[varID]);
+            field1.init(varList1.vars[varID]);
             cdo_read_record(streamID1, field1);
 
-            field2.init(varList2[varID]);
+            field2.init(varList2.vars[varID]);
 
             remap_field(mapdata, field1, field2, operfunc);
 
@@ -697,22 +779,11 @@ public:
         tsID++;
       }
   }
+
   void
-  close()
+  close() override
   {
     cdo_stream_close(streamID2);
     cdo_stream_close(streamID1);
-
-    cdo_finish();
   }
 };
-
-void *
-Remapstat(void *process)
-{
-  ModuleRemapstat remapstat;
-  remapstat.init(process);
-  remapstat.run();
-  remapstat.close();
-  return nullptr;
-}
