@@ -15,15 +15,26 @@
 
 #include "cdo_options.h"
 #include "process_int.h"
-#include "cdo_vlist.h"
 #include "cdo_zaxis.h"
 
-class ModuleReplace
+class Replace : public Process
 {
+public:
+  using Process::Process;
+  inline static CdoModule module = {
+    .name = "Replace",
+    .operators = { { "replace", ReplaceHelp } },
+    .aliases = {},
+    .mode = EXPOSED,     // Module mode: 0:intern 1:extern
+    .number = CDI_REAL,  // Allowed number type
+    .constraints = { 2, 1, NoRestriction },
+  };
+  inline static RegisterEntry<Replace> registration = RegisterEntry<Replace>(module);
+
   static const int MaxVars = 1024;
   int nchvars = 0;
   int idx;
-  size_t nmiss = 0;
+  size_t numMissVals = 0;
 
   CdoStreamID streamID1;
   CdoStreamID streamID2;
@@ -36,18 +47,17 @@ class ModuleReplace
 
   int vars1[MaxVars], vars2[MaxVars];
   std::vector<std::vector<int>> varlevel;
-  std::vector<std::vector<size_t>> varnmiss2;
+  std::vector<std::vector<size_t>> varnumMissVals2;
 
   Varray<double> array;
-  VarList varList1, varList2;
+  VarList varList1;
+  VarList varList2;
   Varray2D<double> vardata2;
 
 public:
   void
-  init(void *process)
+  init() override
   {
-    cdo_initialize(process);
-
     operator_check_argc(0);
 
     streamID1 = cdo_open_read(0);
@@ -60,82 +70,77 @@ public:
 
     auto vlistID2 = cdo_stream_inq_vlist(streamID2);
 
-    varList_init(varList1, vlistID1);
-    varList_init(varList2, vlistID2);
+    varList1 = VarList(vlistID1);
+    varList2 = VarList(vlistID2);
 
     // compare all variables in vlistID2
 
-    auto nvars1 = vlistNvars(vlistID1);
-    auto nvars2 = vlistNvars(vlistID2);
+    auto numVars1 = varList1.numVars();
+    auto numVars2 = varList2.numVars();
 
-    for (int varID2 = 0; varID2 < nvars2; ++varID2)
+    for (int varID2 = 0; varID2 < numVars2; ++varID2)
       {
+        const auto &var2 = varList2.vars[varID2];
         int varID1;
-        for (varID1 = 0; varID1 < nvars1; ++varID1)
+        for (varID1 = 0; varID1 < numVars1; ++varID1)
           {
-            if (varList1[varID1].name == varList2[varID2].name) break;
+            if (varList1.vars[varID1].name == var2.name) break;
           }
 
-        if (varID1 < nvars1)
+        if (varID1 < numVars1)
           {
-            auto gridsize1 = varList1[varID1].gridsize;
-            auto nlevel1 = varList1[varID1].nlevels;
+            const auto &var1 = varList1.vars[varID1];
 
-            auto gridsize2 = varList2[varID2].gridsize;
-            auto nlevel2 = varList2[varID2].nlevels;
+            if (var1.gridsize != var2.gridsize) cdo_abort("Variables have different gridsize!");
+            if (var1.nlevels < var2.nlevels) cdo_abort("Variables have different number of levels!");
 
-            if (gridsize1 != gridsize2) cdo_abort("Variables have different gridsize!");
-
-            if (nlevel1 < nlevel2) cdo_abort("Variables have different number of levels!");
-
-            if (Options::cdoVerbose) cdo_print("Variable %s replaced.", varList1[varID1].name);
+            if (Options::cdoVerbose) cdo_print("Variable %s replaced.", var1.name);
 
             vars1[nchvars] = varID1;
             vars2[nchvars] = varID2;
             nchvars++;
             if (nchvars > MaxVars) cdo_abort("Internal problem - too many variables!");
           }
-        else { cdo_warning("Variable %s not found!", varList2[varID2].name); }
+        else { cdo_warning("Variable %s not found!", var2.name); }
       }
 
     if (nchvars)
       {
         vardata2.resize(nchvars);
-        varnmiss2.resize(nchvars);
+        varnumMissVals2.resize(nchvars);
         varlevel.resize(nchvars);
         for (idx = 0; idx < nchvars; idx++)
           {
-            auto varID1 = vars1[idx];
-            auto varID2 = vars2[idx];
-            auto nlevel1 = varList1[varID1].nlevels;
-            auto nlevel2 = varList2[varID2].nlevels;
-            auto gridsize = varList2[varID2].gridsize;
-            vardata2[idx].resize(nlevel2 * gridsize);
-            varnmiss2[idx].resize(nlevel2);
-            varlevel[idx].resize(nlevel1);
+            const auto &var1 = varList1.vars[vars1[idx]];
+            const auto &var2 = varList2.vars[vars2[idx]];
+
+            auto gridsize = var2.gridsize;
+            vardata2[idx].resize(var2.nlevels * gridsize);
+            varnumMissVals2[idx].resize(var2.nlevels);
+            varlevel[idx].resize(var1.nlevels);
             /*
-            for ( levelID = 0; levelID < nlevel1; levelID++ )
+            for ( levelID = 0; levelID < var1.nlevels; levelID++ )
               varlevel[idx][levelID] = levelID;
             */
-            if (nlevel2 <= nlevel1)
+            if (var2.nlevels <= var1.nlevels)
               {
-                Varray<double> level1(nlevel1), level2(nlevel2);
-                cdo_zaxis_inq_levels(vlistInqVarZaxis(vlistID1, varID1), level1.data());
-                cdo_zaxis_inq_levels(vlistInqVarZaxis(vlistID2, varID2), level2.data());
+                Varray<double> level1(var1.nlevels), level2(var2.nlevels);
+                cdo_zaxis_inq_levels(var1.zaxisID, level1.data());
+                cdo_zaxis_inq_levels(var2.zaxisID, level2.data());
 
-                for (int levelID = 0; levelID < nlevel1; ++levelID) varlevel[idx][levelID] = -1;
+                for (int levelID = 0; levelID < var1.nlevels; ++levelID) varlevel[idx][levelID] = -1;
 
-                for (int l2 = 0; l2 < nlevel2; ++l2)
+                for (int l2 = 0; l2 < var2.nlevels; ++l2)
                   {
                     int l1;
-                    for (l1 = 0; l1 < nlevel1; ++l1)
-                      if (IS_EQUAL(level2[l2], level1[l1]))
+                    for (l1 = 0; l1 < var1.nlevels; ++l1)
+                      if (is_equal(level2[l2], level1[l1]))
                         {
                           varlevel[idx][l1] = l2;
                           break;
                         }
 
-                    if (l1 == nlevel1) cdo_warning("Variable %s on level %g not found!", varList2[varID2].name, level2[l2]);
+                    if (l1 == var1.nlevels) cdo_warning("Variable %s on level %g not found!", var2.name, level2[l2]);
                   }
               }
           }
@@ -153,8 +158,9 @@ public:
 
     nts2 = vlistNtsteps(vlistID2);
   }
+
   void
-  run()
+  run() override
   {
     int tsID = 0;
     while (true)
@@ -177,9 +183,9 @@ public:
                 for (idx = 0; idx < nchvars; idx++)
                   if (vars2[idx] == varID)
                     {
-                      auto offset = varList2[varID].gridsize * levelID;
-                      cdo_read_record(streamID2, &vardata2[idx][offset], &nmiss);
-                      varnmiss2[idx][levelID] = nmiss;
+                      auto offset = varList2.vars[varID].gridsize * levelID;
+                      cdo_read_record(streamID2, &vardata2[idx][offset], &numMissVals);
+                      varnumMissVals2[idx][levelID] = numMissVals;
                       break;
                     }
               }
@@ -200,39 +206,28 @@ public:
                   auto levelID2 = varlevel[idx][levelID];
                   if (levelID2 != -1)
                     {
-                      auto offset = varList1[varID].gridsize * levelID2;
+                      auto offset = varList1.vars[varID].gridsize * levelID2;
                       parray = &vardata2[idx][offset];
-                      nmiss = varnmiss2[idx][levelID2];
+                      numMissVals = varnumMissVals2[idx][levelID2];
                       break;
                     }
                 }
 
-            if (idx == nchvars) cdo_read_record(streamID1, parray, &nmiss);
+            if (idx == nchvars) cdo_read_record(streamID1, parray, &numMissVals);
 
             cdo_def_record(streamID3, varID, levelID);
-            cdo_write_record(streamID3, parray, nmiss);
+            cdo_write_record(streamID3, parray, numMissVals);
           }
 
         tsID++;
       }
   }
+
   void
-  close()
+  close() override
   {
     cdo_stream_close(streamID3);
     cdo_stream_close(streamID2);
     cdo_stream_close(streamID1);
-
-    cdo_finish();
   }
 };
-
-void *
-Replace(void *process)
-{
-  ModuleReplace replace;
-  replace.init(process);
-  replace.run();
-  replace.close();
-  return nullptr;
-}
