@@ -5,8 +5,6 @@
 
 */
 
-#include <algorithm>  // sort
-
 #include <cdi.h>
 
 #include "cdo_options.h"
@@ -22,9 +20,8 @@ constexpr int MAX_CHILDS = 9;
 struct CellIndex
 {
   long ncells;
-  Varray<long> neighbor;  // neighbor cell index
-  Varray<long> parent;    // parent cell index
-  Varray<long> child;     // child cell index
+  Varray<long> parent;  // parent cell index
+  Varray<long> child;   // child cell index
   std::string filename;
 };
 
@@ -39,7 +36,7 @@ read_cellindex(const std::string &filename, CellIndex &cellindex)
 {
   auto streamID = stream_open_read_locked(filename.c_str());
   auto vlistID = streamInqVlist(streamID);
-  auto ngrids = vlistNgrids(vlistID);
+  auto ngrids = vlistNumGrids(vlistID);
   int gridID = -1;
   for (int index = 0; index < ngrids; ++index)
     {
@@ -95,11 +92,11 @@ read_cellindex(const std::string &filename, CellIndex &cellindex)
   for (int recID = 0; recID < nrecs; ++recID)
     {
       int varID, levelID;
-      size_t nmiss;
+      size_t numMissVals;
       streamInqRecord(streamID, &varID, &levelID);
       if (varID == pid /* || varID == nid || varID == cid */)
         {
-          streamReadRecord(streamID, data.data(), &nmiss);
+          streamReadRecord(streamID, data.data(), &numMissVals);
           // if (varID == pid)
           {
             if (Options::cdoVerbose) cdo_print("Read parent_cell_index");
@@ -122,7 +119,7 @@ read_grid(const std::string &filename)
 {
   auto streamID = stream_open_read_locked(filename.c_str());
   auto vlistID = streamInqVlist(streamID);
-  auto ngrids = vlistNgrids(vlistID);
+  auto ngrids = vlistNumGrids(vlistID);
   int gridID = -1;
   for (int index = 0; index < ngrids; ++index)
     {
@@ -184,12 +181,6 @@ struct SortInfo
   int p, i;
 };
 
-static bool
-cmpsinfo(const SortInfo &a, const SortInfo &b)
-{
-  return a.p < b.p;
-}
-
 static void
 compute_child_from_parent(CellIndex &cellindex1, CellIndex &cellindex2)
 {
@@ -210,7 +201,7 @@ compute_child_from_parent(CellIndex &cellindex1, CellIndex &cellindex2)
             sinfo[j].p = parent1[j];
             sinfo[j].i = idx1[j];
           }
-        std::sort(sinfo.begin(), sinfo.end(), cmpsinfo);
+        ranges::sort(sinfo, {}, &SortInfo::p);
         for (long j = 0; j < ncells1; ++j)
           {
             parent1[j] = sinfo[j].p;
@@ -242,7 +233,7 @@ read_coordinates(const std::string &filename, long n, Varray<double> &lon, Varra
 {
   auto streamID = streamOpenRead(filename.c_str());
   auto vlistID = streamInqVlist(streamID);
-  auto ngrids = vlistNgrids(vlistID);
+  auto ngrids = vlistNumGrids(vlistID);
   int gridID = -1;
   for (int index = 0; index < ngrids; ++index)
     {
@@ -263,11 +254,12 @@ read_coordinates(const std::string &filename, long n, Varray<double> &lon, Varra
 }
 
 static void
-read_coordinates(const std::string &filename, long n, Varray<double> &lon, Varray<double> &lat, int nv, Varray<double> &lon_bnds, Varray<double> &lat_bnds)
+read_coordinates(const std::string &filename, long n, Varray<double> &lon, Varray<double> &lat, int nv, Varray<double> &lon_bnds,
+                 Varray<double> &lat_bnds)
 {
   auto streamID = streamOpenRead(filename.c_str());
   auto vlistID = streamInqVlist(streamID);
-  auto ngrids = vlistNgrids(vlistID);
+  auto ngrids = vlistNumGrids(vlistID);
   int gridID = -1;
   for (int index = 0; index < ngrids; ++index)
     {
@@ -300,8 +292,8 @@ constexpr int MAX_SEARCH = 128;  // the triangles are distorted!
 
 static void
 compute_child_from_bounds(CellIndex &cellindex2, long ncells2, Varray<double> &grid_center_lon2, Varray<double> &grid_center_lat2,
-                          Varray<double> &grid_corner_lon2, Varray<double> &grid_corner_lat2, long ncells1, const Varray<double> &grid_center_lon1,
-                          const Varray<double> &grid_center_lat1)
+                          Varray<double> &grid_corner_lon2, Varray<double> &grid_corner_lat2, long ncells1,
+                          const Varray<double> &grid_center_lon1, const Varray<double> &grid_center_lat1)
 {
   if (Options::cdoVerbose) cdo_print("%s", __func__);
 
@@ -312,7 +304,7 @@ compute_child_from_bounds(CellIndex &cellindex2, long ncells2, Varray<double> &g
   GridPointSearch gps;
   grid_point_search_create(gps, xIsCyclic, dims, ncells1, grid_center_lon1, grid_center_lat1);
   knnWeightsType knnWeights(MAX_SEARCH);
-  size_t *nbr_addr = &knnWeights.m_addr[0];
+  size_t *nbrIndices = &knnWeights.m_indices[0];
 
   int ncorner = 3;
   Point3D centerPoint3D;
@@ -347,7 +339,7 @@ compute_child_from_bounds(CellIndex &cellindex2, long ncells2, Varray<double> &g
       double centerCoordinates[3];
       for (int i = 0; i < MAX_SEARCH; ++i)
         {
-          auto cellNo1 = nbr_addr[i];
+          auto cellNo1 = nbrIndices[i];
           if (cellNo1 < SIZE_MAX)
             {
               gcLLtoXYZ(grid_center_lon1[cellNo1], grid_center_lat1[cellNo1], centerCoordinates);
@@ -466,8 +458,20 @@ samplegrid(double missval, long nci, Varray<CellIndex> &cellindex, const Varray<
     }
 }
 
-class ModuleSamplegridicon
+class Samplegridicon : public Process
 {
+public:
+  using Process::Process;
+  inline static CdoModule module = {
+    .name = "Samplegridicon",
+    .operators = { { "samplegridicon", 0, 0, "samplegrids" } },
+    .aliases = {},
+    .mode = EXPOSED,     // Module mode: 0:intern 1:extern
+    .number = CDI_REAL,  // Allowed number type
+    .constraints = { 1, 2, OnlyFirst },
+  };
+  inline static RegisterEntry<Samplegridicon> registration = RegisterEntry<Samplegridicon>(module);
+
   CdoStreamID streamID1;
   CdoStreamID streamID2;
   CdoStreamID streamID3;
@@ -491,12 +495,8 @@ class ModuleSamplegridicon
 
 public:
   void
-  init(void *process)
+  init() override
   {
-    cdo_initialize(process);
-
-    cdo_operator_add("samplegridicon", 0, 0, "sample grids");
-
     nsamplegrids = cdo_operator_argc();
     if (nsamplegrids < 2) cdo_abort("Parameter missing!");
 
@@ -532,7 +532,7 @@ public:
     vlistDefTaxis(vlistID2, taxisID2);
     vlistDefTaxis(vlistID3, taxisID3);
 
-    auto ngrids = vlistNgrids(vlistID1);
+    auto ngrids = vlistNumGrids(vlistID1);
     for (int index = 0; index < ngrids; ++index)
       {
         auto gridID = vlistGrid(vlistID1, index);
@@ -558,7 +558,7 @@ public:
   }
 
   void
-  run()
+  run() override
   {
     int tsID = 0;
     while (true)
@@ -575,20 +575,20 @@ public:
           {
             int varID, levelID;
             cdo_inq_record(streamID1, &varID, &levelID);
-            size_t nmiss;
-            cdo_read_record(streamID1, array1.data(), &nmiss);
+            size_t numMissVals;
+            cdo_read_record(streamID1, array1.data(), &numMissVals);
 
             auto missval = vlistInqVarMissval(vlistID1, varID);
 
             samplegrid(missval, nsamplegrids, cellindex, array1, array2, array3);
 
-            nmiss = varray_num_mv(gridsize2, array2, missval);
+            numMissVals = varray_num_mv(gridsize2, array2, missval);
             cdo_def_record(streamID2, varID, levelID);
-            cdo_write_record(streamID2, array2.data(), nmiss);
+            cdo_write_record(streamID2, array2.data(), numMissVals);
 
-            nmiss = varray_num_mv(gridsize2, array3, missval);
+            numMissVals = varray_num_mv(gridsize2, array3, missval);
             cdo_def_record(streamID3, varID, levelID);
-            cdo_write_record(streamID3, array3.data(), nmiss);
+            cdo_write_record(streamID3, array3.data(), numMissVals);
           }
 
         tsID++;
@@ -596,26 +596,11 @@ public:
   }
 
   void
-  close()
+  close() override
   {
     cdo_stream_close(streamID3);
     cdo_stream_close(streamID2);
     cdo_stream_close(streamID1);
-
     vlistDestroy(vlistID2);
-    gridDestroy(gridID2);
-
-    cdo_finish();
   }
 };
-
-void *
-Samplegridicon(void *process)
-{
-  ModuleSamplegridicon samplegridicon;
-  samplegridicon.init(process);
-  samplegridicon.run();
-  samplegridicon.close();
-
-  return nullptr;
-}
