@@ -5,11 +5,12 @@
 
 */
 
+#include <fstream>
+
 #include <cdi.h>
 
 #include "process_int.h"
 #include "cdo_zaxis.h"
-#include "readline.h"
 #include "cdi_lockedIO.h"
 
 // NOTE: All operators in this module works only on GRIB edition 1 files!
@@ -149,8 +150,23 @@ int getNumberOfDeleteSelectionTuples();
 
 int multiSelectionParser(const char *filenameOrString);
 
-class ModuleSelmulti
+class Selmulti : public Process
 {
+public:
+  using Process::Process;
+  inline static CdoModule module = {
+    .name = "Selmulti",
+    .operators = { { "selmulti", 0, 0, "filename/string with selection specification", SelmultiHelp },
+                   { "delmulti", 0, 0, "filename/string with selection specification", SelmultiHelp },
+                   { "changemulti", 0, 0, "filename/string with selection specification", SelmultiHelp } },
+    .aliases = {},
+    .mode = EXPOSED,     // Module mode: 0:intern 1:extern
+    .number = CDI_REAL,  // Allowed number type
+    .constraints = { 1, 1, NoRestriction },
+  };
+  inline static RegisterEntry<Selmulti> registration = RegisterEntry<Selmulti>(module);
+
+  int SELMULTI, DELMULTI, CHANGEMULTI;
   CdoStreamID streamID1;
   CdoStreamID streamID2;
 
@@ -161,7 +177,7 @@ class ModuleSelmulti
   int vlistID2;
 
   int operatorID;
-  int nvars;
+  int numVars;
   size_t gridsizemax;
 
   bool lcopy = false;
@@ -169,20 +185,13 @@ class ModuleSelmulti
   VarList varList1;
   Varray<double> array;
 
-  int SELMULTI, DELMULTI, CHANGEMULTI;
-
 public:
   void
-  init(void *process)
+  init() override
   {
-
-    cdo_initialize(process);
-
-    // clang-format off
-   SELMULTI    = cdo_operator_add("selmulti",    0, 0, "filename/string with selection specification");
-   DELMULTI    = cdo_operator_add("delmulti",    0, 0, "filename/string with selection specification");
-   CHANGEMULTI = cdo_operator_add("changemulti", 0, 0, "filename/string with selection specification");
-    // clang-format on
+    SELMULTI = module.get_id("selmulti");
+    DELMULTI = module.get_id("delmulti");
+    CHANGEMULTI = module.get_id("changemulti");
 
     operatorID = cdo_operator_id();
 
@@ -221,17 +230,17 @@ public:
 
     vlistID1 = cdo_stream_inq_vlist(streamID1);
 
-    varList_init(varList1, vlistID1);
+    varList1 = VarList(vlistID1);
 
     vlistClearFlag(vlistID1);
-    nvars = vlistNvars(vlistID1);
+    numVars = varList1.numVars();
 
-    Debug(cdoDebugExt, " Total number of variables: %d", nvars);
+    Debug(cdoDebugExt, " Total number of variables: %d", numVars);
 
-    for (int varID = 0; varID < nvars; ++varID)
+    for (int varID = 0; varID < numVars; ++varID)
       {
-        const auto &var = varList1[varID];
-        const auto ltype = zaxis_to_ltype(var.zaxisID);
+        const auto &var = varList1.vars[varID];
+        auto ltype = zaxis_to_ltype(var.zaxisID);
 
         for (int levelID = 0; levelID < var.nlevels; ++levelID)
           {
@@ -253,9 +262,9 @@ public:
                 // (int)level, nvars, varID);
                 // Note: When the list is Empty then function
                 // checkListContainsInt() also returns true !
-                const auto selcode = checkListContainsInt(var.code, tuplerec->codeLST, tuplerec->ncodes);
-                const auto selltype = checkListContainsInt(ltype, tuplerec->levelTypeLST, tuplerec->nlevelTypes);
-                const auto sellevel = checkListContainsInt((int) level, tuplerec->levelLST, tuplerec->nlevels);
+                auto selcode = checkListContainsInt(var.code, tuplerec->codeLST, tuplerec->ncodes);
+                auto selltype = checkListContainsInt(ltype, tuplerec->levelTypeLST, tuplerec->nlevelTypes);
+                auto sellevel = checkListContainsInt((int) level, tuplerec->levelLST, tuplerec->nlevels);
                 if (selcode && selltype && sellevel)
                   {
                     if (operatorID == SELMULTI)
@@ -361,11 +370,11 @@ public:
     vlistDefNtsteps(vlistID2, vlistNtsteps(vlistID1));
 
     {
-      nvars = vlistNvars(vlistID2);
+      numVars = vlistNvars(vlistID2);
       int varID;
-      for (varID = 0; varID < nvars; ++varID)
+      for (varID = 0; varID < numVars; ++varID)
         if (vlistInqVarTimetype(vlistID2, varID) != TIME_CONSTANT) break;
-      if (varID == nvars) vlistDefNtsteps(vlistID2, 0);
+      if (varID == numVars) vlistDefNtsteps(vlistID2, 0);
     }
 
     taxisID1 = vlistInqTaxis(vlistID1);
@@ -379,13 +388,14 @@ public:
     if (vlistNumber(vlistID1) != CDI_REAL) gridsizemax *= 2;
     array = Varray<double>(gridsizemax);
   }
+
   void
-  run()
+  run() override
   {
     int tsID = 0;
     while (true)
       {
-        const auto nrecs = cdo_stream_inq_timestep(streamID1, tsID);
+        auto nrecs = cdo_stream_inq_timestep(streamID1, tsID);
         if (nrecs == 0) break;
 
         cdo_taxis_copy_timestep(taxisID2, taxisID1);
@@ -395,24 +405,24 @@ public:
           {
             int varID, levelID;
             cdo_inq_record(streamID1, &varID, &levelID);
-            const auto &var = varList1[varID];
+            const auto &var = varList1.vars[varID];
 
             if (vlistInqFlag(vlistID1, varID, levelID) == true)
               {
                 int simpleMath = 0;  // 1:  simple array arithmetics ( *,+), 0: do nothing
                 double scale = 1.0;
                 double offset = 0.0;
-                const auto code = var.code;
-                const auto level = zaxisInqLevel(var.zaxisID, levelID);
-                const auto ltype = zaxis_to_ltype(var.zaxisID);
+                auto code = var.code;
+                auto level = zaxisInqLevel(var.zaxisID, levelID);
+                auto ltype = zaxis_to_ltype(var.zaxisID);
                 for (int ii = 0; ii < NUMTUPLES; ++ii)
                   {
                     TUPLEREC *tuplerec = getSelTuple(ii);
                     // Note: When the list is Empty then function
                     // checkListContainsInt() also returns true !
-                    const auto selcode = checkListContainsInt(code, tuplerec->codeLST, tuplerec->ncodes);
-                    const auto selltype = checkListContainsInt(ltype, tuplerec->levelTypeLST, tuplerec->nlevelTypes);
-                    const auto sellevel = checkListContainsInt((int) level, tuplerec->levelLST, tuplerec->nlevels);
+                    auto selcode = checkListContainsInt(code, tuplerec->codeLST, tuplerec->ncodes);
+                    auto selltype = checkListContainsInt(ltype, tuplerec->levelTypeLST, tuplerec->nlevelTypes);
+                    auto sellevel = checkListContainsInt((int) level, tuplerec->levelLST, tuplerec->nlevels);
                     lcopy = true;
                     if (selcode && selltype && sellevel)
                       {
@@ -421,8 +431,8 @@ public:
                             if (cdoDebugExt)
                               cdo_print(" Processing: (code %d, ltype %d, level %d);  nvars=%d, varID=%d => (selcode %d, selltype "
                                         "%d, sellevel %d) => change (%d,%d,%d)",
-                                        code, ltype, (int) level, nvars, varID, selcode, selltype, sellevel, tuplerec->changedCode,
-                                        tuplerec->changedLevelType, tuplerec->changedLevel);
+                                        code, ltype, (int) level, numVars, varID, selcode, selltype, sellevel,
+                                        tuplerec->changedCode, tuplerec->changedLevelType, tuplerec->changedLevel);
                             if ((tuplerec->changedCode == -1) && (tuplerec->changedLevelType == -1)
                                 && (tuplerec->changedLevel == -1))
                               cdo_print(" WARNING: Cannot CHANGE identification!");
@@ -441,7 +451,7 @@ public:
                             if (cdoDebugExt)
                               cdo_print(" Processing: (code %d, ltype %d, level %d);  nvars=%d, varID=%d => (selcode %d, "
                                         "selltype %d, sellevel %d)",
-                                        code, ltype, (int) level, nvars, varID, selcode, selltype, sellevel);
+                                        code, ltype, (int) level, numVars, varID, selcode, selltype, sellevel);
                           }
                         simpleMath = tuplerec->simpleMath;  // 1:  simple array arithmetics ( *,+),
                                                             // 0: do nothing
@@ -455,8 +465,8 @@ public:
                       }
                   }  // end of for (ii=0; ii<NUMTUPLES ..
 
-                const auto varID2 = vlistFindVar(vlistID2, varID);
-                const auto levelID2 = vlistFindLevel(vlistID2, varID, levelID);
+                auto varID2 = vlistFindVar(vlistID2, varID);
+                auto levelID2 = vlistFindLevel(vlistID2, varID, levelID);
 
                 // tijdelijk PATCH M.K.
                 if ((varID2 == -1) || (levelID2 == -1))
@@ -477,8 +487,8 @@ public:
                   }
                 else
                   {
-                    size_t nmiss;
-                    cdo_read_record(streamID1, array.data(), &nmiss);
+                    size_t numMissVals;
+                    cdo_read_record(streamID1, array.data(), &numMissVals);
 
                     if (!simpleMath)
                       {
@@ -501,7 +511,7 @@ public:
                             }
                       }
 
-                    cdo_write_record(streamID2, array.data(), nmiss);
+                    cdo_write_record(streamID2, array.data(), numMissVals);
                   }  // end of else ( lcopy )
               }      // end if ( vlistInqFlag(vlistID1, varID, levelID) == true )
           }          // end for ( recID ..
@@ -509,30 +519,19 @@ public:
         tsID++;
       }  // end while
   }
+
   void
-  close()
+  close() override
   {
     cdo_stream_close(streamID1);
     cdo_stream_close(streamID2);
 
     vlistDestroy(vlistID2);
 
-    cdo_finish();
-
     cdoDebugExt = 0;
   }
 };
 
-void *
-Selmulti(void *process)
-{
-  ModuleSelmulti selmulti;
-  selmulti.init(process);
-  selmulti.run();
-  selmulti.close();
-  return nullptr;
-}
-
 TUPLEREC *
 TUPLERECNew()
 {
@@ -608,7 +607,7 @@ static char *
 removeSpaces(char *pline)
 {
   if (pline == nullptr) return nullptr;
-  while (isspace((int) *pline)) pline++;
+  while (std::isspace((int) *pline)) pline++;
   return pline;
 }
 
@@ -616,9 +615,9 @@ static char *
 skipSeparator(char *pline)
 {
   if (pline == nullptr) return nullptr;
-  while (isspace((int) *pline)) pline++;
+  while (std::isspace((int) *pline)) pline++;
   if (*pline == '=' || *pline == ':' || *pline == '/' || *pline == ',') pline++;
-  while (isspace((int) *pline)) pline++;
+  while (std::isspace((int) *pline)) pline++;
   return pline;
 }
 
@@ -627,7 +626,7 @@ goToNextSeparator(char *pline)
 {
   if (pline == nullptr) return nullptr;
   int separatorFound = 0;
-  while (isspace((int) *pline) || !separatorFound)
+  while (std::isspace((int) *pline) || !separatorFound)
     {
       if (*pline == '\0') return nullptr;
       pline++;
@@ -643,7 +642,7 @@ goToNextSeparator(char *pline)
     }
   if (separatorFound) pline++;
   Debug(cdoDebugExt >= 100, "goToNextSeparator():  pline= ('%s') ", pline);
-  // while ( isspace((int) *pline) ) pline++;
+  // while ( std::isspace((int) *pline) ) pline++;
   pline = removeSpaces(pline);
   return pline;
 }
@@ -725,12 +724,17 @@ findTupleEnd(char *str)
 }
 
 static char *
-readlineForParsing(FILE *gfp, char *strToParsePtr, char *line)
+readlineForParsing(std::ifstream &gfile, char *strToParsePtr, char *line)
 {
-  if (gfp != nullptr)  // file is open => we parse text from a file
+  if (gfile.is_open())  // file is open => we parse text from a file
     {
-      int status = cdo::readline(gfp, line, MAX_LINE_LEN);
-      return (status == 0) ? nullptr : line;
+      std::string strLine;
+      if (std::getline(gfile, strLine))
+        {
+          strcpy(line, strLine.c_str());
+          return line;
+        }
+      else { return nullptr; }
     }
   else if (strToParsePtr != nullptr)  // we parse a given string
     {
@@ -761,14 +765,13 @@ readlineForParsing(FILE *gfp, char *strToParsePtr, char *line)
 int
 multiSelectionParser(const char *filenameOrString)
 {
-  char line[MAX_LINE_LEN], *pline;
+  std::ifstream gfile;
   char *strpos;
   char *parEnd;
   int val;
   float floatval;
   TUPLEREC *tuplerec;
   int selectionRec;
-  FILE *gfp = nullptr;
   char strToParse[MAX_LINE_LEN];
   char *strToParsePtr = nullptr;
   strToParse[0] = 0;
@@ -793,22 +796,23 @@ multiSelectionParser(const char *filenameOrString)
     }
   else
     {
-      gfp = std::fopen(filenameOrString, "r");
+      gfile.open(filenameOrString);
       Debug(cdoDebugExt, " Parsing file:  %s", filenameOrString);
-      if (gfp == nullptr)
+      if (!gfile.is_open())
         {
           cdo_abort(" Missing file:  %s", filenameOrString);
           return 0;
         }
     }
 
-  while ((strToParsePtr = readlineForParsing(gfp, strToParsePtr, line)))
+  char line[MAX_LINE_LEN];
+  while ((strToParsePtr = readlineForParsing(gfile, strToParsePtr, line)))
     {
       if (line[0] == '#') continue;
       if (line[0] == '\0') continue;
-      pline = line;
+      char *pline = line;
       if (cdoDebugExt >= 30) cdo_print(": Line: %s", pline);
-      while (isspace((int) *pline)) pline++;
+      while (std::isspace((int) *pline)) pline++;
       if (pline[0] == '\0') continue;
       strpos = strContains(pline, "SELECT, ");
       selectionRec = 0;  // default is 0;  sel_or_del_or_change:  0:  operator
@@ -1166,8 +1170,9 @@ multiSelectionParser(const char *filenameOrString)
 
             }  // end while pline
         }      // end if "("
-    }          // end while ( cdo::readline(gfp, line, MAX_LINE_LEN) )
-  if (gfp != nullptr) std::fclose(gfp);
+    }          // end while ( readlineForParsing() )
+
+  if (gfile.is_open()) gfile.close();
 
   printSelectionTuples();
   return 1;
@@ -1191,7 +1196,7 @@ printSelectionTuples()
                   tuplerec->nlevels);
       for (ri = 0; ri < tuplerec->ncodes; ri++)
         {
-          sprintf(bff, "%d", tuplerec->codeLST[ri]);
+          std::snprintf(bff, sizeof(bff), "%d", tuplerec->codeLST[ri]);
           strcat(strval, bff);
           if ((ri + 1) < tuplerec->ncodes)
             strcat(strval, "/");
@@ -1200,7 +1205,7 @@ printSelectionTuples()
         }
       for (ri = 0; ri < tuplerec->nlevelTypes; ri++)
         {
-          sprintf(bff, "%d", tuplerec->levelTypeLST[ri]);
+          std::snprintf(bff, sizeof(bff), "%d", tuplerec->levelTypeLST[ri]);
           strcat(strval, bff);
           if ((ri + 1) < tuplerec->nlevelTypes)
             strcat(strval, "/");
@@ -1209,7 +1214,7 @@ printSelectionTuples()
         }
       for (ri = 0; ri < tuplerec->nlevels; ri++)
         {
-          sprintf(bff, "%d", tuplerec->levelLST[ri]);
+          std::snprintf(bff, sizeof(bff), "%d", tuplerec->levelLST[ri]);
           strcat(strval, bff);
           if ((ri + 1) < tuplerec->nlevels)
             strcat(strval, "/");
@@ -1219,7 +1224,7 @@ printSelectionTuples()
 
       if (tuplerec->simpleMath)
         {
-          sprintf(bff, " {scale = %f; offset = %f}", tuplerec->scale, tuplerec->offset);
+          std::snprintf(bff, sizeof(bff), " {scale = %f; offset = %f}", tuplerec->scale, tuplerec->offset);
           strcat(strval, bff);
         }
 
