@@ -16,7 +16,7 @@
 
 template <typename T>
 static void
-isosurface_kernel(double isoval, size_t nmiss, const Varray<double> &levels, int nlevels, size_t gridsize, T missval,
+isosurface_kernel(double isoval, size_t numMissVals, const Varray<double> &levels, int nlevels, size_t gridsize, T missval,
                   const Varray<const T *> &data3D, Varray<T> &data2D)
 {
 #ifdef _OPENMP
@@ -31,7 +31,7 @@ isosurface_kernel(double isoval, size_t nmiss, const Varray<double> &levels, int
           const double val1 = data3D[k][i];
           const double val2 = data3D[k + 1][i];
 
-          if (nmiss)
+          if (numMissVals)
             {
               auto hasMissvals1 = dbl_is_equal(val1, missval);
               auto hasMissvals2 = dbl_is_equal(val2, missval);
@@ -56,20 +56,20 @@ isosurface(double isoval, int nlevels, const Varray<double> &levels, const Field
   auto gridsize = gridInqSize(field3D[0].grid);
   auto missval = field3D[0].missval;
 
-  auto nmiss = field3D[0].nmiss;
-  for (int k = 1; k < nlevels; ++k) nmiss += field3D[k].nmiss;
+  auto numMissVals = field3D[0].numMissVals;
+  for (int k = 1; k < nlevels; ++k) numMissVals += field3D[k].numMissVals;
 
   if (field3D[0].memType == MemType::Float)
     {
       Varray<const float *> data3D(nlevels);
       for (int k = 0; k < nlevels; ++k) data3D[k] = field3D[k].vec_f.data();
-      isosurface_kernel(isoval, nmiss, levels, nlevels, gridsize, (float) missval, data3D, field2D.vec_f);
+      isosurface_kernel(isoval, numMissVals, levels, nlevels, gridsize, (float) missval, data3D, field2D.vec_f);
     }
   else
     {
       Varray<const double *> data3D(nlevels);
       for (int k = 0; k < nlevels; ++k) data3D[k] = field3D[k].vec_d.data();
-      isosurface_kernel(isoval, nmiss, levels, nlevels, gridsize, missval, data3D, field2D.vec_d);
+      isosurface_kernel(isoval, numMissVals, levels, nlevels, gridsize, missval, data3D, field2D.vec_d);
     }
 
   field_num_mv(field2D);
@@ -89,7 +89,7 @@ layer_value_min_kernel(int nlevels, size_t gridsize, T missval, const Varray<con
       for (int k = 0; k < nlevels; ++k)
         {
           auto val = data3D[k][i];
-          if (!dbl_is_equal(val, missval))
+          if (dbl_is_not_equal(val, missval))
             {
               data2D[i] = val;
               break;
@@ -134,7 +134,7 @@ layer_value_max_kernel(int nlevels, size_t gridsize, T missval, const Varray<con
       for (int k = nlevels - 1; k >= 0; --k)
         {
           auto val = data3D[k][i];
-          if (!dbl_is_equal(val, missval))
+          if (dbl_is_not_equal(val, missval))
             {
               data2D[i] = val;
               break;
@@ -165,8 +165,24 @@ layer_value_max(int nlevels, const FieldVector &field3D, Field &field2D)
   field_num_mv(field2D);
 }
 
-class ModuleSelsurface
+class Selsurface : public Process
 {
+public:
+  using Process::Process;
+  inline static CdoModule module = {
+    .name = "Selsurface",
+    // clang-format off
+    .operators = { { "isosurface", SelsurfaceHelp },
+                   { "bottomvalue", SelsurfaceHelp },
+                   { "topvalue", SelsurfaceHelp } },
+    // clang-format on
+    .aliases = {},
+    .mode = EXPOSED,     // Module mode: 0:intern 1:extern
+    .number = CDI_REAL,  // Allowed number type
+    .constraints = { 1, 1, NoRestriction },
+  };
+  inline static RegisterEntry<Selsurface> registration = RegisterEntry<Selsurface>(module);
+  int ISOSURFACE, BOTTOMVALUE, TOPVALUE;
   CdoStreamID streamID1;
   CdoStreamID streamID2;
 
@@ -186,7 +202,7 @@ class ModuleSelsurface
 
   VarList varList1;
   Field field2;
-  FieldVector2D vars1;
+  FieldVector2D varsData1;
 
   int nlevels;
   Varray<double> levels;
@@ -194,19 +210,13 @@ class ModuleSelsurface
   bool isPositive;
   bool isReverse;
 
-  int ISOSURFACE, BOTTOMVALUE, TOPVALUE;
-
 public:
   void
-  init(void *process)
+  init() override
   {
-    cdo_initialize(process);
-
-    // clang-format off
-   ISOSURFACE  = cdo_operator_add("isosurface",  0,  0, nullptr);
-   BOTTOMVALUE = cdo_operator_add("bottomvalue", 0,  0, nullptr);
-   TOPVALUE    = cdo_operator_add("topvalue",    0,  0, nullptr);
-    // clang-format on
+    ISOSURFACE = module.get_id("isosurface");
+    BOTTOMVALUE = module.get_id("bottomvalue");
+    TOPVALUE = module.get_id("topvalue");
 
     operatorID = cdo_operator_id();
 
@@ -228,7 +238,7 @@ public:
     taxisID2 = taxisDuplicate(taxisID1);
     vlistDefTaxis(vlistID2, taxisID2);
 
-    auto nzaxis = vlistNzaxis(vlistID1);
+    auto nzaxis = vlistNumZaxis(vlistID1);
     for (int i = 0; i < nzaxis; ++i)
       {
         auto zaxisID = vlistZaxis(vlistID1, i);
@@ -256,32 +266,31 @@ public:
     streamID2 = cdo_open_write(1);
     cdo_def_vlist(streamID2, vlistID2);
 
-    varList_init(varList1, vlistID1);
+    varList1 = VarList(vlistID1);
 
     nvars = vlistNvars(vlistID1);
     isVar3D = std::vector<bool>(nvars);
     foundVar = std::vector<bool>(nvars);
 
-    fields_from_vlist(vlistID1, vars1);
+    field2D_init(varsData1, varList1);
 
     for (int varID = 0; varID < nvars; ++varID)
       {
-        const auto &var = varList1[varID];
+        const auto &var = varList1.vars[varID];
         isVar3D[varID] = (var.zaxisID == zaxisID1);
       }
   }
 
   void
-  run()
+  run() override
   {
-
     auto bottom_value_func = isReverse ? layer_value_max : layer_value_min;
     auto top_value_func = isReverse ? layer_value_min : layer_value_max;
     if (isPositive && positive_is_down(zaxisID1)) std::swap(bottom_value_func, top_value_func);
+
     int tsID = 0;
     while (true)
       {
-
         auto nrecs = cdo_stream_inq_timestep(streamID1, tsID);
         if (nrecs == 0) break;
 
@@ -294,8 +303,8 @@ public:
           {
             int varID, levelID;
             cdo_inq_record(streamID1, &varID, &levelID);
-            const auto &var = varList1[varID];
-            auto &field1 = vars1[varID][levelID];
+            const auto &var = varList1.vars[varID];
+            auto &field1 = varsData1[varID][levelID];
             field1.init(var);
             cdo_read_record(streamID1, field1);
             foundVar[varID] = true;
@@ -305,14 +314,14 @@ public:
           {
             if (foundVar[varID])
               {
-                const auto &var = varList1[varID];
+                const auto &var = varList1.vars[varID];
                 if (isVar3D[varID])
                   {
                     field2.init(var);
                     // clang-format off
-                  if      (operatorID == ISOSURFACE)  isosurface(isoval, nlevels, levels, vars1[varID], field2);
-                  else if (operatorID == BOTTOMVALUE) bottom_value_func(nlevels, vars1[varID], field2);
-                  else if (operatorID == TOPVALUE)    top_value_func(nlevels, vars1[varID], field2);
+                    if      (operatorID == ISOSURFACE)  isosurface(isoval, nlevels, levels, varsData1[varID], field2);
+                    else if (operatorID == BOTTOMVALUE) bottom_value_func(nlevels, varsData1[varID], field2);
+                    else if (operatorID == TOPVALUE)    top_value_func(nlevels, varsData1[varID], field2);
                     // clang-format on
 
                     cdo_def_record(streamID2, varID, 0);
@@ -323,7 +332,7 @@ public:
                     for (int levelID = 0; levelID < var.nlevels; ++levelID)
                       {
                         cdo_def_record(streamID2, varID, levelID);
-                        cdo_write_record(streamID2, vars1[varID][levelID]);
+                        cdo_write_record(streamID2, varsData1[varID][levelID]);
                       }
                   }
               }
@@ -332,24 +341,13 @@ public:
         tsID++;
       }
   }
-  void
-  close()
-  {
 
+  void
+  close() override
+  {
     cdo_stream_close(streamID2);
     cdo_stream_close(streamID1);
 
     vlistDestroy(vlistID2);
-
-    cdo_finish();
   }
 };
-void *
-Selsurface(void *process)
-{
-  ModuleSelsurface selsurface;
-  selsurface.init(process);
-  selsurface.run();
-  selsurface.close();
-  return nullptr;
-}
