@@ -68,12 +68,12 @@ find_variables(const std::string &varName, int vlistID, std::vector<std::string>
 
   constexpr int Undefined = -99;
   cdiID = Undefined;
-  auto nvars = vlistNvars(vlistID);
-  // auto ngrids = vlistNgrids(vlistID);
-  auto nzaxis = vlistNzaxis(vlistID);
+  auto numVars = vlistNvars(vlistID);
+  // auto ngrids = vlistNumGrids(vlistID);
+  auto nzaxis = vlistNumZaxis(vlistID);
   if (varName.size())
     {
-      for (int idx = 0; idx < nvars; idx++)
+      for (int idx = 0; idx < numVars; idx++)
         {
           auto name = cdo::inq_var_name(vlistID, idx);
           if (wildcardmatch(varName, name) == 0)
@@ -185,30 +185,23 @@ find_attribute(int cdiID, int varID, const std::string &attrName, int &dtype)
 
   if (varID != CDI_GLOBAL && attrValues.empty())
     {
+      VarList varList(cdiID);
+      const auto &var = varList.vars[varID];
       auto stdname = cdo::inq_key_string(cdiID, varID, CDI_KEY_STDNAME);
-      auto longname = cdo::inq_var_longname(cdiID, varID);
-      auto units = cdo::inq_var_units(cdiID, varID);
-
-      auto param = vlistInqVarParam(cdiID, varID);
-      char paramstr[32];
-      param_to_string(param, paramstr, sizeof(paramstr));
-
-      auto code = vlistInqVarCode(cdiID, varID);
       auto table = vlistInqVarTable(cdiID, varID);
-      auto missval = vlistInqVarMissval(cdiID, varID);
 
       double addoffset = 0.0, scalefactor = 1.0;
       cdiInqKeyFloat(cdiID, varID, CDI_KEY_ADDOFFSET, &addoffset);
       cdiInqKeyFloat(cdiID, varID, CDI_KEY_SCALEFACTOR, &scalefactor);
 
       // clang-format off
-      if      (attrName == "long_name")     attrValues.push_back(longname);
+      if      (attrName == "long_name")     attrValues.push_back(var.longname);
       else if (attrName == "standard_name") attrValues.push_back(stdname);
-      else if (attrName == "units")         attrValues.push_back(units);
-      else if (attrName == "param")         attrValues.push_back(paramstr);
-      else if (attrName == "code")          attrValues.push_back(std::to_string(code));
+      else if (attrName == "units")         attrValues.push_back(var.units);
+      else if (attrName == "param")         attrValues.push_back(param_to_string(var.param));
+      else if (attrName == "code")          attrValues.push_back(std::to_string(var.code));
       else if (attrName == "table")         attrValues.push_back(std::to_string(table));
-      else if (attrName == "missing_value") attrValues.push_back(std::to_string(missval));
+      else if (attrName == "missing_value") attrValues.push_back(std::to_string(var.missval));
       else if (attrName == "add_offset")    attrValues.push_back(std::to_string(addoffset));
       else if (attrName == "scale_factor")  attrValues.push_back(std::to_string(scalefactor));
       // clang-format off
@@ -293,6 +286,8 @@ set_attribute(int cdiID, int varID, const std::string &attName, int dtype, const
         {
           if (i > 0 && value[i - 1] == '\\' && value[i] == 'n')
             outvalue[outlen - 1] = '\n';
+          else if (i > 0 && value[i - 1] == '\\' && value[i] == '"')
+            outvalue[outlen - 1] = '\"';
           else
             outvalue[outlen++] = value[i];
         }
@@ -349,44 +344,51 @@ set_attributes(const KVList &kvlist, int vlistID)
     }
 }
 
-class ModuleSetattribute
+class Setattribute : public Process
 {
+public:
+  using Process::Process;
+  inline static CdoModule module = {
+    .name = "Setattribute",
+    .operators = { { "setattribute", 0, 0, "attributes", SetattributeHelp } },
+    .aliases = {},
+    .mode = EXPOSED,     // Module mode: 0:intern 1:extern
+    .number = CDI_REAL,  // Allowed number type
+    .constraints = { 1, 1, NoRestriction },
+  };
+  inline static RegisterEntry<Setattribute> registration = RegisterEntry<Setattribute>(module);
+
   CdoStreamID streamID1;
   CdoStreamID streamID2;
 
   int taxisID1;
   int taxisID2;
 
-  Field field;
   bool dataIsUnchanged;
 
   VarList varList1;
 
 public:
   void
-  init(void *process)
+  init() override
   {
-    cdo_initialize(process);
-
-    cdo_operator_add("setattribute", 0, 0, "attributes");
-
     dataIsUnchanged = data_is_unchanged();
 
     auto operatorID = cdo_operator_id();
 
     operator_input_arg(cdo_operator_enter(operatorID));
 
-    auto natts = cdo_operator_argc();
-    if (natts == 0) cdo_abort("Parameter missing!");
+    auto numAtts = cdo_operator_argc();
+    if (numAtts == 0) cdo_abort("Parameter missing!");
 
     PMList pmlist;
     KVList kvlist;
     kvlist.name = cdo_module_name();
-    if (kvlist.parse_arguments(natts, cdo_get_oper_argv()) != 0) cdo_abort("Parse error!");
+    if (kvlist.parse_arguments(cdo_get_oper_argv()) != 0) cdo_abort("Parse error!");
     if (Options::cdoVerbose) kvlist.print();
 
     auto pkvlist = &kvlist;
-    if (natts == 1)
+    if (numAtts == 1)
       {
         KeyValues &kv = kvlist.front();
         if (kv.key == "FILE")
@@ -407,6 +409,8 @@ public:
     auto vlistID1 = cdo_stream_inq_vlist(streamID1);
     auto vlistID2 = vlistDuplicate(vlistID1);
 
+    varList1 = VarList(vlistID1);
+
     set_attributes(*pkvlist, vlistID2);
 
     taxisID1 = vlistInqTaxis(vlistID1);
@@ -415,13 +419,13 @@ public:
 
     streamID2 = cdo_open_write(1);
     cdo_def_vlist(streamID2, vlistID2);
-
-    varList_init(varList1, vlistID1);
   }
 
   void
-  run()
+  run() override
   {
+    Field field;
+
     int tsID = 0;
     while (true)
       {
@@ -440,7 +444,7 @@ public:
             if (dataIsUnchanged) { cdo_copy_record(streamID2, streamID1); }
             else
               {
-                field.init(varList1[varID]);
+                field.init(varList1.vars[varID]);
                 cdo_read_record(streamID1, field);
                 cdo_write_record(streamID2, field);
               }
@@ -451,21 +455,9 @@ public:
   }
 
   void
-  close()
+  close() override
   {
     cdo_stream_close(streamID1);
     cdo_stream_close(streamID2);
-    cdo_finish();
   }
 };
-
-void *
-Setattribute(void *process)
-{
-  ModuleSetattribute setattribute;
-  setattribute.init(process);
-  setattribute.run();
-  setattribute.close();
-
-  return nullptr;
-}
