@@ -31,9 +31,9 @@ static void
 set_value(Field &field, double value)
 {
   if (field.memType == MemType::Float)
-    field.nmiss = set_value(field.size, field.vec_f, (float) field.missval, (float) value);
+    field.numMissVals = set_value(field.size, field.vec_f, (float) field.missval, (float) value);
   else
-    field.nmiss = set_value(field.size, field.vec_d, field.missval, value);
+    field.numMissVals = set_value(field.size, field.vec_d, field.missval, value);
 }
 
 template <typename T>
@@ -54,9 +54,9 @@ static void
 set_value(Field &field, double value, const Varray<size_t> &cells)
 {
   if (field.memType == MemType::Float)
-    field.nmiss = set_value(field.size, field.vec_f, (float) field.missval, (float) value, cells);
+    field.numMissVals = set_value(field.size, field.vec_f, (float) field.missval, (float) value, cells);
   else
-    field.nmiss = set_value(field.size, field.vec_d, field.missval, value, cells);
+    field.numMissVals = set_value(field.size, field.vec_d, field.missval, value, cells);
 }
 
 static void
@@ -88,7 +88,7 @@ setgridcell_get_parameter(double &constant, Varray<size_t> &cells, std::vector<s
 
       KVList kvlist;
       kvlist.name = cdo_module_name();
-      if (kvlist.parse_arguments(pargc, pargv) != 0) cdo_abort("Parse error!");
+      if (kvlist.parse_arguments(pargv) != 0) cdo_abort("Parse error!");
       if (Options::cdoVerbose) kvlist.print();
 
       for (const auto &kv : kvlist)
@@ -126,8 +126,19 @@ setgridcell_get_parameter(double &constant, Varray<size_t> &cells, std::vector<s
     }
 }
 
-class ModuleSetgridcell
+class Setgridcell : public Process
 {
+public:
+  using Process::Process;
+  inline static CdoModule module = {
+    .name = "Setgridcell",
+    .operators = { { "setgridcell", 0, 0, "value=constant[,cell=gridcellindices(1-N)]", SetgridcellHelp } },
+    .aliases = {},
+    .mode = EXPOSED,     // Module mode: 0:intern 1:extern
+    .number = CDI_REAL,  // Allowed number type
+    .constraints = { 1, 1, NoRestriction },
+  };
+  inline static RegisterEntry<Setgridcell> registration = RegisterEntry<Setgridcell>(module);
 
   Field field;
 
@@ -138,7 +149,7 @@ class ModuleSetgridcell
   int taxisID2;
 
   int numVars;
-  VarList varList;
+  VarList varList1;
   Varray<bool> selectVars;
 
   std::vector<std::string> varnames;
@@ -149,12 +160,8 @@ class ModuleSetgridcell
 
 public:
   void
-  init(void *process)
+  init() override
   {
-    cdo_initialize(process);
-
-    cdo_operator_add("setgridcell", 0, 0, "value=constant[, cell=grid cell indices (1-N)]");
-
     operator_input_arg(cdo_operator_enter(0));
 
     const auto nparam = cdo_operator_argc();
@@ -180,16 +187,16 @@ public:
 
     streamID1 = cdo_open_read(0);
 
-    const auto vlistID1 = cdo_stream_inq_vlist(streamID1);
-    const auto vlistID2 = vlistDuplicate(vlistID1);
+    auto vlistID1 = cdo_stream_inq_vlist(streamID1);
+    auto vlistID2 = vlistDuplicate(vlistID1);
 
     taxisID1 = vlistInqTaxis(vlistID1);
     taxisID2 = taxisDuplicate(taxisID1);
     vlistDefTaxis(vlistID2, taxisID2);
 
-    varList_init(varList, vlistID1);
+    varList1 = VarList(vlistID1);
 
-    numVars = varList.size();
+    numVars = varList1.numVars();
     selectVars = Varray<bool>(numVars, !varnames.size());
     if (varnames.size())
       {
@@ -198,7 +205,7 @@ public:
             auto varFound = false;
             for (int varID = 0; varID < numVars; ++varID)
               {
-                if (varname == varList[varID].name)
+                if (varname == varList1.vars[varID].name)
                   {
                     selectVars[varID] = true;
                     varFound = true;
@@ -214,12 +221,12 @@ public:
   }
 
   void
-  run()
+  run() override
   {
     int tsID = 0;
     while (true)
       {
-        const auto nrecs = cdo_stream_inq_timestep(streamID1, tsID);
+        auto nrecs = cdo_stream_inq_timestep(streamID1, tsID);
         if (nrecs == 0) break;
 
         cdo_taxis_copy_timestep(taxisID2, taxisID1);
@@ -229,7 +236,7 @@ public:
           {
             int varID, levelID;
             cdo_inq_record(streamID1, &varID, &levelID);
-            field.init(varList[varID]);
+            field.init(varList1.vars[varID]);
             cdo_read_record(streamID1, field);
 
             if (selectVars[varID])
@@ -249,22 +256,11 @@ public:
         tsID++;
       }
   }
+
   void
-  close()
+  close() override
   {
     cdo_stream_close(streamID2);
     cdo_stream_close(streamID1);
-
-    cdo_finish();
   }
 };
-
-void *
-Setgridcell(void *process)
-{
-  ModuleSetgridcell setgridcell;
-  setgridcell.init(process);
-  setgridcell.run();
-  setgridcell.close();
-  return nullptr;
-}
