@@ -350,7 +350,7 @@ get_parameter(void)
 
       KVList kvlist;
       kvlist.name = cdo_module_name();
-      if (kvlist.parse_arguments(pargc, pargv) != 0) cdo_abort("Parse error!");
+      if (kvlist.parse_arguments(pargv) != 0) cdo_abort("Parse error!");
       if (Options::cdoVerbose) kvlist.print();
 
       for (const auto &kv : kvlist)
@@ -382,10 +382,7 @@ gen_index_grid(int gridID1, Halo &halo)
       halo.east = parameter_to_long(cdo_operator_argv(0));
       halo.west = parameter_to_long(cdo_operator_argv(1));
     }
-  else
-    {
-      halo = get_parameter();
-    }
+  else { halo = get_parameter(); }
 
   auto isCircularGrid = gridIsCircular(gridID1);
   long nlon = gridInqXsize(gridID1);
@@ -481,7 +478,7 @@ get_gridID(int vlistID1)
 {
   int gridID1 = -1;
 
-  auto ngrids = vlistNgrids(vlistID1);
+  auto ngrids = vlistNumGrids(vlistID1);
   int ndiffgrids = 0;
   int index;
   for (index = 1; index < ngrids; ++index)
@@ -502,8 +499,20 @@ get_gridID(int vlistID1)
   return gridID1;
 }
 
-class SetHalo
+class Sethalo : public Process
 {
+public:
+  using Process::Process;
+  inline static CdoModule module = {
+    .name = "Sethalo",
+    .operators = { { "sethalo", SethaloHelp }, { "tpnhalo", SethaloHelp } },
+    .aliases = {},
+    .mode = EXPOSED,     // Module mode: 0:intern 1:extern
+    .number = CDI_REAL,  // Allowed number type
+    .constraints = { 1, 1, NoRestriction },
+  };
+  inline static RegisterEntry<Sethalo> registration = RegisterEntry<Sethalo>(module);
+
   int SETHALO;
   int operatorID;
 
@@ -517,7 +526,7 @@ class SetHalo
 
   int gridID1 = -1;
 
-  VarList varList;
+  VarList varList1;
 
   std::vector<bool> vars;
   Varray<double> array1;
@@ -525,14 +534,9 @@ class SetHalo
 
 public:
   void
-  init(void *process)
+  init() override
   {
-    cdo_initialize(process);
-
-    // clang-format off
-    SETHALO = cdo_operator_add("sethalo", 0, 0, nullptr);
-              cdo_operator_add("tpnhalo", 0, 0, nullptr);
-    // clang-format on
+    SETHALO = module.get_id("sethalo");
 
     operatorID = cdo_operator_id();
 
@@ -549,7 +553,7 @@ public:
     taxisID2 = taxisDuplicate(taxisID1);
     vlistDefTaxis(vlistID2, taxisID2);
 
-    auto ngrids = vlistNgrids(vlistID1);
+    auto ngrids = vlistNumGrids(vlistID1);
     for (int index = 0; index < ngrids; ++index)
       {
         if (gridID1 == vlistGrid(vlistID1, index))
@@ -559,11 +563,11 @@ public:
           }
       }
 
-    varList_init(varList, vlistID1);
+    varList1 = VarList(vlistID1);
 
     auto nvars = vlistNvars(vlistID1);
     vars = std::vector<bool>(nvars);
-    for (int varID = 0; varID < nvars; ++varID) vars[varID] = (gridID1 == varList[varID].gridID);
+    for (int varID = 0; varID < nvars; ++varID) vars[varID] = (gridID1 == varList1.vars[varID].gridID);
 
     streamID2 = cdo_open_write(1);
     cdo_def_vlist(streamID2, vlistID2);
@@ -573,7 +577,7 @@ public:
   }
 
   void
-  run()
+  run() override
   {
     int tsID = 0;
     while (true)
@@ -591,20 +595,20 @@ public:
 
             if (vars[varID])
               {
-                size_t nmiss;
-                cdo_read_record(streamID1, array1.data(), &nmiss);
+                size_t numMissVals;
+                cdo_read_record(streamID1, array1.data(), &numMissVals);
 
                 auto recalcNumMiss = false;
-                auto missval = varList[varID].missval;
+                auto missval = varList1.vars[varID].missval;
                 if (operatorID == SETHALO)
                   recalcNumMiss = regular_halo(array1, gridID1, array2, halo, missval);
                 else
                   tripolar_halo(array1, gridID1, array2);
 
-                if (nmiss || recalcNumMiss) nmiss = varray_num_mv(array2.size(), array2, missval);
+                if (numMissVals || recalcNumMiss) numMissVals = varray_num_mv(array2.size(), array2, missval);
 
                 cdo_def_record(streamID2, varID, levelID);
-                cdo_write_record(streamID2, array2.data(), nmiss);
+                cdo_write_record(streamID2, array2.data(), numMissVals);
               }
           }
 
@@ -613,22 +617,9 @@ public:
   }
 
   void
-  close()
+  close() override
   {
     cdo_stream_close(streamID2);
     cdo_stream_close(streamID1);
-
-    cdo_finish();
   }
 };
-
-void *
-Sethalo(void *process)
-{
-  SetHalo sethalo;
-  sethalo.init(process);
-  sethalo.run();
-  sethalo.close();
-
-  return nullptr;
-}
