@@ -20,44 +20,45 @@
 
 #include "process_int.h"
 #include "param_conversion.h"
+#include "progress.h"
 
 template <typename T>
 static size_t
 set_missval(size_t gridsize, Varray<T> &array, T missval, T new_missval)
 {
-  size_t nmiss = 0;
+  size_t numMissVals = 0;
   for (size_t i = 0; i < gridsize; ++i)
     if (DBL_IS_EQUAL(array[i], missval) || DBL_IS_EQUAL(array[i], (float) missval) || DBL_IS_EQUAL(array[i], new_missval)
         || DBL_IS_EQUAL(array[i], (float) new_missval))
       {
         array[i] = new_missval;
-        nmiss++;
+        numMissVals++;
       }
 
-  return nmiss;
+  return numMissVals;
 }
 
 static void
 set_missval(Field &field, double new_missval)
 {
   if (field.memType == MemType::Float)
-    field.nmiss = set_missval(field.size, field.vec_f, (float) field.missval, (float) new_missval);
+    field.numMissVals = set_missval(field.size, field.vec_f, (float) field.missval, (float) new_missval);
   else
-    field.nmiss = set_missval(field.size, field.vec_d, field.missval, new_missval);
+    field.numMissVals = set_missval(field.size, field.vec_d, field.missval, new_missval);
 }
 
 template <typename T>
 static size_t
 set_const_to_miss(size_t gridsize, Varray<T> &array, T missval, T rconst)
 {
-  size_t nmiss = 0;
+  size_t numMissVals = 0;
   if (std::isnan(rconst))
     {
       for (size_t i = 0; i < gridsize; ++i)
         if (std::isnan(array[i]))
           {
             array[i] = missval;
-            nmiss++;
+            numMissVals++;
           }
     }
   else
@@ -66,20 +67,20 @@ set_const_to_miss(size_t gridsize, Varray<T> &array, T missval, T rconst)
         if (DBL_IS_EQUAL(array[i], rconst) || DBL_IS_EQUAL(array[i], (float) rconst))
           {
             array[i] = missval;
-            nmiss++;
+            numMissVals++;
           }
     }
 
-  return nmiss;
+  return numMissVals;
 }
 
 static void
 set_const_to_miss(Field &field, double rconst)
 {
   if (field.memType == MemType::Float)
-    field.nmiss += set_const_to_miss(field.size, field.vec_f, (float) field.missval, (float) rconst);
+    field.numMissVals += set_const_to_miss(field.size, field.vec_f, (float) field.missval, (float) rconst);
   else
-    field.nmiss += set_const_to_miss(field.size, field.vec_d, field.missval, rconst);
+    field.numMissVals += set_const_to_miss(field.size, field.vec_d, field.missval, rconst);
 }
 
 template <typename T>
@@ -96,33 +97,33 @@ static void
 set_miss_to_const(Field &field, double rconst)
 {
   if (field.memType == MemType::Float)
-    field.nmiss = set_miss_to_const(field.size, field.vec_f, (float) field.missval, (float) rconst);
+    field.numMissVals = set_miss_to_const(field.size, field.vec_f, (float) field.missval, (float) rconst);
   else
-    field.nmiss = set_miss_to_const(field.size, field.vec_d, field.missval, rconst);
+    field.numMissVals = set_miss_to_const(field.size, field.vec_d, field.missval, rconst);
 }
 
 template <typename T>
 static size_t
 set_range_to_miss(size_t gridsize, Varray<T> &array, T missval, T rmin, T rmax)
 {
-  size_t nmiss = 0;
+  size_t numMissVals = 0;
   for (size_t i = 0; i < gridsize; ++i)
     if (array[i] >= rmin && array[i] <= rmax)
       {
         array[i] = missval;
-        nmiss++;
+        numMissVals++;
       }
 
-  return nmiss;
+  return numMissVals;
 }
 
 static void
 set_range_to_miss(Field &field, double rmin, double rmax)
 {
   if (field.memType == MemType::Float)
-    field.nmiss += set_range_to_miss(field.size, field.vec_f, (float) field.missval, (float) rmin, (float) rmax);
+    field.numMissVals += set_range_to_miss(field.size, field.vec_f, (float) field.missval, (float) rmin, (float) rmax);
   else
-    field.nmiss += set_range_to_miss(field.size, field.vec_d, field.missval, rmin, rmax);
+    field.numMissVals += set_range_to_miss(field.size, field.vec_d, field.missval, rmin, rmax);
 }
 
 template <typename T>
@@ -132,34 +133,49 @@ set_valid_range(size_t gridsize, Varray<T> &array, T missval, T rmin, T rmax)
   for (size_t i = 0; i < gridsize; ++i)
     if (array[i] < rmin || array[i] > rmax) array[i] = missval;
 
-  const auto nmiss = varray_num_mv(gridsize, array, missval);
+  auto numMissVals = varray_num_mv(gridsize, array, missval);
 
-  return nmiss;
+  return numMissVals;
 }
 
 static void
 set_valid_range(Field &field, double rmin, double rmax)
 {
   if (field.memType == MemType::Float)
-    field.nmiss = set_valid_range(field.size, field.vec_f, (float) field.missval, (float) rmin, (float) rmax);
+    field.numMissVals = set_valid_range(field.size, field.vec_f, (float) field.missval, (float) rmin, (float) rmax);
   else
-    field.nmiss = set_valid_range(field.size, field.vec_d, field.missval, rmin, rmax);
+    field.numMissVals = set_valid_range(field.size, field.vec_d, field.missval, rmin, rmax);
 }
 
-class ModuleSetmiss
+class Setmiss : public Process
 {
+public:
+  using Process::Process;
+  inline static CdoModule module = {
+    .name = "Setmiss",
+    .operators = { { "setmissval", 0, 0, "missing value", SetmissHelp },
+                   { "setctomiss", 0, 0, "constant", SetmissHelp },
+                   { "setmisstoc", 0, 0, "constant", SetmissHelp },
+                   { "setrtomiss", 0, 0, "range(min,max)", SetmissHelp },
+                   { "setvrange", 0, 0, "range(min,max)", SetmissHelp } },
+    .aliases = {},
+    .mode = EXPOSED,     // Module mode: 0:intern 1:extern
+    .number = CDI_REAL,  // Allowed number type
+    .constraints = { 1, 1, NoRestriction },
+  };
+  inline static RegisterEntry<Setmiss> registration = RegisterEntry<Setmiss>(module);
+
+  int SETMISSVAL, SETCTOMISS, SETMISSTOC, SETRTOMISS, SETVRANGE;
 
   CdoStreamID streamID1;
   CdoStreamID streamID2;
   int taxisID1;
   int taxisID2;
 
-  VarList varList;
+  VarList varList1;
   Field field;
   int operatorID;
 
-  int SETMISSVAL, SETCTOMISS, SETMISSTOC, SETRTOMISS, SETVRANGE;
-
   double rconst = 0.0;
   double rmin = 0.0;
   double rmax = 0.0;
@@ -167,17 +183,13 @@ class ModuleSetmiss
 
 public:
   void
-  init(void *process)
+  init() override
   {
-    cdo_initialize(process);
-
-    // clang-format off
-   SETMISSVAL = cdo_operator_add("setmissval", 0, 0, "missing value");
-   SETCTOMISS = cdo_operator_add("setctomiss", 0, 0, "constant");
-   SETMISSTOC = cdo_operator_add("setmisstoc", 0, 0, "constant");
-   SETRTOMISS = cdo_operator_add("setrtomiss", 0, 0, "range (min, max)");
-   SETVRANGE  = cdo_operator_add("setvrange",  0, 0, "range (min, max)");
-    // clang-format on
+    SETMISSVAL = module.get_id("setmissval");
+    SETCTOMISS = module.get_id("setctomiss");
+    SETMISSTOC = module.get_id("setmisstoc");
+    SETRTOMISS = module.get_id("setrtomiss");
+    SETVRANGE = module.get_id("setvrange");
 
     operatorID = cdo_operator_id();
 
@@ -200,18 +212,18 @@ public:
 
     streamID1 = cdo_open_read(0);
 
-    const auto vlistID1 = cdo_stream_inq_vlist(streamID1);
-    const auto vlistID2 = vlistDuplicate(vlistID1);
+    auto vlistID1 = cdo_stream_inq_vlist(streamID1);
+    auto vlistID2 = vlistDuplicate(vlistID1);
 
     taxisID1 = vlistInqTaxis(vlistID1);
     taxisID2 = taxisDuplicate(taxisID1);
     vlistDefTaxis(vlistID2, taxisID2);
 
-    varList_init(varList, vlistID1);
+    varList1 = VarList(vlistID1);
 
     if (operatorID == SETMISSVAL)
       {
-        const auto nvars = vlistNvars(vlistID2);
+        auto nvars = vlistNvars(vlistID2);
         for (int varID = 0; varID < nvars; ++varID) vlistDefVarMissval(vlistID2, varID, new_missval);
       }
     else if (operatorID == SETMISSTOC)
@@ -219,7 +231,7 @@ public:
         auto nvars = vlistNvars(vlistID2);
         for (int varID = 0; varID < nvars; ++varID)
           {
-            const auto &var = varList[varID];
+            const auto &var = varList1.vars[varID];
             if (DBL_IS_EQUAL(rconst, var.missval))
               {
                 cdo_warning("Missing value and constant have the same value!");
@@ -240,32 +252,37 @@ public:
     streamID2 = cdo_open_write(1);
     cdo_def_vlist(streamID2, vlistID2);
   }
+
   void
-  run()
+  run() override
   {
+    auto numSteps = varList1.numSteps();
+    cdo::Progress progress;
+
     int tsID = 0;
     while (true)
       {
-        const auto nrecs = cdo_stream_inq_timestep(streamID1, tsID);
+        auto nrecs = cdo_stream_inq_timestep(streamID1, tsID);
         if (nrecs == 0) break;
 
+        if (numSteps > 1) progress.update((tsID + 1.0) / numSteps);
+
         cdo_taxis_copy_timestep(taxisID2, taxisID1);
         cdo_def_timestep(streamID2, tsID);
 
         for (int recID = 0; recID < nrecs; ++recID)
           {
-            int varID, levelID;
-            cdo_inq_record(streamID1, &varID, &levelID);
-            const auto &var = varList[varID];
+            auto [varID, levelID] = cdo_inq_record(streamID1);
+            const auto &var = varList1.vars[varID];
             field.init(var);
             cdo_read_record(streamID1, field);
 
             // clang-format off
-          if      (operatorID == SETMISSVAL) set_missval(field, new_missval);
-          else if (operatorID == SETCTOMISS) set_const_to_miss(field, rconst);
-          else if (operatorID == SETMISSTOC) set_miss_to_const(field, rconst);
-          else if (operatorID == SETRTOMISS) set_range_to_miss(field, rmin, rmax);
-          else if (operatorID == SETVRANGE)  set_valid_range(field, rmin, rmax);
+            if      (operatorID == SETMISSVAL) set_missval(field, new_missval);
+            else if (operatorID == SETCTOMISS) set_const_to_miss(field, rconst);
+            else if (operatorID == SETMISSTOC) set_miss_to_const(field, rconst);
+            else if (operatorID == SETRTOMISS) set_range_to_miss(field, rmin, rmax);
+            else if (operatorID == SETVRANGE)  set_valid_range(field, rmin, rmax);
             // clang-format on
 
             cdo_def_record(streamID2, varID, levelID);
@@ -275,22 +292,11 @@ public:
         tsID++;
       }
   }
+
   void
-  close()
+  close() override
   {
     cdo_stream_close(streamID2);
     cdo_stream_close(streamID1);
-
-    cdo_finish();
   }
 };
-void *
-Setmiss(void *process)
-{
-  ModuleSetmiss setmiss;
-  setmiss.init(process);
-  setmiss.run();
-  setmiss.close();
-
-  return nullptr;
-}
