@@ -33,7 +33,7 @@ enum pt_mode_t
 };
 
 static void
-apply_parameterList(pt_mode_t ptmode, PMList &pmlist, int nvars, int vlistID2, std::vector<CmorVar> &vars)
+apply_parameterList(pt_mode_t ptmode, PMList &pmlist, int numVars, int vlistID2, std::vector<CmorVar> &vars)
 {
   const std::vector<std::string> hentry = { "Header" };
   const std::vector<std::string> ventry = { "variable_entry", "parameter" };
@@ -41,6 +41,8 @@ apply_parameterList(pt_mode_t ptmode, PMList &pmlist, int nvars, int vlistID2, s
   char paramstr[32];
   int codenum = 0;
 
+  VarList varList2(vlistID2);
+
   // search for global missing value
   auto hasMissvals = false;
   double missval = 0.0;
@@ -59,18 +61,18 @@ apply_parameterList(pt_mode_t ptmode, PMList &pmlist, int nvars, int vlistID2, s
   }
 
   int numVarsFound = 0;
-  for (int varID = 0; varID < nvars; ++varID)
+  for (int varID = 0; varID < numVars; ++varID)
     {
       auto &var = vars[varID];
       var.name = cdo::inq_var_name(vlistID2, varID);
 
+      const auto &var2 = varList2.vars[varID];
       if (hasMissvals)
         {
-          auto missval_old = vlistInqVarMissval(vlistID2, varID);
-          if (!dbl_is_equal(missval, missval_old))
+          if (dbl_is_not_equal(missval, var2.missval))
             {
               var.changemissval = true;
-              var.missval_old = missval_old;
+              var.missvalOld = var2.missval;
               vlistDefVarMissval(vlistID2, varID, missval);
             }
         }
@@ -78,7 +80,7 @@ apply_parameterList(pt_mode_t ptmode, PMList &pmlist, int nvars, int vlistID2, s
       const KVList *kvlist = nullptr;
       if (ptmode == CODE_NUMBER)
         {
-          codenum = vlistInqVarCode(vlistID2, varID);
+          codenum = var2.code;
           std::snprintf(valstr, sizeof(valstr), "%d", codenum);
           kvlist = pmlist.searchKVListVentry("code", valstr, ventry);
           if (kvlist)
@@ -86,7 +88,7 @@ apply_parameterList(pt_mode_t ptmode, PMList &pmlist, int nvars, int vlistID2, s
               auto tableID = vlistInqVarTable(vlistID2, varID);
               auto tabnum = tableInqNum(tableID);
               int levtype = 0;
-              cdiInqKeyInt(vlistInqVarZaxis(vlistID2, varID), CDI_GLOBAL, CDI_KEY_TYPEOFFIRSTFIXEDSURFACE, &levtype);
+              cdiInqKeyInt(var2.zaxisID, CDI_GLOBAL, CDI_KEY_TYPEOFFIRSTFIXEDSURFACE, &levtype);
               auto table = tabnum;
               auto ltype = levtype;
               {
@@ -102,14 +104,14 @@ apply_parameterList(pt_mode_t ptmode, PMList &pmlist, int nvars, int vlistID2, s
         }
       else if (ptmode == PARAMETER_ID)
         {
-          auto param = vlistInqVarParam(vlistID2, varID);
+          auto param = var2.param;
           cdiParamToString(param, paramstr, sizeof(paramstr));
           std::snprintf(valstr, sizeof(valstr), "%s", paramstr);
           kvlist = pmlist.searchKVListVentry("param", valstr, ventry);
           if (kvlist)
             {
               int levtype = 0;
-              cdiInqKeyInt(vlistInqVarZaxis(vlistID2, varID), CDI_GLOBAL, CDI_KEY_TYPEOFFIRSTFIXEDSURFACE, &levtype);
+              cdiInqKeyInt(var2.zaxisID, CDI_GLOBAL, CDI_KEY_TYPEOFFIRSTFIXEDSURFACE, &levtype);
               auto kv = kvlist->search("ltype");
               auto ltype = (kv && kv->nvalues == 1) ? parameter_to_int(kv->values[0]) : levtype;
               if (levtype != ltype) kvlist = nullptr;
@@ -121,7 +123,7 @@ apply_parameterList(pt_mode_t ptmode, PMList &pmlist, int nvars, int vlistID2, s
         {
           numVarsFound++;
           int pnum, ptab, pdum;
-          cdiDecodeParam(vlistInqVarParam(vlistID2, varID), &pnum, &ptab, &pdum);
+          cdiDecodeParam(var2.param, &pnum, &ptab, &pdum);
           auto hasValidMin = false, hasValidMax = false;
 
           for (const auto &kv : *kvlist)
@@ -151,8 +153,24 @@ apply_parameterList(pt_mode_t ptmode, PMList &pmlist, int nvars, int vlistID2, s
     }
 }
 
-class ModuleSetpartab
+class Setpartab : public Process
 {
+public:
+  using Process::Process;
+  inline static CdoModule module = {
+    .name = "Setpartab",
+    .operators = { { "setcodetab", 0, 0, "parameter code table name", SetHelp },
+                   { "setpartabc", 0, 0, "parameter table name", SetpartabHelp },
+                   { "setpartabp", 0, 0, "parameter table name", SetpartabHelp },
+                   { "setpartabn", 0, 0, "parameter table name", SetpartabHelp } },
+    .aliases = { { "setpartab", "setcodetab" }, { "setpartabv", "setpartabn" } },
+    .mode = EXPOSED,     // Module mode: 0:intern 1:extern
+    .number = CDI_REAL,  // Allowed number type
+    .constraints = { 1, 1, NoRestriction },
+  };
+  inline static RegisterEntry<Setpartab> registration = RegisterEntry<Setpartab>(module);
+
+  int SETCODETAB, SETPARTABC, SETPARTABP, SETPARTABN;
   int tableID = -1;
   int tableformat = 0;
   bool deleteVars = false;
@@ -167,24 +185,19 @@ class ModuleSetpartab
   int vlistID2;
 
   std::vector<CmorVar> vars;
-  int nvars;
+  int numVars;
 
   Varray<double> array;
   VarList varList2;
 
-  int SETCODETAB, SETPARTABC, SETPARTABP, SETPARTABN;
-
 public:
   void
-  init(void *process)
+  init() override
   {
-
-    cdo_initialize(process);
-
-    SETCODETAB = cdo_operator_add("setcodetab", 0, 0, "parameter code table name");
-    SETPARTABC = cdo_operator_add("setpartabc", 0, 0, "parameter table name");
-    SETPARTABP = cdo_operator_add("setpartabp", 0, 0, "parameter table name");
-    SETPARTABN = cdo_operator_add("setpartabn", 0, 0, "parameter table name");
+    SETCODETAB = module.get_id("setcodetab");
+    SETPARTABC = module.get_id("setpartabc");
+    SETPARTABP = module.get_id("setpartabp");
+    SETPARTABN = module.get_id("setpartabn");
 
     auto operatorID = cdo_operator_id();
 
@@ -243,17 +256,17 @@ public:
     vlistID2 = vlistDuplicate(vlistID1);
     // vlistPrint(vlistID2);
 
-    nvars = vlistNvars(vlistID2);
-    vars = std::vector<CmorVar>(nvars);
+    numVars = vlistNvars(vlistID2);
+    vars = std::vector<CmorVar>(numVars);
 
     if (convertData)
-      for (int varID = 0; varID < nvars; ++varID) vars[varID].convert = true;
+      for (int varID = 0; varID < numVars; ++varID) vars[varID].convert = true;
 
     if (tableformat == 0)
       {
-        // for (int varID = 0; varID < nvars; ++varID) vlistDefVarTable(vlistID2, varID, tableID);
+        // for (int varID = 0; varID < numVars; ++varID) vlistDefVarTable(vlistID2, varID, tableID);
         char name[CDI_MAX_NAME], longname[CDI_MAX_NAME], units[CDI_MAX_NAME];
-        for (int varID = 0; varID < nvars; ++varID)
+        for (int varID = 0; varID < numVars; ++varID)
           {
             auto param = vlistInqVarParam(vlistID2, varID);
             int pdis, pcat, pnum;
@@ -288,10 +301,10 @@ public:
           pmlist.read_namelist(fp, filename);
           std::fclose(fp);
 
-          apply_parameterList(ptmode, pmlist, nvars, vlistID2, vars);
+          apply_parameterList(ptmode, pmlist, numVars, vlistID2, vars);
         }
 
-        for (int varID = 0; varID < nvars; ++varID)
+        for (int varID = 0; varID < numVars; ++varID)
           if (vars[varID].remove)
             {
               deleteVars = true;
@@ -303,7 +316,7 @@ public:
             vlistClearFlag(vlistID1);
             vlistClearFlag(vlistID2);
 
-            for (int varID = 0; varID < nvars; ++varID)
+            for (int varID = 0; varID < numVars; ++varID)
               {
                 auto zaxisID = vlistInqVarZaxis(vlistID2, varID);
                 auto nlevs = zaxisInqSize(zaxisID);
@@ -328,7 +341,7 @@ public:
             if (vlistNvars(vlistID2) == 0) cdo_abort("No variable selected!");
           }
 
-        for (int varID = 0; varID < nvars; ++varID)
+        for (int varID = 0; varID < numVars; ++varID)
           {
             auto &var = vars[varID];
             if (!var.convert) var.changeunits = false;
@@ -350,11 +363,11 @@ public:
     if (vlistNumber(vlistID1) != CDI_REAL) gridsizemax *= 2;
     array = Varray<double>(gridsizemax);
 
-    varList_init(varList2, vlistID2);
+    varList2 = VarList(vlistID2);
   }
 
   void
-  run()
+  run() override
   {
     int tsID = 0;
     while (true)
@@ -365,7 +378,7 @@ public:
         cdo_taxis_copy_timestep(taxisID2, taxisID1);
         cdo_def_timestep(streamID2, tsID);
 
-        cmor_check_init(nvars, vars);
+        cmor_check_init(numVars, vars);
 
         for (int recID = 0; recID < nrecs; ++recID)
           {
@@ -389,17 +402,18 @@ public:
 
             cdo_def_record(streamID2, varID2, levelID2);
 
-            size_t nmiss;
-            cdo_read_record(streamID1, array.data(), &nmiss);
+            size_t numMissVals;
+            cdo_read_record(streamID1, array.data(), &numMissVals);
 
-            auto missval = varList2[varID2].missval;
-            auto gridsize = varList2[varID2].nwpv * varList2[varID2].gridsize;
+            const auto &var2 = varList2.vars[varID2];
+            auto missval = var2.missval;
+            auto gridsize = var2.nwpv * var2.gridsize;
 
-            if (nmiss && var.changemissval)
+            if (numMissVals && var.changemissval)
               {
                 for (size_t i = 0; i < gridsize; ++i)
                   {
-                    if (dbl_is_equal(array[i], var.missval_old)) array[i] = missval;
+                    if (dbl_is_equal(array[i], var.missvalOld)) array[i] = missval;
                   }
               }
 
@@ -407,7 +421,7 @@ public:
               {
                 for (size_t i = 0; i < gridsize; ++i)
                   {
-                    if (!dbl_is_equal(array[i], missval)) array[i] *= var.factor;
+                    if (dbl_is_not_equal(array[i], missval)) array[i] *= var.factor;
                   }
               }
 
@@ -417,7 +431,7 @@ public:
                 int nerr = 0;
                 for (size_t i = 0; i < gridsize; ++i)
                   {
-                    if (!dbl_is_equal(array[i], missval))
+                    if (dbl_is_not_equal(array[i], missval))
                       {
                         array[i] = cv_convert_double((const cv_converter *) var.ut_converter, array[i]);
                         if (ut_get_status() != UT_SUCCESS) nerr++;
@@ -432,40 +446,29 @@ public:
               }
 #endif
 
-            cdo_write_record(streamID2, array.data(), nmiss);
+            cdo_write_record(streamID2, array.data(), numMissVals);
 
             cmor_check_prep(var, gridsize, missval, array.data());
           }
 
-        cmor_check_eval(vlistID2, nvars, vars);
+        cmor_check_eval(vlistID2, numVars, vars);
 
         tsID++;
       }
   }
+
   void
-  close()
+  close() override
   {
 
     cdo_stream_close(streamID2);
     cdo_stream_close(streamID1);
 
 #ifdef HAVE_UDUNITS2
-    for (int varID = 0; varID < nvars; ++varID)
+    for (int varID = 0; varID < numVars; ++varID)
       if (vars[varID].changeunits) cdo_convert_free(vars[varID].ut_converter);
 
     cdo_convert_destroy();
 #endif
-
-    cdo_finish();
   }
 };
-void *
-Setpartab(void *process)
-{
-  ModuleSetpartab setpartab;
-  setpartab.init(process);
-  setpartab.run();
-  setpartab.close();
-
-  return nullptr;
-}
