@@ -162,8 +162,31 @@ timeunits_is_valid(int timeUnits)
           || timeUnits == TUNIT_DAY || timeUnits == TUNIT_MONTH || timeUnits == TUNIT_YEAR);
 }
 
-class ModuleSettime
+class Settime : public Process
 {
+public:
+  using Process::Process;
+  inline static CdoModule module = {
+    .name = "Settime",
+    .operators = { { "setyear", 0, 1, "year", SettimeHelp },
+                   { "setmon", 0, 1, "month", SettimeHelp },
+                   { "setday", 0, 1, "day", SettimeHelp },
+                   { "setdate", 0, 1, "date(format:YYYY-MM-DD)", SettimeHelp },
+                   { "settime", 0, 1, "time(format:hh:mm:ss)", SettimeHelp },
+                   { "settunits", 0, 1, "timeunits(seconds,minutes,hours,days,months,years)", SettimeHelp },
+                   { "settaxis", 0, -2, "date<,time<,increment>>(formatYYYY-MM-DD,hh:mm:ss)", SettimeHelp },
+                   { "settbounds", 0, 1, "frequency(hour,day,month,year)", SettimeHelp },
+                   { "setreftime", 0, -2, "date<,time<,units>>(formatYYYY-MM-DD,hh:mm:ss)", SettimeHelp },
+                   { "setcalendar", 0, 1, "calendar(standard,proleptic_gregorian,360_day,365_day,366_day)", SettimeHelp },
+                   { "shifttime", 0, 1, "shiftvalue", SettimeHelp } },
+    .aliases = {},
+    .mode = EXPOSED,     // Module mode: 0:intern 1:extern
+    .number = CDI_BOTH,  // Allowed number type
+    .constraints = { 1, 1, NoRestriction },
+  };
+  inline static RegisterEntry<Settime> registration = RegisterEntry<Settime>(module);
+
+  int SETYEAR, SETMON, SETDAY, SETDATE, SETTIME, SETTUNITS, SETTAXIS, SETTBOUNDS, SETREFTIME, SETCALENDAR, SHIFTTIME;
   int64_t newval = 0;
   int timeUnits = TUNIT_DAY;
   int64_t ijulinc = 0;
@@ -178,8 +201,6 @@ class ModuleSettime
   CdiDateTime vDateTimeBounds[2]{};
   JulianDate julianDate;
 
-  int SETYEAR, SETMON, SETDAY, SETDATE, SETTIME, SETTUNITS, SETTAXIS, SETTBOUNDS, SETREFTIME, SETCALENDAR, SHIFTTIME;
-
   CdoStreamID streamID1;
   CdoStreamID streamID2 = CDO_STREAM_UNDEF;
 
@@ -198,24 +219,19 @@ class ModuleSettime
 
 public:
   void
-  init(void *process)
+  init() override
   {
-
-    cdo_initialize(process);
-
-    // clang-format off
-   SETYEAR     = cdo_operator_add("setyear",      0,  1, "year");
-   SETMON      = cdo_operator_add("setmon",       0,  1, "month");
-   SETDAY      = cdo_operator_add("setday",       0,  1, "day");
-   SETDATE     = cdo_operator_add("setdate",      0,  1, "date (format: YYYY-MM-DD)");
-   SETTIME     = cdo_operator_add("settime",      0,  1, "time (format: hh:mm:ss)");
-   SETTUNITS   = cdo_operator_add("settunits",    0,  1, "time units (seconds, minutes, hours, days, months, years)");
-   SETTAXIS    = cdo_operator_add("settaxis",     0, -2, "date<,time<,increment>> (format YYYY-MM-DD,hh:mm:ss)");
-   SETTBOUNDS  = cdo_operator_add("settbounds",   0,  1, "frequency (hour, day, month, year)");
-   SETREFTIME  = cdo_operator_add("setreftime",   0, -2, "date<,time<,units>> (format YYYY-MM-DD,hh:mm:ss)");
-   SETCALENDAR = cdo_operator_add("setcalendar",  0,  1, "calendar (standard, proleptic_gregorian, 360_day, 365_day, 366_day)");
-   SHIFTTIME   = cdo_operator_add("shifttime",    0,  1, "shift value");
-    // clang-format on
+    SETYEAR = module.get_id("setyear");
+    SETMON = module.get_id("setmon");
+    SETDAY = module.get_id("setday");
+    SETDATE = module.get_id("setdate");
+    SETTIME = module.get_id("settime");
+    SETTUNITS = module.get_id("settunits");
+    SETTAXIS = module.get_id("settaxis");
+    SETTBOUNDS = module.get_id("settbounds");
+    SETREFTIME = module.get_id("setreftime");
+    SETCALENDAR = module.get_id("setcalendar");
+    SHIFTTIME = module.get_id("shifttime");
 
     operatorID = cdo_operator_id();
     // nargs = cdo_operator_f2(operatorID);
@@ -275,9 +291,9 @@ public:
     ntsteps = vlistNtsteps(vlistID1);
     auto nvars = vlistNvars(vlistID1);
 
-    varList_init(varList1, vlistID1);
+    varList1 = VarList(vlistID1);
 
-    if (ntsteps == 1 && varList_numVaryingVars(varList1) == 0) ntsteps = 0;
+    if (ntsteps == 1 && varList1.numVaryingVars() == 0) ntsteps = 0;
 
     if (ntsteps == 0)
       {
@@ -354,7 +370,7 @@ public:
   }
 
   void
-  run()
+  run() override
   {
     int tsID = 0;
     while (true)
@@ -463,7 +479,7 @@ public:
             cdo_inq_record(streamID1, &varID, &levelID);
             cdo_def_record(streamID2, varID, levelID);
 
-            field.init(varList1[varID]);
+            field.init(varList1.vars[varID]);
             cdo_read_record(streamID1, field);
             cdo_write_record(streamID2, field);
           }
@@ -472,23 +488,10 @@ public:
       }
   }
   void
-  close()
+  close() override
   {
 
     cdo_stream_close(streamID2);
     cdo_stream_close(streamID1);
-
-    cdo_finish();
   }
 };
-
-void *
-Settime(void *process)
-{
-
-  ModuleSettime settime;
-  settime.init(process);
-  settime.run();
-  settime.close();
-  return nullptr;
-}
