@@ -19,7 +19,6 @@
 #include "printinfo.h"
 #include "mpmo_color.h"
 #include "process_int.h"
-#include "compare.h"
 #include "util_string.h"
 #include "datetime.h"
 #include "cdo_default_values.h"
@@ -108,20 +107,17 @@ limit_string_length(char *string, size_t maxlen)
 }
 
 static void
-print_vars_info(int operfunc, bool ensembleInfo, int vlistID, bool xsInfo)
+print_vars_info(int operfunc, bool ensembleInfo, const VarList &varList, int vlistID, bool xsInfo)
 {
   char tmpname[CDI_MAX_NAME];
   char paramstr[32];
 
-  VarList varList;
-  varList_init(varList, vlistID);
-
-  auto nvars = vlistNvars(vlistID);
+  auto numVars = varList.numVars();
   auto nsubtypes = vlistNsubtypes(vlistID);
 
-  for (int varID = 0; varID < nvars; ++varID)
+  for (int varID = 0; varID < numVars; ++varID)
     {
-      const auto &var = varList[varID];
+      const auto &var = varList.vars[varID];
 
       auto tabnum = tableInqNum(vlistInqVarTable(vlistID, varID));
 
@@ -147,7 +143,7 @@ print_vars_info(int operfunc, bool ensembleInfo, int vlistID, bool xsInfo)
       fprintf(stdout, "%c ", var.isConstant ? 'c' : 'v');
 
       // tsteptype
-      fprintf(stdout, "%-8s ", cdo::get_steptype_name(var.tsteptype));
+      fprintf(stdout, "%-8s ", cdo::get_steptype_name(var.stepType));
 
       // ensemble information
       if (ensembleInfo)
@@ -184,15 +180,15 @@ print_vars_info(int operfunc, bool ensembleInfo, int vlistID, bool xsInfo)
 
       // datatype
       set_text_color(stdout, BLUE);
-      fprintf(stdout, " %-3s", cdo::datatype_to_cstr(var.datatype));
-
-      // memType
-      if (xsInfo) fprintf(stdout, "   %-3s", memtype_to_cstr(var.memType));
+      fprintf(stdout, " %-3s", cdo::datatype_to_cstr(var.dataType));
 
       auto compType = vlistInqVarCompType(vlistID, varID);
       auto isCompressed = (compType != CDI_COMPRESS_NONE);
       fprintf(stdout, "%c ", isCompressed ? (int) comptype_to_name(compType)[0] : ' ');
 
+      // memType
+      if (xsInfo) fprintf(stdout, "   %-3s", memtype_to_cstr(var.memType));
+
       reset_text_color(stdout);
 
       fprintf(stdout, ": ");
@@ -402,9 +398,8 @@ private:
 
 public:
   void
-  init()
+  init() override
   {
-
     auto operatorID = cdo_operator_id();
 
     operfunc = cdo_operator_f1(operatorID);
@@ -415,12 +410,15 @@ public:
   }
 
   void
-  run()
+  run() override
   {
     for (int indf = 0; indf < cdo_stream_cnt(); indf++)
       {
         auto streamID = cdo_open_read(indf);
         auto vlistID = cdo_stream_inq_vlist(streamID);
+
+        VarList varList(vlistID);
+
         auto nsubtypes = vlistNsubtypes(vlistID);
 
         set_text_color(stdout, BRIGHT);
@@ -442,7 +440,7 @@ public:
         reset_text_color(stdout);
         fprintf(stdout, "\n");
 
-        int numVariables = vlistNvars(vlistID);
+        auto numVars = varList.numVars();
         int numRecordsConst = 0;
         int numRecordsVar = 0;
         int numValues = 0;
@@ -451,11 +449,9 @@ public:
         size_t outputSize = 0;
         if (Options::test && xsInfo)
           {
-            VarList varList;
-            varList_init(varList, vlistID);
-            for (int varID = 0; varID < numVariables; ++varID)
+            for (int varID = 0; varID < numVars; ++varID)
               {
-                const auto &var = varList[varID];
+                const auto &var = varList.vars[varID];
 
                 if (var.isConstant)
                   numRecordsConst += var.nlevels;
@@ -468,15 +464,15 @@ public:
                 // auto numBytes = (var.memType == MemType::Double) ? 8 : 4;
                 // memorySize += size * numBytes;
 
-                auto numBits = get_num_input_bits(var.datatype);
+                auto numBits = get_num_input_bits(var.dataType);
                 inputSize += (size * numBits) / 8;
 
-                numBits = get_num_output_bits(var.datatype);
+                numBits = get_num_output_bits(var.dataType);
                 outputSize += (size * numBits) / 8;
               }
           }
 
-        print_vars_info(operfunc, ensembleInfo, vlistID, xsInfo);
+        print_vars_info(operfunc, ensembleInfo, varList, vlistID, xsInfo);
 
         set_text_color(stdout, BRIGHT);
         fprintf(stdout, "   Grid coordinates");
@@ -530,7 +526,7 @@ public:
 
             int numFields = numRecordsVar * numTimesteps + numRecordsConst;
             fprintf(stdout, "%33s : %d\n", "number of fields", numFields);
-            fprintf(stdout, "%33s : %d\n", "number of variables", numVariables);
+            fprintf(stdout, "%33s : %d\n", "number of variables", numVars);
             if (numTimesteps) fprintf(stdout, "%33s : %d\n", "number of timesteps", numTimesteps);
             fprintf(stdout, "%33s : %d\n", "number of values", numTimesteps * numValues);
             // fprintf(stdout, "%33s : %s\n", "required memory", num_values_to_byte_cstr(memorySize));
@@ -543,7 +539,7 @@ public:
   }
 
   void
-  close()
+  close() override
   {
   }
 };
