@@ -11,14 +11,10 @@
      Sorttimestamp    sorttimestamp         Sort all timesteps
 */
 
-#include <algorithm>  // sort
-
 #include <cdi.h>
-#include "julian_date.h"
 
 #include "cdo_options.h"
 #include "process_int.h"
-#include "cdo_vlist.h"
 #include "printinfo.h"
 #include "field_functions.h"
 
@@ -30,18 +26,25 @@ struct TimeInfo
   double datetime;
 };
 
-static bool
-cmpdatetime(const TimeInfo &a, const TimeInfo &b)
-{
-  return a.datetime < b.datetime;
-}
-class ModuleSortTimestamp
+class Sorttimestamp : public Process
 {
+public:
+  using Process::Process;
+  inline static CdoModule module = {
+    .name = "Sorttimestamp",
+    .operators = { { "sorttimestamp" }, { "sorttaxis" } },
+    .aliases = {},
+    .mode = EXPOSED,     // Module mode: 0:intern 1:extern
+    .number = CDI_REAL,  // Allowed number type
+    .constraints = { -1, 1, NoRestriction },
+  };
+  inline static RegisterEntry<Sorttimestamp> registration = RegisterEntry<Sorttimestamp>(module);
+
   int varID, levelID;
   int lasttsID = -1;
   int nalloc = 0;
   int vlistID2 = -1, taxisID2 = -1;
-  int nvars = 0;
+  int numVars = 0;
 
   CdoStreamID streamID1;
   CdoStreamID streamID2;
@@ -51,10 +54,8 @@ class ModuleSortTimestamp
 
 public:
   void
-  init(void *process)
+  init() override
   {
-    cdo_initialize(process);
-
     skipSameTime = getenv_skip_same_time();
 
     if (cdo_operator_argc() == 1)
@@ -68,9 +69,9 @@ public:
   }
 
   void
-  run()
+  run() override
   {
-    FieldVector3D vars;
+    FieldVector3D varsData;
     std::vector<CdiDateTime> vDateTimes;
     auto nfiles = cdo_stream_cnt() - 1;
 
@@ -81,8 +82,7 @@ public:
         auto vlistID1 = cdo_stream_inq_vlist(streamID1);
         auto taxisID1 = vlistInqTaxis(vlistID1);
 
-        VarList varList1;
-        varList_init(varList1, vlistID1);
+        VarList varList1(vlistID1);
 
         if (fileID == 0)
           {
@@ -94,9 +94,9 @@ public:
                 taxisDeleteBounds(taxisID2);
               }
           }
-        else { vlist_compare(vlistID2, vlistID1, CmpVlist::All); }
+        else { vlist_compare(vlistID2, vlistID1, CmpVarList::All); }
 
-        nvars = vlistNvars(vlistID1);
+        numVars = varList1.numVars();
 
         int tsID = 0;
         while (true)
@@ -109,18 +109,18 @@ public:
                 constexpr int NALLOC_INC = 1024;
                 nalloc += NALLOC_INC;
                 vDateTimes.resize(nalloc);
-                vars.resize(nalloc);
+                varsData.resize(nalloc);
               }
 
             vDateTimes[xtsID] = taxisInqVdatetime(taxisID1);
 
-            fields_from_vlist(vlistID1, vars[xtsID]);
+            field2D_init(varsData[xtsID], varList1);
 
             for (int recID = 0; recID < nrecs; ++recID)
               {
                 cdo_inq_record(streamID1, &varID, &levelID);
-                auto &field = vars[xtsID][varID][levelID];
-                field.init(varList1[varID]);
+                auto &field = varsData[xtsID][varID][levelID];
+                field.init(varList1.vars[varID]);
                 cdo_read_record(streamID1, field);
               }
 
@@ -140,12 +140,12 @@ public:
       {
         auto julday = date_to_julday(calendar, cdiDate_get(vDateTimes[tsID].date));
         auto secofday = time_to_sec(cdiTime_get(vDateTimes[tsID].time));
-        const double jdatetime = julday + secofday / 86400.0;
+        double jdatetime = julday + secofday / 86400.0;
         timeinfo[tsID].index = tsID;
         timeinfo[tsID].datetime = jdatetime;
       }
 
-    std::stable_sort(timeinfo.begin(), timeinfo.end(), cmpdatetime);
+    ranges::stable_sort(timeinfo, {}, &TimeInfo::datetime);
 
     vlistDefTaxis(vlistID2, taxisID2);
 
@@ -170,8 +170,8 @@ public:
               {
                 auto lskip = false;
                 auto xtsID2 = timeinfo[lasttsID].index;
-                const auto &field1 = vars[xtsID][0][0];
-                const auto &field2 = vars[xtsID2][0][0];
+                const auto &field1 = varsData[xtsID][0][0];
+                const auto &field2 = varsData[xtsID2][0][0];
                 if (field1.memType == MemType::Float)
                   {
                     if (field1.vec_f == field2.vec_f) lskip = true;
@@ -196,12 +196,13 @@ public:
         taxisDefVdatetime(taxisID2, vDateTimes[xtsID]);
         cdo_def_timestep(streamID2, tsID2++);
 
-        for (varID = 0; varID < nvars; ++varID)
+        VarList varList2(vlistID2);
+
+        for (varID = 0; varID < numVars; ++varID)
           {
-            auto nlevels = zaxisInqSize(vlistInqVarZaxis(vlistID2, varID));
-            for (levelID = 0; levelID < nlevels; ++levelID)
+            for (levelID = 0; levelID < varList2.vars[varID].nlevels; ++levelID)
               {
-                auto &field = vars[xtsID][varID][levelID];
+                auto &field = varsData[xtsID][varID][levelID];
                 if (field.hasData())
                   {
                     cdo_def_record(streamID2, varID, levelID);
@@ -213,20 +214,8 @@ public:
   }
 
   void
-  close()
+  close() override
   {
     cdo_stream_close(streamID2);
-
-    cdo_finish();
   }
 };
-
-void *
-Sorttimestamp(void *process)
-{
-  ModuleSortTimestamp sorttimestamp;
-  sorttimestamp.init(process);
-  sorttimestamp.run();
-  sorttimestamp.close();
-  return nullptr;
-}
