@@ -17,59 +17,33 @@
 #include <cstdlib>
 #include <cassert>
 #include <cdi.h>
-#include "julian_date.h"
 
 #include "cdo_options.h"
+#include "cdo_data.h"
 #include "process_int.h"
 #include "cdo_zaxis.h"
 #include "param_conversion.h"
 #include <mpim_grid.h>
 #include "grid_healpix.h"
 #include "griddes.h"
-#include "constants.h"
 #include "stdnametable.h"
 #include "param_conversion.h"
 
-static constexpr double etopo_scale = 3.0;
-static constexpr double etopo_offset = 11000.0;
-static constexpr unsigned short etopo[] = {
-#include "etopo.dat"
-};
-
-static constexpr double temp_scale = 500.0;
-static constexpr double temp_offset = -220.0;
-static constexpr unsigned short temp[] = {
-#include "temp.dat"
-};
-
-static constexpr double mask_scale = 1.0;
-static constexpr double mask_offset = 0.0;
-static constexpr unsigned short mask[] = {
-#include "mask.dat"
-};
-
-// Some Constants for creating temperatur and pressure for the standard atmosphere
-constexpr double T_ZERO = 213.0;
-constexpr double T_DELTA = 75.0;
-constexpr double SCALEHEIGHT = 10000.0;  // [m]
-
-static double
-std_atm_temperatur(double height)
+static int
+random_init(int operatorID)
 {
-  // Compute the temperatur for the given height (in meters) according to the solution of the hydrostatic atmosphere
-  return (T_ZERO + T_DELTA * std::exp((-1) * (height / SCALEHEIGHT)));
-}
-
-static double
-std_atm_pressure(double height)
-{
-  constexpr double P_ZERO = 1013.25;  // surface pressure [hPa]
-  constexpr double CC_R = 287.05;     // specific gas constant for air
-  constexpr double TMP4PRESSURE = (C_EARTH_GRAV * SCALEHEIGHT) / (CC_R * T_ZERO);
-
-  // Compute the pressure for the given height (in meters) according to the solution of the hydrostatic atmosphere
-  return (P_ZERO
-          * std::exp((-1) * TMP4PRESSURE * std::log((std::exp(height / SCALEHEIGHT) * T_ZERO + T_DELTA) / (T_ZERO + T_DELTA))));
+  unsigned int seed = Options::Random_Seed;
+  operator_input_arg(cdo_operator_enter(operatorID));
+  if (cdo_operator_argc() < 1) cdo_abort("Too few arguments!");
+  if (cdo_operator_argc() > 2) cdo_abort("Too many arguments!");
+  auto gridID = cdo_define_grid(cdo_operator_argv(0));
+  if (cdo_operator_argc() == 2)
+    {
+      auto idum = parameter_to_int(cdo_operator_argv(1));
+      if (idum >= 0 && idum < 0x7FFFFFFF) seed = idum;
+    }
+  std::srand(seed);
+  return gridID;
 }
 
 static void
@@ -95,6 +69,22 @@ conv_generic_grid(int gridID, size_t gridsize, Varray<double> &xvals2D, Varray<d
       }
 }
 
+static int
+generate_full_point_grid_radian(int gridID, Varray<double> &xvals, Varray<double> &yvals)
+{
+  gridID = generate_full_point_grid(gridID);
+  if (!gridHasCoordinates(gridID)) cdo_abort("Target cell center coordinates missing!");
+
+  gridInqXvals(gridID, xvals.data());
+  gridInqYvals(gridID, yvals.data());
+
+  // Convert lat/lon units if required
+  cdo_grid_to_radian(gridID, CDI_XAXIS, xvals, "grid center lon");
+  cdo_grid_to_radian(gridID, CDI_YAXIS, yvals, "grid center lat");
+
+  return gridID;
+}
+
 static size_t
 calc_index_ii(size_t nx, double xval)
 {
@@ -207,61 +197,6 @@ remap_nn_reg2d(size_t nx, size_t ny, const Varray<float> &data, int gridID, Varr
     remap_nn_reg2d_to_nonreg2d(nx, ny, data, gridID, array);
 }
 
-static int
-random_init(int operatorID)
-{
-  unsigned int seed = Options::Random_Seed;
-  operator_input_arg(cdo_operator_enter(operatorID));
-  if (cdo_operator_argc() < 1) cdo_abort("Too few arguments!");
-  if (cdo_operator_argc() > 2) cdo_abort("Too many arguments!");
-  auto gridID = cdo_define_grid(cdo_operator_argv(0));
-  if (cdo_operator_argc() == 2)
-    {
-      auto idum = parameter_to_int(cdo_operator_argv(1));
-      if (idum >= 0 && idum < 0x7FFFFFFF) seed = idum;
-    }
-  std::srand(seed);
-  return gridID;
-}
-
-static void
-random_compute(size_t gridsize, Varray<float> &array)
-{
-  for (size_t i = 0; i < gridsize; ++i) array[i] = ((double) std::rand()) / ((double) RAND_MAX);
-}
-
-static void
-sincos_compute(size_t gridsize, Varray<float> &array, const Varray<double> &xvals, const Varray<double> &yvals)
-{
-  for (size_t i = 0; i < gridsize; ++i) array[i] = std::cos(1.0 * xvals[i]) * std::sin(2.0 * yvals[i]);
-}
-
-static void
-coshill_compute(size_t gridsize, Varray<float> &array, const Varray<double> &xvals, const Varray<double> &yvals)
-{
-  for (size_t i = 0; i < gridsize; ++i) array[i] = 2.0 - std::cos(std::acos(std::cos(xvals[i]) * std::cos(yvals[i])) / 1.2);
-}
-
-static void
-testfield_compute(size_t gridsize, Varray<float> &array, const Varray<double> &xvals, const Varray<double> &yvals)
-{
-  double xyz[3];
-  for (size_t i = 0; i < gridsize; ++i)
-    {
-      gcLLtoXYZ(xvals[i], yvals[i], xyz);
-      auto x = xyz[0];
-      auto y = xyz[1];
-      auto z = xyz[2];
-      array[i] = 1.0 + std::pow(x, 8.0) + std::exp(2.0 * y * y * y) + std::exp(2.0 * x * x) + 10.0 * x * y * z;
-    }
-}
-
-static void
-unpack_data(size_t datasize, Varray<float> &data, double scale, double offset, const unsigned short *zdata)
-{
-  for (size_t i = 0; i < datasize; ++i) data[i] = zdata[i] / scale - offset;
-}
-
 static int
 define_point_grid()
 {
@@ -313,59 +248,65 @@ define_temperature_attributes(int vlistID, int varID)
   cdiDefKeyString(vlistID, varID, CDI_KEY_UNITS, "K");
 }
 
-class ModuleVargen
+class Vargen : public Process
 {
-private:
-  int RANDOM;
-  int SINCOS;
-  int COSHILL;
-  int TESTFIELD;
-  int CONST;
-  int SEQ;
-  int TOPO;
-  int TEMP;
-  int MASK;
-  int STDATM;
+public:
+  using Process::Process;
+  inline static CdoModule module = {
+    .name = "Vargen",
+    // clang-format off
+    .operators = { { "random", 0, 0, "grid description file or name, <seed>", VargenHelp },
+                   { "const", 0, 0, "constant value, grid description file or name", VargenHelp },
+                   { "sincos", 0, 0, "grid description file or name", VargenHelp },
+                   { "coshill", 0, 0, "grid description file or name", VargenHelp },
+                   { "testfield", 0, 0, "grid description file or name", VargenHelp },
+                   { "seq", 0, 0, "start,end,<increment>", VargenHelp },
+                   { "topo", VargenHelp },
+                   { "temp", VargenHelp },
+                   { "mask", VargenHelp },
+                   { "stdatm", 0, 0, "height levels[m]", VargenHelp } },
+    // clang-format on
+    .aliases = { { "for", "seq" } },
+    .mode = EXPOSED,     // Module mode: 0:intern 1:extern
+    .number = CDI_REAL,  // Allowed number type
+    .constraints = { 0, 1, NoRestriction },
+  };
+  inline static RegisterEntry<Vargen> registration = RegisterEntry<Vargen>(module);
 
+  int RANDOM, SINCOS, COSHILL, TESTFIELD, CONST, SEQ, TOPO, TEMP, MASK, STDATM;
+
+private:
   static constexpr size_t nlat = 360, nlon = 720;
   double lon[nlon], lat[nlat];
-  int nlevels = 1;
   int gridID = -1, gridIDdata = -1;
   double rstart = 0.0, rstop = 0.0, rinc = 0.0;
   double rconst = 0.0;
   std::vector<double> levels;
   int ntimesteps;
-  int julday;
 
   int taxisID;
   CdoStreamID streamID;
-  int varID;
   int varID2;
   int vlistID;
   int operatorID;
-  size_t gridsize;
-  Varray<float> array;
-  int nvars;
 
 public:
   void
-  init(void *process)
+  init() override
   {
-    cdo_initialize(process);
+    int nlevels = 1;
 
-    // clang-format off
-    RANDOM    = cdo_operator_add("random",    0, 0, "grid description file or name, <seed>");
-    SINCOS    = cdo_operator_add("sincos",    0, 0, "grid description file or name");
-    COSHILL   = cdo_operator_add("coshill",   0, 0, "grid description file or name");
-    //not usesd todo: make unavailable for non developers
-    TESTFIELD = cdo_operator_add("testfield", 0, 0, "grid description file or name");
-    CONST     = cdo_operator_add("const",     0, 0, "constant value, grid description file or name");
-    SEQ       = cdo_operator_add("seq",       0, 0, "start, end, <increment>");
-    TOPO      = cdo_operator_add("topo",      0, 0, nullptr);
-    TEMP      = cdo_operator_add("temp",      0, 0, nullptr);
-    MASK      = cdo_operator_add("mask",      0, 0, nullptr);
-    STDATM    = cdo_operator_add("stdatm",    0, 0, "height levels [m]");
-    // clang-format on
+    RANDOM = module.get_id("random");
+    SINCOS = module.get_id("sincos");
+    COSHILL = module.get_id("coshill");
+    // not usesd todo: make unavailable for non developers
+    TESTFIELD = module.get_id("testfield");
+    CONST = module.get_id("const");
+    SEQ = module.get_id("seq");
+    TOPO = module.get_id("topo");
+    TEMP = module.get_id("temp");
+    MASK = module.get_id("mask");
+    STDATM = module.get_id("stdatm");
 
     operatorID = cdo_operator_id();
 
@@ -431,7 +372,7 @@ public:
 
     auto timetype = (operatorID == SEQ) ? TIME_VARYING : TIME_CONSTANT;
 
-    varID = vlistDefVar(vlistID, gridID, zaxisID, timetype);
+    auto varID = vlistDefVar(vlistID, gridID, zaxisID, timetype);
     /*
        For the standard atmosphere two output variables are generated: pressure and temperature.
        The first (varID) is pressure, second (varID2) is temperature. Add an additional variable for the standard atmosphere.
@@ -461,20 +402,20 @@ public:
 
     cdo_def_vlist(streamID, vlistID);
 
-    gridsize = gridInqSize(gridID);
-    array = Varray<float>(gridsize);
-
     ntimesteps = (operatorID == SEQ) ? 1.001 + ((rstop - rstart) / rinc) : 1;
     if (operatorID != SEQ) vlistDefNtsteps(vlistID, 0);
-
-    julday = date_to_julday(CALENDAR_PROLEPTIC, 10101);
-
-    nvars = vlistNvars(vlistID);
   }
 
   void
-  run()
+  run() override
   {
+    VarList varList(vlistID);
+
+    auto julday = date_to_julday(CALENDAR_PROLEPTIC, 10101);
+
+    size_t gridsize = gridInqSize(gridID);
+    Varray<float> array(gridsize);
+
     for (int tsID = 0; tsID < ntimesteps; ++tsID)
       {
         auto rval = rstart + rinc * tsID;
@@ -484,63 +425,45 @@ public:
         cdo_def_timestep(streamID, tsID);
 
         // this should either be 1 or 2, two for atmosphere
-        for (varID = 0; varID < nvars; ++varID)
+        for (int varID = 0; varID < varList.numVars(); ++varID)
           {
-            nlevels = zaxisInqSize(vlistInqVarZaxis(vlistID, varID));
-            for (int levelID = 0; levelID < nlevels; ++levelID)
+            for (int levelID = 0; levelID < varList.vars[varID].nlevels; ++levelID)
               {
                 cdo_def_record(streamID, varID, levelID);
 
-                if (operatorID == RANDOM) { random_compute(gridsize, array); }
+                if (operatorID == RANDOM) { cdo::fill_random(array); }
                 else if (operatorID == SINCOS || operatorID == COSHILL || operatorID == TESTFIELD)
                   {
                     Varray<double> xvals(gridsize), yvals(gridsize);
 
                     if (grid_is_distance_generic(gridID)) { conv_generic_grid(gridID, gridsize, xvals, yvals); }
-                    else
-                      {
-                        gridID = generate_full_point_grid(gridID);
-                        if (!gridHasCoordinates(gridID)) cdo_abort("Target cell center coordinates missing!");
-
-                        gridInqXvals(gridID, xvals.data());
-                        gridInqYvals(gridID, yvals.data());
-
-                        // Convert lat/lon units if required
-                        cdo_grid_to_radian(gridID, CDI_XAXIS, xvals, "grid center lon");
-                        cdo_grid_to_radian(gridID, CDI_YAXIS, yvals, "grid center lat");
-                      }
+                    else { gridID = generate_full_point_grid_radian(gridID, xvals, yvals); }
 
                     // clang-format off
-                    if      (operatorID == SINCOS)    sincos_compute(gridsize, array, xvals, yvals);
-                    else if (operatorID == COSHILL)   coshill_compute(gridsize, array, xvals, yvals);
-                    else if (operatorID == TESTFIELD) testfield_compute(gridsize, array, xvals, yvals);
+                    if      (operatorID == SINCOS)    cdo::fill_sincos(array, xvals, yvals);
+                    else if (operatorID == COSHILL)   cdo::fill_coshill(array, xvals, yvals);
+                    else if (operatorID == TESTFIELD) cdo::fill_testfield(array, xvals, yvals);
                     // clang-format on
                   }
-                else if (operatorID == CONST)
-                  {
-                    for (size_t i = 0; i < gridsize; ++i) array[i] = rconst;
-                  }
+                else if (operatorID == CONST) { ranges::fill(array, rconst); }
                 else if (operatorID == TOPO || operatorID == TEMP || operatorID == MASK)
                   {
-                    auto datasize = gridInqSize(gridIDdata);
-                    Varray<float> data(datasize);
+                    Varray<float> data;
 
                     // clang-format off
-                    if      (operatorID == TOPO) unpack_data(datasize, data, etopo_scale, etopo_offset, etopo);
-                    else if (operatorID == TEMP) unpack_data(datasize, data, temp_scale, temp_offset, temp);
-                    else if (operatorID == MASK) unpack_data(datasize, data, mask_scale, mask_offset, mask);
+                    if      (operatorID == TOPO) data = cdo::unpack_data(cdo::topoData);
+                    else if (operatorID == TEMP) data = cdo::unpack_data(cdo::tempData);
+                    else if (operatorID == MASK) data = cdo::unpack_data(cdo::maskData);
                     // clang-format on
 
                     if (gridID != gridIDdata && gridIDdata != -1) { remap_nn_reg2d(nlon, nlat, data, gridID, array); }
-                    else
-                      {
-                        for (size_t i = 0; i < gridsize; ++i) array[i] = data[i];
-                      }
+                    else { array = data; }
                   }
                 else if (operatorID == SEQ) { array[0] = rval; }
                 else if (operatorID == STDATM)
                   {
-                    array[0] = (varID == varID2) ? std_atm_temperatur(levels[levelID]) : std_atm_pressure(levels[levelID]);
+                    array[0]
+                        = (varID == varID2) ? cdo::std_atm_temperatur(levels[levelID]) : cdo::std_atm_pressure(levels[levelID]);
                   }
 
                 cdo_write_record_f(streamID, array.data(), 0);
@@ -550,22 +473,10 @@ public:
   }
 
   void
-  close()
+  close() override
   {
     cdo_stream_close(streamID);
 
     vlistDestroy(vlistID);
-
-    cdo_finish();
   }
 };
-
-void *
-Vargen(void *process)
-{
-  ModuleVargen vargen;
-  vargen.init(process);
-  vargen.run();
-  vargen.close();
-  return nullptr;
-}
