@@ -12,6 +12,7 @@
 
 #include <cdi.h>
 
+#include "arithmetic.h"
 #include "process_int.h"
 #include <mpim_grid.h>
 #include "cdi_lockedIO.h"
@@ -20,6 +21,7 @@
 static void
 var_rms(const Varray<double> &w, const FieldVector &field1, const FieldVector &field2, Field &field3)
 {
+  auto is_EQ = dbl_is_equal;
   auto grid1 = field1[0].grid;
   auto grid2 = field2[0].grid;
   auto missval1 = field1[0].missval;
@@ -30,16 +32,17 @@ var_rms(const Varray<double> &w, const FieldVector &field1, const FieldVector &f
   auto len = gridInqSize(grid1);
   if (len != gridInqSize(grid2)) cdo_abort("fields have different size!");
 
-  // if ( nmiss1 )
+  // if ( numMissVals1 )
   {
     for (size_t k = 0; k < nlev; ++k)
       {
         auto array1 = field1[k].vec_d;
         auto array2 = field2[k].vec_d;
-        for (size_t i = 0; i < len; ++i) /*	  if ( !DBL_IS_EQUAL(w[i], missval1) ) */
+        for (size_t i = 0; i < len; ++i)
+          //	  if ( !is_EQ(w[i], missval1) )
           {
-            rsum = ADDMN(rsum, MULMN(w[i], MULMN(SUBMN(array2[i], array1[i]), SUBMN(array2[i], array1[i]))));
-            rsumw = ADDMN(rsumw, w[i]);
+            rsum = ADDM(rsum, MULM(w[i], MULM(SUBM(array2[i], array1[i]), SUBM(array2[i], array1[i]))));
+            rsumw = ADDM(rsumw, w[i]);
           }
       }
   }
@@ -54,17 +57,29 @@ else
   }
   */
 
-  const double ravg = SQRTMN(DIVMN(rsum, rsumw));
+  auto ravg = SQRTM(DIVM(rsum, rsumw));
 
-  size_t rnmiss = 0;
-  if (DBL_IS_EQUAL(ravg, missval1)) rnmiss++;
+  size_t rnumMissVals = 0;
+  if (is_EQ(ravg, missval1)) rnumMissVals++;
 
   field3.vec_d[0] = ravg;
-  field3.nmiss = rnmiss;
+  field3.numMissVals = rnumMissVals;
 }
 
-class ModuleVarrms
+class Varrms : public Process
 {
+public:
+  using Process::Process;
+  inline static CdoModule module = {
+    .name = "Varrms",
+    .operators = { { "varrms" } },
+    .aliases = {},
+    .mode = EXPOSED,     // Module mode: 0:intern 1:extern
+    .number = CDI_REAL,  // Allowed number type
+    .constraints = { 2, 1, NoRestriction },
+  };
+  inline static RegisterEntry<Varrms> registration = RegisterEntry<Varrms>(module);
+
 private:
   CdoStreamID streamID1;
   CdoStreamID streamID2;
@@ -76,8 +91,8 @@ private:
   int vlistID1;
   int vlistID3;
 
-  FieldVector2D vars1;
-  FieldVector2D vars2;
+  FieldVector2D varsData1;
+  FieldVector2D varsData2;
 
   int nvars;
   int lastgrid = -1;
@@ -87,14 +102,12 @@ private:
 
   Field field3;
   Varray<double> weights;
+  VarList varList1;
 
 public:
   void
-  init(void *process)
+  init() override
   {
-
-    cdo_initialize(process);
-
     operator_check_argc(0);
 
     streamID1 = cdo_open_read(0);
@@ -121,7 +134,7 @@ public:
     taxisID3 = taxisDuplicate(taxisID1);
     vlistDefTaxis(vlistID3, taxisID3);
 
-    auto ngrids = vlistNgrids(vlistID1);
+    auto ngrids = vlistNumGrids(vlistID1);
     int index = 0;
     auto gridID1 = vlistGrid(vlistID1, index);
 
@@ -134,8 +147,10 @@ public:
     streamID3 = cdo_open_write(2);
     cdo_def_vlist(streamID3, vlistID3);
 
-    fields_from_vlist(vlistID1, vars1, FIELD_VEC);
-    fields_from_vlist(vlistID2, vars2, FIELD_VEC);
+    varList1 = VarList(vlistID1);
+    VarList varList2(vlistID2);
+    field2D_init(varsData1, varList1, FIELD_VEC);
+    field2D_init(varsData2, varList2, FIELD_VEC);
 
     auto gridsizemax = vlistGridsizeMax(vlistID1);
     if (needWeights) weights.resize(gridsizemax);
@@ -145,7 +160,7 @@ public:
   }
 
   void
-  run()
+  run() override
   {
     int tsID = 0;
     while (true)
@@ -161,21 +176,21 @@ public:
 
         for (int recID = 0; recID < nrecs; ++recID)
           {
-            size_t nmiss;
+            size_t numMissVals;
             int varID, levelID;
             cdo_inq_record(streamID1, &varID, &levelID);
-            cdo_read_record(streamID1, vars1[varID][levelID].vec_d.data(), &nmiss);
-            if (nmiss) cdo_abort("Missing values unsupported for this operator!");
+            cdo_read_record(streamID1, varsData1[varID][levelID].vec_d.data(), &numMissVals);
+            if (numMissVals) cdo_abort("Missing values unsupported for this operator!");
 
             cdo_inq_record(streamID2, &varID, &levelID);
-            cdo_read_record(streamID2, vars2[varID][levelID].vec_d.data(), &nmiss);
-            if (nmiss) cdo_abort("Missing values unsupported for this operator!");
+            cdo_read_record(streamID2, varsData2[varID][levelID].vec_d.data(), &numMissVals);
+            if (numMissVals) cdo_abort("Missing values unsupported for this operator!");
           }
 
         for (int varID = 0; varID < nvars; ++varID)
           {
             auto wstatus = false;
-            auto gridID = vars1[varID][0].grid;
+            auto gridID = varsData1[varID][0].grid;
             if (needWeights && gridID != lastgrid)
               {
                 lastgrid = gridID;
@@ -185,11 +200,11 @@ public:
             if (wstatus != 0 && tsID == 0 && code != oldcode)
               cdo_warning("Using constant area weights for code %d!", oldcode = code);
 
-            field3.missval = vars1[varID][0].missval;
-            var_rms(weights, vars1[varID], vars2[varID], field3);
+            field3.missval = varsData1[varID][0].missval;
+            var_rms(weights, varsData1[varID], varsData2[varID], field3);
 
             cdo_def_record(streamID3, varID, 0);
-            cdo_write_record(streamID3, field3.vec_d.data(), field3.nmiss);
+            cdo_write_record(streamID3, field3.vec_d.data(), field3.numMissVals);
           }
 
         tsID++;
@@ -197,24 +212,12 @@ public:
   }
 
   void
-  close()
+  close() override
   {
     cdo_stream_close(streamID3);
     cdo_stream_close(streamID2);
     cdo_stream_close(streamID1);
 
     vlistDestroy(vlistID3);
-
-    cdo_finish();
   }
 };
-void *
-Varrms(void *process)
-{
-  ModuleVarrms varrms;
-  varrms.init(process);
-  varrms.run();
-  varrms.close();
-
-  return nullptr;
-}
