@@ -18,7 +18,6 @@
 #include "griddes.h"
 #include <mpim_grid.h>
 #include "remap_vars.h"
-#include "cdo_zaxis.h"
 
 #ifdef HAVE_LIBNETCDF
 static void
@@ -94,7 +93,7 @@ cdf_read_var_double(int ncfileid, const char *name, double *array)
 }
 
 static void
-cdf_read_coordinate_radian(int ncfileid, const char *name, size_t size, Varray<double> &varray)
+cdf_read_coordinate_radian(int ncfileid, const char *name, Varray<double> &varray)
 {
   int ncvarid;
   nce(nc_inq_varid(ncfileid, name, &ncvarid));
@@ -115,10 +114,10 @@ struct RemapAttributes
 struct RemapGridW
 {
   std::string name;
-  int rank;         // rank of the grid
-  size_t ncells;    // total points on the grid
-  size_t ncorners;  // number of corners for each grid cell
-  size_t dims[2];   // size of grid dimension
+  int rank = 0;               // rank of the grid
+  size_t ncells = 0;          // total points on the grid
+  size_t ncorners = 0;        // number of corners for each grid cell
+  size_t dims[2] = { 0, 0 };  // size of grid dimension
 
   std::vector<int> mask;  // flag which cells participate
 
@@ -130,14 +129,8 @@ struct RemapGridW
   Varray<double> cell_area;  // total area of each grid cell
   Varray<double> cell_frac;  // fractional area of grid cells participating in remapping
 
-  RemapGridW() : rank(0), ncells(0), ncorners(0)
-  {
-    dims[0] = 0;
-    dims[1] = 0;
-  }
-
   void
-  setNumCells(size_t ncells_)
+  set_num_cells(size_t ncells_)
   {
     ncells = ncells_;
 
@@ -150,7 +143,7 @@ struct RemapGridW
   }
 
   void
-  setNumCorners(size_t ncorners_)
+  set_num_corners(size_t ncorners_)
   {
     ncorners = ncorners_;
 
@@ -171,11 +164,11 @@ struct RemapVarsW
   RemapMethod mapType{ RemapMethod::UNDEF };  // identifier for remapping method
   NormOpt normOpt{ NormOpt::NONE };           // option for normalization (conserv only)
   size_t numLinks = 0;                        // number of links for remapping
-  size_t num_wts = 0;                         // number of weights used in remapping
+  size_t numWeights = 0;                      // number of weights used in remapping
 
-  std::vector<size_t> srcCellIndices;  // source grid indices for each link
-  std::vector<size_t> tgtCellIndices;  // target grid indices for each link
-  Varray<double> wts;                  // map weights for each link [numLinks*num_wts]
+  Varray<size_t> srcCellIndices;  // source grid indices for each link
+  Varray<size_t> tgtCellIndices;  // target grid indices for each link
+  Varray<double> weights;         // map weights for each link [numLinks*numWeights]
 };
 
 static void
@@ -184,8 +177,8 @@ remapVarsWInit(RemapMethod mapType, int remapOrder, RemapVarsW &rv)
   rv.sort_add = (mapType == RemapMethod::CONSERV_SCRIP);
 
   // Determine the number of weights
-  rv.num_wts = (mapType == RemapMethod::CONSERV_SCRIP) ? 3 : ((mapType == RemapMethod::BICUBIC) ? 4 : 1);
-  if (mapType == RemapMethod::CONSERV && remapOrder == 2) rv.num_wts = 3;
+  rv.numWeights = (mapType == RemapMethod::CONSERV_SCRIP) ? 3 : ((mapType == RemapMethod::BICUBIC) ? 4 : 1);
+  if (mapType == RemapMethod::CONSERV && remapOrder == 2) rv.numWeights = 3;
 }
 
 static NormOpt
@@ -215,17 +208,15 @@ read_remapgrid_scrip(int ncfileid, const std::string &prefix, bool lgridarea, Re
 {
   // Read all variables of the grid
   cdf_read_var_size(ncfileid, (prefix + "_dims").c_str(), 2, grid.dims);
-
   cdf_read_var_int(ncfileid, (prefix + "_imask").c_str(), grid.mask.data());
 
-  cdf_read_coordinate_radian(ncfileid, (prefix + "_center_lat").c_str(), grid.ncells, grid.cell_center_lat);
-  cdf_read_coordinate_radian(ncfileid, (prefix + "_center_lon").c_str(), grid.ncells, grid.cell_center_lon);
+  cdf_read_coordinate_radian(ncfileid, (prefix + "_center_lat").c_str(), grid.cell_center_lat);
+  cdf_read_coordinate_radian(ncfileid, (prefix + "_center_lon").c_str(), grid.cell_center_lon);
 
   if (grid.ncorners)
     {
-      size_t size = grid.ncorners * grid.ncells;
-      cdf_read_coordinate_radian(ncfileid, (prefix + "_corner_lat").c_str(), size, grid.cell_corner_lat);
-      cdf_read_coordinate_radian(ncfileid, (prefix + "_corner_lon").c_str(), size, grid.cell_corner_lon);
+      cdf_read_coordinate_radian(ncfileid, (prefix + "_corner_lat").c_str(), grid.cell_corner_lat);
+      cdf_read_coordinate_radian(ncfileid, (prefix + "_corner_lon").c_str(), grid.cell_corner_lon);
     }
 
   if (lgridarea) cdf_read_var_double(ncfileid, (prefix + "_area").c_str(), grid.cell_area.data());
@@ -241,11 +232,12 @@ read_remapweights_scrip(int ncfileid, RemapVarsW &rv)
   for (size_t i = 0; i < rv.numLinks; ++i) rv.srcCellIndices[i]--;
   for (size_t i = 0; i < rv.numLinks; ++i) rv.tgtCellIndices[i]--;
 
-  cdf_read_var_double(ncfileid, "remap_matrix", rv.wts.data());
+  cdf_read_var_double(ncfileid, "remap_matrix", rv.weights.data());
 }
 
 static RemapSwitches
-readRemapFileScrip(const std::string &remapFile, RemapAttributes &remapAtts, RemapGridW &srcGrid, RemapGridW &tgtGrid, RemapVarsW &rv)
+readRemapFileScrip(const std::string &remapFile, RemapAttributes &remapAtts, RemapGridW &srcGrid, RemapGridW &tgtGrid,
+                   RemapVarsW &rv)
 {
   // The routine reads a NetCDF file to extract remapping info in SCRIP format
 
@@ -298,11 +290,11 @@ readRemapFileScrip(const std::string &remapFile, RemapAttributes &remapAtts, Rem
   if (Options::cdoVerbose) cdo_print("Remapping between: %s and %s", srcGrid.name, tgtGrid.name);
 
   // Read dimension information
-  srcGrid.setNumCells(cdf_read_dimlen(ncfileid, "src_grid_size"));
-  tgtGrid.setNumCells(cdf_read_dimlen(ncfileid, "dst_grid_size"));
+  srcGrid.set_num_cells(cdf_read_dimlen(ncfileid, "src_grid_size"));
+  tgtGrid.set_num_cells(cdf_read_dimlen(ncfileid, "dst_grid_size"));
 
-  srcGrid.setNumCorners(cdf_read_dimlen(ncfileid, "src_grid_corners"));
-  tgtGrid.setNumCorners(cdf_read_dimlen(ncfileid, "dst_grid_corners"));
+  srcGrid.set_num_corners(cdf_read_dimlen(ncfileid, "src_grid_corners"));
+  tgtGrid.set_num_corners(cdf_read_dimlen(ncfileid, "dst_grid_corners"));
 
   srcGrid.rank = cdf_read_dimlen(ncfileid, "src_grid_rank");
   tgtGrid.rank = cdf_read_dimlen(ncfileid, "dst_grid_rank");
@@ -312,14 +304,14 @@ readRemapFileScrip(const std::string &remapFile, RemapAttributes &remapAtts, Rem
 
   rv.numLinks = cdf_read_dimlen(ncfileid, "numLinks");
   //  if ( rv.numLinks == 0 ) cdo_abort("Number of remap links is 0, no remap weights found!");
-  rv.num_wts = cdf_read_dimlen(ncfileid, "num_wgts");
+  rv.numWeights = cdf_read_dimlen(ncfileid, "num_wgts");
 
   // Allocate address and weight arrays
   if (rv.numLinks > 0)
     {
       rv.srcCellIndices.resize(rv.numLinks);
       rv.tgtCellIndices.resize(rv.numLinks);
-      rv.wts.resize(rv.num_wts * rv.numLinks);
+      rv.weights.resize(rv.numWeights * rv.numLinks);
     }
 
   read_remapgrid_scrip(ncfileid, "src_grid", lgridarea, srcGrid);
@@ -379,10 +371,7 @@ cdfWriteVarSize(int ncfileid, int ncvarid, nc_type sizetype, size_t len, size_t
       nce(nc_put_var_int(ncfileid, ncvarid, iarray.data()));
     }
 #ifdef HAVE_NETCDF4
-  else
-    {
-      nce(nc_put_var_ulonglong(ncfileid, ncvarid, (unsigned long long *) array));
-    }
+  else { nce(nc_put_var_ulonglong(ncfileid, ncvarid, (unsigned long long *) array)); }
 #endif
 }
 
@@ -392,7 +381,7 @@ checkRemapFilesize(const RemapGridW &srcGrid, const RemapGridW &tgtGrid, const R
   size_t nlinks = rv.numLinks;
   size_t nele1 = 4 * 8 + 4 + srcGrid.ncorners * 2 * 8;
   size_t nele2 = 4 * 8 + 4 + tgtGrid.ncorners * 2 * 8;
-  size_t filesize = srcGrid.ncells * nele1 + tgtGrid.ncells * nele2 + nlinks * (4 + 4 + rv.num_wts * 8);
+  size_t filesize = srcGrid.ncells * nele1 + tgtGrid.ncells * nele2 + nlinks * (4 + 4 + rv.numWeights * 8);
 
   if (Options::cdoVerbose)
     {
@@ -609,7 +598,7 @@ write_remapfile_scrip(const std::string &remapFile, const RemapAttributes &remap
 
   // Define map size dimensions
   auto nc_numlinks_id = cdf_def_dim(ncfileid, "numLinks", rv.numLinks);
-  auto nc_numwgts_id = cdf_def_dim(ncfileid, "num_wgts", rv.num_wts);
+  auto nc_numwgts_id = cdf_def_dim(ncfileid, "num_wgts", rv.numWeights);
 
   // Define mapping arrays
   auto nc_srcadd_id = cdf_def_var(ncfileid, "src_address", sizetype, 1, &nc_numlinks_id);
@@ -633,7 +622,7 @@ write_remapfile_scrip(const std::string &remapFile, const RemapAttributes &remap
 
   cdfWriteVarSize(ncfileid, nc_srcadd_id, sizetype, rv.numLinks, rv.srcCellIndices.data());
   cdfWriteVarSize(ncfileid, nc_dstadd_id, sizetype, rv.numLinks, rv.tgtCellIndices.data());
-  cdfWriteVarDouble(ncfileid, nc_rmpmatrix_id, rv.wts.data());
+  cdfWriteVarDouble(ncfileid, nc_rmpmatrix_id, rv.weights.data());
 
   nce(nc_close(ncfileid));
 
@@ -646,6 +635,8 @@ static void
 check_areas(size_t n_a, const Varray<double> &area_a, const Varray<double> &area_b, size_t n_s, const Varray<size_t> &col,
             const Varray<size_t> &row, const Varray<double> &S)
 {
+  if (area_a.size() == 0 || area_b.size() == 0) return;
+
   Varray<double> sum(n_a, 0.0);
 
   // sum weighted ratio of true areas
@@ -655,8 +646,13 @@ check_areas(size_t n_a, const Varray<double> &area_a, const Varray<double> &area
 
   // check that sums are equal to 1 (within tolerance of 1.e-6)
   for (size_t i = 0; i < n_a; ++i)  // loop over all source cells
-    if (std::fabs(sum[i] - 1.0) > 1.e-6) printf("ERROR\n");
-  printf("OK\n");
+    {
+      if (std::fabs(sum[i] - 1.0) > 1.e-6)
+        {
+          cdo_print("srcCellIndex=%zu areaSum=%g", i + 1, sum[i]);
+          break;
+        }
+    }
 }
 
 static void
@@ -669,7 +665,7 @@ verify_weights(const std::string &remapFile)
   (void) readRemapFileScrip(remapFile, remapAtts, srcGrid, tgtGrid, remapVars);
 
   check_areas(srcGrid.ncells, srcGrid.cell_area, tgtGrid.cell_area, remapVars.numLinks, remapVars.srcCellIndices,
-              remapVars.tgtCellIndices, remapVars.wts);
+              remapVars.tgtCellIndices, remapVars.weights);
 }
 
 static void
@@ -696,25 +692,33 @@ write_remap_scrip(const std::string &remapFileIn, const std::string &remapFileOu
 }
 #endif
 
-class ModuleVerifyweights
+class Verifyweights : public Process
 {
+public:
+  using Process::Process;
+  inline static CdoModule module = {
+    .name = "Verifyweights",
+    .operators = { { "verifyweights", 0, 1, "remap filename" }, { "writeremapscrip", 0, 2, "input and output remap filename" } },
+    .aliases = {},
+    .mode = EXPOSED,     // Module mode: 0:intern 1:extern
+    .number = CDI_REAL,  // Allowed number type
+    .constraints = { 0, 0, NoRestriction },
+  };
+  inline static RegisterEntry<Verifyweights> registration = RegisterEntry<Verifyweights>(module);
+
+  int VERIFYWEIGHTS, WRITEREMAPSCRIP;
   int operatorID;
   std::string remapFileIn;
 
-  int VERIFYWEIGHTS, WRITEREMAPSCRIP;
-
 public:
-
   void
-  init(void *process)
+  init() override
   {
-    cdo_initialize(process);
-
 #ifndef HAVE_LIBNETCDF
     cdo_abort("NetCDF support not compiled in!");
 #else
-    VERIFYWEIGHTS = cdo_operator_add("verifyweights", 0, 1, "remap file name");
-    WRITEREMAPSCRIP = cdo_operator_add("writeremapscrip", 0, 2, "input and output remap file name");
+    VERIFYWEIGHTS = module.get_id("verifyweights");
+    WRITEREMAPSCRIP = module.get_id("writeremapscrip");
 
     operatorID = cdo_operator_id();
 
@@ -727,7 +731,7 @@ public:
   }
 
   void
-  run()
+  run() override
   {
 #ifdef HAVE_LIBNETCDF
     if (operatorID == VERIFYWEIGHTS)
@@ -738,18 +742,7 @@ public:
   }
 
   void
-  close()
+  close() override
   {
-    cdo_finish();
   }
 };
-
-void *
-Verifyweights(void *process)
-{
-  ModuleVerifyweights verifyweights;
-  verifyweights.init(process);
-  verifyweights.run();
-  verifyweights.close();
-  return nullptr;
-}
