@@ -5,14 +5,9 @@
 
 */
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
-
 #include <cdi.h>
 
 #include "process_int.h"
-#include "cdo_vlist.h"
 #include "param_conversion.h"
 #include "cdo_options.h"
 #include "cimdOmp.h"
@@ -20,8 +15,20 @@
 #include "pmlist.h"
 #include "fill_1d.h"
 
-class ModuleVertfillmiss
+class Vertfillmiss : public Process
 {
+public:
+  using Process::Process;
+  inline static CdoModule module = {
+    .name = "Vertfillmiss",
+    .operators = { { "vertfillmiss", VertfillmissHelp } },
+    .aliases = {},
+    .mode = EXPOSED,     // Module mode: 0:intern 1:extern
+    .number = CDI_REAL,  // Allowed number type
+    .constraints = { 1, 1, NoRestriction },
+  };
+  inline static RegisterEntry<Vertfillmiss> registration = RegisterEntry<Vertfillmiss>(module);
+
 private:
   CdoStreamID streamID1;
   CdoStreamID streamID2;
@@ -31,11 +38,10 @@ private:
   int taxisID1;
   int taxisID2;
 
-  VarList varList;
-  FieldVector2D vars;
+  VarList varList1;
+  FieldVector2D varsData;
 
   int calendar;
-  int nvars;
 
   FillMethod method{ FillMethod::Nearest };
   int limit{ 0 };
@@ -44,22 +50,6 @@ private:
   Varray2D<double> dataValues2D;
   Varray<double> levelValues;
 
-  void
-  add_operators()
-  {
-    cdo_operator_add("vertfillmiss", 0, 0, nullptr);
-  }
-
-  FillMethod
-  convert_fillmethod(const std::string &methodStr)
-  {
-    auto fillMethod = string_to_fillmethod(methodStr);
-
-    if (fillMethod == FillMethod::Undefined) cdo_abort("method=%s unsupported!", methodStr);
-
-    return fillMethod;
-  }
-
   void
   get_parameter()
   {
@@ -70,7 +60,7 @@ private:
 
         KVList kvlist;
         kvlist.name = cdo_module_name();
-        if (kvlist.parse_arguments(pargc, pargv) != 0) cdo_abort("Parse error!");
+        if (kvlist.parse_arguments(pargv) != 0) cdo_abort("Parse error!");
         if (Options::cdoVerbose) kvlist.print();
 
         for (const auto &kv : kvlist)
@@ -81,7 +71,7 @@ private:
             const auto &value = kv.values[0];
 
             // clang-format off
-            if      (key == "method")   method = convert_fillmethod(parameter_to_word(value));
+            if      (key == "method")   method = convert<FillMethod>(value);
             else if (key == "limit")    limit = parameter_to_int(value);
             else if (key == "max_gaps") maxGaps = parameter_to_int(value);
             else cdo_abort("Invalid parameter key >%s<!", key);
@@ -92,12 +82,8 @@ private:
 
 public:
   void
-  init(void *process)
+  init() override
   {
-    cdo_initialize(process);
-
-    add_operators();
-
     get_parameter();
     limit = std::max(limit, 0);
     maxGaps = std::max(maxGaps, 0);
@@ -116,25 +102,23 @@ public:
 
     calendar = taxisInqCalendar(taxisID1);
 
-    varList_init(varList, vlistID1);
-
-    nvars = vlistNvars(vlistID1);
+    varList1 = VarList(vlistID1);
 
     int maxLevels = 0;
-    for (auto &var : varList) maxLevels = std::max(maxLevels, var.nlevels);
+    for (auto &var : varList1.vars) maxLevels = std::max(maxLevels, var.nlevels);
 
     dataValues2D = Varray2D<double>(Threading::ompNumThreads);
     for (auto &dataValues : dataValues2D) dataValues.resize(maxLevels);
 
     levelValues.resize(maxLevels);
 
-    fields_from_vlist(vlistID1, vars);
+    field2D_init(varsData, varList1);
   }
 
   void
   fillmiss(int varID)
   {
-    const auto &var = varList[varID];
+    const auto &var = varList1.vars[varID];
     auto fieldMemType = var.memType;
     auto gridsize = var.gridsize;
     auto numLevels = var.nlevels;
@@ -151,9 +135,9 @@ public:
         auto &dataValues = dataValues2D[ompthID];
 
         if (fieldMemType == MemType::Float)
-          for (int levelID = 0; levelID < numLevels; ++levelID) dataValues[levelID] = vars[varID][levelID].vec_f[i];
+          for (int levelID = 0; levelID < numLevels; ++levelID) dataValues[levelID] = varsData[varID][levelID].vec_f[i];
         else
-          for (int levelID = 0; levelID < numLevels; ++levelID) dataValues[levelID] = vars[varID][levelID].vec_d[i];
+          for (int levelID = 0; levelID < numLevels; ++levelID) dataValues[levelID] = varsData[varID][levelID].vec_d[i];
 
         if (method == FillMethod::Nearest)
           fill_1d_nearest(numLevels, levelValues, dataValues, missval, limit, maxGaps);
@@ -165,15 +149,16 @@ public:
           fill_1d_backward(numLevels, dataValues, missval, limit, maxGaps);
 
         if (fieldMemType == MemType::Float)
-          for (int levelID = 0; levelID < numLevels; ++levelID) vars[varID][levelID].vec_f[i] = dataValues[levelID];
+          for (int levelID = 0; levelID < numLevels; ++levelID) varsData[varID][levelID].vec_f[i] = dataValues[levelID];
         else
-          for (int levelID = 0; levelID < numLevels; ++levelID) vars[varID][levelID].vec_d[i] = dataValues[levelID];
+          for (int levelID = 0; levelID < numLevels; ++levelID) varsData[varID][levelID].vec_d[i] = dataValues[levelID];
       }
   }
 
   void
-  run()
+  run() override
   {
+    auto numVars = varList1.numVars();
     int tsID = 0;
     while (true)
       {
@@ -185,29 +170,28 @@ public:
 
         for (int recID = 0; recID < nrecs; ++recID)
           {
-            int varID, levelID;
-            cdo_inq_record(streamID1, &varID, &levelID);
-            auto &field = vars[varID][levelID];
-            field.init(varList[varID]);
+            auto [varID, levelID] = cdo_inq_record(streamID1);
+            auto &field = varsData[varID][levelID];
+            field.init(varList1.vars[varID]);
             cdo_read_record(streamID1, field);
           }
 
-        for (int varID = 0; varID < nvars; ++varID)
+        for (int varID = 0; varID < numVars; ++varID)
           {
-            int numLevels = varList[varID].nlevels;
+            int numLevels = varList1.vars[varID].nlevels;
             if (numLevels > 1)
               {
                 size_t numMissVals = 0;
-                for (int levelID = 0; levelID < numLevels; ++levelID) { numMissVals += vars[varID][levelID].nmiss; }
+                for (int levelID = 0; levelID < numLevels; ++levelID) { numMissVals += varsData[varID][levelID].numMissVals; }
                 if (numMissVals > 0) fillmiss(varID);
               }
           }
 
-        for (int varID = 0; varID < nvars; ++varID)
+        for (int varID = 0; varID < numVars; ++varID)
           {
-            for (int levelID = 0; levelID < varList[varID].nlevels; ++levelID)
+            for (int levelID = 0; levelID < varList1.vars[varID].nlevels; ++levelID)
               {
-                auto &field = vars[varID][levelID];
+                auto &field = varsData[varID][levelID];
                 if (field.hasData())
                   {
                     cdo_def_record(streamID2, varID, levelID);
@@ -222,22 +206,9 @@ public:
   }
 
   void
-  close()
+  close() override
   {
     cdo_stream_close(streamID2);
     cdo_stream_close(streamID1);
-
-    cdo_finish();
   }
 };
-
-void *
-Vertfillmiss(void *process)
-{
-  ModuleVertfillmiss verfillmiss;
-  verfillmiss.init(process);
-  verfillmiss.run();
-  verfillmiss.close();
-
-  return nullptr;
-}
