@@ -16,7 +16,6 @@
 #include <iostream>
 #include <algorithm>
 #include <vector>
-#include <limits>
 
 #ifdef HAVE_EXECINFO_H
 #include <execinfo.h>
@@ -30,13 +29,14 @@
 #include <cfenv>
 #include <sys/stat.h>
 #include <sys/resource.h>  // getrlimit
-#include <unistd.h> /* sysconf */
+#include <unistd.h>        /* sysconf */
 #include <cstring>
 #include <cstdlib>
 #include <csignal>
 
 #include <cdi.h>
 
+#include "cpp_lib.h"
 #include "cdo_getopt.h"
 #include "cdo_rlimit.h"
 #include <mpim_grid.h>
@@ -44,7 +44,6 @@
 #include "cdo_default_values.h"
 #include "param_conversion.h"
 #include "progress.h"
-
 #include "module_info.h"
 #include "percentiles.h"
 #include "util_wildcards.h"
@@ -58,7 +57,6 @@
 #include "cdo_output.h"
 #include "cdo_features.h"
 #include "cdo_zaxis.h"
-#include "compare.h"
 #include "table.h"
 #include "datetime.h"
 #include "remap_grid_cell_search.h"
@@ -70,8 +68,9 @@
 static ProcessManager g_processManager;
 
 void
-cdo_exit()
+cdo_exit(std::string msg = "")
 {
+  (void) msg;
   g_processManager.kill_processes();
   exit(EXIT_FAILURE);
 }
@@ -178,6 +177,17 @@ cdo_version()
 #ifdef CXX_VERSION
   fprintf(fp, "CXX version : %s\n", CXX_VERSION);
 #endif
+  fprintf(fp, "CXX library :");
+#ifdef HAVE_LIB_RANGES
+  fprintf(fp, " ranges");
+#endif
+#ifdef HAVE_LIB_RANGES_ZIP
+  fprintf(fp, " ranges_zip");
+#endif
+#ifdef HAVE_LIB_MDSPAN
+  fprintf(fp, " mdspan");
+#endif
+  fprintf(fp, "\n");
 #endif
 #ifdef C_COMPILER
   fprintf(fp, "C Compiler: %s\n", C_COMPILER);
@@ -241,8 +251,8 @@ print_category(const std::string &p_category, FILE *p_target)
 static void
 cdo_usage()
 {
-  FILE *target = stderr;
-  std::string pad = CLIOptions::pad_size_terminal('-');
+  auto target = stderr;
+  auto pad = CLIOptions::pad_size_terminal('-');
   fprintf(target, "%s", pad.c_str());
   fprintf(target, "  Usage : cdo  [Options]  Operator1  [-Operator2  [-OperatorN]]\n");
   pad = CLIOptions::pad_size_terminal('-');
@@ -293,11 +303,7 @@ cdo_init_is_tty()
   fstat(0, &statbuf);
   if (S_ISCHR(statbuf.st_mode)) cdo::stdinIsTerminal = true;
   fstat(1, &statbuf);
-  if (S_ISCHR(statbuf.st_mode))
-    {
-      cdo::stdoutIsTerminal = true;
-      progress::stdoutIsTerminal = true;
-    }
+  if (S_ISCHR(statbuf.st_mode)) { cdo::stdoutIsTerminal = true; }
   fstat(2, &statbuf);
   if (S_ISCHR(statbuf.st_mode)) cdo::stderrIsTerminal = true;
 }
@@ -972,7 +978,7 @@ predefined_tables(int p_padding)
   constexpr int id_padding = 4;
   int padding = p_padding + id_padding;
   int numTables = tableInqNumber();
-  std::string tables{"Predefined tables: "};
+  std::string tables{ "Predefined tables: " };
   for (int id = 0; id < numTables; id++)
     {
       if (id % 7 == 6) tables += std::string("\n") + std::string(padding, ' ');
@@ -1246,19 +1252,19 @@ setup_cli_options()
 
   CLIOptions::option("help", "h")
       ->describe_argument("operator")
-      ->add_effect([&](const std::string &operator_name) { cdo_print_help(Factory::get_help(operator_name)); })
+      ->add_effect([&](const std::string &operator_name) { cdo_print_help(operator_name); })
       ->on_empty_argument([]() { cdo_usage(); })
       ->aborts_program(true)
       ->set_category("Help")
       ->add_help("Shows either help information for the given operator or the usage of CDO.");
 
   CLIOptions::option("history")
-      ->add_effect([&]() { Options::CDO_Append_History = 1; })
+      ->add_effect([&]() { Options::CDO_Append_History = true; })
       ->set_category("History")
       ->add_help("Do append to NetCDF \"history\" global attribute.");
 
   CLIOptions::option("no_history")
-      ->add_effect([&]() { Options::CDO_Append_History = 0; })
+      ->add_effect([&]() { Options::CDO_Append_History = false; })
       ->set_category("History")
       ->add_help("Do not append to NetCDF \"history\" global attribute.");
 
@@ -1402,7 +1408,6 @@ setup_cli_options()
       ->add_effect([&]() {
         Options::silentMode = true;
         MpMO::enable_silent_mode(Options::silentMode);
-        progress::silentMode = true;
       })
       ->set_category("Output")
       ->add_help("Silent mode.");
