@@ -14,6 +14,8 @@
 
 #include <cdi.h>
 
+#include <map>
+
 #ifdef HAVE_NETCDF_META_H
 #include <netcdf_meta.h>
 #endif
@@ -45,9 +47,10 @@ extern "C"
 }
 #endif
 
-#include <stdio.h>
+#include <cstdio>
+#include <cstring>
+#include <iostream>
 
-#include "process_int.h"
 #include "cdo_features.h"
 #include "cdo_rlimit.h"
 #include "cimdOmp.h"
@@ -57,13 +60,93 @@ extern "C"
 
 extern "C" size_t getMemorySize(void);
 
+namespace CdoFeatures
+{
+
+#ifdef HAVE_LIBHDF5
+constexpr bool have_hdf5 = true;
+#else
+constexpr bool have_hdf5 = false;
+#endif
+
+#ifdef HAVE_LIBCGRIBEX
+constexpr bool have_cgribex = true;
+#else
+constexpr bool have_cgribex = false;
+#endif
+
+#ifdef HAVE_LIBCMOR
+constexpr bool have_cmor = true;
+#else
+constexpr bool have_cmor = false;
+#endif
+
+#ifdef HAVE_LIBMAGICS
+constexpr bool have_magics = true;
+#else
+constexpr bool have_magics = false;
+#endif
+
+#ifdef _OPENMP
+constexpr bool have_openmp = true;
+#else
+constexpr bool have_openmp = false;
+#endif
+
+#ifdef HAVE_LIBPROJ
+constexpr bool have_proj = true;
+#else
+constexpr bool have_proj = false;
+#endif
+
+#ifdef HAVE_LIBPTHREAD
+constexpr bool have_threads = true;
+#else
+constexpr bool have_threads = false;
+#endif
+
+#ifdef HAVE_WORDEXP_H
+constexpr bool have_wordexp = true;
+#else
+constexpr bool have_wordexp = false;
+#endif
+
+#ifdef HIRLAM_EXTENSIONS
+constexpr bool has_hirlam_extensions = true;
+#else
+constexpr bool has_hirlam_extensions = false;
+#endif
+
+static std::map<std::string, std::pair<std::string, bool>> configMap
+    = { { "has-srv", { "SERVICE", cdiHaveFiletype(CDI_FILETYPE_SRV) } },
+        { "has-ext", { "EXTRA", cdiHaveFiletype(CDI_FILETYPE_EXT) } },
+        { "has-ieg", { "IEG", cdiHaveFiletype(CDI_FILETYPE_IEG) } },
+        { "has-grb", { "GRIB 1", cdiHaveFiletype(CDI_FILETYPE_GRB) } },
+        { "has-grb1", { "GRIB 1", cdiHaveFiletype(CDI_FILETYPE_GRB) } },
+        { "has-grb2", { "GRIB 2", cdiHaveFiletype(CDI_FILETYPE_GRB2) } },
+        { "has-nc", { "NetCDF", cdiHaveFiletype(CDI_FILETYPE_NC) } },
+        { "has-nc2", { "NetCDF 2", cdiHaveFiletype(CDI_FILETYPE_NC2) } },
+        { "has-nc4", { "NetCDF 4", cdiHaveFiletype(CDI_FILETYPE_NC4) } },
+        { "has-nc4c", { "NetCDF 4 classic", cdiHaveFiletype(CDI_FILETYPE_NC4C) } },
+        { "has-nc5", { "NetCDF 5", cdiHaveFiletype(CDI_FILETYPE_NC5) } },
+        { "has-nczarr", { "NetCDF 4 zarr", cdiHaveFiletype(CDI_FILETYPE_NCZARR) } },
+        { "has-hdf5", { "HDF5", have_hdf5 } },
+        { "has-cgribex", { "CGRIBEX", have_cgribex } },
+        { "has-cmor", { "CMOR", have_cmor } },
+        { "has-magics", { "MAGICS", have_magics } },
+        { "has-openmp", { "OPENMP", have_openmp } },
+        { "has-proj", { "PROJ", have_proj } },
+        { "has-threads", { "PTHREADS", have_threads } },
+        { "has-wordexp", { "WORDEXP", have_wordexp } },
+        { "has-hirlam_extensions", { "HIRLAM_EXTENSIONS", has_hirlam_extensions } } };
+
 void
 cdo_print_features(void)
 {
   constexpr size_t gigaByte = 1024 * 1024 * 1024;
   auto fp = stdout;
   fprintf(fp, "Features: ");
-  size_t rssCur = (size_t)cdo::get_rss_cur() / gigaByte;
+  size_t rssCur = (size_t) cdo::get_rss_cur() / gigaByte;
   size_t memorySize = getMemorySize() / gigaByte;
   if (rssCur > 0 && rssCur < memorySize) fprintf(fp, "%zu/", rssCur);
   if (memorySize > 0) fprintf(fp, "%zuGB ", memorySize);
@@ -73,10 +156,23 @@ cdo_print_features(void)
   if (numProcs < concurrentThreads) fprintf(fp, "%u/", numProcs);
 #endif
   fprintf(fp, "%uthreads", concurrentThreads);
+#ifdef HAVE_OPENMP4
+  auto numDevices = omp_get_num_devices();
+  if (numDevices > 0) fprintf(fp, " %ddevices", numDevices);
+#endif
   fprintf(fp, " c++%d", (int) ((__cplusplus - 200000) / 100));
+#ifdef __FAST_MATH__
+  fprintf(fp, "/fastmath");
+#endif
 #ifdef _OPENMP
   fprintf(fp, " OpenMP");
-#if defined(HAVE_OPENMP45)
+#if defined(HAVE_OPENMP52)
+  fprintf(fp, "52");
+#elif defined(HAVE_OPENMP51)
+  fprintf(fp, "51");
+#elif defined(HAVE_OPENMP5)
+  fprintf(fp, "5");
+#elif defined(HAVE_OPENMP45)
   fprintf(fp, "45");
 #elif defined(HAVE_OPENMP4)
   fprintf(fp, "4");
@@ -244,73 +340,20 @@ cdo_print_libraries(void)
   fprintf(fp, "\n");
 }
 
+void
+print_argument_options()
+{
+  fprintf(stdout, "Available config option:\n");
+  fprintf(stdout, "\n");
+  for (const auto &entry : configMap)
+    fprintf(stdout, "  %-12s  whether %s is enabled\n", entry.first.c_str(), entry.second.first.c_str());
+}
+
 int
 cdo_print_config(const std::string &option)
 {
   int status = EXIT_SUCCESS;
 
-  std::map<std::string, std::pair<std::string, bool>> configMap;
-
-  // clang-format off
-  configMap["has-srv"]               = {"SERVICE",           cdiHaveFiletype(CDI_FILETYPE_SRV)};
-  configMap["has-ext"]               = {"EXTRA",             cdiHaveFiletype(CDI_FILETYPE_EXT)};
-  configMap["has-ieg"]               = {"IEG",               cdiHaveFiletype(CDI_FILETYPE_IEG)};
-  configMap["has-grb"]               = {"GRIB 1",            cdiHaveFiletype(CDI_FILETYPE_GRB)};
-  configMap["has-grb1"]              = {"GRIB 1",            cdiHaveFiletype(CDI_FILETYPE_GRB)};
-  configMap["has-grb2"]              = {"GRIB 2",            cdiHaveFiletype(CDI_FILETYPE_GRB2)};
-  configMap["has-nc"]                = {"NetCDF",            cdiHaveFiletype(CDI_FILETYPE_NC)};
-  configMap["has-nc2"]               = {"NetCDF 2",          cdiHaveFiletype(CDI_FILETYPE_NC2)};
-  configMap["has-nc4"]               = {"NetCDF 4",          cdiHaveFiletype(CDI_FILETYPE_NC4)};
-  configMap["has-nc4c"]              = {"NetCDF 4 classic",  cdiHaveFiletype(CDI_FILETYPE_NC4C)};
-  configMap["has-nc5"]               = {"NetCDF 5",          cdiHaveFiletype(CDI_FILETYPE_NC5)};
-  configMap["has-nczarr"]            = {"NetCDF 4 zarr",     cdiHaveFiletype(CDI_FILETYPE_NCZARR)};
-  configMap["has-hdf5"]              = {"HDF5",              false};
-  configMap["has-cgribex"]           = {"CGRIBEX",           false};
-  configMap["has-cmor"]              = {"CMOR",              false};
-  configMap["has-magics"]            = {"MAGICS",            false};
-  configMap["has-openmp"]            = {"OPENMP",            false};
-  configMap["has-proj"]              = {"PROJ",              false};
-  configMap["has-threads"]           = {"PTHREADS",          false};
-  configMap["has-wordexp"]           = {"WORDEXP",           false};
-  configMap["has-hirlam_extensions"] = {"HIRLAM_EXTENSIONS", false};
-  // clang-format on
-
-#ifdef HAVE_LIBHDF5
-  configMap["has-hdf5"].second = true;
-#endif
-
-#ifdef HAVE_LIBCGRIBEX
-  configMap["has-cgribex"].second = true;
-#endif
-
-#ifdef HAVE_LIBCMOR
-  configMap["has-cmor"].second = true;
-#endif
-
-#ifdef HAVE_LIBMAGICS
-  configMap["has-magics"].second = true;
-#endif
-
-#ifdef _OPENMP
-  configMap["has-openmp"].second = true;
-#endif
-
-#ifdef HAVE_LIBPROJ
-  configMap["has-proj"].second = true;
-#endif
-
-#ifdef HAVE_LIBPTHREAD
-  configMap["has-threads"].second = true;
-#endif
-
-#ifdef HAVE_WORDEXP_H
-  configMap["has-wordexp"].second = true;
-#endif
-
-#ifdef HIRLAM_EXTENSIONS
-  configMap["has-hirlam_extensions"].second = true;
-#endif
-
   if ("all-json" == option || "all" == option)
     {
       std::cout << "{";
@@ -338,10 +381,7 @@ cdo_print_config(const std::string &option)
         {
           fprintf(stdout, "unknown config option: %s\n", option.c_str());
           fprintf(stdout, "\n");
-          fprintf(stdout, "Available config option:\n");
-          fprintf(stdout, "\n");
-          for (const auto &entry : configMap)
-            fprintf(stdout, "  %-12s  whether %s is enabled\n", entry.first.c_str(), entry.second.first.c_str());
+          print_argument_options();
 
           status = EXIT_FAILURE;
         }
@@ -349,3 +389,4 @@ cdo_print_config(const std::string &option)
 
   return status;
 }
+};  // namespace CdoFeatures
