@@ -9,6 +9,7 @@
 
 #include "cdo_options.h"
 #include "cdo_output.h"
+#include "cdo_varlist.h"
 #include "compare.h"
 #include "cdi_lockedIO.h"
 #include "varray.h"
@@ -17,18 +18,15 @@ int
 cdo_read_timestepmask(const char *maskfile, std::vector<bool> &imask)
 {
   auto streamID = stream_open_read_locked(maskfile);
-  const auto vlistID = streamInqVlist(streamID);
+  auto vlistID = streamInqVlist(streamID);
+  VarList varList(vlistID);
+  const auto &var0 = varList.vars[0];
 
-  const auto nvars = vlistNvars(vlistID);
-  if (nvars > 1) cdo_abort("timestepmask %s contains more than one variable!", maskfile);
+  if (varList.numVars() > 1) cdo_abort("timestepmask %s contains more than one variable!", maskfile);
+  if (var0.nlevels > 1) cdo_abort("timestepmask %s has more than one level!", maskfile);
+  if (var0.gridsize > 1) cdo_abort("timestepmask %s has more than one gridpoint!", maskfile);
 
-  const auto nlev = zaxisInqSize(vlistInqVarZaxis(vlistID, 0));
-  if (nlev > 1) cdo_abort("timestepmask %s has more than one level!", maskfile);
-
-  const auto gridsize = gridInqSize(vlistInqVarGrid(vlistID, 0));
-  if (gridsize > 1) cdo_abort("timestepmask %s has more than one gridpoint!", maskfile);
-
-  auto nts = vlistNtsteps(vlistID);
+  auto nts = varList.numSteps();
   if (nts == -1)
     {
       nts = 0;
@@ -53,12 +51,12 @@ cdo_read_timestepmask(const char *maskfile, std::vector<bool> &imask)
       if (nrecs != 1) cdo_abort("Internal error; unexprected number of records!");
 
       int varID, levelID;
-      size_t nmiss;
+      size_t numMissVals;
       double value;
       streamInqRecord(streamID, &varID, &levelID);
-      streamReadRecord(streamID, &value, &nmiss);
+      streamReadRecord(streamID, &value, &numMissVals);
 
-      imask[tsID] = !(nmiss || IS_EQUAL(value, 0));
+      imask[tsID] = !(numMissVals || IS_EQUAL(value, 0));
 
       tsID++;
     }
@@ -71,25 +69,21 @@ cdo_read_timestepmask(const char *maskfile, std::vector<bool> &imask)
 static void
 read_one_field(const char *text, const char *filename, Varray<double> &array)
 {
-  const auto streamID = stream_open_read_locked(filename);
-  const auto vlistID = streamInqVlist(streamID);
+  auto streamID = stream_open_read_locked(filename);
+  auto vlistID = streamInqVlist(streamID);
+  VarList varList(vlistID);
+  const auto &var0 = varList.vars[0];
 
-  const auto nvars = vlistNvars(vlistID);
-  if (nvars > 1) cdo_abort("%s file %s contains more than one variable!", text, filename);
+  if (varList.numVars() > 1) cdo_abort("%s file %s contains more than one variable!", text, filename);
+  if (var0.nlevels > 1) cdo_abort("%s file %s has more than one level!", text, filename);
+  if (varList.numRecords() != 1) cdo_abort("%s file %s contains more than one field!", text, filename);
 
-  const auto nlev = zaxisInqSize(vlistInqVarZaxis(vlistID, 0));
-  if (nlev > 1) cdo_abort("%s file %s has more than one level!", text, filename);
-
-  const auto nrecs = streamInqTimestep(streamID, 0);
-  if (nrecs != 1) cdo_abort("%s file %s contains more than one field!", text, filename);
-
-  const auto gridsize = gridInqSize(vlistInqVarGrid(vlistID, 0));
-  array.resize(gridsize);
+  array.resize(var0.gridsize);
 
   int varID, levelID;
-  size_t nmiss;
+  size_t numMissVals;
   streamInqRecord(streamID, &varID, &levelID);
-  streamReadRecord(streamID, array.data(), &nmiss);
+  streamReadRecord(streamID, array.data(), &numMissVals);
   streamClose(streamID);
 }
 
@@ -99,7 +93,7 @@ cdo_read_mask(const char *maskfile, std::vector<bool> &imask)
   Varray<double> array;
   read_one_field("Mask", maskfile, array);
 
-  const auto gridsize = array.size();
+  auto gridsize = array.size();
   imask.resize(gridsize);
 
   for (size_t i = 0; i < gridsize; ++i) imask[i] = is_not_equal(array[i], 0);
@@ -113,7 +107,7 @@ cdo_read_index(const char *indexfile, std::vector<int> &index)
   Varray<double> array;
   read_one_field("Index", indexfile, array);
 
-  const auto gridsize = array.size();
+  auto gridsize = array.size();
   index.resize(gridsize);
 
   for (size_t i = 0; i < gridsize; ++i) index[i] = (int) std::lround(array[i]) - 1;
