@@ -8,17 +8,14 @@
 
 */
 
-#include <assert.h>
-#include <string.h>
+#include <cstring>
 
 #include <cdi.h>
 
-#include "cdo_vlist.h"
 #include <mpim_grid.h>
 #include "process_int.h"
 #include "ecacore.h"
 #include "ecautil.h"
-#include "util_files.h"
 #include "util_date.h"
 #include "datetime.h"
 #include "field_functions.h"
@@ -66,9 +63,11 @@ eca1(const ECA_REQUEST_1 &request)
   auto ivlistID = cdo_stream_inq_vlist(istreamID);
   auto ovlistID = vlistCreate();
 
-  auto gridID = vlistInqVarGrid(ivlistID, FIRST_VAR_ID);
-  auto zaxisID = vlistInqVarZaxis(ivlistID, FIRST_VAR_ID);
-  auto missval = vlistInqVarMissval(ivlistID, FIRST_VAR_ID);
+  VarList varList1(ivlistID);
+  const auto varFirst = varList1.vars[FIRST_VAR_ID];
+  auto gridID = varFirst.gridID;
+  auto zaxisID = varFirst.zaxisID;
+  auto missval = varFirst.missval;
 
   auto varID = vlistDefVar(ovlistID, gridID, zaxisID, TIME_VARYING);
 
@@ -163,7 +162,7 @@ eca1(const ECA_REQUEST_1 &request)
                   if (request.var1.f2 != &vfarnum2)
                     {
                       field_fill(var12[levelID], missval);
-                      var12[levelID].nmiss = gridsize;
+                      var12[levelID].numMissVals = gridsize;
                     }
                   field_fill(samp1[levelID], missval);
                   if (!samp2[levelID].empty()) field_fill(samp2[levelID], 0.0);
@@ -171,13 +170,13 @@ eca1(const ECA_REQUEST_1 &request)
                   if (IS_SET(request.var2.h2)) field_fill(var21[levelID], missval);
                   if (IS_SET(request.var2.h3)) field_fill(var23[levelID], missval);
 
-                  samp1[levelID].nmiss = gridsize;
-                  if (IS_SET(request.var1.f3)) var13[levelID].nmiss = gridsize;
-                  if (IS_SET(request.var2.h2)) var21[levelID].nmiss = gridsize;
-                  if (IS_SET(request.var2.h3)) var23[levelID].nmiss = gridsize;
+                  samp1[levelID].numMissVals = gridsize;
+                  if (IS_SET(request.var1.f3)) var13[levelID].numMissVals = gridsize;
+                  if (IS_SET(request.var2.h2)) var21[levelID].numMissVals = gridsize;
+                  if (IS_SET(request.var2.h3)) var23[levelID].numMissVals = gridsize;
                 }
 
-              cdo_read_record(istreamID, field1.vec_d.data(), &field1.nmiss);
+              cdo_read_record(istreamID, field1.vec_d.data(), &field1.numMissVals);
               field1.grid = var12[levelID].grid;
               field1.missval = var12[levelID].missval;
 
@@ -186,14 +185,14 @@ eca1(const ECA_REQUEST_1 &request)
               if (IS_SET(request.var2.h2))
                 {
                   field2.vec_d = field1.vec_d;
-                  field2.nmiss = field1.nmiss;
+                  field2.numMissVals = field1.numMissVals;
                   field2.grid = field1.grid;
                   field2.missval = field1.missval;
                 }
 
               if (IS_SET(request.var1.f1)) request.var1.f1(field1, request.var1.f1arg);
 
-              if (field1.nmiss || !samp2[levelID].empty())
+              if (field1.numMissVals || !samp2[levelID].empty())
                 {
                   if (samp2[levelID].empty())
                     {
@@ -213,7 +212,7 @@ eca1(const ECA_REQUEST_1 &request)
               if (IS_SET(request.var1.f3) && request.var1.f2 == &vfarnum2)
                 {
                   varray_copy(gridsize, var12[levelID].vec_d, field3.vec_d);
-                  field3.nmiss = var12[levelID].nmiss;
+                  field3.numMissVals = var12[levelID].numMissVals;
                   field3.grid = var12[levelID].grid;
                   field3.missval = var12[levelID].missval;
                 }
@@ -226,7 +225,7 @@ eca1(const ECA_REQUEST_1 &request)
                   if (IS_NOT_SET(request.var2.h2))
                     {
                       varray_copy(gridsize, var12[levelID].vec_d, field2.vec_d);
-                      field2.nmiss = var12[levelID].nmiss;
+                      field2.numMissVals = var12[levelID].numMissVals;
                       field2.grid = var12[levelID].grid;
                       field2.missval = var12[levelID].missval;
                     }
@@ -253,18 +252,18 @@ eca1(const ECA_REQUEST_1 &request)
                       auto len = field1.size;
                       if (len != field2.size) cdo_abort("Fields have different size (%s)", __func__);
 
-                      if (field2.nmiss)
+                      if (field2.numMissVals)
                         {
                           for (size_t i = 0; i < len; ++i)
                             {
-                              if (!dbl_is_equal(array2[i], missval2) && !dbl_is_equal(array2[i], 0.0)) array1[i] = 0.0;
+                              if (dbl_is_not_equal(array2[i], missval2) && dbl_is_not_equal(array2[i], 0.0)) array1[i] = 0.0;
                             }
                         }
                       else
                         {
                           for (size_t i = 0; i < len; ++i)
                             {
-                              if (!dbl_is_equal(array2[i], 0.0)) array1[i] = 0.0;
+                              if (dbl_is_not_equal(array2[i], 0.0)) array1[i] = 0.0;
                             }
                         }
                       request.var1.f3(var13[levelID], field3);
@@ -327,7 +326,7 @@ eca1(const ECA_REQUEST_1 &request)
           cdo_def_timestep(ostreamID, otsID);
         }
 
-      if (otsID && vlistInqVarTimetype(ivlistID, FIRST_VAR_ID) == TIME_CONSTANT) continue;
+      if (otsID && varFirst.timeType == TIME_CONSTANT) continue;
 
       varID = 0;
       for (int levelID = 0; levelID < nlevels; ++levelID)
@@ -337,7 +336,7 @@ eca1(const ECA_REQUEST_1 &request)
           vfarsel(rvar, samp1[levelID]);
 
           cdo_def_record(ostreamID, varID, levelID);
-          cdo_write_record(ostreamID, rvar.vec_d.data(), rvar.nmiss);
+          cdo_write_record(ostreamID, rvar.vec_d.data(), rvar.numMissVals);
         }
 
       if (IS_SET(request.var2.h2) || IS_SET(request.var2.h3))
@@ -350,7 +349,7 @@ eca1(const ECA_REQUEST_1 &request)
               vfarsel(rvar, samp1[levelID]);
 
               cdo_def_record(ostreamID, varID, levelID);
-              cdo_write_record(ostreamID, rvar.vec_d.data(), rvar.nmiss);
+              cdo_write_record(ostreamID, rvar.vec_d.data(), rvar.numMissVals);
             }
         }
 
@@ -380,12 +379,17 @@ eca2(const ECA_REQUEST_2 &request)
   auto ivlistID2 = cdo_stream_inq_vlist(istreamID2);
   auto ovlistID = vlistCreate();
 
-  vlist_compare(ivlistID1, ivlistID2, CmpVlist::All);
+  VarList varList1(ivlistID1);
+  VarList varList2(ivlistID2);
+  const auto var1First = varList1.vars[FIRST_VAR_ID];
+  const auto var2First = varList2.vars[FIRST_VAR_ID];
 
-  auto gridID = vlistInqVarGrid(ivlistID1, FIRST_VAR_ID);
-  auto zaxisID = vlistInqVarZaxis(ivlistID1, FIRST_VAR_ID);
-  auto missval1 = vlistInqVarMissval(ivlistID1, FIRST_VAR_ID);
-  auto missval2 = vlistInqVarMissval(ivlistID2, FIRST_VAR_ID);
+  varList_compare(varList1, varList2);
+
+  auto gridID = var1First.gridID;
+  auto zaxisID = var1First.zaxisID;
+  auto missval1 = var1First.missval;
+  auto missval2 = var2First.missval;
 
   varID = vlistDefVar(ovlistID, gridID, zaxisID, TIME_VARYING);
 
@@ -428,7 +432,7 @@ eca2(const ECA_REQUEST_2 &request)
   field2.resize(gridsize);
   field3.resize(gridsize);
   constexpr int MaxDays = 373;
-  FieldVector2D vars2[MaxDays];
+  FieldVector2D varsData2[MaxDays];
 
   auto nlevels = zaxisInqSize(zaxisID);
 
@@ -461,15 +465,15 @@ eca2(const ECA_REQUEST_2 &request)
       auto dayOfYear = decode_day_of_year(ivDateTime.date);
       if (dayOfYear < 0 || dayOfYear >= MaxDays) cdo_abort("Day %d out of range!", dayOfYear);
 
-      if (!vars2[dayOfYear].size()) fields_from_vlist(ivlistID2, vars2[dayOfYear], FIELD_VEC);
+      if (!varsData2[dayOfYear].size()) field2D_init(varsData2[dayOfYear], varList2, FIELD_VEC);
 
       for (int recID = 0; recID < nrecs; ++recID)
         {
           int levelID;
           cdo_inq_record(istreamID2, &varID, &levelID);
           if (varID != FIRST_VAR_ID) continue;
-          auto &rvar = vars2[dayOfYear][0][levelID];
-          cdo_read_record(istreamID2, rvar.vec_d.data(), &rvar.nmiss);
+          auto &rvar = varsData2[dayOfYear][0][levelID];
+          cdo_read_record(istreamID2, rvar.vec_d.data(), &rvar.numMissVals);
         }
 
       itsID++;
@@ -490,7 +494,7 @@ eca2(const ECA_REQUEST_2 &request)
           auto ivDateTime = taxisInqVdatetime(itaxisID1);
 
           auto dayOfYear = decode_day_of_year(ivDateTime.date);
-          if (!vars2[dayOfYear].size()) cdo_abort("Input streams have different time values!");
+          if (!varsData2[dayOfYear].size()) cdo_abort("Input streams have different time values!");
 
           if (numSets == 0) inDateTime21 = ivDateTime;
 
@@ -512,7 +516,7 @@ eca2(const ECA_REQUEST_2 &request)
                   if (request.var1.f4 != &vfarnum2)
                     {
                       field_fill(var14[levelID], missval1);
-                      var14[levelID].nmiss = gridsize;
+                      var14[levelID].numMissVals = gridsize;
                     }
                   field_fill(samp1[levelID], missval1);
                   field_fill(samp2[levelID], missval1);
@@ -521,20 +525,20 @@ eca2(const ECA_REQUEST_2 &request)
                   if (IS_SET(request.var1.f5)) field_fill(var15[levelID], missval1);
                   if (IS_SET(request.var2.h2)) field_fill(var22[levelID], missval1);
 
-                  samp1[levelID].nmiss = gridsize;
-                  samp2[levelID].nmiss = gridsize;
-                  if (request.var1.epilog == PERCENT_OF_TOTAL_AMOUNT) total[levelID].nmiss = gridsize;
-                  if (IS_SET(request.var1.f5)) var15[levelID].nmiss = gridsize;
-                  if (IS_SET(request.var2.h2)) var22[levelID].nmiss = gridsize;
+                  samp1[levelID].numMissVals = gridsize;
+                  samp2[levelID].numMissVals = gridsize;
+                  if (request.var1.epilog == PERCENT_OF_TOTAL_AMOUNT) total[levelID].numMissVals = gridsize;
+                  if (IS_SET(request.var1.f5)) var15[levelID].numMissVals = gridsize;
+                  if (IS_SET(request.var2.h2)) var22[levelID].numMissVals = gridsize;
                 }
 
-              cdo_read_record(istreamID1, field1.vec_d.data(), &field1.nmiss);
+              cdo_read_record(istreamID1, field1.vec_d.data(), &field1.numMissVals);
               field1.grid = gridID;
               field1.missval = missval1;
-              field2.grid = vars2[dayOfYear][0][levelID].grid;
-              field2.nmiss = vars2[dayOfYear][0][levelID].nmiss;
+              field2.grid = varsData2[dayOfYear][0][levelID].grid;
+              field2.numMissVals = varsData2[dayOfYear][0][levelID].numMissVals;
               field2.missval = missval2;
-              field_copy(vars2[dayOfYear][0][levelID], field2);
+              field_copy(varsData2[dayOfYear][0][levelID], field2);
 
               vfarnum(samp1[levelID], field1);
               vfarnum(samp2[levelID], field2);
@@ -544,7 +548,7 @@ eca2(const ECA_REQUEST_2 &request)
               if (IS_SET(request.var1.f1)) request.var1.f1(field1, request.var1.f1arg);
               if (IS_SET(request.var1.f2)) request.var1.f2(field2, request.var1.f2arg);
 
-              if (field1.nmiss || !samp3[levelID].empty())
+              if (field1.numMissVals || !samp3[levelID].empty())
                 {
                   if (samp3[levelID].empty())
                     {
@@ -561,7 +565,7 @@ eca2(const ECA_REQUEST_2 &request)
               if (IS_SET(request.var1.f5) && request.var1.f4 == &vfarnum2)
                 {
                   varray_copy(gridsize, var14[levelID].vec_d, field3.vec_d);
-                  field3.nmiss = var14[levelID].nmiss;
+                  field3.numMissVals = var14[levelID].numMissVals;
                   field3.grid = var14[levelID].grid;
                   field3.missval = var14[levelID].missval;
                 }
@@ -572,7 +576,7 @@ eca2(const ECA_REQUEST_2 &request)
               if (IS_SET(request.var2.h2))
                 {
                   varray_copy(gridsize, var14[levelID].vec_d, field2.vec_d);
-                  field2.nmiss = var14[levelID].nmiss;
+                  field2.numMissVals = var14[levelID].numMissVals;
                   field2.grid = var14[levelID].grid;
                   field2.missval = var14[levelID].missval;
 
@@ -592,18 +596,18 @@ eca2(const ECA_REQUEST_2 &request)
                       auto len = field1.size;
                       if (len != field3.size) cdo_abort("Fields have different size (%s)", __func__);
 
-                      if (field1.nmiss)
+                      if (field1.numMissVals)
                         {
                           for (size_t i = 0; i < len; ++i)
                             {
-                              if (!dbl_is_equal(array2[i], missvaltemp) && !dbl_is_equal(array2[i], 0.0)) array1[i] = 0.0;
+                              if (dbl_is_not_equal(array2[i], missvaltemp) && dbl_is_not_equal(array2[i], 0.0)) array1[i] = 0.0;
                             }
                         }
                       else
                         {
                           for (size_t i = 0; i < len; ++i)
                             {
-                              if (!dbl_is_equal(array2[i], 0.0)) array1[i] = 0.0;
+                              if (dbl_is_not_equal(array2[i], 0.0)) array1[i] = 0.0;
                             }
                         }
                       request.var1.f5(var15[levelID], field3, request.var1.f5arg);
@@ -673,7 +677,7 @@ eca2(const ECA_REQUEST_2 &request)
           cdo_def_timestep(ostreamID, otsID);
         }
 
-      if (otsID && vlistInqVarTimetype(ivlistID1, FIRST_VAR_ID) == TIME_CONSTANT) continue;
+      if (otsID && var1First.timeType == TIME_CONSTANT) continue;
 
       varID = 0;
       for (int levelID = 0; levelID < nlevels; ++levelID)
@@ -684,7 +688,7 @@ eca2(const ECA_REQUEST_2 &request)
           vfarsel(rvar, samp2[levelID]);
 
           cdo_def_record(ostreamID, varID, levelID);
-          cdo_write_record(ostreamID, rvar.vec_d.data(), rvar.nmiss);
+          cdo_write_record(ostreamID, rvar.vec_d.data(), rvar.numMissVals);
         }
 
       if (IS_SET(request.var2.h2))
@@ -698,7 +702,7 @@ eca2(const ECA_REQUEST_2 &request)
               vfarsel(rvar, samp2[levelID]);
 
               cdo_def_record(ostreamID, varID, levelID);
-              cdo_write_record(ostreamID, rvar.vec_d.data(), rvar.nmiss);
+              cdo_write_record(ostreamID, rvar.vec_d.data(), rvar.numMissVals);
             }
         }
 
@@ -730,11 +734,15 @@ eca3(const ECA_REQUEST_3 &request)
   auto ivlistID2 = cdo_stream_inq_vlist(istreamID2);
   auto ovlistID = vlistCreate();
 
-  vlist_compare(ivlistID1, ivlistID2, CmpVlist::All);
+  VarList varList1(ivlistID1);
+  VarList varList2(ivlistID2);
+  const auto var1First = varList1.vars[FIRST_VAR_ID];
 
-  auto gridID = vlistInqVarGrid(ivlistID1, FIRST_VAR_ID);
-  auto zaxisID = vlistInqVarZaxis(ivlistID1, FIRST_VAR_ID);
-  auto missval = vlistInqVarMissval(ivlistID1, FIRST_VAR_ID);
+  varList_compare(varList1, varList2);
+
+  auto gridID = var1First.gridID;
+  auto zaxisID = var1First.zaxisID;
+  auto missval = var1First.missval;
 
   varID = vlistDefVar(ovlistID, gridID, zaxisID, TIME_VARYING);
 
@@ -823,15 +831,15 @@ eca3(const ECA_REQUEST_3 &request)
                       var1[levelID].vec_d[i] = missval;
                       var2[levelID].vec_d[i] = missval;
                     }
-                  var1[levelID].nmiss = gridsize;
-                  var2[levelID].nmiss = gridsize;
+                  var1[levelID].numMissVals = gridsize;
+                  var2[levelID].numMissVals = gridsize;
                 }
 
-              cdo_read_record(istreamID1, field1.vec_d.data(), &field1.nmiss);
+              cdo_read_record(istreamID1, field1.vec_d.data(), &field1.numMissVals);
               field1.grid = var1[levelID].grid;
               field1.missval = var1[levelID].missval;
 
-              cdo_read_record(istreamID2, field2.vec_d.data(), &field2.nmiss);
+              cdo_read_record(istreamID2, field2.vec_d.data(), &field2.numMissVals);
               field2.grid = var1[levelID].grid;
               field2.missval = var1[levelID].missval;
 
@@ -861,13 +869,13 @@ eca3(const ECA_REQUEST_3 &request)
           cdo_def_timestep(ostreamID, otsID);
         }
 
-      if (otsID && vlistInqVarTimetype(ivlistID1, FIRST_VAR_ID) == TIME_CONSTANT) continue;
+      if (otsID && var1First.timeType == TIME_CONSTANT) continue;
 
       varID = 0;
       for (int levelID = 0; levelID < nlevels; ++levelID)
         {
           cdo_def_record(ostreamID, varID, levelID);
-          cdo_write_record(ostreamID, var1[levelID].vec_d.data(), var1[levelID].nmiss);
+          cdo_write_record(ostreamID, var1[levelID].vec_d.data(), var1[levelID].numMissVals);
         }
 
       if (nrecs == 0) break;
@@ -885,7 +893,7 @@ fldhvs(const FieldVector &fieldVector, const size_t nlevels)
 {
   for (size_t level = 0; level < nlevels; level++)
     {
-      if (fieldVector[level].nmiss != fieldVector[level].size) return true;
+      if (fieldVector[level].numMissVals != fieldVector[level].size) return true;
     }
 
   return false;
@@ -911,12 +919,16 @@ eca4(const ECA_REQUEST_4 &request)
   auto ivlistID2 = cdo_stream_inq_vlist(istreamID2);
   auto ovlistID = vlistCreate();
 
-  int gridID = vlistInqVarGrid(ivlistID1, FIRST_VAR_ID);
-  if (gridInqSize(gridID) != gridInqSize(vlistInqVarGrid(ivlistID2, FIRST_VAR_ID)))
-    cdo_abort("Grid sizes of the input fields do not match!");
+  VarList varList1(ivlistID1);
+  VarList varList2(ivlistID2);
+  const auto var1First = varList1.vars[FIRST_VAR_ID];
+  const auto var2First = varList2.vars[FIRST_VAR_ID];
 
-  auto zaxisID = vlistInqVarZaxis(ivlistID1, FIRST_VAR_ID);
-  auto missval = vlistInqVarMissval(ivlistID1, FIRST_VAR_ID);
+  auto gridID = var1First.gridID;
+  if (var1First.gridsize != var2First.gridsize) cdo_abort("Grid sizes of the input fields do not match!");
+
+  auto zaxisID = var1First.zaxisID;
+  auto missval = var1First.missval;
 
   auto ovarID1 = vlistDefVar(ovlistID, gridID, zaxisID, TIME_VARYING);
 
@@ -1008,9 +1020,9 @@ eca4(const ECA_REQUEST_4 &request)
     {
       int levelID;
       cdo_inq_record(istreamID2, &varID, &levelID);
-      cdo_read_record(istreamID2, mask.vec_d.data(), &mask.nmiss);
+      cdo_read_record(istreamID2, mask.vec_d.data(), &mask.numMissVals);
       mask.grid = gridID;
-      mask.missval = vlistInqVarMissval(ivlistID2, 0);
+      mask.missval = var2First.missval;
 
       request.s3(mask, request.s3arg);
     }
@@ -1057,11 +1069,11 @@ eca4(const ECA_REQUEST_4 &request)
                   field_fill(startDateWithHist[0][levelID], missval);
                   field_fill(endDateWithHist[0][levelID], missval);
 
-                  gslDuration[levelID].nmiss = 0;
-                  gslFirstDay[levelID].nmiss = 0;
+                  gslDuration[levelID].numMissVals = 0;
+                  gslFirstDay[levelID].numMissVals = 0;
                   // reinitialize the current year
-                  startDateWithHist[0][levelID].nmiss = gridsize;
-                  endDateWithHist[0][levelID].nmiss = gridsize;
+                  startDateWithHist[0][levelID].numMissVals = gridsize;
+                  endDateWithHist[0][levelID].numMissVals = gridsize;
                 }
               // init the history ONCE
               if (0 == itsID)
@@ -1069,13 +1081,13 @@ eca4(const ECA_REQUEST_4 &request)
                   field_fill(startDateWithHist[1][levelID], missval);
                   field_fill(endDateWithHist[1][levelID], missval);
 
-                  startDateWithHist[1][levelID].nmiss = gridsize;
-                  endDateWithHist[1][levelID].nmiss = gridsize;
+                  startDateWithHist[1][levelID].numMissVals = gridsize;
+                  endDateWithHist[1][levelID].numMissVals = gridsize;
                 }
 
-              cdo_read_record(istreamID1, fieldGt.vec_d.data(), &fieldGt.nmiss);
+              cdo_read_record(istreamID1, fieldGt.vec_d.data(), &fieldGt.numMissVals);
               fieldLt.vec_d = fieldGt.vec_d;
-              fieldLt.nmiss = fieldGt.nmiss;
+              fieldLt.numMissVals = fieldGt.numMissVals;
               fieldGt.grid = startCount[levelID].grid;
               fieldGt.missval = startCount[levelID].missval;
               fieldLt.grid = startCount[levelID].grid;
@@ -1088,20 +1100,20 @@ eca4(const ECA_REQUEST_4 &request)
                   for (size_t i = 0; i < gridsize; ++i)
                     {
                       if (yvals[i] >= 0.0)
-                        if (!dbl_is_equal(startCount[levelID].vec_d[i], missval))
+                        if (dbl_is_not_equal(startCount[levelID].vec_d[i], missval))
                           {
                             startCount[levelID].vec_d[i] = missval;
-                            startCount[levelID].nmiss++;
+                            startCount[levelID].numMissVals++;
                           }
                     }
                   // reset southern endCount
                   for (size_t i = 0; i < gridsize; ++i)
                     {
                       if (yvals[i] < 0.0)
-                        if (!dbl_is_equal(endCount[levelID].vec_d[i], missval))
+                        if (dbl_is_not_equal(endCount[levelID].vec_d[i], missval))
                           {
                             endCount[levelID].vec_d[i] = missval;
-                            endCount[levelID].nmiss++;
+                            endCount[levelID].numMissVals++;
                           }
                     }
 
@@ -1122,10 +1134,10 @@ eca4(const ECA_REQUEST_4 &request)
                         {
                           if (yvals[i] >= 0.0)
                             {
-                              if (!dbl_is_equal(endCount[levelID].vec_d[i], missval))
+                              if (dbl_is_not_equal(endCount[levelID].vec_d[i], missval))
                                 {
                                   endCount[levelID].vec_d[i] = missval;
-                                  endCount[levelID].nmiss++;
+                                  endCount[levelID].numMissVals++;
                                 }
                             }
                         }
@@ -1139,10 +1151,10 @@ eca4(const ECA_REQUEST_4 &request)
                         {
                           if (yvals[i] < 0.0)
                             {
-                              if (!dbl_is_equal(startCount[levelID].vec_d[i], missval))
+                              if (dbl_is_not_equal(startCount[levelID].vec_d[i], missval))
                                 {
                                   startCount[levelID].vec_d[i] = missval;
-                                  startCount[levelID].nmiss++;
+                                  startCount[levelID].numMissVals++;
                                 }
                             }
                         }
@@ -1236,13 +1248,13 @@ eca4(const ECA_REQUEST_4 &request)
                         }
                     }
                 }
-              // update nmiss for saving data in GRIB
-              startCount[levelID].nmiss = field_num_miss(startCount[levelID]);
-              endCount[levelID].nmiss = field_num_miss(endCount[levelID]);
-              startDateWithHist[1][levelID].nmiss = field_num_miss(startDateWithHist[1][levelID]);
-              startDateWithHist[0][levelID].nmiss = field_num_miss(startDateWithHist[0][levelID]);
-              endDateWithHist[1][levelID].nmiss = field_num_miss(endDateWithHist[1][levelID]);
-              endDateWithHist[0][levelID].nmiss = field_num_miss(endDateWithHist[0][levelID]);
+              // update numMissVals for saving data in GRIB
+              startCount[levelID].numMissVals = field_num_miss(startCount[levelID]);
+              endCount[levelID].numMissVals = field_num_miss(endCount[levelID]);
+              startDateWithHist[1][levelID].numMissVals = field_num_miss(startDateWithHist[1][levelID]);
+              startDateWithHist[0][levelID].numMissVals = field_num_miss(startDateWithHist[0][levelID]);
+              endDateWithHist[1][levelID].numMissVals = field_num_miss(endDateWithHist[1][levelID]);
+              endDateWithHist[0][levelID].numMissVals = field_num_miss(endDateWithHist[0][levelID]);
             }
 
           ovDateTime = ivDateTime;
