@@ -19,7 +19,7 @@
 #include "ecautil.h"
 
 /**
- * Counts the number of nonmissing values. The result of the operation
+ * Counts the number of nonumMissValsing values. The result of the operation
  * is computed according to the following rules:
  *
  * field1  field2  mode  result
@@ -53,7 +53,7 @@ count(Field &field1, const Field &field2, double mode)
   auto len = field1.size;
   if (len != field2.size) cdo_abort("Fields have different size (%s)", __func__);
 
-  if (field1.nmiss)
+  if (field1.numMissVals)
     {
       for (size_t i = 0; i < len; ++i)
         {
@@ -63,7 +63,7 @@ count(Field &field1, const Field &field2, double mode)
               continue;
             }
 
-          if (!dbl_is_equal(array1[i], missval1))
+          if (dbl_is_not_equal(array1[i], missval1))
             {
               if (is_equal(mode, 0.0) || is_equal(mode, 1.0))
                 array1[i] += 1.0;
@@ -81,11 +81,11 @@ count(Field &field1, const Field &field2, double mode)
             }
         }
 
-      field1.nmiss = field_num_miss(field1);
+      field1.numMissVals = field_num_miss(field1);
     }
   else
     {
-      if (field2.nmiss)
+      if (field2.numMissVals)
         {
           for (size_t i = 0; i < len; ++i)
             {
@@ -138,7 +138,7 @@ selcomp(Field &field1, const Field &field2, int (*compare)(double, double))
   auto len = field1.size;
   if (len != field2.size) cdo_abort("Fields have different size (%s)", __func__);
 
-  if (field1.nmiss || field2.nmiss)
+  if (field1.numMissVals || field2.numMissVals)
     {
       for (size_t i = 0; i < len; ++i)
         if (dbl_is_equal(array1[i], missval1) || dbl_is_equal(array2[i], missval2) || !compare(array1[i], array2[i]))
@@ -150,7 +150,7 @@ selcomp(Field &field1, const Field &field2, int (*compare)(double, double))
         if (!compare(array1[i], array2[i])) array1[i] = missval1;
     }
 
-  field1.nmiss = field_num_miss(field1);
+  field1.numMissVals = field_num_miss(field1);
 }
 
 /**
@@ -180,7 +180,7 @@ selcompc(Field &field, double c, int (*compare)(double, double))
     {
       for (size_t i = 0; i < len; ++i) array[i] = missval;
     }
-  else if (field.nmiss)
+  else if (field.numMissVals)
     {
       for (size_t i = 0; i < len; ++i)
         if (dbl_is_equal(array[i], missval) || !compare(array[i], c)) array[i] = missval;
@@ -191,7 +191,7 @@ selcompc(Field &field, double c, int (*compare)(double, double))
         if (!compare(array[i], c)) array[i] = missval;
     }
 
-  field.nmiss = field_num_miss(field);
+  field.numMissVals = field_num_miss(field);
 }
 
 static int
@@ -227,7 +227,7 @@ eq(double a, double b)
 static int
 ne(double a, double b)
 {
-  return !dbl_is_equal(a, b);
+  return dbl_is_not_equal(a, b);
 }
 
 void
@@ -259,7 +259,7 @@ vfarsel(Field &field1, const Field &field2)
   auto len = field1.size;
   if (len != field2.size) cdo_abort("Fields have different gridsize (%s)", __func__);
 
-  if (field2.nmiss)
+  if (field2.numMissVals)
     {
       for (size_t i = 0; i < len; ++i)
         if (dbl_is_equal(array2[i], missval2) || dbl_is_equal(array2[i], 0.0)) array1[i] = missval1;
@@ -270,7 +270,7 @@ vfarsel(Field &field1, const Field &field2)
         if (is_equal(array2[i], 0.0)) array1[i] = missval1;
     }
 
-  field1.nmiss = field_num_miss(field1);
+  field1.numMissVals = field_num_miss(field1);
 }
 
 void
@@ -493,8 +493,8 @@ compute_gsl(int nlevels, size_t gridsize, std::vector<double> &yvals, double mis
 
   for (int levelID = 0; levelID < nlevels; ++levelID)
     {
-      gslDuration[levelID].nmiss = field_num_miss(gslDuration[levelID]);
-      gslFirstDay[levelID].nmiss = field_num_miss(gslFirstDay[levelID]);
+      gslDuration[levelID].numMissVals = field_num_miss(gslDuration[levelID]);
+      gslFirstDay[levelID].numMissVals = field_num_miss(gslFirstDay[levelID]);
     }
 }
 
@@ -511,12 +511,12 @@ write_gsl_stream(CdoStreamID ostreamID, int otaxisID, int otsID, int ovarID1, in
   for (int levelID = 0; levelID < nlevels; ++levelID)
     {
       cdo_def_record(ostreamID, ovarID1, levelID);
-      cdo_write_record(ostreamID, gslDuration[levelID].vec_d.data(), gslDuration[levelID].nmiss);
+      cdo_write_record(ostreamID, gslDuration[levelID].vec_d.data(), gslDuration[levelID].numMissVals);
     }
 
   for (int levelID = 0; levelID < nlevels; ++levelID)
     {
       cdo_def_record(ostreamID, ovarID2, levelID);
-      cdo_write_record(ostreamID, gslFirstDay[levelID].vec_d.data(), gslFirstDay[levelID].nmiss);
+      cdo_write_record(ostreamID, gslFirstDay[levelID].vec_d.data(), gslFirstDay[levelID].numMissVals);
     }
 }
