@@ -260,7 +260,7 @@ param_meta_copy(ParamEntry &out, const ParamEntry &in)
   out.nlat = in.nlat;
   out.nlev = in.nlev;
   out.missval = in.missval;
-  out.nmiss = 0;
+  out.numMissVals = 0;
   out.coord = 0;
   out.hasMV = true;
   out.name = "";
@@ -274,7 +274,7 @@ expr_con_var(int init, int oper, const nodeType *p1, const nodeType *p2)
 {
   auto ngp = (p2->param.ngp > 0) ? p2->param.ngp : 1;
   auto nlev = (p2->param.nlev > 0) ? p2->param.nlev : 1;
-  auto hasMV = (p2->param.nmiss > 0);
+  auto hasMV = (p2->param.numMissVals > 0);
   auto datatype = p2->param.datatype;
   auto missval2 = p2->param.missval;
 
@@ -298,7 +298,7 @@ expr_con_var(int init, int oper, const nodeType *p1, const nodeType *p2)
 
       oper_expr_con_var(oper, hasMV, n, missval2, odat, cval, idat);
 
-      p->param.nmiss = array_num_mv(n, odat, missval2);
+      p->param.numMissVals = array_num_mv(n, odat, missval2);
     }
 
   return p;
@@ -309,7 +309,7 @@ expr_var_con(int init, int oper, const nodeType *p1, const nodeType *p2)
 {
   auto ngp = (p1->param.ngp > 0) ? p1->param.ngp : 1;
   auto nlev = (p1->param.nlev > 0) ? p1->param.nlev : 1;
-  auto hasMV = (p1->param.nmiss > 0);
+  auto hasMV = (p1->param.numMissVals > 0);
   auto datatype = p1->param.datatype;
   auto missval1 = p1->param.missval;
 
@@ -333,7 +333,7 @@ expr_var_con(int init, int oper, const nodeType *p1, const nodeType *p2)
 
       oper_expr_var_con(oper, hasMV, n, missval1, odat, idat, cval);
 
-      p->param.nmiss = array_num_mv(n, odat, missval1);
+      p->param.numMissVals = array_num_mv(n, odat, missval1);
     }
 
   return p;
@@ -343,8 +343,8 @@ static nodeType *
 expr_var_var(int init, int oper, nodeType *p1, nodeType *p2)
 {
   auto px = p1;
-  auto nmiss1 = p1->param.nmiss;
-  auto nmiss2 = p2->param.nmiss;
+  auto numMissVals1 = p1->param.numMissVals;
+  auto numMissVals2 = p2->param.numMissVals;
   auto missval1 = p1->param.missval;
   auto missval2 = p2->param.missval;
 
@@ -411,7 +411,7 @@ expr_var_var(int init, int oper, nodeType *p1, nodeType *p2)
     }
 
   p->param.name = tmpVarName;
-  // printf("%s %s nmiss %zu %zu\n", p->u.var.name.c_str(), px->param.name.c_str(), nmiss1, nmiss2);
+  // printf("%s %s numMissVals %zu %zu\n", p->u.var.name.c_str(), px->param.name.c_str(), numMissVals1, numMissVals2);
 
   if (!init)
     {
@@ -426,7 +426,7 @@ expr_var_var(int init, int oper, nodeType *p1, nodeType *p2)
           const auto idat1 = p1->param.data + loff1;
           const auto idat2 = p2->param.data + loff2;
           auto odat = p->param.data + loff;
-          auto hasMV = (nmiss1 > 0 || nmiss2 > 0);
+          auto hasMV = (numMissVals1 > 0 || numMissVals2 > 0);
 
           if (ngp1 != ngp2)
             {
@@ -447,7 +447,7 @@ expr_var_var(int init, int oper, nodeType *p1, nodeType *p2)
           else { oper_expr_var_var(oper, hasMV, ngp, missval1, missval2, odat, idat1, idat2); }
         }
 
-      p->param.nmiss = array_num_mv(ngp * nlev, p->param.data, missval1);
+      p->param.numMissVals = array_num_mv(ngp * nlev, p->param.data, missval1);
     }
 
   return p;
@@ -480,12 +480,12 @@ ex_copy_var(int init, nodeType *p2, const nodeType *p1)
 
   if (!init)
     {
-      if (copyConstValue1) { varray_fill(ngp2 * nlev2, p2->param.data, p1->param.data[0]); }
+      if (copyConstValue1) { ranges::fill_n(p2->param.data, ngp2 * nlev2, p1->param.data[0]); }
       else
         {
           array_copy(ngp2 * nlev2, p1->param.data, p2->param.data);
           p2->param.missval = p1->param.missval;
-          p2->param.nmiss = p1->param.nmiss;
+          p2->param.numMissVals = p1->param.numMissVals;
         }
     }
 }
@@ -518,7 +518,7 @@ ex_copy_con(int init, nodeType *p2, const nodeType *p1)
   if (!init)
     {
       assert(p2->param.data != nullptr);
-      varray_fill(ngp * nlev, p2->param.data, cval);
+      ranges::fill_n(p2->param.data, ngp * nlev, cval);
     }
 }
 
@@ -670,8 +670,7 @@ ex_fun_std(int funcID, size_t n, bool hasMV, double mv, const double *p1data, do
 }
 
 void
-func_expr_con_var(int funcID, bool hasMV, size_t n, double mv, double *odat, double cval,
-                  const double *idat)
+func_expr_con_var(int funcID, bool hasMV, size_t n, double mv, double *odat, double cval, const double *idat)
 {
   auto &funcEntry = funcSymbolTable[funcID];
   auto exprfunc = (double (*)(double, double)) funcEntry.func;
@@ -697,8 +696,7 @@ func_expr_con_var(int funcID, bool hasMV, size_t n, double mv, double *odat, dou
 }
 
 void
-func_expr_var_con(int funcID, bool hasMV, size_t n, double mv, double *odat, const double *idat,
-                  double cval)
+func_expr_var_con(int funcID, bool hasMV, size_t n, double mv, double *odat, const double *idat, double cval)
 {
   auto &funcEntry = funcSymbolTable[funcID];
   auto exprfunc = (double (*)(double, double)) funcEntry.func;
@@ -724,8 +722,7 @@ func_expr_var_con(int funcID, bool hasMV, size_t n, double mv, double *odat, con
 }
 
 void
-func_expr_var_var(int funcID, bool hasMV, size_t n, double mv1, double mv2, double *odat,
-                  const double *idat1, const double *idat2)
+func_expr_var_var(int funcID, bool hasMV, size_t n, double mv1, double mv2, double *odat, const double *idat1, const double *idat2)
 {
   auto &funcEntry = funcSymbolTable[funcID];
   auto exprfunc = (double (*)(double, double)) funcEntry.func;
@@ -762,7 +759,7 @@ ex_fun_var(int init, int funcID, nodeType *p1)
   auto ngp = (p1->param.ngp > 0) ? p1->param.ngp : 1;
   auto nlev = (p1->param.nlev > 0) ? p1->param.nlev : 1;
   auto nlat = p1->param.nlat;
-  auto nmiss = p1->param.nmiss;
+  auto numMissVals = p1->param.numMissVals;
   auto missval = p1->param.missval;
 
   auto p = new nodeType;
@@ -806,7 +803,7 @@ ex_fun_var(int init, int funcID, nodeType *p1)
       double *pdata = p->param.data;
       double *p1data = p1->param.data;
 
-      if (functype == FT_STD) { ex_fun_std(funcID, ngp * nlev, nmiss, missval, p1data, pdata); }
+      if (functype == FT_STD) { ex_fun_std(funcID, ngp * nlev, numMissVals, missval, p1data, pdata); }
       else if (functype == FT_FLD)
         {
           Field field;
@@ -820,7 +817,7 @@ ex_fun_var(int init, int funcID, nodeType *p1)
           auto exprfunc = (double (*)(const Field &)) funcEntry.func;
           for (size_t k = 0; k < nlev; ++k)
             {
-              fld_field_init(field, nmiss, missval, ngp, p1data + k * ngp, p1->param.weight);
+              fld_field_init(field, numMissVals, missval, ngp, p1data + k * ngp, p1->param.weight);
               pdata[k] = exprfunc(field);
             }
         }
@@ -832,9 +829,9 @@ ex_fun_var(int init, int funcID, nodeType *p1)
           auto exprfunc = (void (*)(const Field &, Field &)) funcEntry.func;
           for (size_t k = 0; k < nlev; ++k)
             {
-              fld_field_init(field1, nmiss, missval, ngp, &p1data[k * ngp], nullptr);
+              fld_field_init(field1, numMissVals, missval, ngp, &p1data[k * ngp], nullptr);
               field1.grid = gridID;
-              fld_field_init(field2, nmiss, missval, nlat, &pdata[k * nlat], nullptr);
+              fld_field_init(field2, numMissVals, missval, nlat, &pdata[k * nlat], nullptr);
               exprfunc(field1, field2);
               array_copy(nlat, field2.vec_d.data(), &pdata[k * nlat]);
             }
@@ -848,7 +845,7 @@ ex_fun_var(int init, int funcID, nodeType *p1)
           for (size_t i = 0; i < ngp; ++i)
             {
               for (size_t k = 0; k < nlev; ++k) field.vec_d[k] = p1data[k * ngp + i];
-              fld_field_init(field, nmiss, missval, nlev, nullptr, nullptr);
+              fld_field_init(field, numMissVals, missval, nlev, nullptr, nullptr);
               pdata[i] = exprfunc(field);
             }
         }
@@ -856,7 +853,7 @@ ex_fun_var(int init, int funcID, nodeType *p1)
       else
         cdo_abort("Intermal error, wrong function type (%d) for %s()!", functype, funcname);
 
-      if (pdata) p->param.nmiss = array_num_mv(p->param.ngp * p->param.nlev, pdata, missval);
+      if (pdata) p->param.numMissVals = array_num_mv(p->param.ngp * p->param.nlev, pdata, missval);
     }
 
   if (p1->isTmpObj) node_delete(p1);
@@ -894,7 +891,7 @@ func_con_var(int init, int funcID, const nodeType *p1, const nodeType *p2)
 
   auto ngp = (p2->param.ngp > 0) ? p2->param.ngp : 1;
   auto nlev = (p2->param.nlev > 0) ? p2->param.nlev : 1;
-  auto hasMV = (p2->param.nmiss > 0);
+  auto hasMV = (p2->param.numMissVals > 0);
   auto missval2 = p2->param.missval;
 
   auto n = ngp * nlev;
@@ -916,7 +913,7 @@ func_con_var(int init, int funcID, const nodeType *p1, const nodeType *p2)
 
       func_expr_con_var(funcID, hasMV, n, missval2, odat, cval, idat);
 
-      p->param.nmiss = array_num_mv(n, odat, missval2);
+      p->param.numMissVals = array_num_mv(n, odat, missval2);
     }
 
   return p;
@@ -931,7 +928,7 @@ func_var_con(int init, int funcID, const nodeType *p1, const nodeType *p2)
 
   auto ngp = (p1->param.ngp > 0) ? p1->param.ngp : 1;
   auto nlev = (p1->param.nlev > 0) ? p1->param.nlev : 1;
-  auto hasMV = (p1->param.nmiss > 0);
+  auto hasMV = (p1->param.numMissVals > 0);
   auto missval1 = p1->param.missval;
 
   auto n = ngp * nlev;
@@ -953,7 +950,7 @@ func_var_con(int init, int funcID, const nodeType *p1, const nodeType *p2)
 
       func_expr_var_con(funcID, hasMV, n, missval1, odat, idat, cval);
 
-      p->param.nmiss = array_num_mv(n, odat, missval1);
+      p->param.numMissVals = array_num_mv(n, odat, missval1);
     }
 
   return p;
@@ -967,8 +964,8 @@ func_var_var(int init, int funcID, nodeType *p1, nodeType *p2)
   if (functype != FT_STD) cdo_abort("Intermal error, wrong function type (%d) for %s()!", functype, funcEntry.name);
 
   auto px = p1;
-  auto nmiss1 = p1->param.nmiss;
-  auto nmiss2 = p2->param.nmiss;
+  auto numMissVals1 = p1->param.numMissVals;
+  auto numMissVals2 = p2->param.numMissVals;
   auto missval1 = p1->param.missval;
   auto missval2 = p2->param.missval;
 
@@ -1035,7 +1032,7 @@ func_var_var(int init, int funcID, nodeType *p1, nodeType *p2)
     }
 
   p->param.name = tmpVarName;
-  // printf("%s %s nmiss %zu %zu\n", p->u.var.name.c_str(), px->param.name.c_str(), nmiss1, nmiss2);
+  // printf("%s %s numMissVals %zu %zu\n", p->u.var.name.c_str(), px->param.name.c_str(), numMissVals1, numMissVals2);
 
   if (!init)
     {
@@ -1050,7 +1047,7 @@ func_var_var(int init, int funcID, nodeType *p1, nodeType *p2)
           const auto idat1 = p1->param.data + loff1;
           const auto idat2 = p2->param.data + loff2;
           auto odat = p->param.data + loff;
-          auto hasMV = (nmiss1 > 0 || nmiss2 > 0);
+          auto hasMV = (numMissVals1 > 0 || numMissVals2 > 0);
 
           if (ngp1 != ngp2)
             {
@@ -1071,7 +1068,7 @@ func_var_var(int init, int funcID, nodeType *p1, nodeType *p2)
           else { func_expr_var_var(funcID, hasMV, ngp, missval1, missval2, odat, idat1, idat2); }
         }
 
-      p->param.nmiss = array_num_mv(ngp * nlev, p->param.data, missval1);
+      p->param.numMissVals = array_num_mv(ngp * nlev, p->param.data, missval1);
     }
 
   return p;
@@ -1141,7 +1138,7 @@ ex_remap(int init, int funcID, nodeType *p1, nodeType *p2)
   auto gridID = p1->param.gridID;
   auto ngp = (p1->param.ngp > 0) ? p1->param.ngp : 1;
   auto nlev = (p1->param.nlev > 0) ? p1->param.nlev : 1;
-  auto nmiss = p1->param.nmiss;
+  auto numMissVals = p1->param.numMissVals;
   auto missval = p1->param.missval;
 
   auto p = new nodeType;
@@ -1173,10 +1170,10 @@ ex_remap(int init, int funcID, nodeType *p1, nodeType *p2)
           auto exprfunc = (void (*)(const Field &, Field &)) funcEntry.func;
           for (size_t k = 0; k < nlev; ++k)
             {
-              fld_field_init(field1, nmiss, missval, ngp, &p1data[k * ngp], nullptr);
+              fld_field_init(field1, numMissVals, missval, ngp, &p1data[k * ngp], nullptr);
               field1.grid = gridID;
               field2.grid = gridID2;
-              fld_field_init(field2, nmiss, missval, ngp2, &pdata[k * ngp2], nullptr);
+              fld_field_init(field2, numMissVals, missval, ngp2, &pdata[k * ngp2], nullptr);
               exprfunc(field1, field2);
               array_copy(ngp2, field2.vec_d.data(), &pdata[k * ngp2]);
             }
@@ -1253,7 +1250,7 @@ ex_fun1c(int init, int funcID, nodeType *p1, nodeType *p2, ParseParamType &parse
 
   auto ngp = (p1->param.ngp > 0) ? p1->param.ngp : 1;
   auto nlev = (p1->param.nlev > 0) ? p1->param.nlev : 1;
-  auto nmiss = p1->param.nmiss;
+  auto numMissVals = p1->param.numMissVals;
   auto missval = p1->param.missval;
 
   auto p = new nodeType;
@@ -1314,8 +1311,8 @@ ex_fun1c(int init, int funcID, nodeType *p1, nodeType *p2, ParseParamType &parse
       pdata = p->param.data;
       const auto p1data = p1->param.data + ngp * levidx;
       array_copy(ngp, p1data, pdata);
-      if (nmiss) nmiss = array_num_mv(ngp, pdata, missval);
-      p->param.nmiss = nmiss;
+      if (numMissVals) numMissVals = array_num_mv(ngp, pdata, missval);
+      p->param.numMissVals = numMissVals;
     }
 
   if (p1->isTmpObj) node_delete(p1);
@@ -1341,7 +1338,7 @@ ex_fun2c(int init, int funcID, nodeType *p1, nodeType *p2, nodeType *p3, ParsePa
 
   auto ngp = (p1->param.ngp > 0) ? p1->param.ngp : 1;
   auto nlev = (p1->param.nlev > 0) ? p1->param.nlev : 1;
-  auto nmiss = p1->param.nmiss;
+  auto numMissVals = p1->param.numMissVals;
   auto missval = p1->param.missval;
 
   auto p = new nodeType;
@@ -1420,8 +1417,8 @@ ex_fun2c(int init, int funcID, nodeType *p1, nodeType *p2, nodeType *p3, ParsePa
       auto paramdata = p->param.data;
       const auto p1data = p1->param.data + ngp * levidx1;
       array_copy(ngp * nlevout, p1data, paramdata);
-      if (nmiss) nmiss = array_num_mv(ngp * nlevout, paramdata, missval);
-      p->param.nmiss = nmiss;
+      if (numMissVals) numMissVals = array_num_mv(ngp * nlevout, paramdata, missval);
+      p->param.numMissVals = numMissVals;
     }
 
   if (p1->isTmpObj) node_delete(p1);
@@ -1476,7 +1473,7 @@ ex_uminus_var(int init, nodeType *p1)
 {
   auto ngp = (p1->param.ngp > 0) ? p1->param.ngp : 1;
   auto nlev = (p1->param.nlev > 0) ? p1->param.nlev : 1;
-  auto nmiss = p1->param.nmiss;
+  auto numMissVals = p1->param.numMissVals;
   auto missval = p1->param.missval;
 
   auto p = new nodeType;
@@ -1493,7 +1490,7 @@ ex_uminus_var(int init, nodeType *p1)
       double *pdata = p->param.data;
       const double *p1data = p1->param.data;
 
-      if (nmiss)
+      if (numMissVals)
         {
           for (size_t i = 0; i < ngp * nlev; ++i) pdata[i] = dbl_is_equal(p1data[i], missval) ? missval : -(p1data[i]);
         }
@@ -1502,7 +1499,7 @@ ex_uminus_var(int init, nodeType *p1)
           for (size_t i = 0; i < ngp * nlev; ++i) pdata[i] = -(p1data[i]);
         }
 
-      p->param.nmiss = nmiss;
+      p->param.numMissVals = numMissVals;
     }
 
   return p;
@@ -1549,7 +1546,7 @@ ex_not_var(int init, nodeType *p1)
 {
   auto ngp = (p1->param.ngp > 0) ? p1->param.ngp : 1;
   auto nlev = (p1->param.nlev > 0) ? p1->param.nlev : 1;
-  auto nmiss = p1->param.nmiss;
+  auto numMissVals = p1->param.numMissVals;
   auto missval = p1->param.missval;
 
   auto p = new nodeType;
@@ -1566,7 +1563,7 @@ ex_not_var(int init, nodeType *p1)
       double *pdata = p->param.data;
       const double *p1data = p1->param.data;
 
-      if (nmiss)
+      if (numMissVals)
         {
           for (size_t i = 0; i < ngp * nlev; ++i) pdata[i] = dbl_is_equal(p1data[i], missval) ? missval : unary_op_not(p1data[i]);
         }
@@ -1575,7 +1572,7 @@ ex_not_var(int init, nodeType *p1)
           for (size_t i = 0; i < ngp * nlev; ++i) pdata[i] = unary_op_not(p1data[i]);
         }
 
-      p->param.nmiss = nmiss;
+      p->param.numMissVals = numMissVals;
     }
 
   return p;
@@ -1621,14 +1618,8 @@ static void
 str_add_node_info(char *string, size_t stringlen, nodeType *p, const char *ext)
 {
   auto len = strlen(string);
-  if (p->type == NodeEnum::typeCon)
-    {
-      snprintf(string + len, stringlen - len, "%g%s", p->con().value, ext);
-    }
-  else
-    {
-      snprintf(string + len, stringlen - len, "%s[N%zu][L%zu]%s", p->var().name.c_str(), p->param.ngp, p->param.nlev, ext);
-    }
+  if (p->type == NodeEnum::typeCon) { snprintf(string + len, stringlen - len, "%g%s", p->con().value, ext); }
+  else { snprintf(string + len, stringlen - len, "%s[N%zu][L%zu]%s", p->var().name.c_str(), p->param.ngp, p->param.nlev, ext); }
 }
 
 static nodeType *
@@ -1660,19 +1651,16 @@ ex_ifelse(int init, nodeType *p1, nodeType *p2, nodeType *p3)
   auto nlev = (p0->param.nlev > 0) ? p0->param.nlev : 1;
   auto px = p0;
 
-  size_t nmiss1 = 0;
+  size_t numMissVals1 = 0;
   auto missval1 = missval;
   const double *pdata1 = nullptr;
   size_t ngp1 = 1;
   size_t nlev1 = 1;
 
-  if (p1->type == NodeEnum::typeCon)
-    {
-      pdata1 = &p1->con().value;
-    }
+  if (p1->type == NodeEnum::typeCon) { pdata1 = &p1->con().value; }
   else
     {
-      nmiss1 = p1->param.nmiss;
+      numMissVals1 = p1->param.numMissVals;
       ngp1 = (p1->param.ngp > 0) ? p1->param.ngp : 1;
       nlev1 = (p1->param.nlev > 0) ? p1->param.nlev : 1;
       missval1 = p1->param.missval;
@@ -1700,10 +1688,7 @@ ex_ifelse(int init, nodeType *p1, nodeType *p2, nodeType *p3)
   size_t ngp2 = 1;
   size_t nlev2 = 1;
 
-  if (p2->type == NodeEnum::typeCon)
-    {
-      pdata2 = &p2->con().value;
-    }
+  if (p2->type == NodeEnum::typeCon) { pdata2 = &p2->con().value; }
   else
     {
       ngp2 = (p2->param.ngp > 0) ? p2->param.ngp : 1;
@@ -1733,10 +1718,7 @@ ex_ifelse(int init, nodeType *p1, nodeType *p2, nodeType *p3)
   size_t ngp3 = 1;
   size_t nlev3 = 1;
 
-  if (p3->type == NodeEnum::typeCon)
-    {
-      pdata3 = &p3->con().value;
-    }
+  if (p3->type == NodeEnum::typeCon) { pdata3 = &p3->con().value; }
   else
     {
       ngp3 = (p3->param.ngp > 0) ? p3->param.ngp : 1;
@@ -1771,7 +1753,7 @@ ex_ifelse(int init, nodeType *p1, nodeType *p2, nodeType *p3)
 
   if (!init)
     {
-      size_t nmiss = 0;
+      size_t numMissVals = 0;
 
       p->param.data = new double[ngp * nlev];
 
@@ -1796,7 +1778,7 @@ ex_ifelse(int init, nodeType *p1, nodeType *p2, nodeType *p3)
               auto ival2 = idat2[(ngp2 > 1) ? i : 0];
               auto ival3 = idat3[(ngp3 > 1) ? i : 0];
 
-              if (nmiss1 && dbl_is_equal(ival1, missval1))
+              if (numMissVals1 && dbl_is_equal(ival1, missval1))
                 odat[i] = missval1;
               else if (is_not_equal(ival1, 0.0))
                 odat[i] = dbl_is_equal(ival2, missval2) ? missval1 : ival2;
@@ -1804,10 +1786,10 @@ ex_ifelse(int init, nodeType *p1, nodeType *p2, nodeType *p3)
                 odat[i] = dbl_is_equal(ival3, missval3) ? missval1 : ival3;
             }
 
-          nmiss += array_num_mv(ngp, odat, missval1);
+          numMissVals += array_num_mv(ngp, odat, missval1);
         }
 
-      p->param.nmiss = nmiss;
+      p->param.numMissVals = numMissVals;
     }
 
   if (p1->isTmpObj) node_delete(p1);
@@ -1895,7 +1877,7 @@ add_new_constant(const std::string &varname, ParseParamType &parseArg, std::vect
   params[varID].nlat = 0;
   params[varID].nlev = 0;
   params[varID].missval = -9.e33;
-  params[varID].nmiss = 0;
+  params[varID].numMissVals = 0;
   params[varID].name = varname;
   parseArg.nparams++;
   parseArg.cnparams++;
@@ -1911,7 +1893,7 @@ add_new_param(const std::string &varname, ParseParamType &parseArg, std::vector<
   params[varID].isValid = true;
   params[varID].hasMV = param.hasMV;
   params[varID].name = varname;
-  params[varID].nmiss = param.nmiss;
+  params[varID].numMissVals = param.numMissVals;
   if (param.units.size()) params[varID].units = param.units;
   if (param.longname.size()) params[varID].longname = param.longname;
   parseArg.nparams++;
@@ -1955,7 +1937,7 @@ expr_run_var_grid(const std::string &vnm, int coord, ParseParamType &parseArg)
 {
   auto &params = parseArg.params;
 
-  auto varname = vnm.substr(0, vnm.size() -2);
+  auto varname = vnm.substr(0, vnm.size() - 2);
   auto varID = param_search_name(parseArg.nparams, params, varname);
   if (varID == -1) cdo_abort("Coordinate %c: variable >%s< not found!", coord, varname);
 
@@ -1991,7 +1973,7 @@ expr_run_var_zaxis(const std::string &vnm, int coord, ParseParamType &parseArg)
 {
   auto &params = parseArg.params;
 
-  auto varname = vnm.substr(0, vnm.size() -2);
+  auto varname = vnm.substr(0, vnm.size() - 2);
   auto varID = param_search_name(parseArg.nparams, params, varname);
   if (varID == -1) cdo_abort("Coordinate %c: variable >%s< not found!", coord, varname);
 
@@ -2046,7 +2028,7 @@ expr_run_var(nodeType *p, ParseParamType &parseArg)
   if (varID == -1) { cdo_abort("Variable >%s< not found!", vnm); }
   else if (init)
     {
-      if (varID < parseArg.nvars1 && !parseArg.needed[varID]) parseArg.needed[varID] = true;
+      if (varID < parseArg.numVars1 && !parseArg.needed[varID]) parseArg.needed[varID] = true;
     }
 
   param_meta_copy(p->param, params[varID]);
@@ -2060,7 +2042,7 @@ expr_run_var(nodeType *p, ParseParamType &parseArg)
   if (!init)
     {
       p->param.data = params[varID].data;
-      p->param.nmiss = params[varID].nmiss;
+      p->param.numMissVals = params[varID].numMissVals;
     }
 
   if (parseArg.debug)
@@ -2191,7 +2173,7 @@ expr_run_opr(nodeType *p, ParseParamType &parseArg)
             if (varID >= 0)
               {
                 // printf("  found %s\n", varname2);
-                if (varID < parseArg.nvars1)
+                if (varID < parseArg.numVars1)
                   {
                     if (rnode->param.nlev > 0 && params[varID].nlev != rnode->param.nlev)
                       cdo_abort("The number of layers must not change (name=%s layers: in=%zu out=%zu)!", params[varID].name,
@@ -2227,7 +2209,7 @@ expr_run_opr(nodeType *p, ParseParamType &parseArg)
             p->isTmpObj = false;
 
             ex_copy(init, p, rnode);
-            params[varID].nmiss = p->param.nmiss;
+            params[varID].numMissVals = p->param.numMissVals;
           }
 
         if (rnode && rnode->isTmpObj)
@@ -2272,3 +2254,16 @@ expr_run(nodeType *p, ParseParamType &parseArg)
 
   return nullptr;
 }
+int
+params_get_coord_ID(const ParseParamType &parseArg, int coord, int cdiID)
+{
+  auto ncoords = parseArg.numCoords;
+  for (int coordID = 0; coordID < ncoords; ++coordID)
+    {
+      if (parseArg.coords[coordID].coord == coord && parseArg.coords[coordID].cdiID == cdiID) return coordID;
+    }
+
+  cdo_abort("%s: coordinate %c not found!", __func__, coord);
+
+  return -1;
+}
