@@ -5,7 +5,7 @@
 
 */
 
-#include "process_int.h"
+#include "cdo_output.h"
 #include "cdo_zaxis.h"
 #include "cimdOmp.h"
 #include <mpim_grid.h>
@@ -350,10 +350,10 @@ oper_expr_var_var(int oper, bool hasMV, size_t n, double mv1, double mv2, double
 }
 
 void
-fld_field_init(Field &field, size_t nmiss, double missval, size_t ngp, double *array, double *w)
+fld_field_init(Field &field, size_t numMissVals, double missval, size_t ngp, double *array, double *w)
 {
   field.size = ngp;
-  field.nmiss = nmiss;
+  field.numMissVals = numMissVals;
   field.missval = missval;
   if (array != nullptr) array_copy(ngp, array, field.vec_d.data());
   if (w != nullptr) array_copy(ngp, w, field.weightv.data());
@@ -384,8 +384,7 @@ void
 vert_weights(int zaxisID, size_t nlev, Varray<double> &weights)
 {
   Varray<double> thickness(nlev);
-  weights.resize(nlev);
-  varray_fill(nlev, weights, 1.0);
+  weights.resize(nlev, 1.0);
 
   if (nlev > 1)
     {
