@@ -5,114 +5,109 @@
 
 */
 
-#include <algorithm>
 #include <cdi.h>
 
+#include "arithmetic.h"
 #include "cdo_options.h"
-#include "process_int.h"
-#include "field.h"
+#include "cdo_output.h"
+#include "cimdOmp.h"
+#include "field_functions.h"
 
 // clang-format off
 
 // arithmetic
-auto arith_add   = [](const double a, const double b) { return a + b; };
-auto arith_sub   = [](const double a, const double b) { return a - b; };
-auto arith_mul   = [](const double a, const double b) { return a * b; };
-auto arith_div   = [](const double a, const double b) { return a / b; };
-auto arith_min   = [](const auto a, const auto b) { return (b > a) ? a : b; };
-auto arith_max   = [](const auto a, const auto b) { return (b < a) ? a : b; };
-auto arith_sumq  = [](const double a, const double b) { return a + b * b; };
-auto arith_atan2 = [](const double a, const double b) { return std::atan2(a, b); };
+const auto arith_add   = [](double a, double b) { return a + b; };
+const auto arith_sub   = [](double a, double b) { return a - b; };
+const auto arith_mul   = [](double a, double b) { return a * b; };
+const auto arith_div   = [](double a, double b) { return a / b; };
+const auto arith_min   = [](auto a, auto b) { return (b > a) ? a : b; };
+const auto arith_max   = [](auto a, auto b) { return (b < a) ? a : b; };
+const auto arith_sumq  = [](double a, double b) { return a + b * b; };
+const auto arith_atan2 = [](double a, double b) { return std::atan2(a, b); };
 
 // arithmetic with missing values
-auto arith_add_mv = [](auto &a, const auto mv_a, const auto b, const auto mv_b, auto isEQ)
-                    { a = (isEQ(a, mv_a) || isEQ(b, mv_b)) ? mv_a : arith_add(a, b); };
-auto arith_sub_mv = [](auto &a, const auto mv_a, const auto b, const auto mv_b, auto isEQ)
-                    { a = (isEQ(a, mv_a) || isEQ(b, mv_b)) ? mv_a : arith_sub(a, b); };
-auto arith_mul_mv = [](auto &a, const auto mv_a, const auto b, const auto mv_b, auto isEQ)
-                    { a = (isEQ(a, 0) || isEQ(b, 0)) ? 0 : ((isEQ(a, mv_a) || isEQ(b, mv_b)) ? mv_a : arith_mul(a, b)); };
-auto arith_div_mv = [](auto &a, const auto mv_a, const auto b, const auto mv_b, auto isEQ)
-                    { a = (isEQ(a, mv_a) || isEQ(b, mv_b) || isEQ(b, 0)) ? mv_a : arith_div(a, b); };
-auto arith_min_mv = [](auto &a, const auto mv_a, const auto b, const auto mv_b, auto isEQ)
-                    { a = isEQ(b, mv_b) ? a : (isEQ(a, mv_a) ? b : arith_min(a, b)); };
-auto arith_max_mv = [](auto &a, const auto mv_a, const auto b, const auto mv_b, auto isEQ)
-                    { a = isEQ(b, mv_b) ? a : (isEQ(a, mv_a) ? b : arith_max(a, b)); };
-auto arith_sum_mv = [](auto &a, const auto mv_a, const auto b, const auto mv_b, auto isEQ)
-                    { if (!isEQ(b, mv_b)) a = (isEQ(a, mv_a) ? b : arith_add(a, b)); };
-auto arith_sumq_mv = [](auto &a, const auto mv_a, const auto b, const auto mv_b, auto isEQ)
-                     { if (!isEQ(b, mv_b)) a = (isEQ(a, mv_a) ? arith_mul(b, b) : arith_sumq(a, b)); };
-auto arith_atan2_mv = [](auto &a, const auto mv_a, const auto b, const auto mv_b, auto isEQ)
-                      { a = (isEQ(a, mv_a) || isEQ(b, mv_b)) ? mv_a : arith_atan2(a, b); };
-auto arith_vinit_mv = [](auto &a, const auto mv_a, const auto b, const auto mv_b, auto isEQ)
-                      { a = !isEQ(b, mv_b); (void)mv_a; };
-auto arith_vincr_mv = [](auto &a, const auto mv_a, const auto b, const auto mv_b, auto isEQ)
-                      { if (!isEQ(b, mv_b)) a = a + 1; (void)mv_a; };
+const auto arith_add_mv = [](auto &a, auto mv_a, auto b, auto mv_b, auto is_EQ)
+                          { a = (is_EQ(a, mv_a) || is_EQ(b, mv_b)) ? mv_a : arith_add(a, b); };
+const auto arith_sub_mv = [](auto &a, auto mv_a, auto b, auto mv_b, auto is_EQ)
+                          { a = (is_EQ(a, mv_a) || is_EQ(b, mv_b)) ? mv_a : arith_sub(a, b); };
+const auto arith_mul_mv = [](auto &a, auto mv_a, auto b, auto mv_b, auto is_EQ)
+                          { a = (is_EQ(a, 0) || is_EQ(b, 0)) ? 0 : ((is_EQ(a, mv_a) || is_EQ(b, mv_b)) ? mv_a : arith_mul(a, b)); };
+const auto arith_div_mv = [](auto &a, auto mv_a, auto b, auto mv_b, auto is_EQ)
+                          { a = (is_EQ(a, mv_a) || is_EQ(b, mv_b) || is_EQ(b, 0)) ? mv_a : arith_div(a, b); };
+const auto arith_min_mv = [](auto &a, auto mv_a, auto b, auto mv_b, auto is_EQ)
+                          { a = is_EQ(b, mv_b) ? a : (is_EQ(a, mv_a) ? b : arith_min(a, b)); };
+const auto arith_max_mv = [](auto &a, auto mv_a, auto b, auto mv_b, auto is_EQ)
+                          { a = is_EQ(b, mv_b) ? a : (is_EQ(a, mv_a) ? b : arith_max(a, b)); };
+const auto arith_sum_mv = [](auto &a, auto mv_a, auto b, auto mv_b, auto is_EQ)
+                          { if (!is_EQ(b, mv_b)) a = (is_EQ(a, mv_a) ? b : arith_add(a, b)); };
+const auto arith_sumq_mv = [](auto &a, auto mv_a, auto b, auto mv_b, auto is_EQ)
+                           { if (!is_EQ(b, mv_b)) a = (is_EQ(a, mv_a) ? arith_mul(b, b) : arith_sumq(a, b)); };
+const auto arith_atan2_mv = [](auto &a, auto mv_a, auto b, auto mv_b, auto is_EQ)
+                            { a = (is_EQ(a, mv_a) || is_EQ(b, mv_b)) ? mv_a : arith_atan2(a, b); };
+const auto arith_vinit_mv = [](auto &a, auto mv_a, auto b, auto mv_b, auto is_EQ)
+                            { a = !is_EQ(b, mv_b); (void)mv_a; };
+const auto arith_vincr_mv = [](auto &a, auto mv_a, auto b, auto mv_b, auto is_EQ)
+                            { if (!is_EQ(b, mv_b)) a = a + 1; (void)mv_a; };
 // clang-format on
 
 template <typename T1, typename T2>
 static void
-varray2_div(const size_t len, Varray<T1> &v1, const Varray<T2> &v2, const double missval1)
+varray2_div(size_t n, Varray<T1> &v1, const Varray<T2> &v2, double missval1)
 {
-  assert(len > 0);
+  assert(n > 0);
   assert(v1.size() > 0);
   assert(v2.size() > 0);
-  assert(len <= v1.size());
-  assert(len <= v2.size());
+  assert(n <= v1.size());
+  assert(n <= v2.size());
 
 #ifdef _OPENMP
-#pragma omp parallel for default(shared) schedule(static)
+#pragma omp parallel for if (n > cdoMinLoopSize) default(shared) schedule(static)
 #endif
-  for (size_t i = 0; i < len; ++i)
-    {
-      v1[i] = is_equal(v2[i], 0.0) ? missval1 : arith_div(v1[i], v2[i]);
-    }
+  for (size_t i = 0; i < n; ++i) { v1[i] = is_equal(v2[i], 0.0) ? missval1 : arith_div(v1[i], v2[i]); }
 }
 
 template <typename T1, typename T2, typename FUNC>
 static void
-varray2_arith(const size_t len, Varray<T1> &v1, const Varray<T2> &v2, FUNC func)
+varray2_arith(size_t n, Varray<T1> &v1, const Varray<T2> &v2, FUNC func)
 {
-  assert(len > 0);
+  assert(n > 0);
   assert(v1.size() > 0);
   assert(v2.size() > 0);
-  assert(len <= v1.size());
-  assert(len <= v2.size());
+  assert(n <= v1.size());
+  assert(n <= v2.size());
 
-#ifdef _OPENMP
-#pragma omp parallel for default(shared) schedule(static)
+#ifdef HAVE_OPENMP4
+#pragma omp parallel for simd if (n > cdoMinLoopSize) default(shared) schedule(static)
 #endif
-  for (size_t i = 0; i < len; ++i)
-    {
-      v1[i] = func(v1[i], v2[i]);
-    }
+  for (size_t i = 0; i < n; ++i) { v1[i] = func(v1[i], v2[i]); }
 }
 
 template <typename T1, typename T2, typename FUNC>
 static void
-varray2_arith_mv(const size_t len, Varray<T1> &v1, const Varray<T2> &v2, const double missval1, const double missval2, FUNC func)
+varray2_arith_mv(size_t n, Varray<T1> &v1, const Varray<T2> &v2, double missval1, double missval2, FUNC func)
 {
-  assert(len > 0);
+  assert(n > 0);
   assert(v1.size() > 0);
   assert(v2.size() > 0);
-  assert(len <= v1.size());
-  assert(len <= v2.size());
+  assert(n <= v1.size());
+  assert(n <= v2.size());
 
-  const T1 mv1 = missval1;
-  const T2 mv2 = missval2;
+  T1 mv1 = missval1;
+  T2 mv2 = missval2;
 
   if (std::isnan(missval1) || std::isnan(missval2))
     {
 #ifdef _OPENMP
-#pragma omp parallel for default(shared) schedule(static)
+#pragma omp parallel for if (n > cdoMinLoopSize) default(shared) schedule(static)
 #endif
-      for (size_t i = 0; i < len; ++i) func(v1[i], mv1, v2[i], mv2, dbl_is_equal);
+      for (size_t i = 0; i < n; ++i) { func(v1[i], mv1, v2[i], mv2, dbl_is_equal); }
     }
   else
     {
 #ifdef _OPENMP
-#pragma omp parallel for default(shared) schedule(static)
+#pragma omp parallel for if (n > cdoMinLoopSize) default(shared) schedule(static)
 #endif
-      for (size_t i = 0; i < len; ++i) func(v1[i], mv1, v2[i], mv2, is_equal);
+      for (size_t i = 0; i < n; ++i) { func(v1[i], mv1, v2[i], mv2, is_equal); }
     }
 }
 
@@ -131,7 +126,7 @@ field2_vinit(Field &field1, const Field &field2)
   else
     varray2_arith_mv(field1.size, field1.vec_d, field2.vec_d, field1.missval, field2.missval, arith_vinit_mv);
 
-  field1.nmiss = field2.nmiss;
+  field1.numMissVals = field2.numMissVals;
 }
 
 // increment valid values
@@ -149,15 +144,15 @@ field2_vincr(Field &field1, const Field &field2)
   else
     varray2_arith_mv(field1.size, field1.vec_d, field2.vec_d, field1.missval, field2.missval, arith_vincr_mv);
 
-  field1.nmiss = field2.nmiss;
+  field1.numMissVals = field2.numMissVals;
 }
 
 // init valid values
 template <typename T1, typename T2>
 static void
-field2_vinit(const size_t len, Varray<T1> &v1, const Varray<T2> &v2, const T2 missval, int vinit)
+field2_vinit(size_t len, Varray<T1> &v1, const Varray<T2> &v2, T2 missval, int vinit)
 {
-  for (size_t i = 0; i < len; ++i) v1[i] = (DBL_IS_EQUAL(v2[i], missval)) ? 0 : vinit;
+  for (size_t i = 0; i < len; ++i) v1[i] = (dbl_is_equal(v2[i], missval)) ? 0 : vinit;
 }
 
 void
@@ -174,16 +169,16 @@ field2_vinit(Field &field1, const Field &field2, int vinit)
   else
     field2_vinit(field1.size, field1.vec_d, field2.vec_d, field1.missval, vinit);
 
-  field1.nmiss = field2.nmiss;
+  field1.numMissVals = field2.numMissVals;
 }
 
 // increment valid values
 template <typename T1, typename T2>
 static void
-field2_vincr(const size_t len, Varray<T1> &v1, const Varray<T2> &v2, const T2 missval, int vincr)
+field2_vincr(size_t len, Varray<T1> &v1, const Varray<T2> &v2, T2 missval, int vincr)
 {
   for (size_t i = 0; i < len; ++i)
-    if (!DBL_IS_EQUAL(v2[i], missval)) v1[i] += vincr;
+    if (dbl_is_not_equal(v2[i], missval)) v1[i] += vincr;
 }
 
 void
@@ -200,7 +195,7 @@ field2_vincr(Field &field1, const Field &field2, int vincr)
   else
     field2_vincr(field1.size, field1.vec_d, field2.vec_d, field1.missval, vincr);
 
-  field1.nmiss = field2.nmiss;
+  field1.numMissVals = field2.numMissVals;
 }
 
 void
@@ -208,7 +203,7 @@ field2_add(Field &field1, const Field &field2)
 {
   if (field1.size != field2.size) cdo_abort("Fields have different size (%s)", __func__);
 
-  if (field1.nmiss || field2.nmiss)
+  if (field1.numMissVals || field2.numMissVals)
     {
       if (memtype_is_float_float(field1.memType, field2.memType))
         varray2_arith_mv(field1.size, field1.vec_f, field2.vec_f, field1.missval, field2.missval, arith_add_mv);
@@ -239,7 +234,7 @@ field2_sum(Field &field1, const Field &field2)
 {
   if (field1.size != field2.size) cdo_abort("Fields have different size (%s)", __func__);
 
-  if (field1.nmiss || field2.nmiss)
+  if (field1.numMissVals || field2.numMissVals)
     {
       if (memtype_is_float_float(field1.memType, field2.memType))
         varray2_arith_mv(field1.size, field1.vec_f, field2.vec_f, field1.missval, field2.missval, arith_sum_mv);
@@ -268,34 +263,34 @@ field2_sum(Field &field1, const Field &field2)
 void
 field2_sumw(Field &field1, const Field &field2, double w)
 {
-  const auto missval1 = field1.missval;
-  const auto missval2 = field2.missval;
+  auto missval1 = field1.missval;
+  auto missval2 = field2.missval;
   auto &array1 = field1.vec_d;
   const auto &array2 = field2.vec_d;
 
   auto len = field1.size;
   if (len != field2.size) cdo_abort("Fields have different size (%s)", __func__);
 
-  if (field1.nmiss || field2.nmiss)
+  if (field1.numMissVals || field2.numMissVals)
     {
 #ifdef _OPENMP
-#pragma omp parallel for default(shared) schedule(static)
+#pragma omp parallel for if (len > cdoMinLoopSize) default(shared) schedule(static)
 #endif
       for (size_t i = 0; i < len; ++i)
-        if (!DBL_IS_EQUAL(array2[i], missval2))
+        if (dbl_is_not_equal(array2[i], missval2))
           {
-            if (!DBL_IS_EQUAL(array1[i], missval1))
+            if (dbl_is_not_equal(array1[i], missval1))
               array1[i] += w * array2[i];
             else
               array1[i] = w * array2[i];
           }
 
-      field1.nmiss = field_num_miss(field1);
+      field1.numMissVals = field_num_miss(field1);
     }
   else
     {
 #ifdef _OPENMP
-#pragma omp parallel for default(shared) schedule(static)
+#pragma omp parallel for if (len > cdoMinLoopSize) default(shared) schedule(static)
 #endif
       for (size_t i = 0; i < len; ++i) array1[i] += w * array2[i];
     }
@@ -309,7 +304,7 @@ field2_sumw(Field &field1, const Field &field2, double w)
  * change anything (they do not start a non-period by setting occurrence to zero).
  */
 void
-field2_sumtr(Field &occur, const Field &field, const double refval)
+field2_sumtr(Field &occur, const Field &field, double refval)
 {
   auto omissval = occur.missval;
   auto fmissval = field.missval;
@@ -319,32 +314,32 @@ field2_sumtr(Field &occur, const Field &field, const double refval)
   auto len = occur.size;
   if (len != field.size) cdo_abort("Fields have different size (%s)", __func__);
 
-  if (occur.nmiss || field.nmiss)
+  if (occur.numMissVals || field.numMissVals)
     {
 #ifdef _OPENMP
-#pragma omp parallel for default(shared) schedule(static)
+#pragma omp parallel for if (len > cdoMinLoopSize) default(shared) schedule(static)
 #endif
       for (size_t i = 0; i < len; ++i)
-        if (!DBL_IS_EQUAL(farray[i], fmissval))
+        if (dbl_is_not_equal(farray[i], fmissval))
           {
-            if (!DBL_IS_EQUAL(oarray[i], omissval))
-              oarray[i] = (DBL_IS_EQUAL(farray[i], refval)) ? 0.0 : oarray[i] + 1.0;
+            if (dbl_is_not_equal(oarray[i], omissval))
+              oarray[i] = (dbl_is_equal(farray[i], refval)) ? 0.0 : oarray[i] + 1.0;
             else
-              oarray[i] = (DBL_IS_EQUAL(farray[i], refval)) ? 0.0 : 1.0;
+              oarray[i] = (dbl_is_equal(farray[i], refval)) ? 0.0 : 1.0;
           }
         else
           {
-            if (!DBL_IS_EQUAL(oarray[i], omissval)) oarray[i] = 0.0;
+            if (dbl_is_not_equal(oarray[i], omissval)) oarray[i] = 0.0;
           }
 
-      occur.nmiss = field_num_miss(occur);
+      occur.numMissVals = field_num_miss(occur);
     }
   else
     {
 #ifdef _OPENMP
-#pragma omp parallel for default(shared) schedule(static)
+#pragma omp parallel for if (len > cdoMinLoopSize) default(shared) schedule(static)
 #endif
-      for (size_t i = 0; i < len; ++i) oarray[i] = (DBL_IS_EQUAL(farray[i], refval)) ? 0.0 : oarray[i] + 1.0;
+      for (size_t i = 0; i < len; ++i) oarray[i] = (dbl_is_equal(farray[i], refval)) ? 0.0 : oarray[i] + 1.0;
     }
 }
 
@@ -353,7 +348,7 @@ field2_sumq(Field &field1, const Field &field2)
 {
   if (field1.size != field2.size) cdo_abort("Fields have different size (%s)", __func__);
 
-  if (field1.nmiss || field2.nmiss)
+  if (field1.numMissVals || field2.numMissVals)
     {
       if (memtype_is_float_float(field1.memType, field2.memType))
         varray2_arith_mv(field1.size, field1.vec_f, field2.vec_f, field1.missval, field2.missval, arith_sumq_mv);
@@ -387,36 +382,36 @@ field2_sumsumq(Field &field1, Field &field2, const Field &field3)
 }
 
 void
-field2_sumqw(Field &field1, const Field &field2, const double w)
+field2_sumqw(Field &field1, const Field &field2, double w)
 {
-  const auto missval1 = field1.missval;
-  const auto missval2 = field2.missval;
+  auto missval1 = field1.missval;
+  auto missval2 = field2.missval;
   auto &array1 = field1.vec_d;
   const auto &array2 = field2.vec_d;
 
-  const auto len = field1.size;
+  auto len = field1.size;
   if (len != field2.size) cdo_abort("Fields have different size (%s)", __func__);
 
-  if (field1.nmiss || field2.nmiss)
+  if (field1.numMissVals || field2.numMissVals)
     {
 #ifdef _OPENMP
-#pragma omp parallel for default(shared) schedule(static)
+#pragma omp parallel for if (len > cdoMinLoopSize) default(shared) schedule(static)
 #endif
       for (size_t i = 0; i < len; ++i)
-        if (!DBL_IS_EQUAL(array2[i], missval2))
+        if (dbl_is_not_equal(array2[i], missval2))
           {
-            if (!DBL_IS_EQUAL(array1[i], missval1))
+            if (dbl_is_not_equal(array1[i], missval1))
               array1[i] += w * array2[i] * array2[i];
             else
               array1[i] = w * array2[i] * array2[i];
           }
 
-      field1.nmiss = field_num_miss(field1);
+      field1.numMissVals = field_num_miss(field1);
     }
   else
     {
 #ifdef _OPENMP
-#pragma omp parallel for default(shared) schedule(static)
+#pragma omp parallel for if (len > cdoMinLoopSize) default(shared) schedule(static)
 #endif
       for (size_t i = 0; i < len; ++i) array1[i] += w * array2[i] * array2[i];
     }
@@ -427,7 +422,7 @@ field2_sub(Field &field1, const Field &field2)
 {
   if (field1.size != field2.size) cdo_abort("Fields have different size (%s)", __func__);
 
-  if (field1.nmiss || field2.nmiss)
+  if (field1.numMissVals || field2.numMissVals)
     {
       if (memtype_is_float_float(field1.memType, field2.memType))
         varray2_arith_mv(field1.size, field1.vec_f, field2.vec_f, field1.missval, field2.missval, arith_sub_mv);
@@ -458,7 +453,7 @@ field2_mul(Field &field1, const Field &field2)
 {
   if (field1.size != field2.size) cdo_abort("Fields have different size (%s)", __func__);
 
-  if (field1.nmiss || field2.nmiss)
+  if (field1.numMissVals || field2.numMissVals)
     {
       if (memtype_is_float_float(field1.memType, field2.memType))
         varray2_arith_mv(field1.size, field1.vec_f, field2.vec_f, field1.missval, field2.missval, arith_mul_mv);
@@ -489,7 +484,7 @@ field2_div(Field &field1, const Field &field2)
 {
   if (field1.size != field2.size) cdo_abort("Fields have different size (%s)", __func__);
 
-  if (field1.nmiss || field2.nmiss)
+  if (field1.numMissVals || field2.numMissVals)
     {
       if (memtype_is_float_float(field1.memType, field2.memType))
         varray2_arith_mv(field1.size, field1.vec_f, field2.vec_f, field1.missval, field2.missval, arith_div_mv);
@@ -537,18 +532,18 @@ field2_atan2(Field &field1, const Field &field2)
 void
 field2_set_miss(Field &field1, const Field &field2)
 {
-  const auto missval1 = field1.missval;
+  auto missval1 = field1.missval;
   auto &array1 = field1.vec_d;
   const auto &array2 = field2.vec_d;
 
-  const auto len = field1.size;
+  auto len = field1.size;
   if (len != field2.size) cdo_abort("Fields have different size (%s)", __func__);
 
-  if (field1.nmiss)
+  if (field1.numMissVals)
     {
-      for (size_t i = 0; i < len; ++i) array1[i] = DBL_IS_EQUAL(array1[i], missval1) ? array2[i] : array1[i];
+      for (size_t i = 0; i < len; ++i) array1[i] = dbl_is_equal(array1[i], missval1) ? array2[i] : array1[i];
 
-      field1.nmiss = field_num_miss(field1);
+      field1.numMissVals = field_num_miss(field1);
     }
 }
 
@@ -557,7 +552,7 @@ field2_min(Field &field1, const Field &field2)
 {
   if (field1.size != field2.size) cdo_abort("Fields have different size (%s)", __func__);
 
-  if (field1.nmiss || field2.nmiss)
+  if (field1.numMissVals || field2.numMissVals)
     {
       if (memtype_is_float_float(field1.memType, field2.memType))
         varray2_arith_mv(field1.size, field1.vec_f, field2.vec_f, field1.missval, field2.missval, arith_min_mv);
@@ -588,7 +583,7 @@ field2_max(Field &field1, const Field &field2)
 {
   if (field1.size != field2.size) cdo_abort("Fields have different size (%s)", __func__);
 
-  if (field1.nmiss || field2.nmiss)
+  if (field1.numMissVals || field2.numMissVals)
     {
       if (memtype_is_float_float(field1.memType, field2.memType))
         varray2_arith_mv(field1.size, field1.vec_f, field2.vec_f, field1.missval, field2.missval, arith_max_mv);
@@ -621,32 +616,29 @@ field2_maxmin(Field &field1, Field &field2, const Field &field3)
   field2_max(field1, field3);
 }
 
-auto field2_set_index = [](auto &v1, auto &v2, const auto v3, const auto idx) {
+auto field2_set_index = [](auto &v1, auto &v2, auto v3, auto idx) {
   v2 = v3;
   v1 = idx;
 };
 
 template <typename T>
 void
-field2_minidx(size_t nmiss3, size_t len, double missval3, Field &field1, Field &field2, const Varray<T> &v3, int idx)
+field2_minidx(size_t numMissVals3, size_t len, double missval3, Field &field1, Field &field2, const Varray<T> &v3, int idx)
 {
-  const auto missval1 = field1.missval;
-  const auto missval2 = field2.missval;
+  auto missval1 = field1.missval;
+  auto missval2 = field2.missval;
   auto &v1 = field1.vec_d;
   auto &v2 = field2.vec_d;
 
-  if (field2.nmiss || nmiss3)
+  if (field2.numMissVals || numMissVals3)
     {
       for (size_t i = 0; i < len; ++i)
         {
-          if (DBL_IS_EQUAL(v3[i], missval3))
+          if (dbl_is_equal(v3[i], missval3))
             {
-              if (DBL_IS_EQUAL(v2[i], missval2)) v1[i] = missval1;
-            }
-          else if (v3[i] < v2[i] || DBL_IS_EQUAL(v2[i], missval2))
-            {
-              field2_set_index(v1[i], v2[i], v3[i], idx);
+              if (dbl_is_equal(v2[i], missval2)) v1[i] = missval1;
             }
+          else if (v3[i] < v2[i] || dbl_is_equal(v2[i], missval2)) { field2_set_index(v1[i], v2[i], v3[i], idx); }
         }
 
       field_num_mv(field1);
@@ -668,32 +660,29 @@ field2_minidx(Field &field1, Field &field2, const Field &field3, int idx)
   if (field2.size != field3.size) cdo_abort("Fields have different size (%s)", __func__);
 
   if (field3.memType == MemType::Float)
-    field2_minidx(field3.nmiss, field3.size, field3.missval, field1, field2, field3.vec_f, idx);
+    field2_minidx(field3.numMissVals, field3.size, field3.missval, field1, field2, field3.vec_f, idx);
   else
-    field2_minidx(field3.nmiss, field3.size, field3.missval, field1, field2, field3.vec_d, idx);
+    field2_minidx(field3.numMissVals, field3.size, field3.missval, field1, field2, field3.vec_d, idx);
 }
 
 template <typename T>
 void
-field2_maxidx(size_t nmiss3, size_t len, double missval3, Field &field1, Field &field2, const Varray<T> &v3, int idx)
+field2_maxidx(size_t numMissVals3, size_t len, double missval3, Field &field1, Field &field2, const Varray<T> &v3, int idx)
 {
-  const auto missval1 = field1.missval;
-  const auto missval2 = field2.missval;
+  auto missval1 = field1.missval;
+  auto missval2 = field2.missval;
   auto &v1 = field1.vec_d;
   auto &v2 = field2.vec_d;
 
-  if (field2.nmiss || nmiss3)
+  if (field2.numMissVals || numMissVals3)
     {
       for (size_t i = 0; i < len; ++i)
         {
-          if (DBL_IS_EQUAL(v3[i], missval3))
+          if (dbl_is_equal(v3[i], missval3))
             {
-              if (DBL_IS_EQUAL(v2[i], missval2)) v1[i] = missval1;
-            }
-          else if (v3[i] > v2[i] || DBL_IS_EQUAL(v2[i], missval2))
-            {
-              field2_set_index(v1[i], v2[i], v3[i], idx);
+              if (dbl_is_equal(v2[i], missval2)) v1[i] = missval1;
             }
+          else if (v3[i] > v2[i] || dbl_is_equal(v2[i], missval2)) { field2_set_index(v1[i], v2[i], v3[i], idx); }
         }
 
       field_num_mv(field1);
@@ -715,56 +704,57 @@ field2_maxidx(Field &field1, Field &field2, const Field &field3, int idx)
   if (field2.size != field3.size) cdo_abort("Fields have different size (%s)", __func__);
 
   if (field3.memType == MemType::Float)
-    field2_maxidx(field3.nmiss, field3.size, field3.missval, field1, field2, field3.vec_f, idx);
+    field2_maxidx(field3.numMissVals, field3.size, field3.missval, field1, field2, field3.vec_f, idx);
   else
-    field2_maxidx(field3.nmiss, field3.size, field3.missval, field1, field2, field3.vec_d, idx);
+    field2_maxidx(field3.numMissVals, field3.size, field3.missval, field1, field2, field3.vec_d, idx);
 }
 
 static size_t
-field_set_nmiss(const size_t len, Varray<double> &v, double missval)
+field_set_numMissVals(size_t len, Varray<double> &v, double missval)
 {
-  size_t nmiss = 0;
+  size_t numMissVals = 0;
 
   if (std::isnan(missval))
     {
       for (size_t i = 0; i < len; ++i)
-        if (DBL_IS_EQUAL(v[i], missval) || v[i] < 0)
+        if (dbl_is_equal(v[i], missval) || v[i] < 0)
           {
             v[i] = missval;
-            nmiss++;
+            numMissVals++;
           }
     }
   else
     {
       for (size_t i = 0; i < len; ++i)
-        if (IS_EQUAL(v[i], missval) || v[i] < 0)
+        if (is_equal(v[i], missval) || v[i] < 0)
           {
             v[i] = missval;
-            nmiss++;
+            numMissVals++;
           }
     }
 
-  return nmiss;
+  return numMissVals;
 }
 
 void
-field2_var(Field &field1, const Field &field2, const Field &field3, const int divisor)
+field2_var(Field &field1, const Field &field2, const Field &field3, int divisor)
 {
-  const auto missval1 = field1.missval;
-  const auto missval2 = field2.missval;
+  auto missval1 = field1.missval;
+  auto missval2 = field2.missval;
   auto &array1 = field1.vec_d;
   const auto &array2 = field2.vec_d;
   const auto &array3 = field3.vec_d;
 
-  const auto len = field1.size;
+  auto len = field1.size;
   if (len != field2.size) cdo_abort("Fields have different size (%s)", __func__);
 
-  if (field1.nmiss || field2.nmiss)
+  if (field1.numMissVals || field2.numMissVals)
     {
+      auto is_EQ = dbl_is_equal;
       for (size_t i = 0; i < len; ++i)
         {
-          const auto temp = DIVMN(MULMN(array1[i], array1[i]), array3[i]);
-          array1[i] = DIVMN(SUBMN(array2[i], temp), array3[i] - divisor);
+          auto temp = DIVM(MULM(array1[i], array1[i]), array3[i]);
+          array1[i] = DIVM(SUBM(array2[i], temp), array3[i] - divisor);
           if (array1[i] < 0 && array1[i] > -1.e-5) array1[i] = 0;
         }
     }
@@ -772,63 +762,60 @@ field2_var(Field &field1, const Field &field2, const Field &field3, const int di
     {
       for (size_t i = 0; i < len; ++i)
         {
-          const auto temp = DIV(MUL(array1[i], array1[i]), array3[i]);
+          auto temp = DIV(MUL(array1[i], array1[i]), array3[i]);
           array1[i] = DIV(SUB(array2[i], temp), array3[i] - divisor);
           if (array1[i] < 0 && array1[i] > -1.e-5) array1[i] = 0;
         }
     }
 
-  field1.nmiss = field_set_nmiss(len, array1, missval1);
+  field1.numMissVals = field_set_numMissVals(len, array1, missval1);
 }
 
 void
-field2_std(Field &field1, const Field &field2, const Field &field3, const int divisor)
+field2_std(Field &field1, const Field &field2, const Field &field3, int divisor)
 {
-  const auto missval1 = field1.missval;
+  auto missval1 = field1.missval;
   auto &array1 = field1.vec_d;
 
-  const auto len = field1.size;
+  auto len = field1.size;
   if (len != field2.size) cdo_abort("Fields have different size (%s)", __func__);
 
   field2_var(field1, field2, field3, divisor);
 
-  size_t nmiss = 0;
+  size_t numMissVals = 0;
   for (size_t i = 0; i < len; ++i)
-    if (DBL_IS_EQUAL(array1[i], missval1) || array1[i] < 0)
+    if (dbl_is_equal(array1[i], missval1) || array1[i] < 0)
       {
         array1[i] = missval1;
-        nmiss++;
+        numMissVals++;
       }
-    else
-      {
-        array1[i] = IS_NOT_EQUAL(array1[i], 0) ? std::sqrt(array1[i]) : 0;
-      }
-  field1.nmiss = nmiss;
+    else { array1[i] = is_not_equal(array1[i], 0) ? std::sqrt(array1[i]) : 0; }
+  field1.numMissVals = numMissVals;
 }
 
 void
-fieldc_var(Field &field1, const Field &field2, const int nsets, const int divisor)
+fieldc_var(Field &field1, const Field &field2, int numSets, int divisor)
 {
-  const auto nsetx = nsets - divisor;
-  const auto missval1 = field1.missval;
-  const auto missval2 = field2.missval;
+  auto nsetx = numSets - divisor;
+  auto missval1 = field1.missval;
+  auto missval2 = field2.missval;
   auto &array1 = field1.vec_d;
   const auto &array2 = field2.vec_d;
-  double temp;
 
-  const auto len = field1.size;
+  auto len = field1.size;
   if (len != field2.size) cdo_abort("Fields have different size (%s)", __func__);
 
   if (nsetx == 0)
     {
       for (size_t i = 0; i < len; ++i) array1[i] = missval1;
     }
-  else if (field1.nmiss || field2.nmiss)
+  else if (field1.numMissVals || field2.numMissVals)
     {
+      auto is_EQ = dbl_is_equal;
       for (size_t i = 0; i < len; ++i)
         {
-          temp = MULMN(array1[i], array1[i]) / nsets;
-          array1[i] = SUBMN(array2[i], temp) / nsetx;
+          auto temp = MULM(array1[i], array1[i]) / numSets;
+          array1[i] = SUBM(array2[i], temp) / nsetx;
           if (array1[i] < 0 && array1[i] > -1.e-5) array1[i] = 0;
         }
     }
@@ -836,61 +823,58 @@ fieldc_var(Field &field1, const Field &field2, const int nsets, const int diviso
     {
       for (size_t i = 0; i < len; ++i)
         {
-          temp = MUL(array1[i], array1[i]) / nsets;
+          auto temp = MUL(array1[i], array1[i]) / numSets;
           array1[i] = SUB(array2[i], temp) / nsetx;
           if (array1[i] < 0 && array1[i] > -1.e-5) array1[i] = 0;
         }
     }
 
-  field1.nmiss = field_set_nmiss(len, array1, missval1);
+  field1.numMissVals = field_set_numMissVals(len, array1, missval1);
 }
 
 void
-fieldc_std(Field &field1, const Field &field2, const int nsets, const int divisor)
+fieldc_std(Field &field1, const Field &field2, int numSets, int divisor)
 {
-  const auto missval1 = field1.missval;
+  auto missval1 = field1.missval;
   auto &array1 = field1.vec_d;
 
-  const auto len = field1.size;
+  auto len = field1.size;
   if (len != field2.size) cdo_abort("Fields have different size (%s)", __func__);
 
-  fieldc_var(field1, field2, nsets, divisor);
+  fieldc_var(field1, field2, numSets, divisor);
 
-  size_t nmiss = 0;
+  size_t numMissVals = 0;
   for (size_t i = 0; i < len; ++i)
-    if (DBL_IS_EQUAL(array1[i], missval1) || array1[i] < 0)
+    if (dbl_is_equal(array1[i], missval1) || array1[i] < 0)
       {
         array1[i] = missval1;
-        nmiss++;
-      }
-    else
-      {
-        array1[i] = IS_NOT_EQUAL(array1[i], 0) ? std::sqrt(array1[i]) : 0;
+        numMissVals++;
       }
+    else { array1[i] = is_not_equal(array1[i], 0) ? std::sqrt(array1[i]) : 0; }
 
-  field1.nmiss = nmiss;
+  field1.numMissVals = numMissVals;
 }
 
 void
 field2_moq(Field &field1, const Field &field2)
 {
-  const auto missval1 = field1.missval;
-  const auto missval2 = field2.missval;
+  auto missval1 = field1.missval;
+  auto missval2 = field2.missval;
   auto &array1 = field1.vec_d;
   const auto &array2 = field2.vec_d;
 
-  const auto len = field1.size;
+  auto len = field1.size;
   if (len != field2.size) cdo_abort("Fields have different size (%s)", __func__);
 
-  if (field2.nmiss)
+  if (field2.numMissVals)
     {
       for (size_t i = 0; i < len; ++i)
-        if (!DBL_IS_EQUAL(array2[i], missval2))
+        if (dbl_is_not_equal(array2[i], missval2))
           array1[i] = array2[i] * array2[i];
         else
           array1[i] = missval1;
 
-      field1.nmiss = field_num_miss(field1);
+      field1.numMissVals = field_num_miss(field1);
     }
   else
     {
@@ -899,25 +883,25 @@ field2_moq(Field &field1, const Field &field2)
 }
 
 void
-field2_moqw(Field &field1, const Field &field2, const double w)
+field2_moqw(Field &field1, const Field &field2, double w)
 {
-  const auto missval1 = field1.missval;
-  const auto missval2 = field2.missval;
+  auto missval1 = field1.missval;
+  auto missval2 = field2.missval;
   auto &array1 = field1.vec_d;
   const auto &array2 = field2.vec_d;
 
-  const auto len = field1.size;
+  auto len = field1.size;
   if (len != field2.size) cdo_abort("Fields have different size (%s)", __func__);
 
-  if (field2.nmiss)
+  if (field2.numMissVals)
     {
       for (size_t i = 0; i < len; ++i)
-        if (!DBL_IS_EQUAL(array2[i], missval2))
+        if (dbl_is_not_equal(array2[i], missval2))
           array1[i] = w * array2[i] * array2[i];
         else
           array1[i] = missval1;
 
-      field1.nmiss = field_num_miss(field1);
+      field1.numMissVals = field_num_miss(field1);
     }
   else
     {
@@ -926,7 +910,7 @@ field2_moqw(Field &field1, const Field &field2, const double w)
 }
 
 /**
- * Counts the number of nonmissing values. The result of the operation is computed according to the following rules:
+ * Counts the number of nonumMissValsing values. The result of the operation is computed according to the following rules:
  *
  * field1  field2  result
  * a       b       a + 1
@@ -939,7 +923,7 @@ field2_moqw(Field &field1, const Field &field2, const double w)
  */
 template <typename T1, typename T2>
 static void
-varray2_count_mv(const size_t len, Varray<T1> &v1, const Varray<T2> &v2, const double mv1, const double mv2)
+varray2_count_mv(size_t len, Varray<T1> &v1, const Varray<T2> &v2, double mv1, double mv2)
 {
   assert(len > 0);
   assert(v1.size() > 0);
@@ -948,9 +932,9 @@ varray2_count_mv(const size_t len, Varray<T1> &v1, const Varray<T2> &v2, const d
   assert(len <= v2.size());
 
   for (size_t i = 0; i < len; ++i)
-    if (!DBL_IS_EQUAL(v2[i], mv2))
+    if (dbl_is_not_equal(v2[i], mv2))
       {
-        if (!DBL_IS_EQUAL(v1[i], mv1))
+        if (dbl_is_not_equal(v1[i], mv1))
           v1[i] += 1.0;
         else
           v1[i] = 1.0;
@@ -959,11 +943,11 @@ varray2_count_mv(const size_t len, Varray<T1> &v1, const Varray<T2> &v2, const d
 
 template <typename T>
 static void
-varray2_count(const size_t len, Varray<T> &v)
+varray2_count(size_t len, Varray<T> &v)
 {
   assert(len > 0);
 
-  for (size_t i = 0; i < len; ++i)  v[i] += 1.0;
+  for (size_t i = 0; i < len; ++i) v[i] += 1.0;
 }
 
 void
@@ -971,7 +955,7 @@ field2_count(Field &field1, const Field &field2)
 {
   if (field1.size != field2.size) cdo_abort("Fields have different size (%s)", __func__);
 
-  if (field1.nmiss || field2.nmiss)
+  if (field1.numMissVals || field2.numMissVals)
     {
       if (memtype_is_float_float(field1.memType, field2.memType))
         varray2_count_mv(field1.size, field1.vec_f, field2.vec_f, field1.missval, field2.missval);
@@ -982,7 +966,7 @@ field2_count(Field &field1, const Field &field2)
       else
         varray2_count_mv(field1.size, field1.vec_d, field2.vec_d, field1.missval, field2.missval);
 
-      field1.nmiss = field_num_miss(field1);
+      field1.numMissVals = field_num_miss(field1);
     }
   else
     {
@@ -991,7 +975,6 @@ field2_count(Field &field1, const Field &field2)
       else
         varray2_count(field1.size, field1.vec_d);
     }
-
 }
 
 void
