@@ -178,7 +178,7 @@ rect_grid_search2(long &imin, long &imax, double xmin, double xmax, long nxm, co
   return lfound;
 }
 
-static double
+double
 intlinarr2p(long nxm, long nym, double **fieldm, const Varray<double> &xm, const Varray<double> &ym, double x, double y)
 {
   long ii, jj;
@@ -216,7 +216,7 @@ intlinarr2(T missval, int lonIsCircular, size_t nxm, size_t nym, const Varray<T>
   for (size_t jj = 0; jj < nym; ++jj)
     for (size_t ii = 0; ii < nlon1; ++ii) grid1_mask[jj * nlon1 + ii] = !DBL_IS_EQUAL(varray1[jj * nlon1 + ii], missval);
 
-  progress::init();
+  cdo::Progress progress;
 
 #ifdef _OPENMP
 #pragma omp parallel for default(shared)
@@ -228,7 +228,7 @@ intlinarr2(T missval, int lonIsCircular, size_t nxm, size_t nym, const Varray<T>
       varray2[i] = missval;
 
       atomicCount++;
-      if (cdo_omp_get_thread_num() == 0) progress::update(0, 1, (double) atomicCount / gridsize2);
+      if (cdo_omp_get_thread_num() == 0) progress.update((double) atomicCount / gridsize2);
 
       size_t ii, jj;
       auto lfound = rect_grid_search(ii, jj, x[i], y[i], nxm, nym, xm, ym);
@@ -263,8 +263,6 @@ intlinarr2(T missval, int lonIsCircular, size_t nxm, size_t nym, const Varray<T>
           varray2[i] = fwsum;
         }
     }
-
-  progress::update(0, 1, 1);
 }
 
 void
@@ -811,5 +809,5 @@ interpolate(const Field &field1, Field &field2)
         }
     }
 
-  field2.nmiss = varray_num_mv(gridsize_o, arrayOut, missval);
+  field2.numMissVals = varray_num_mv(gridsize_o, arrayOut, missval);
 }
