@@ -40,7 +40,7 @@ hist_init_bins(int nbins, T *bins)
 }
 
 static void
-histDefBounds(Histogram &hist, float a, float b)
+histDefBounds(HistogramEntry &hist, float a, float b)
 {
   assert(hist.nbins > 0);
 
@@ -63,7 +63,7 @@ calc_bin(int nbins, float histMin, float histStep, float value)
 
 template <typename T>
 static void
-histBinAddValue(Histogram &hist, T *bins, float value)
+histBinAddValue(HistogramEntry &hist, T *bins, float value)
 {
   auto bin = calc_bin(hist.nbins, hist.min, hist.step, value);
   if (bin >= 0 && bin < hist.nbins) bins[bin]++;
@@ -71,14 +71,14 @@ histBinAddValue(Histogram &hist, T *bins, float value)
 
 template <typename T>
 static void
-histBinSubValue(Histogram &hist, T *bins, float value)
+histBinSubValue(HistogramEntry &hist, T *bins, float value)
 {
   auto bin = calc_bin(hist.nbins, hist.min, hist.step, value);
   if (bin >= 0 && bin < hist.nbins && bins[bin] > 0) bins[bin]--;
 }
 
 static void
-histBin(Histogram &hist)
+histBin(HistogramEntry &hist)
 {
   assert(hist.nsamp == hist.capacity);
 
@@ -99,7 +99,7 @@ histBin(Histogram &hist)
 }
 /* unused
 static int
-histReset(Histogram &hist)
+histReset(HistogramEntry &hist)
 {
   assert(hist.nbins > 0);
 
@@ -122,7 +122,7 @@ histReset(Histogram &hist)
 */
 
 static void
-histCheckValue(const Histogram &hist, float &value)
+histCheckValue(const HistogramEntry &hist, float &value)
 {
   // 2011-08-01 Uwe Schulzweida: added check for rounding errors
   if (value < hist.min && (hist.min - value) < 1.e5f) value = hist.min;
@@ -130,7 +130,7 @@ histCheckValue(const Histogram &hist, float &value)
 }
 
 static int
-histAddValue(Histogram &hist, float value)
+histAddValue(HistogramEntry &hist, float value)
 {
   assert(hist.nbins > 0);
 
@@ -156,7 +156,7 @@ histAddValue(Histogram &hist, float value)
 }
 
 static void
-histRemoveValue(Histogram &hist, float value)
+histRemoveValue(HistogramEntry &hist, float value)
 {
   auto fltptr = FLT_PTR(hist.ptr);
 
@@ -176,7 +176,7 @@ histRemoveValue(Histogram &hist, float value)
 }
 
 static int
-histSubValue(Histogram &hist, float value)
+histSubValue(HistogramEntry &hist, float value)
 {
   assert(hist.nbins > 0);
 
@@ -220,7 +220,7 @@ histGetBin(int nbins, double s, const T *ptr)
 }
 
 static double
-histGetPercentile(const Histogram &hist, double p)
+histGetPercentile(const HistogramEntry &hist, double p)
 {
   assert(hist.nsamp > 0);
   assert(hist.nbins > 0);
@@ -240,7 +240,7 @@ histGetPercentile(const Histogram &hist, double p)
 
       auto bin = hist.isUint32 ? histGetBin(hist.nbins, s, INT_PTR(hist.ptr)) : histGetBin(hist.nbins, s, SHR_PTR(hist.ptr));
 
-      //assert(hist.step > 0.0f);
+      // assert(hist.step > 0.0f);
 
       return hist.min + bin * hist.step;
     }
@@ -286,7 +286,7 @@ HistogramSet::createVarLevels(int varID, int nlevels, size_t nhists)
 
 template <typename T1, typename T2>
 static void
-def_bounds(size_t nhists, std::vector<Histogram> &hists, const Varray<T1> &v1, const Varray<T2> &v2, float mv1, float mv2)
+def_bounds(size_t nhists, std::vector<HistogramEntry> &hists, const Varray<T1> &v1, const Varray<T2> &v2, float mv1, float mv2)
 {
   assert(!v1.empty());
   assert(!v2.empty());
@@ -333,15 +333,15 @@ HistogramSet::defVarLevelBounds(int varID, int levelID, const Field &field1, con
 
 template <typename T>
 static int
-histAddVarLevelValues(size_t nhists, std::vector<Histogram> &hists, const Varray<T> &v, size_t nmiss, T mv)
+histAddVarLevelValues(size_t nhists, std::vector<HistogramEntry> &hists, const Varray<T> &v, size_t numMissVals, T mv)
 {
   assert(!v.empty());
 
   int nign = 0;
-  if (nmiss)
+  if (numMissVals)
     {
       for (size_t i = 0; i < nhists; ++i)
-        if (!dbl_is_equal(v[i], mv)) nign += histAddValue(hists[i], v[i]);
+        if (dbl_is_not_equal(v[i], mv)) nign += histAddValue(hists[i], v[i]);
     }
   else
     {
@@ -353,15 +353,15 @@ histAddVarLevelValues(size_t nhists, std::vector<Histogram> &hists, const Varray
 
 template <typename T>
 static int
-histSubVarLevelValues(size_t nhists, std::vector<Histogram> &hists, const Varray<T> &v, size_t nmiss, T mv)
+histSubVarLevelValues(size_t nhists, std::vector<HistogramEntry> &hists, const Varray<T> &v, size_t numMissVals, T mv)
 {
   assert(!v.empty());
 
   int nign = 0;
-  if (nmiss)
+  if (numMissVals)
     {
       for (size_t i = 0; i < nhists; ++i)
-        if (!dbl_is_equal(v[i], mv)) nign += histSubValue(hists[i], v[i]);
+        if (dbl_is_not_equal(v[i], mv)) nign += histSubValue(hists[i], v[i]);
     }
   else
     {
@@ -387,9 +387,9 @@ HistogramSet::addVarLevelValues(int varID, int levelID, const Field &field)
   int nign = 0;
 
   if (field.memType == MemType::Float)
-    nign = histAddVarLevelValues(nhists, hists, field.vec_f, field.nmiss, (float) field.missval);
+    nign = histAddVarLevelValues(nhists, hists, field.vec_f, field.numMissVals, (float) field.missval);
   else
-    nign = histAddVarLevelValues(nhists, hists, field.vec_d, field.nmiss, field.missval);
+    nign = histAddVarLevelValues(nhists, hists, field.vec_d, field.numMissVals, field.missval);
 
   if (nign)
     {
@@ -416,9 +416,9 @@ HistogramSet::subVarLevelValues(int varID, int levelID, const Field &field)
   int nign = 0;
 
   if (field.memType == MemType::Float)
-    nign = histSubVarLevelValues(nhists, hists, field.vec_f, field.nmiss, (float) field.missval);
+    nign = histSubVarLevelValues(nhists, hists, field.vec_f, field.numMissVals, (float) field.missval);
   else
-    nign = histSubVarLevelValues(nhists, hists, field.vec_d, field.nmiss, field.missval);
+    nign = histSubVarLevelValues(nhists, hists, field.vec_d, field.numMissVals, field.missval);
 
   if (nign)
     {
@@ -451,26 +451,23 @@ HistogramSet::reset(int varID, int levelID)
 
 template <typename T>
 static size_t
-calcPercentile(size_t nhists, const std::vector<Histogram> &hists, double p, Varray<T> &v, T mv)
+calcPercentile(size_t nhists, const std::vector<HistogramEntry> &hists, double p, Varray<T> &v, T mv)
 {
   assert(!v.empty());
 
-  size_t nmiss = 0;
+  size_t numMissVals = 0;
 
   for (size_t i = 0; i < nhists; ++i)
     {
-      if (hists[i].nsamp)
-        {
-          v[i] = histGetPercentile(hists[i], p);
-        }
+      if (hists[i].nsamp) { v[i] = histGetPercentile(hists[i], p); }
       else
         {
           v[i] = mv;
-          nmiss++;
+          numMissVals++;
         }
     }
 
-  return nmiss;
+  return numMissVals;
 }
 
 void
@@ -487,7 +484,7 @@ HistogramSet::getVarLevelPercentiles(Field &field, int varID, int levelID, doubl
   const auto &hists = this->histograms[varID][levelID];
 
   if (field.memType == MemType::Float)
-    field.nmiss = calcPercentile(nhists, hists, p, field.vec_f, (float) field.missval);
+    field.numMissVals = calcPercentile(nhists, hists, p, field.vec_f, (float) field.missval);
   else
-    field.nmiss = calcPercentile(nhists, hists, p, field.vec_d, field.missval);
+    field.numMissVals = calcPercentile(nhists, hists, p, field.vec_d, field.missval);
 }
