@@ -136,7 +136,7 @@ RemapType
   int nused = 0;
   int gridID = -1;
   size_t gridsize = 0;
-  size_t nmiss = 0;
+  size_t numMissVals = 0;
   RemapGrid srcGrid;
   RemapGrid tgtGrid;
   RemapVars vars;
@@ -155,7 +155,7 @@ void remap_set_int(int remapvar, int value);
 
 void remap_init_grids(RemapMethod mapType, bool doExtrapolate, int gridID1, RemapGrid &srcGrid, int gridID2, RemapGrid &tgtGrid);
 
-void remap_grid_free(RemapGrid &grid);
+void remap_grid_free(RemapGrid &grid, bool removeMask = true);
 void remap_grid_alloc(RemapMethod mapType, RemapGrid &grid);
 void remap_search_init(RemapMethod mapType, RemapSearch &search, RemapGrid &srcGrid, RemapGrid &tgtGrid);
 void remap_search_free(RemapSearch &search);
@@ -185,8 +185,8 @@ void remap_gradients(RemapGrid &grid, const Field &field, RemapGradients &gradie
 void sort_add(size_t numLinks, size_t numWeights, size_t *add1, size_t *add2, double *weights);
 void sort_iter(size_t numLinks, size_t numWeights, size_t *add1, size_t *add2, double *weights, int parent);
 
-void remap_write_data_scrip(const char *weightsfile, const RemapSwitches &remapSwitches, RemapGrid &srcGrid, RemapGrid &tgtGrid,
-                            RemapVars &rv);
+void remap_write_data_scrip(const std::string &weightsfile, const RemapSwitches &remapSwitches, RemapGrid &srcGrid,
+                            RemapGrid &tgtGrid, RemapVars &rv);
 RemapSwitches remap_read_data_scrip(const std::string &weightsfile, int gridID1, int gridID2, RemapGrid &srcGrid,
                                     RemapGrid &tgtGrid, RemapVars &rv);
 
@@ -217,18 +217,18 @@ void remap_check_area(size_t grid_size, const Varray<double> &cell_area, const c
 
 template <typename T>
 void
-remap_set_mask(size_t gridsize, const Varray<T> &v, size_t nmiss, T missval, Varray<short> &mask)
+remap_set_mask(size_t gridsize, const Varray<T> &v, size_t numMissVals, T missval, Varray<short> &mask)
 {
   mask.resize(gridsize);
 
-  if (nmiss)
+  if (numMissVals)
     {
       if (std::isnan(missval))
         {
 #ifdef _OPENMP
 #pragma omp parallel for default(shared) schedule(static)
 #endif
-          for (size_t i = 0; i < gridsize; ++i) mask[i] = !dbl_is_equal(v[i], missval);
+          for (size_t i = 0; i < gridsize; ++i) mask[i] = dbl_is_not_equal(v[i], missval);
         }
       else
         {
@@ -245,12 +245,12 @@ remap_set_mask(size_t gridsize, const Varray<T> &v, size_t nmiss, T missval, Var
 }
 
 inline void
-remap_set_mask(size_t gridsize, const Field &field1, size_t nmiss, double missval, Varray<short> &imask)
+remap_set_mask(size_t gridsize, const Field &field1, size_t numMissVals, double missval, Varray<short> &imask)
 {
   if (field1.memType == MemType::Float)
-    remap_set_mask(gridsize, field1.vec_f, nmiss, (float) missval, imask);
+    remap_set_mask(gridsize, field1.vec_f, numMissVals, (float) missval, imask);
   else
-    remap_set_mask(gridsize, field1.vec_d, nmiss, missval, imask);
+    remap_set_mask(gridsize, field1.vec_d, numMissVals, missval, imask);
 }
 
 #endif /* REMAP_H */
