@@ -6,8 +6,8 @@
 */
 
 #include <atomic>
-#include <algorithm>  // std::sort
 
+#include "cpp_lib.h"
 #include "process_int.h"
 #include "cdo_timer.h"
 #include <mpim_grid.h>
@@ -60,14 +60,26 @@ renormalize_weights(const double (&srcLats)[4], double (&weights)[4][4])
   for (int i = 0; i < 4; ++i) weights[i][2] = 0.0;
   for (int i = 0; i < 4; ++i) weights[i][3] = 0.0;
 }
-
+/*
+#ifdef HAVE_LIB_RANGES_ZIP
+#include <ranges>
 static void
-bicubic_sort_weights(size_t (&indices)[4], double (&weights)[4][4])
+bicubic_sort_weights_by_index_zip(size_t (&indices)[4], double (&weights)[4][4])
+{
+  auto r = std::views::zip(indices, weights);
+  std::sort(r.begin(), r.end, [](auto a, auto b) {
+    auto [ai, aw] = a;  // std::tuple
+    auto [bi, bw] = b;
+    return (ai < bi);
+  });
+}
+#endif
+*/
+static void
+bicubic_sort_weights_by_index(size_t (&indices)[4], double (&weights)[4][4])
 {
   constexpr size_t numWeights = 4;
 
-  if (is_sorted_list(numWeights, indices)) return;
-
   struct IndexWeightX
   {
     size_t index;
@@ -82,8 +94,7 @@ bicubic_sort_weights(size_t (&indices)[4], double (&weights)[4][4])
       for (int k = 0; k < 4; ++k) indexWeights[i].weight[k] = weights[i][k];
     }
 
-  auto comp_index = [](const auto &a, const auto &b) noexcept { return a.index < b.index; };
-  std::sort(indexWeights.begin(), indexWeights.end(), comp_index);
+  ranges::sort(indexWeights, {}, &IndexWeightX::index);
 
   for (size_t i = 0; i < numWeights; ++i)
     {
@@ -92,6 +103,19 @@ bicubic_sort_weights(size_t (&indices)[4], double (&weights)[4][4])
     }
 }
 
+static void
+bicubic_sort_weights(size_t (&indices)[4], double (&weights)[4][4])
+{
+  constexpr size_t numWeights = 4;
+  if (is_sorted_list(numWeights, indices)) return;
+
+  // #ifdef HAVE_LIB_RANGES_ZIP
+  //   bicubic_sort_weights_by_index_zip(indices, weights);
+  // #else
+  bicubic_sort_weights_by_index(indices, weights);
+  // #endif
+}
+
 static void
 bicubic_warning()
 {
@@ -114,7 +138,7 @@ bicubic_warning()
 void
 remap_bicubic_weights(RemapSearch &rsearch, RemapVars &rv)
 {
-  auto srcGrid = rsearch.srcGrid;
+  const auto srcGrid = rsearch.srcGrid;
   auto tgtGrid = rsearch.tgtGrid;
 
   if (Options::cdoVerbose) cdo_print("Called %s()", __func__);
@@ -122,8 +146,7 @@ remap_bicubic_weights(RemapSearch &rsearch, RemapVars &rv)
   if (srcGrid->rank != 2) cdo_abort("Can't do bicubic interpolation if the source grid is not a regular 2D grid!");
 
   cdo::timer timer;
-
-  progress::init();
+  cdo::Progress progress;
 
   // Compute mappings from source to target grid
 
@@ -142,7 +165,7 @@ remap_bicubic_weights(RemapSearch &rsearch, RemapVars &rv)
   for (size_t tgtCellIndex = 0; tgtCellIndex < tgtGridSize; ++tgtCellIndex)
     {
       atomicCount++;
-      if (cdo_omp_get_thread_num() == 0) progress::update(0, 1, (double) atomicCount / tgtGridSize);
+      if (cdo_omp_get_thread_num() == 0) progress.update((double) atomicCount / tgtGridSize);
 
       weightLinks[tgtCellIndex].nlinks = 0;
 
@@ -153,7 +176,7 @@ remap_bicubic_weights(RemapSearch &rsearch, RemapVars &rv)
       double srcLats[4];     //  latitudes  of four bilinear corners
       double srcLons[4];     //  longitudes of four bilinear corners
       double weights[4][4];  //  bicubic weights for four corners
-      size_t indices[4];     //  address for the four source points
+      size_t indices[4];     //  indices for the four source points
 
       // Find nearest square of grid points on source grid
       auto searchResult = remap_search_square(rsearch, llpoint, indices, srcLats, srcLons);
@@ -193,10 +216,10 @@ remap_bicubic_weights(RemapSearch &rsearch, RemapVars &rv)
         }
     }
 
-  progress::update(0, 1, 1);
-
   weight_links_4_to_remap_links(tgtGridSize, weightLinks, rv);
 
+  rv.linksPerValue = 4;
+
   if (Options::cdoVerbose) cdo_print("%s: %.2f seconds", __func__, timer.elapsed());
 }  // remap_bicubic_weights
 
@@ -226,7 +249,7 @@ bicubic_remap(const Varray<T> &srcArray, const double (&weights)[4][4], const si
 
 template <typename T>
 static void
-remap_bicubic(RemapSearch &rsearch, const Varray<T> &srcArray, Varray<T> &tgtArray, T missval, size_t nmiss)
+remap_bicubic(RemapSearch &rsearch, const Varray<T> &srcArray, Varray<T> &tgtArray, T missval, size_t numMissVals)
 {
   auto srcGrid = rsearch.srcGrid;
   auto tgtGrid = rsearch.tgtGrid;
@@ -236,14 +259,13 @@ remap_bicubic(RemapSearch &rsearch, const Varray<T> &srcArray, Varray<T> &tgtArr
   if (srcGrid->rank != 2) cdo_abort("Can't do bicubic interpolation if the source grid is not a regular 2D grid!");
 
   cdo::timer timer;
-
-  progress::init();
+  cdo::Progress progress;
 
   auto tgtGridSize = tgtGrid->size;
   auto srcGridSize = srcGrid->size;
 
-  Varray<short> srcGridMask;
-  if (nmiss) remap_set_mask(srcGridSize, srcArray, nmiss, missval, srcGridMask);
+  Varray<short> srcGridMask(srcGridSize, 1);
+  if (numMissVals) remap_set_mask(srcGridSize, srcArray, numMissVals, missval, srcGridMask);
 
   // Compute mappings from source to target grid
 
@@ -260,7 +282,7 @@ remap_bicubic(RemapSearch &rsearch, const Varray<T> &srcArray, Varray<T> &tgtArr
   for (size_t tgtCellIndex = 0; tgtCellIndex < tgtGridSize; ++tgtCellIndex)
     {
       atomicCount++;
-      if (cdo_omp_get_thread_num() == 0) progress::update(0, 1, (double) atomicCount / tgtGridSize);
+      if (cdo_omp_get_thread_num() == 0) progress.update((double) atomicCount / tgtGridSize);
 
       tgtArray[tgtCellIndex] = missval;
 
@@ -271,7 +293,7 @@ remap_bicubic(RemapSearch &rsearch, const Varray<T> &srcArray, Varray<T> &tgtArr
       double srcLats[4];     //  latitudes  of four bilinear corners
       double srcLons[4];     //  longitudes of four bilinear corners
       double weights[4][4];  //  bicubic weights for four corners
-      size_t indices[4];     //  address for the four source points
+      size_t indices[4];     //  indices for the four source points
 
       // Find nearest square of grid points on source grid
       auto searchResult = remap_search_square(rsearch, llpoint, indices, srcLats, srcLons);
@@ -310,8 +332,6 @@ remap_bicubic(RemapSearch &rsearch, const Varray<T> &srcArray, Varray<T> &tgtArr
         }
     }
 
-  progress::update(0, 1, 1);
-
   if (Options::cdoVerbose) cdo_print("%s: %.2f seconds", __func__, timer.elapsed());
 }  // remap_bicubic
 
@@ -321,7 +341,7 @@ remap_bicubic(RemapSearch &rsearch, const Field &field1, Field &field2)
   if (field1.memType != field2.memType) cdo_abort("Interal error, memType of field1 and field2 differ!");
 
   if (field1.memType == MemType::Float)
-    remap_bicubic(rsearch, field1.vec_f, field2.vec_f, (float) field1.missval, field1.nmiss);
+    remap_bicubic(rsearch, field1.vec_f, field2.vec_f, (float) field1.missval, field1.numMissVals);
   else
-    remap_bicubic(rsearch, field1.vec_d, field2.vec_d, field1.missval, field1.nmiss);
+    remap_bicubic(rsearch, field1.vec_d, field2.vec_d, field1.missval, field1.numMissVals);
 }
