@@ -10,7 +10,6 @@
 #include "remap.h"
 #include "remap_store_link_cnsrv.h"
 #include "cdo_options.h"
-#include "progress.h"
 #include "cimdOmp.h"
 
 #define ZERO 0.0
@@ -1221,8 +1220,6 @@ remap_conserv_weights_scrip(RemapSearch &rsearch, RemapVars &rv)
 
   if (Options::cdoVerbose) cdo_print("Called %s()", __func__);
 
-  progress::init();
-
   long numWeights = rv.numWeights;
 
   GridStore grid_store;
@@ -1280,9 +1277,10 @@ remap_conserv_weights_scrip(RemapSearch &rsearch, RemapVars &rv)
   srch_corners = tgtNumCorners;
 
 #ifdef _OPENMP
-#pragma omp parallel for default(none) shared(findex, rsearch, numWeights, src_centroid_lon, src_centroid_lat, grid_store, rv,      \
-                                              Options::cdoVerbose, max_subseg, srch_corner_lat, srch_corner_lon, maxSearchCells, \
-                                              srcNumCorners, srch_corners, srcGrid, tgtGrid, tgtGridSize, srcGridSize, srch_add)
+#pragma omp parallel for default(none)                                                                                       \
+    shared(findex, rsearch, numWeights, src_centroid_lon, src_centroid_lat, grid_store, rv, Options::cdoVerbose, max_subseg, \
+               srch_corner_lat, srch_corner_lon, maxSearchCells, srcNumCorners, srch_corners, srcGrid, tgtGrid, tgtGridSize, \
+               srcGridSize, srch_add)
 #endif
   for (long srcCellIndex = 0; srcCellIndex < srcGridSize; ++srcCellIndex)
     {
@@ -1292,7 +1290,6 @@ remap_conserv_weights_scrip(RemapSearch &rsearch, RemapVars &rv)
 #pragma omp atomic
 #endif
       findex++;
-      if (ompthID == 0) progress::update(0, 0.5, findex / srcGridSize);
 
       bool lcoinc;  // flag for coincident segments
       bool lthresh = false;
@@ -1467,9 +1464,10 @@ remap_conserv_weights_scrip(RemapSearch &rsearch, RemapVars &rv)
   findex = 0;
 
 #ifdef _OPENMP
-#pragma omp parallel for default(none) shared(findex, rsearch, numWeights, tgt_centroid_lon, tgt_centroid_lat, grid_store, rv,      \
-                                              Options::cdoVerbose, max_subseg, srch_corner_lat, srch_corner_lon, maxSearchCells, \
-                                              tgtNumCorners, srch_corners, srcGrid, tgtGrid, tgtGridSize, srcGridSize, srch_add)
+#pragma omp parallel for default(none)                                                                                       \
+    shared(findex, rsearch, numWeights, tgt_centroid_lon, tgt_centroid_lat, grid_store, rv, Options::cdoVerbose, max_subseg, \
+               srch_corner_lat, srch_corner_lon, maxSearchCells, tgtNumCorners, srch_corners, srcGrid, tgtGrid, tgtGridSize, \
+               srcGridSize, srch_add)
 #endif
   for (long tgtCellIndex = 0; tgtCellIndex < tgtGridSize; ++tgtCellIndex)
     {
@@ -1479,7 +1477,6 @@ remap_conserv_weights_scrip(RemapSearch &rsearch, RemapVars &rv)
 #pragma omp atomic
 #endif
       findex++;
-      if (ompthID == 0) progress::update(0.5, 0.5, findex / tgtGridSize);
 
       bool lcoinc;  // flag for coincident segments
       bool lthresh = false;
@@ -1637,8 +1634,6 @@ remap_conserv_weights_scrip(RemapSearch &rsearch, RemapVars &rv)
         }
     }
 
-  progress::update(0, 1, 1);
-
   /* Finished with all cells: deallocate search arrays */
 
   for (int i = 0; i < Threading::ompNumThreads; ++i) free(srch_corner_lon[i]);
