@@ -26,7 +26,7 @@ remap_distwgt_weights(size_t numNeighbors, RemapSearch &rsearch, RemapVars &rv)
 
   if (Options::cdoVerbose) cdo_print("Called %s()", __func__);
 
-  progress::init();
+  cdo::Progress progress;
 
   // Compute mappings from source to target grid
 
@@ -43,6 +43,7 @@ remap_distwgt_weights(size_t numNeighbors, RemapSearch &rsearch, RemapVars &rv)
 
   // Loop over target grid
 
+  std::atomic<long> linksPerValue{ -1 };
   std::atomic<size_t> atomicCount{ 0 };
 
 #ifdef _OPENMP
@@ -53,7 +54,7 @@ remap_distwgt_weights(size_t numNeighbors, RemapSearch &rsearch, RemapVars &rv)
       auto ompthID = cdo_omp_get_thread_num();
 
       atomicCount++;
-      if (ompthID == 0) progress::update(0, 1, (double) atomicCount / tgtGridSize);
+      if (ompthID == 0) progress.update((double) atomicCount / tgtGridSize);
 
       weightLinks[tgtCellIndex].nlinks = 0;
 
@@ -72,30 +73,40 @@ remap_distwgt_weights(size_t numNeighbors, RemapSearch &rsearch, RemapVars &rv)
         if (knnWgt.m_mask[i]) tgtGrid->cell_frac[tgtCellIndex] = 1.0;
 
       // Store the link
-      store_weightlinks(0, nadds, knnWgt.m_addr.data(), knnWgt.m_dist.data(), tgtCellIndex, weightLinks);
-    }
+      store_weightlinks(0, nadds, knnWgt.m_indices.data(), knnWgt.m_dist.data(), tgtCellIndex, weightLinks);
 
-  progress::update(0, 1, 1);
+      if (numNeighbors > 1 && nadds > 0)
+        {
+          if (linksPerValue == -1)
+            linksPerValue = nadds;
+          else if (linksPerValue > 0 && linksPerValue != (long) nadds)
+            linksPerValue = 0;
+        }
+    }
 
   grid_point_search_delete(rsearch.gps);
 
   weight_links_to_remap_links(0, tgtGridSize, weightLinks, rv);
 
-  if (numNeighbors == 1) rv.linksPerValue = 1;
+  if (numNeighbors == 1)
+    rv.linksPerValue = 1;
+  else if (linksPerValue > 0)
+    rv.linksPerValue = linksPerValue;
 
   if (Options::cdoVerbose) cdo_print("Point search nearest: %.2f seconds", timer.elapsed());
 }  // remap_distwgt_weights
 
 template <typename T>
 static void
-remap_dist_wgt(size_t numNeighbors, RemapSearch &rsearch, const Varray<T> &srcArray, Varray<T> &tgtArray, T missval, size_t nmiss)
+remap_distwgt(size_t numNeighbors, RemapSearch &rsearch, const Varray<T> &srcArray, Varray<T> &tgtArray, T missval,
+              size_t numMissVals)
 {
   auto srcGrid = rsearch.srcGrid;
   auto tgtGrid = rsearch.tgtGrid;
 
   if (Options::cdoVerbose) cdo_print("Called %s()", __func__);
 
-  progress::init();
+  cdo::Progress progress;
 
   // Compute mappings from source to target grid
 
@@ -103,7 +114,7 @@ remap_dist_wgt(size_t numNeighbors, RemapSearch &rsearch, const Varray<T> &srcAr
   auto srcGridSize = srcGrid->size;
 
   Varray<short> srcGridMask;
-  if (nmiss) remap_set_mask(srcGridSize, srcArray, nmiss, missval, srcGridMask);
+  if (numMissVals) remap_set_mask(srcGridSize, srcArray, numMissVals, missval, srcGridMask);
 
   std::vector<knnWeightsType> knnWeights;
   knnWeights.reserve(Threading::ompNumThreads);
@@ -123,7 +134,7 @@ remap_dist_wgt(size_t numNeighbors, RemapSearch &rsearch, const Varray<T> &srcAr
       auto ompthID = cdo_omp_get_thread_num();
 
       atomicCount++;
-      if (ompthID == 0) progress::update(0, 1, (double) atomicCount / tgtGridSize);
+      if (ompthID == 0) progress.update((double) atomicCount / tgtGridSize);
 
       auto &tgtValue = tgtArray[tgtCellIndex];
       tgtValue = missval;
@@ -137,25 +148,22 @@ remap_dist_wgt(size_t numNeighbors, RemapSearch &rsearch, const Varray<T> &srcAr
       remap_search_points(rsearch, llpoint, knnWgt);
 
       // Compute weights based on inverse distance if mask is false, eliminate those points
-      auto nadds
-          = (srcGridMask.size() > 0) ? knnWgt.computeWeights(srcGridMask) : knnWgt.computeWeights();
+      auto nadds = (srcGridMask.size() > 0) ? knnWgt.computeWeights(srcGridMask) : knnWgt.computeWeights();
       if (nadds) tgtValue = knnWgt.arrayWeightsSum(srcArray);
     }
 
-  progress::update(0, 1, 1);
-
   if (Options::cdoVerbose) cdo_print("Point search nearest: %.2f seconds", timer.elapsed());
-}  // remap_dist_wgt
+}  // remap_distwgt
 
 void
-remap_dist_wgt(size_t numNeighbors, RemapSearch &rsearch, const Field &field1, Field &field2)
+remap_distwgt(size_t numNeighbors, RemapSearch &rsearch, const Field &field1, Field &field2)
 {
   if (field1.memType != field2.memType) cdo_abort("Interal error, memType of field1 and field2 differ!");
 
   if (field1.memType == MemType::Float)
-    remap_dist_wgt(numNeighbors, rsearch, field1.vec_f, field2.vec_f, (float) field1.missval, field1.nmiss);
+    remap_distwgt(numNeighbors, rsearch, field1.vec_f, field2.vec_f, (float) field1.missval, field1.numMissVals);
   else
-    remap_dist_wgt(numNeighbors, rsearch, field1.vec_d, field2.vec_d, field1.missval, field1.nmiss);
+    remap_distwgt(numNeighbors, rsearch, field1.vec_d, field2.vec_d, field1.missval, field1.numMissVals);
 }
 
 void
@@ -171,7 +179,7 @@ intgriddis(const Field &field1, Field &field2, size_t numNeighbors)
 
   if (Options::cdoVerbose) cdo_print("Called %s()", __func__);
 
-  progress::init();
+  cdo::Progress progress;
 
   // Interpolate from source to target grid
 
@@ -185,7 +193,7 @@ intgriddis(const Field &field1, Field &field2, size_t numNeighbors)
   auto tgtGridSize = remap.tgtGrid.size;
 
   Varray<short> srcGridMask;
-  if (field1.nmiss) remap_set_mask(srcGridSize, srcArray, field1.nmiss, srcMissval, srcGridMask);
+  if (field1.numMissVals) remap_set_mask(srcGridSize, srcArray, field1.numMissVals, srcMissval, srcGridMask);
 
   std::vector<knnWeightsType> knnWeights;
   knnWeights.reserve(Threading::ompNumThreads);
@@ -197,7 +205,7 @@ intgriddis(const Field &field1, Field &field2, size_t numNeighbors)
 
   // Loop over target grid
 
-  std::atomic<size_t> nmiss{ 0 };
+  std::atomic<size_t> numMissVals{ 0 };
   std::atomic<size_t> atomicCount{ 0 };
 
 #ifdef _OPENMP
@@ -208,7 +216,7 @@ intgriddis(const Field &field1, Field &field2, size_t numNeighbors)
       auto ompthID = cdo_omp_get_thread_num();
 
       atomicCount++;
-      if (ompthID == 0) progress::update(0, 1, (double) atomicCount / tgtGridSize);
+      if (ompthID == 0) progress.update((double) atomicCount / tgtGridSize);
 
       auto &tgtValue = tgtArray[tgtCellIndex];
       tgtValue = tgtMissval;
@@ -222,17 +230,14 @@ intgriddis(const Field &field1, Field &field2, size_t numNeighbors)
       remap_search_points(remap.search, llpoint, knnWgt);
 
       // Compute weights based on inverse distance if mask is false, eliminate those points
-      auto nadds
-          = (srcGridMask.size() > 0) ? knnWgt.computeWeights(srcGridMask) : knnWgt.computeWeights();
+      auto nadds = (srcGridMask.size() > 0) ? knnWgt.computeWeights(srcGridMask) : knnWgt.computeWeights();
       if (nadds)
         tgtValue = knnWgt.arrayWeightsSum(srcArray);
       else
-        nmiss++;
+        numMissVals++;
     }
 
-  progress::update(0, 1, 1);
-
-  field2.nmiss = nmiss;
+  field2.numMissVals = numMissVals;
 
   remap_grid_free(remap.srcGrid);
   remap_grid_free(remap.tgtGrid);
