@@ -869,12 +869,12 @@ remap_stat(int remapOrder, RemapGrid &srcGrid, RemapGrid &tgtGrid, RemapVars &rv
       cdo_print("  Conservation:");
       double sum = 0.0;
       for (size_t i = 0; i < srcGrid.size; ++i)
-        if (!dbl_is_equal(array1[i], mv1)) sum += array1[i] * srcGrid.cell_area[i] * srcGrid.cell_frac[i];
+        if (dbl_is_not_equal(array1[i], mv1)) sum += array1[i] * srcGrid.cell_area[i] * srcGrid.cell_frac[i];
       cdo_print("  Grid1 Integral = %g", sum);
 
       sum = 0;
       for (size_t i = 0; i < tgtGrid.size; ++i)
-        if (!dbl_is_equal(array2[i], mv2)) sum += array2[i] * tgtGrid.cell_area[i] * tgtGrid.cell_frac[i];
+        if (dbl_is_not_equal(array2[i], mv2)) sum += array2[i] * tgtGrid.cell_area[i] * tgtGrid.cell_frac[i];
       cdo_print("  Grid2 Integral = %g", sum);
     }
 
