@@ -8,22 +8,25 @@
 #define VARRAY_H
 
 #include <iostream>  // cerr
+#include <algorithm>
 #include <vector>
 #include <cstddef>
 #include <cfloat>
 #include <cassert>
+#include "cpp_lib.h"
 #include "compare.h"
 
+namespace ranges = std::ranges;
 
 // unary operators
 // clang-format off
 const auto unary_op_abs   = [](double x) noexcept { return std::fabs(x); };
-const auto unary_op_int   = [](double x) noexcept { return (int)(x); };
+const auto unary_op_int   = [](auto   x) noexcept { return (int)(x); };
 const auto unary_op_nint  = [](double x) noexcept { return std::round(x); };
 const auto unary_op_sqr   = [](double x) noexcept { return x * x; };
-const auto unary_op_nop   = [](double x) noexcept { return x; };
+const auto unary_op_nop   = [](auto   x) noexcept { return x; };
 const auto unary_op_reci  = [](double x) noexcept { return 1.0 / x; };
-const auto unary_op_not   = [](double x) noexcept { return is_equal(x, 0.0); };
+const auto unary_op_not   = [](auto   x) noexcept { return is_equal(x, 0); };
 const auto unary_op_exp   = [](double x) noexcept { return std::exp(x); };
 const auto unary_op_log   = [](double x) noexcept { return std::log(x); };
 const auto unary_op_log10 = [](double x) noexcept { return std::log10(x); };
@@ -44,8 +47,8 @@ const auto binary_op_GE  = [](auto x, auto y) noexcept { return x >= y; };
 const auto binary_op_NE  = [](auto x, auto y) noexcept { return is_not_equal(x, y); };
 const auto binary_op_EQ  = [](auto x, auto y) noexcept { return is_equal(x, y); };
 const auto binary_op_LEG = [](auto x, auto y) noexcept { return (x < y) ? -1.0 : (x > y); };
-const auto binary_op_AND = [](double x, double y) noexcept { return is_not_equal(x, 0.0) && is_not_equal(y, 0.0); };
-const auto binary_op_OR  = [](double x, double y) noexcept { return is_not_equal(x, 0.0) || is_not_equal(y, 0.0); };
+const auto binary_op_AND = [](auto x, auto y) noexcept { return is_not_equal(x, 0) && is_not_equal(y, 0); };
+const auto binary_op_OR  = [](auto x, auto y) noexcept { return is_not_equal(x, 0) || is_not_equal(y, 0); };
 const auto binary_op_POW = [](double x, double y) noexcept { return std::pow(x, y); };
 const auto binary_op_ADD = [](double x, double y) noexcept { return x + y; };
 const auto binary_op_SUB = [](double x, double y) noexcept { return x - y; };
@@ -53,7 +56,7 @@ const auto binary_op_MUL = [](double x, double y) noexcept { return x * y; };
 const auto binary_op_DIV = [](double x, double y) noexcept { return x / y; };
 // clang-format on
 
-//#define CHECK_UNUSED_VECTOR 1
+// #define CHECK_UNUSED_VECTOR 1
 
 #ifdef CHECK_UNUSED_VECTOR
 // clang-format off
@@ -76,7 +79,7 @@ CheckVector
   T * end() noexcept { return &ptr[0] + 1; }
   const T * begin() const noexcept { return &ptr[0]; }
   const T * end() const noexcept { return &ptr[0] + 1; }
-  
+
   bool empty() const { return true; }
   size_t size() const { return m_count; }
   void clear() { }
@@ -207,68 +210,11 @@ template <class T, size_t ROW, size_t COL>
 using Matrix = std::array<std::array<T, COL>, ROW>;
 */
 
-// clang-format off
-#define MADDMN(x, y) ((DBL_IS_EQUAL((x), missval1) || DBL_IS_EQUAL((y), missval2)) ? missval1 : (x) + (y))
-#define MSUBMN(x, y) ((DBL_IS_EQUAL((x), missval1) || DBL_IS_EQUAL((y), missval2)) ? missval1 : (x) - (y))
-#define MMULMN(x, y) ((DBL_IS_EQUAL((x), 0) || DBL_IS_EQUAL((y), 0)) ? 0 : (DBL_IS_EQUAL((x), missval1) || DBL_IS_EQUAL((y), missval2)) ? missval1 : (x) * (y))
-#define MDIVMN(x, y) ((DBL_IS_EQUAL((x), missval1) || DBL_IS_EQUAL((y), missval2) || DBL_IS_EQUAL((y), 0)) ? missval1 : (x) / (y))
-#define MPOWMN(x, y) ((DBL_IS_EQUAL((x), missval1) || DBL_IS_EQUAL((y), missval2)) ? missval1 : std::pow((x), (y)))
-#define MSQRTMN(x) ((DBL_IS_EQUAL((x), missval1) || (x) < 0) ? missval1 : std::sqrt(x))
-
-#define ADD(x, y) ((x) + (y))
-#define SUB(x, y) ((x) - (y))
-#define MUL(x, y) ((x) * (y))
-#define DIV(x, y) (IS_EQUAL((y), 0.) ? missval1 : (x) / (y))
-#define POW(x, y) std::pow((x), (y))
-#define SQRT(x) std::sqrt(x)
-
-#define ADDM(x, y) ((IS_EQUAL((x), missval1) || IS_EQUAL((y), missval2)) ? missval1 : (x) + (y))
-#define SUBM(x, y) ((IS_EQUAL((x), missval1) || IS_EQUAL((y), missval2)) ? missval1 : (x) - (y))
-#define MULM(x, y) ((IS_EQUAL((x), 0) || IS_EQUAL((y), 0)) ? 0 : (IS_EQUAL((x), missval1) || IS_EQUAL((y), missval2)) ? missval1 : (x) * (y))
-#define DIVM(x, y) ((IS_EQUAL((x), missval1) || IS_EQUAL((y), missval2) || IS_EQUAL((y), 0)) ? missval1 : (x) / (y))
-#define POWM(x, y) ((IS_EQUAL((x), missval1) || IS_EQUAL((y), missval2)) ? missval1 : std::pow((x), (y)))
-#define SQRTM(x) ((IS_EQUAL((x), missval1) || (x) < 0) ? missval1 : std::sqrt(x))
-
-#define ADDMN(x, y) FADDMN(x, y, missval1, missval2)
-#define SUBMN(x, y) FSUBMN(x, y, missval1, missval2)
-#define MULMN(x, y) FMULMN(x, y, missval1, missval2)
-#define DIVMN(x, y) FDIVMN(x, y, missval1, missval2)
-#define POWMN(x, y) FPOWMN(x, y, missval1, missval2)
-#define SQRTMN(x) FSQRTMN(x, missval1)
-
-static inline double FADDMN(double x, double y, double missval1, double missval2)
-{
-  return MADDMN(x, y);
-}
-static inline double FSUBMN(double x, double y, double missval1, double missval2)
-{
-  return MSUBMN(x, y);
-}
-static inline double FMULMN(double x, double y, double missval1, double missval2)
-{
-  return MMULMN(x, y);
-}
-static inline double FDIVMN(double x, double y, double missval1, double missval2)
-{
-  return MDIVMN(x, y);
-}
-static inline double FPOWMN(double x, double y, double missval1, double missval2)
-{
-  return MPOWMN(x, y);
-}
-static inline double FSQRTMN(double x, double missval1)
-{
-  return MSQRTMN(x);
-}
-// clang-format on
-
-const char *fpe_errstr(int fpeRaised);
+template <typename T>
+MinMaxSum varray_min_max_sum(size_t len, const Varray<T> &v, const MinMaxSum &mms);
 
 template <typename T>
-MinMaxSum varray_min_max_sum(size_t len, const Varray<T> &v, MinMaxSum mms);
-
-template <typename T>
-MinMaxSum varray_min_max_sum_mv(size_t len, const Varray<T> &v, T missval, MinMaxSum mms);
+MinMaxSum varray_min_max_sum_mv(size_t len, const Varray<T> &v, T missval, const MinMaxSum &mms);
 
 template <typename T>
 MinMaxMean varray_min_max_mean(size_t len, const Varray<T> &v);
@@ -282,28 +228,6 @@ MinMax array_min_max_mask(size_t len, const T *array, const Varray<int> &mask);
 void array_add_array(size_t len, double *array1, const double *array2);
 void array_add_array_mv(size_t len, double *array1, const double *array2, double missval);
 
-template <typename T1, typename T2>
-void
-varray_fill(size_t len, T1 *v, T2 value)
-{
-  T1 c = value;
-  for (size_t i = 0; i < len; ++i) v[i] = c;
-}
-
-template <typename T1, typename T2>
-void
-varray_fill(Varray<T1> &v, T2 value)
-{
-  varray_fill(v.size(), v.data(), value);
-}
-
-template <typename T1, typename T2>
-void
-varray_fill(size_t len, Varray<T1> &v, T2 value)
-{
-  varray_fill(len, v.data(), value);
-}
-
 template <typename T1, typename T2>
 void
 array_copy(size_t len, const T1 *array1, T2 *array2)
@@ -334,7 +258,7 @@ varray_transform(const Varray<T> &vIn, Varray<T> &vOut, UnaryOperation unary_op)
   assert(vOut.size() > 0);
   assert(vOut.size() <= vIn.size());
 
-  const auto len = vIn.size();
+  auto len = vIn.size();
   for (size_t i = 0; i < len; ++i) vOut[i] = unary_op(vIn[i]);
 }
 
@@ -404,27 +328,27 @@ template <typename T>
 double varray_weighted_avg_mv(size_t len, const Varray<T> &v, const Varray<double> &w, T missval);
 
 template <typename T>
-double varray_var(size_t len, const Varray<T> &v, size_t nmiss, T missval);
+double varray_var(size_t len, const Varray<T> &v, size_t numMissVals, T missval);
 
 template <typename T>
-double varray_var_1(size_t len, const Varray<T> &v, size_t nmiss, T missval);
+double varray_var_1(size_t len, const Varray<T> &v, size_t numMissVals, T missval);
 
 template <typename T>
-double varray_weighted_var(size_t len, const Varray<T> &v, const Varray<double> &w, size_t nmiss, T missval);
+double varray_weighted_var(size_t len, const Varray<T> &v, const Varray<double> &w, size_t numMissVals, T missval);
 
 template <typename T>
-double varray_weighted_var_1(size_t len, const Varray<T> &v, const Varray<double> &w, size_t nmiss, T missval);
+double varray_weighted_var_1(size_t len, const Varray<T> &v, const Varray<double> &w, size_t numMissVals, T missval);
 
 template <typename T>
-double varray_skew(size_t len, const Varray<T> &v, size_t nmiss, T missval);
+double varray_skew(size_t len, const Varray<T> &v, size_t numMissVals, T missval);
 
 template <typename T>
-double varray_kurt(size_t len, const Varray<T> &v, size_t nmiss, T missval);
+double varray_kurt(size_t len, const Varray<T> &v, size_t numMissVals, T missval);
 
 template <typename T>
-double varray_median(size_t len, const Varray<T> &v, size_t nmiss, T missval);
+double varray_median(size_t len, const Varray<T> &v, size_t numMissVals, T missval);
 
 template <typename T>
-double varray_count(size_t len, const Varray<T> &v, size_t nmiss, T missval);
+double varray_count(size_t len, const Varray<T> &v, size_t numMissVals, T missval);
 
 #endif  //  VARRAY_H
